@@ -19,6 +19,8 @@ import netaddr
 from neutron_lib import constants as lib_constants
 from oslo_log import log as logging
 from oslo_utils import excutils
+from pyroute2.netlink import exceptions \
+    as pyroute2_exc  # pylint: disable=no-name-in-module
 
 from neutron.agent.l3 import dvr_fip_ns
 from neutron.agent.l3 import dvr_router_base
@@ -45,27 +47,63 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         self.rtr_fip_connect = False
         self.fip_ns = None
         self._pending_arp_set = set()
+        self._load_used_fip_information()
 
-        self.load_used_fip_information()
+    def _load_used_fip_information(self):
+        """Load FIP from the FipRulePriorityAllocator state file.
 
-    def load_used_fip_information(self):
-        """Some information needed to remove a floating ip e.g. it's associated
-        ip rule priorities, are stored in memory to avoid extra db lookups.
-        Since this is lost on agent restart we need to reload them.
+        If, for any reason, the FIP is not stored in the state file, this
+        method reads the namespace "ip rule" list and search for the
+        corresponding fixed IP of the FIP. If present, this "ip rule" is
+        (1) deleted, (2) a new rule priority is allocated and (3) the "ip rule"
+        is written again with the new assigned priority.
+
+        At the end of the method, all existing "ip rule" registers in
+        FIP_RT_TBL table (where FIP rules are stored) that don't match with
+        any register memoized in self._rule_priorities is deleted.
         """
         ex_gw_port = self.get_ex_gw_port()
-        if ex_gw_port:
-            fip_ns = self.agent.get_fip_ns(ex_gw_port['network_id'])
+        if not ex_gw_port:
+            return
 
-            for fip in self.get_floating_ips():
-                floating_ip = fip['floating_ip_address']
-                fixed_ip = fip['fixed_ip_address']
-                rule_pr = fip_ns.lookup_rule_priority(floating_ip)
-                if rule_pr:
-                    self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
-                else:
-                    LOG.error("Rule priority not found for floating ip %s",
-                              floating_ip)
+        fip_ns = self.agent.get_fip_ns(ex_gw_port['network_id'])
+        for fip in self.get_floating_ips():
+            floating_ip = fip['floating_ip_address']
+            fixed_ip = fip['fixed_ip_address']
+            if not fixed_ip:
+                continue
+
+            rule_pr = fip_ns.lookup_rule_priority(floating_ip)
+            if rule_pr:
+                self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
+                continue
+
+            rule_pr = fip_ns.allocate_rule_priority(floating_ip)
+            ip_lib.add_ip_rule(self.ns_name, fixed_ip,
+                               table=dvr_fip_ns.FIP_RT_TBL,
+                               priority=rule_pr)
+            self.floating_ips_dict[floating_ip] = (fixed_ip, rule_pr)
+
+        self._cleanup_unused_fip_ip_rules()
+
+    def _cleanup_unused_fip_ip_rules(self):
+        if not self.router_namespace.exists():
+            # It could be a new router, thus the namespace is not created yet.
+            return
+
+        ip_rules = ip_lib.list_ip_rules(self.ns_name,
+                                        lib_constants.IP_VERSION_4)
+        ip_rules = [ipr for ipr in ip_rules
+                    if ipr['table'] == dvr_fip_ns.FIP_RT_TBL]
+        for ip_rule in ip_rules:
+            for fixed_ip, rule_pr in self.floating_ips_dict.values():
+                if (ip_rule['from'] == fixed_ip and
+                        ip_rule['priority'] == rule_pr):
+                    break
+            else:
+                ip_lib.delete_ip_rule(self.ns_name, ip_rule['from'],
+                                      table=dvr_fip_ns.FIP_RT_TBL,
+                                      priority=ip_rule['priority'])
 
     def migrate_centralized_floating_ip(self, fip, interface_name, device):
         # Remove the centralized fip first and then add fip to the host
@@ -181,10 +219,10 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
                                   priority=int(str(rule_pr)))
             self.fip_ns.deallocate_rule_priority(floating_ip)
         else:
-            LOG.error("Unable to find necessary information to complete "
-                      "removal of floating ip rules for %s - will require "
-                      "manual cleanup (see LP 1891673 for details).",
-                      floating_ip)
+            LOG.error('Floating IP %s not stored in this agent. Because of '
+                      'the initialization method '
+                      '"_load_used_fip_information", all floating IPs should '
+                      'be memoized in the local memory.', floating_ip)
 
     def floating_ip_removed_dist(self, fip_cidr):
         """Remove floating IP from FIP namespace."""
@@ -308,46 +346,56 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         device_exists = device.exists()
         return device, device_exists
 
-    def _set_subnet_arp_info(self, subnet_id):
+    def _set_subnet_arp_info(self, subnet):
         """Set ARP info retrieved from Plugin for existing ports."""
         # TODO(Carl) Can we eliminate the need to make this RPC while
         # processing a router.
-        subnet_ports = self.agent.get_ports_by_subnet(subnet_id)
+        subnet_ports = self.agent.get_ports_by_subnet(subnet['id'])
         ignored_device_owners = (
             lib_constants.ROUTER_INTERFACE_OWNERS +
             tuple(common_utils.get_dvr_allowed_address_pair_device_owners()))
-        device, device_exists = self.get_arp_related_dev(subnet_id)
+        device, device_exists = self.get_arp_related_dev(subnet['id'])
 
+        subnet_ip_version = netaddr.IPNetwork(subnet['cidr']).version
         for p in subnet_ports:
             if p['device_owner'] not in ignored_device_owners:
                 for fixed_ip in p['fixed_ips']:
-                    self._update_arp_entry(fixed_ip['ip_address'],
-                                           p['mac_address'],
-                                           subnet_id,
-                                           'add',
-                                           device=device,
-                                           device_exists=device_exists)
+                    if fixed_ip['subnet_id'] == subnet['id']:
+                        self._update_arp_entry(fixed_ip['ip_address'],
+                                               p['mac_address'],
+                                               subnet['id'],
+                                               'add',
+                                               device=device,
+                                               device_exists=device_exists)
                 for allowed_address_pair in p.get('allowed_address_pairs', []):
-                    self._update_arp_entry(allowed_address_pair['ip_address'],
-                                           allowed_address_pair['mac_address'],
-                                           subnet_id,
-                                           'add',
-                                           device=device,
-                                           device_exists=device_exists)
+                    if ('/' not in str(allowed_address_pair['ip_address']) or
+                            common_utils.is_cidr_host(
+                                allowed_address_pair['ip_address'])):
+                        ip_address = common_utils.cidr_to_ip(
+                            allowed_address_pair['ip_address'])
+                        ip_version = common_utils.get_ip_version(ip_address)
+                        if ip_version == subnet_ip_version:
+                            self._update_arp_entry(
+                                ip_address,
+                                allowed_address_pair['mac_address'],
+                                subnet['id'],
+                                'add',
+                                device=device,
+                                device_exists=device_exists)
 
         # subnet_ports does not have snat port if the port is still unbound
         # by the time this function is called. So ensure to add arp entry
         # for snat port if port details are updated in router info.
         for p in self.get_snat_interfaces():
             for fixed_ip in p['fixed_ips']:
-                if fixed_ip['subnet_id'] == subnet_id:
+                if fixed_ip['subnet_id'] == subnet['id']:
                     self._update_arp_entry(fixed_ip['ip_address'],
                                            p['mac_address'],
-                                           subnet_id,
+                                           subnet['id'],
                                            'add',
                                            device=device,
                                            device_exists=device_exists)
-        self._process_arp_cache_for_internal_port(subnet_id)
+        self._process_arp_cache_for_internal_port(subnet['id'])
 
     @staticmethod
     def _get_snat_idx(ip_cidr):
@@ -464,7 +512,7 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         # external_gateway port or the agent_mode.
         ex_gw_port = self.get_ex_gw_port()
         for subnet in port['subnets']:
-            self._set_subnet_arp_info(subnet['id'])
+            self._set_subnet_arp_info(subnet)
             if ex_gw_port:
                 # Check for address_scopes here if gateway exists.
                 address_scopes_match = self._check_if_address_scopes_match(
@@ -669,10 +717,13 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
         if not ext_device_name:
             return ports_scopemark
 
-        ext_scope = self._get_external_address_scope()
-        ext_scope_mark = self.get_address_scope_mark_mask(ext_scope)
-        ports_scopemark[lib_constants.IP_VERSION_4][ext_device_name] = (
-            ext_scope_mark)
+        ext_scope_mark = self._get_port_devicename_scopemark(
+                [ext_port], self.get_internal_device_name,
+                interface_name=ext_device_name)
+        for ip_version in (lib_constants.IP_VERSION_4,
+                           lib_constants.IP_VERSION_6):
+            ports_scopemark[ip_version].update(
+                ext_scope_mark[ip_version])
         return ports_scopemark
 
     def _check_if_floatingip_bound_to_host(self, fip):
@@ -817,15 +868,17 @@ class DvrLocalRouter(dvr_router_base.DvrRouterBase):
 
     def _update_fip_route_table_with_next_hop_routes(self, operation, route,
                                                      fip_ns_name, tbl_index):
-        cmd = ['ip', 'route', operation, 'to', route['destination'],
-               'via', route['nexthop'], 'table', tbl_index]
-        ip_wrapper = ip_lib.IPWrapper(namespace=fip_ns_name)
-        if ip_wrapper.netns.exists(fip_ns_name):
-            ip_wrapper.netns.execute(cmd, check_exit_code=False)
-        else:
+        cmd = (ip_lib.add_ip_route if operation == 'replace' else
+               ip_lib.delete_ip_route)
+        try:
+            cmd(fip_ns_name, route['destination'], via=route['nexthop'],
+                table=tbl_index, proto='boot')
+        except priv_ip_lib.NetworkNamespaceNotFound:
             LOG.debug("The FIP namespace %(ns)s does not exist for "
                       "router %(id)s",
                       {'ns': fip_ns_name, 'id': self.router_id})
+        except (OSError, pyroute2_exc.NetlinkError):
+            pass
 
     def _check_if_route_applicable_to_fip_namespace(self, route,
                                                     agent_gateway_port):
