@@ -35,7 +35,6 @@ from oslo_utils import fileutils
 from oslo_utils import netutils
 from oslo_utils import uuidutils
 
-from neutron._i18n import _
 from neutron.agent.common import utils as agent_common_utils
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
@@ -171,6 +170,14 @@ class NetModel(DictModel):
     def namespace(self):
         return self._ns_name
 
+    # TODO(ralonsoh): remove in Z+.
+    @property
+    def project_id(self):
+        try:
+            return self['project_id']
+        except KeyError:
+            return self['tenant_id']
+
 
 class DhcpBase(object, metaclass=abc.ABCMeta):
 
@@ -195,7 +202,8 @@ class DhcpBase(object, metaclass=abc.ABCMeta):
         self.disable(retain_port=True, block=True)
         self.enable()
 
-    @abc.abstractproperty
+    @property
+    @abc.abstractmethod
     def active(self):
         """Boolean representing the running state of the DHCP server."""
 
@@ -320,17 +328,13 @@ class DhcpLocalProcess(DhcpBase, metaclass=abc.ABCMeta):
     def _get_value_from_conf_file(self, kind, converter=None):
         """A helper function to read a value from one of the state files."""
         file_name = self.get_conf_file_name(kind)
-        msg = _('Error while reading %s')
-
         try:
             with open(file_name, 'r') as f:
-                try:
-                    return converter(f.read()) if converter else f.read()
-                except ValueError:
-                    msg = _('Unable to convert value in %s')
+                return converter(f.read()) if converter else f.read()
+        except ValueError:
+            msg = "Unable to convert value in %s"
         except IOError:
-            msg = _('Unable to access %s')
-
+            msg = "Unable to access %s"
         LOG.debug(msg, file_name)
         return None
 
@@ -675,7 +679,7 @@ class Dnsmasq(DhcpLocalProcess):
 
         In dnsmasq it is not possible to configure two dhcp-host
         entries mapped to a single client mac address with IP
-        addresses in the same subnet. When recieving a requst
+        addresses in the same subnet. When receiving a request
         dnsmasq will match on the first entry in it's config,
         and lease that address. The second entry will never be
         used.
@@ -1149,10 +1153,8 @@ class Dnsmasq(DhcpLocalProcess):
                  addr_mode == constants.IPV6_SLAAC)):
                 continue
             if subnet.dns_nameservers:
-                if ((subnet.ip_version == 4 and
-                     subnet.dns_nameservers == ['0.0.0.0']) or
-                    (subnet.ip_version == 6 and
-                     subnet.dns_nameservers == ['::'])):
+                if common_utils.is_dns_servers_any_address(
+                        subnet.dns_nameservers, subnet.ip_version):
                     # Special case: Do not announce DNS servers
                     options.append(
                         self._format_option(
@@ -1311,6 +1313,7 @@ class Dnsmasq(DhcpLocalProcess):
     def _format_option(self, ip_version, tag, option, *args):
         """Format DHCP option by option name or code."""
         option = str(option)
+        option = option.split("\n", 1)[0]
         pattern = "(tag:(.*),)?(.*)$"
         matches = re.match(pattern, option)
         extra_tag = matches.groups()[0]
@@ -1434,7 +1437,8 @@ class DeviceManager(object):
             skip_subnet = (
                 subnet.ip_version != ip_version or
                 not subnet.enable_dhcp or
-                subnet.gateway_ip is None)
+                subnet.gateway_ip is None or
+                subnet.subnetpool_id == constants.IPV6_PD_POOL_ID)
 
             if skip_subnet:
                 continue
@@ -1587,7 +1591,7 @@ class DeviceManager(object):
             admin_state_up=True,
             device_id=device_id,
             network_id=network.id,
-            tenant_id=network.tenant_id,
+            project_id=network.project_id,
             fixed_ips=unique_ip_subnets)
         return self.plugin.create_dhcp_port({'port': port_dict})
 
