@@ -14,6 +14,7 @@
 #    under the License.
 
 import errno
+from os import path
 import re
 import threading
 import time
@@ -25,16 +26,19 @@ from neutron_lib import exceptions
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import netutils
-from pyroute2.netlink import exceptions as netlink_exceptions
-from pyroute2.netlink import rtnl
-from pyroute2.netlink.rtnl import ifaddrmsg
-from pyroute2.netlink.rtnl import ifinfmsg
-from pyroute2 import netns
+from pyroute2.netlink import exceptions \
+    as netlink_exceptions  # pylint: disable=no-name-in-module
+from pyroute2.netlink import rtnl  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import \
+    ifaddrmsg  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import ifinfmsg  # pylint: disable=no-name-in-module
+from pyroute2 import netns  # pylint: disable=no-name-in-module
 
 from neutron._i18n import _
 from neutron.agent.common import utils
 from neutron.common import utils as common_utils
 from neutron.privileged.agent.linux import ip_lib as privileged
+from neutron.privileged.agent.linux import utils as priv_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -135,7 +139,7 @@ class SubProcessBase(object):
         opt_list = ['-%s' % o for o in options]
         ip_cmd = add_namespace_to_cmd(['ip'], namespace)
         cmd = ip_cmd + opt_list + [command] + list(args)
-        return utils.execute(cmd, run_as_root=run_as_root,
+        return utils.execute(cmd, run_as_root=run_as_root, privsep_exec=True,
                              log_fail_as_error=self.log_fail_as_error)
 
     def set_log_fail_as_error(self, fail_with_error):
@@ -274,11 +278,20 @@ class IPWrapper(SubProcessBase):
                                     vlan_id=vlan_id)
         return IPDevice(name, namespace=self.namespace)
 
-    def add_vxlan(self, name, vni, group=None, dev=None, ttl=None, tos=None,
+    def add_vxlan(self, name, vni, dev, group=None, ttl=None, tos=None,
                   local=None, srcport=None, dstport=None, proxy=False):
-        kwargs = {'vxlan_id': vni}
+        kwargs = {'vxlan_id': vni,
+                  'physical_interface': dev}
         if group:
-            kwargs['vxlan_group'] = group
+            try:
+                ip_version = common_utils.get_ip_version(group)
+                if ip_version == constants.IP_VERSION_6:
+                    kwargs['vxlan_group6'] = group
+                else:
+                    kwargs['vxlan_group'] = group
+            except netaddr.core.AddrFormatError:
+                err_msg = _("Invalid group address: %s") % group
+                raise exceptions.InvalidInput(error_message=err_msg)
         if dev:
             kwargs['physical_interface'] = dev
         if ttl:
@@ -286,7 +299,15 @@ class IPWrapper(SubProcessBase):
         if tos:
             kwargs['vxlan_tos'] = tos
         if local:
-            kwargs['vxlan_local'] = local
+            try:
+                ip_version = common_utils.get_ip_version(local)
+                if ip_version == constants.IP_VERSION_6:
+                    kwargs['vxlan_local6'] = local
+                else:
+                    kwargs['vxlan_local'] = local
+            except netaddr.core.AddrFormatError:
+                err_msg = _("Invalid local address: %s") % local
+                raise exceptions.InvalidInput(error_message=err_msg)
         if proxy:
             kwargs['vxlan_proxy'] = proxy
         # tuple: min,max
@@ -354,7 +375,7 @@ class IPDevice(SubProcessBase):
         try:
             ip_wrapper.netns.execute(["conntrack", "-D", "-d", ip_str],
                                      check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
 
         except RuntimeError:
             LOG.exception("Failed deleting ingress connection state of"
@@ -364,7 +385,7 @@ class IPDevice(SubProcessBase):
         try:
             ip_wrapper.netns.execute(["conntrack", "-D", "-q", ip_str],
                                      check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
         except RuntimeError:
             LOG.exception("Failed deleting egress connection state of"
                           " floatingip %s", ip_str)
@@ -376,7 +397,7 @@ class IPDevice(SubProcessBase):
                '--dport', dport]
         try:
             ip_wrapper.netns.execute(cmd, check_exit_code=True,
-                                     extra_ok_codes=[1])
+                                     extra_ok_codes=[1], privsep_exec=True)
 
         except RuntimeError:
             LOG.exception("Failed deleting ingress connection state of "
@@ -401,23 +422,11 @@ class IPDevice(SubProcessBase):
         self._name = name
 
 
-class IpCommandBase(object):
-    COMMAND = ''
+class IpDeviceCommandBase(object):
 
     def __init__(self, parent):
         self._parent = parent
 
-    def _run(self, options, args):
-        return self._parent._run(options, self.COMMAND, args)
-
-    def _as_root(self, options, args, use_root_namespace=False):
-        return self._parent._as_root(options,
-                                     self.COMMAND,
-                                     args,
-                                     use_root_namespace=use_root_namespace)
-
-
-class IpDeviceCommandBase(IpCommandBase):
     @property
     def name(self):
         return self._parent.name
@@ -428,7 +437,6 @@ class IpDeviceCommandBase(IpCommandBase):
 
 
 class IpLinkCommand(IpDeviceCommandBase):
-    COMMAND = 'link'
 
     def set_address(self, mac_address):
         privileged.set_link_attribute(
@@ -454,6 +462,8 @@ class IpLinkCommand(IpDeviceCommandBase):
         privileged.set_link_attribute(
             self.name, self._parent.namespace, net_ns_fd=namespace)
         self._parent.namespace = namespace
+        common_utils.wait_until_true(lambda: self.exists, timeout=5,
+                                     sleep=0.5)
 
     def set_name(self, name):
         privileged.set_link_attribute(
@@ -516,12 +526,11 @@ class IpLinkCommand(IpDeviceCommandBase):
         return privileged.get_link_vfs(self.name, self._parent.namespace)
 
     def set_vf_feature(self, vf_config):
-        return privileged.set_link_vf_feature(
-            self.name, self._parent.namespace, vf_config)
+        privileged.set_link_vf_feature(self.name, self._parent.namespace,
+                                       vf_config)
 
 
 class IpAddrCommand(IpDeviceCommandBase):
-    COMMAND = 'addr'
 
     def add(self, cidr, scope='global', add_broadcast=True):
         add_ip_address(cidr, self.name, self._parent.namespace, scope,
@@ -594,7 +603,6 @@ class IpAddrCommand(IpDeviceCommandBase):
 
 
 class IpRouteCommand(IpDeviceCommandBase):
-    COMMAND = 'route'
 
     def __init__(self, parent, table=None):
         super(IpRouteCommand, self).__init__(parent)
@@ -657,13 +665,13 @@ class IPRoute(SubProcessBase):
 
 
 class IpNeighCommand(IpDeviceCommandBase):
-    COMMAND = 'neigh'
 
-    def add(self, ip_address, mac_address, **kwargs):
+    def add(self, ip_address, mac_address, nud_state=None, **kwargs):
         add_neigh_entry(ip_address,
                         mac_address,
                         self.name,
-                        self._parent.namespace,
+                        namespace=self._parent.namespace,
+                        nud_state=nud_state,
                         **kwargs)
 
     def delete(self, ip_address, mac_address, **kwargs):
@@ -685,21 +693,33 @@ class IpNeighCommand(IpDeviceCommandBase):
         Given address entry is removed from neighbour cache (ARP or NDP). To
         flush all entries pass string 'all' as an address.
 
+        From https://man.archlinux.org/man/core/iproute2/ip-neighbour.8.en:
+          "the default neighbour states to be flushed do not include permanent
+           and noarp".
+
         :param ip_version: Either 4 or 6 for IPv4 or IPv6 respectively
-        :param ip_address: The prefix selecting the neighbours to flush
+        :param ip_address: The prefix selecting the neighbours to flush or
+                           "all"
         """
-        # NOTE(haleyb): There is no equivalent to 'flush' in pyroute2
-        self._as_root([ip_version], ('flush', 'to', ip_address))
+        cidr = netaddr.IPNetwork(ip_address) if ip_address != 'all' else None
+        for entry in self.dump(ip_version):
+            if entry['state'] in ('permanent', 'noarp'):
+                continue
+            if ip_address == 'all' or entry['dst'] in cidr:
+                self.delete(entry['dst'], entry['lladdr'])
 
 
-class IpNetnsCommand(IpCommandBase):
-    COMMAND = 'netns'
+class IpNetnsCommand(object):
+
+    def __init__(self, parent):
+        self._parent = parent
 
     def add(self, name):
         create_network_namespace(name)
         wrapper = IPWrapper(namespace=name)
         wrapper.netns.execute(['sysctl', '-w',
-                               'net.ipv4.conf.all.promote_secondaries=1'])
+                               'net.ipv4.conf.all.promote_secondaries=1'],
+                              privsep_exec=True)
         return wrapper
 
     def delete(self, name):
@@ -707,7 +727,7 @@ class IpNetnsCommand(IpCommandBase):
 
     def execute(self, cmds, addl_env=None, check_exit_code=True,
                 log_fail_as_error=True, extra_ok_codes=None,
-                run_as_root=False):
+                run_as_root=False, privsep_exec=False):
         ns_params = []
         if self._parent.namespace:
             run_as_root = True
@@ -721,7 +741,8 @@ class IpNetnsCommand(IpCommandBase):
         return utils.execute(cmd, check_exit_code=check_exit_code,
                              extra_ok_codes=extra_ok_codes,
                              log_fail_as_error=log_fail_as_error,
-                             run_as_root=run_as_root)
+                             run_as_root=run_as_root,
+                             privsep_exec=privsep_exec)
 
     def exists(self, name):
         return network_namespace_exists(name)
@@ -824,37 +845,27 @@ def flush_ip_addresses(ip_version, device, namespace=None):
     privileged.flush_ip_addresses(ip_version, device, namespace)
 
 
-def get_routing_table(ip_version, namespace=None):
-    """Return a list of dictionaries, each representing a route.
-
-    @param ip_version: the routes of version to return, for example 4
-    @param namespace
-    @return: a list of dictionaries, each representing a route.
-    The dictionary format is: {'destination': cidr,
-                               'nexthop': ip,
-                               'device': device_name,
-                               'scope': scope}
-    """
-    # oslo.privsep turns lists to tuples in its IPC code. Change it back
-    return list(privileged.get_routing_table(ip_version, namespace))
-
-
 # NOTE(haleyb): These neighbour functions live outside the IpNeighCommand
 # class since not all callers require it.
-def add_neigh_entry(ip_address, mac_address, device, namespace=None, **kwargs):
+def add_neigh_entry(ip_address, mac_address, device, namespace=None,
+                    nud_state=None, **kwargs):
     """Add a neighbour entry.
 
     :param ip_address: IP address of entry to add
     :param mac_address: MAC address of entry to add
     :param device: Device name to use in adding entry
     :param namespace: The name of the namespace in which to add the entry
+    :param nud_state: The NUD (Neighbour Unreachability Detection) state of
+                      the entry; defaults to "permanent"
     """
     ip_version = common_utils.get_ip_version(ip_address)
+    nud_state = nud_state or 'permanent'
     privileged.add_neigh_entry(ip_version,
                                ip_address,
                                mac_address,
                                device,
                                namespace,
+                               nud_state,
                                **kwargs)
 
 
@@ -931,9 +942,16 @@ def network_namespace_exists(namespace, try_is_ready=False, **kwargs):
                          is ready to be operated.
     :param kwargs: Callers add any filters they use as kwargs
     """
+    if not namespace:
+        return False
+
     if not try_is_ready:
-        output = list_network_namespaces(**kwargs)
-        return namespace in output
+        nspath = kwargs.get('nspath') or netns.NETNS_RUN_DIR
+        nspath += '/' + namespace
+        if cfg.CONF.AGENT.use_helper_for_ns_read:
+            return priv_utils.path_exists(nspath)
+        else:
+            return path.exists(nspath)
 
     try:
         privileged.open_namespace(namespace)
@@ -958,8 +976,8 @@ def ensure_device_is_ready(device_name, namespace=None):
         # Ensure the device has a MAC address and is up, even if it is already
         # up.
         if not dev.link.exists or not dev.link.address:
-            LOG.error("Device %s cannot be used as it has no MAC "
-                      "address", device_name)
+            LOG.info("Device %s cannot be used as it has no MAC "
+                     "address", device_name)
             return False
         dev.link.set_up()
     except RuntimeError:
@@ -1014,7 +1032,8 @@ def _arping(ns_name, iface_name, address, count, log_exception):
                           '-w', 2, address]
             try:
                 ip_wrapper.netns.execute(arping_cmd,
-                                         extra_ok_codes=extra_ok_codes)
+                                         extra_ok_codes=extra_ok_codes,
+                                         privsep_exec=True)
             except Exception as exc:
                 # Since this is spawned in a thread and executed 2 seconds
                 # apart, something may have been deleted while we were
@@ -1024,8 +1043,8 @@ def _arping(ns_name, iface_name, address, count, log_exception):
                                                         [address],
                                                         mac=None,
                                                         namespace=ns_name)
-                msg = _("Failed sending gratuitous ARP to %(addr)s on "
-                        "%(iface)s in namespace %(ns)s: %(err)s")
+                msg = ("Failed sending gratuitous ARP to %(addr)s on "
+                       "%(iface)s in namespace %(ns)s: %(err)s")
                 logger_method = LOG.exception
                 if not (log_exception and (first or exists)):
                     logger_method = LOG.info
@@ -1101,7 +1120,8 @@ def sysctl(cmd, namespace=None, log_fail_as_error=True):
     ip_wrapper = IPWrapper(namespace=namespace)
     try:
         ip_wrapper.netns.execute(cmd, run_as_root=True,
-                                 log_fail_as_error=log_fail_as_error)
+                                 log_fail_as_error=log_fail_as_error,
+                                 privsep_exec=True)
     except RuntimeError as rte:
         LOG.warning(
             "Setting %(cmd)s in namespace %(ns)s failed: %(err)s.",
@@ -1127,7 +1147,8 @@ def get_ip_nonlocal_bind(namespace=None):
     """Get kernel option value of ip_nonlocal_bind in given namespace."""
     cmd = ['sysctl', '-bn', IP_NONLOCAL_BIND]
     ip_wrapper = IPWrapper(namespace)
-    return int(ip_wrapper.netns.execute(cmd, run_as_root=True))
+    return int(ip_wrapper.netns.execute(cmd, run_as_root=True,
+                                        privsep_exec=True))
 
 
 def set_ip_nonlocal_bind(value, namespace=None, log_fail_as_error=True):
@@ -1162,7 +1183,8 @@ def get_ipv6_forwarding(device, namespace=None):
     """Get kernel value of IPv6 forwarding for device in given namespace."""
     cmd = ['sysctl', '-b', "net.ipv6.conf.%s.forwarding" % device]
     ip_wrapper = IPWrapper(namespace)
-    return int(ip_wrapper.netns.execute(cmd, run_as_root=True))
+    return int(ip_wrapper.netns.execute(cmd, run_as_root=True,
+                                        privsep_exec=True))
 
 
 def _parse_ip_rule(rule, ip_version):
@@ -1280,9 +1302,13 @@ def _exist_ip_rule(rules, ip, iif, table, priority, to):
     return True
 
 
-def add_ip_rule(namespace, ip, iif=None, table=None, priority=None, to=None):
+def add_ip_rule(namespace, ip, iif=None, table=IP_RULE_TABLES['default'],
+                priority=None, to=None):
     """Create an IP rule in a namespace
 
+    "table" parameter cannot be an empty value (None). By default, this method
+    will assign "IP_RULE_TABLES['default']" value if None is passed.
+
     :param namespace: (string) namespace name
     :param ip: (string) source IP or CIDR address (IPv4, IPv6)
     :param iif: (Optional) (string) input interface name
@@ -1290,6 +1316,7 @@ def add_ip_rule(namespace, ip, iif=None, table=None, priority=None, to=None):
     :param priority: (Optional) (string, int) rule priority
     :param to: (Optional) (string) destination IP or CIDR address (IPv4, IPv6)
     """
+    table = table if table is not None else IP_RULE_TABLES['default']
     ip_version = common_utils.get_ip_version(ip)
     rules = list_ip_rules(namespace, ip_version)
     if _exist_ip_rule(rules, ip, iif, table, priority, to):
@@ -1452,6 +1479,8 @@ def ip_monitor(namespace, queue, event_stop, event_started):
             while True:
                 ip_addresses = _ip.get()
                 for ip_address in ip_addresses:
+                    LOG.debug("IP monitor %s; Adding IP address: %s "
+                              "to the queue.", namespace, ip_address)
                     _queue.put(ip_address)
         except EOFError:
             pass
@@ -1471,6 +1500,8 @@ def ip_monitor(namespace, queue, event_stop, event_started):
         while not event_stop.is_set():
             try:
                 ip_address = _queue.get(timeout=1)
+                LOG.debug("IP monitor %s; IP address to process: %s",
+                          namespace, ip_address)
             except eventlet.queue.Empty:
                 continue
             if 'index' in ip_address and 'prefixlen' in ip_address:
@@ -1480,6 +1511,8 @@ def ip_monitor(namespace, queue, event_stop, event_started):
                 if not name:
                     continue
 
+                LOG.debug("IP monitor %s; Queueing IP address: %s; device: %s",
+                          namespace, ip_address, name)
                 cache_devices[index] = name
                 queue.put(_parse_ip_address(ip_address, name))
 
@@ -1493,14 +1526,14 @@ def ip_monitor(namespace, queue, event_stop, event_started):
 
 
 def add_ip_route(namespace, cidr, device=None, via=None, table=None,
-                 metric=None, scope=None, **kwargs):
+                 metric=None, scope=None, proto='static', **kwargs):
     """Add an IP route"""
     if table:
         table = IP_RULE_TABLES.get(table, table)
     ip_version = common_utils.get_ip_version(cidr or via)
     privileged.add_ip_route(namespace, cidr, ip_version,
                             device=device, via=via, table=table,
-                            metric=metric, scope=scope, **kwargs)
+                            metric=metric, scope=scope, proto=proto, **kwargs)
 
 
 def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
@@ -1510,6 +1543,14 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
         for device in (d for d in devices if d['index'] == index):
             return get_attr(device, 'IFLA_IFNAME')
 
+    def get_proto(proto_number):
+        if isinstance(proto_number, int) and proto_number in rtnl.rt_proto:
+            return rtnl.rt_proto[proto_number]
+        elif isinstance(proto_number, str) and proto_number.isnumeric():
+            return rtnl.rt_proto[int(proto_number)]
+        elif str(proto_number) in constants.IP_PROTOCOL_NUM_TO_NAME_MAP:
+            return constants.IP_PROTOCOL_NUM_TO_NAME_MAP[str(proto_number)]
+
     table = table if table else 'main'
     table = IP_RULE_TABLES.get(table, table)
     routes = privileged.list_ip_routes(namespace, ip_version, device=device,
@@ -1525,16 +1566,30 @@ def list_ip_routes(namespace, ip_version, scope=None, via=None, table=None,
         table = int(get_attr(route, 'RTA_TABLE'))
         metric = (get_attr(route, 'RTA_PRIORITY') or
                   IP_ROUTE_METRIC_DEFAULT[ip_version])
+        proto = get_proto(route['proto'])
         value = {
             'table': IP_RULE_TABLES_NAMES.get(table, table),
             'source_prefix': get_attr(route, 'RTA_PREFSRC'),
             'cidr': cidr,
             'scope': IP_ADDRESS_SCOPE[int(route['scope'])],
-            'device': get_device(int(get_attr(route, 'RTA_OIF')), devices),
-            'via': get_attr(route, 'RTA_GATEWAY'),
             'metric': metric,
+            'proto': proto,
         }
 
+        multipath = get_attr(route, 'RTA_MULTIPATH')
+        if multipath:
+            value['device'] = None
+            mp_via = []
+            for mp in multipath:
+                mp_via.append({'device': get_device(int(mp['oif']), devices),
+                               'via': get_attr(mp, 'RTA_GATEWAY'),
+                               'weight': int(mp['hops']) + 1})
+            value['via'] = mp_via
+        else:
+            value['device'] = get_device(int(get_attr(route, 'RTA_OIF')),
+                                         devices)
+            value['via'] = get_attr(route, 'RTA_GATEWAY')
+
         ret.append(value)
 
     if scope:
