@@ -307,14 +307,8 @@ class IptablesManager(object):
     # Flag to denote iptables supports --random-fully argument
     _random_fully = None
 
-    def __init__(self, _execute=None, state_less=False, use_ipv6=False,
-                 nat=True, namespace=None, binary_name=binary_name,
-                 external_lock=True):
-        if _execute:
-            self.execute = _execute
-        else:
-            self.execute = linux_utils.execute
-
+    def __init__(self, state_less=False, use_ipv6=False, nat=True,
+                 namespace=None, binary_name=binary_name, external_lock=True):
         self.use_ipv6 = use_ipv6
         self.namespace = namespace
         self.iptables_apply_deferred = False
@@ -484,12 +478,14 @@ class IptablesManager(object):
         args = ['iptables-save', '-t', table]
         if self.namespace:
             args = ['ip', 'netns', 'exec', self.namespace] + args
-        return self.execute(args, run_as_root=True).split('\n')
+        return linux_utils.execute(args, run_as_root=True,
+                                   privsep_exec=True).split('\n')
 
     def _get_version(self):
         # Output example is "iptables v1.6.2"
         args = ['iptables', '--version']
-        version = str(self.execute(args, run_as_root=True).split()[1][1:])
+        version = str(linux_utils.execute(
+            args, run_as_root=True, privsep_exec=True).split()[1][1:])
         LOG.debug("IPTables version installed: %s", version)
         return version
 
@@ -501,6 +497,10 @@ class IptablesManager(object):
         version = self._get_version()
         self.__class__._random_fully = utils.is_version_greater_equal(
             version, n_const.IPTABLES_RANDOM_FULLY_VERSION)
+
+        self._random_fully = self._random_fully and \
+            cfg.CONF.AGENT.use_random_fully
+
         return self._random_fully
 
     @property
@@ -514,8 +514,8 @@ class IptablesManager(object):
             args += ['-w', self.xlock_wait_time, '-W', XLOCK_WAIT_INTERVAL]
         try:
             kwargs = {} if lock else {'log_fail_as_error': False}
-            self.execute(args, process_input='\n'.join(commands),
-                         run_as_root=True, **kwargs)
+            linux_utils.execute(args, process_input='\n'.join(commands),
+                                run_as_root=True, privsep_exec=True, **kwargs)
         except RuntimeError as error:
             return error
 
@@ -577,7 +577,8 @@ class IptablesManager(object):
             if self.namespace:
                 args = ['ip', 'netns', 'exec', self.namespace] + args
             try:
-                save_output = self.execute(args, run_as_root=True)
+                save_output = linux_utils.execute(args, run_as_root=True,
+                                                  privsep_exec=True)
             except RuntimeError:
                 # We could be racing with a cron job deleting namespaces.
                 # It is useless to try to apply iptables rules over and
@@ -778,7 +779,16 @@ class IptablesManager(object):
                 args.append('-Z')
             if self.namespace:
                 args = ['ip', 'netns', 'exec', self.namespace] + args
-            current_table = self.execute(args, run_as_root=True)
+
+            # Execute iptables command in the linux host.
+            # When routers migrate from a host,an exception might happen here,
+            # and we do not care about it. Therefore, we do not need to log
+            # this error in production environments. Only when debug mode is
+            # enabled is that we need to log the error. This is used to avoid
+            # generating alarms that will be ignored by  operators.
+            current_table = linux_utils.execute(
+                args, run_as_root=True, privsep_exec=True,
+                log_fail_as_error=cfg.CONF.debug)
             current_lines = current_table.split('\n')
 
             for line in current_lines[2:]:
@@ -844,12 +854,28 @@ def _get_rules_by_chain(rules):
     return by_chain
 
 
+def _ensure_all_mac_addresses_are_uppercase(rules):
+    new_rules = []
+    lowercase_mac_regex = re.compile(r"(?:[0-9a-f]{2}[:]){5}(?:[0-9a-f]{2})")
+    callback = lambda pat: pat.group(0).upper()
+    for rule in rules:
+        new_rules.append(re.sub(lowercase_mac_regex, callback, rule))
+    return new_rules
+
+
 def _generate_chain_diff_iptables_commands(chain, old_chain_rules,
                                            new_chain_rules):
     # keep track of the old index because we have to insert rules
     # in the right position
     old_index = 1
     statements = []
+    # NOTE(slaweq): Different operating systems may return MAC addresses in the
+    # itables-save's output in lowercase or uppercase. As
+    # neutron.agent.linux.iptables_firewall module always keeps rules with MAC
+    # addresses with uppercase, we need to ensure here that all rules are
+    # stored in the same way
+    old_chain_rules = _ensure_all_mac_addresses_are_uppercase(old_chain_rules)
+    new_chain_rules = _ensure_all_mac_addresses_are_uppercase(new_chain_rules)
     for line in difflib.ndiff(old_chain_rules, new_chain_rules):
         if line.startswith('?'):
             # skip ? because that's a guide string for intraline differences
