@@ -16,6 +16,8 @@
 import collections
 import contextlib
 import copy
+import itertools
+import re
 
 import eventlet
 import netaddr
@@ -23,6 +25,7 @@ from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
 from neutron_lib import constants as lib_const
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
 from neutron_lib.plugins import utils as p_utils
 from neutron_lib.utils import helpers
 from oslo_config import cfg
@@ -37,10 +40,9 @@ from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import iptables
 from neutron.agent.linux.openvswitch_firewall import rules
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
-        as ovs_consts
 
 LOG = logging.getLogger(__name__)
+CONJ_ID_REGEX = re.compile(r"conj_id=(\d+),")
 
 
 def _replace_register(flow_params, register_number, register_value):
@@ -149,7 +151,7 @@ class SecurityGroup(object):
                 else:
                     rule['protocol'] = lib_const.IP_PROTOCOL_MAP.get(
                         protocol, protocol)
-            if 'remote_group_id' in rule:
+            if 'remote_group_id' in rule or 'remote_address_group_id' in rule:
                 self.remote_rules.append(rule)
             else:
                 self.raw_rules.append(rule)
@@ -262,33 +264,75 @@ class SGPortMap(object):
 class ConjIdMap(object):
     """Handle conjunction ID allocations and deallocations."""
 
-    def __new__(cls):
+    CONJ_ID_BLOCK_SIZE = 8
+    MAX_CONJ_ID = 2 ** 32 - 8
+
+    def __new__(cls, int_br):
         if not hasattr(cls, '_instance'):
             cls._instance = super(ConjIdMap, cls).__new__(cls)
         return cls._instance
 
-    def __init__(self):
+    def __init__(self, int_br):
         self.id_map = collections.defaultdict(self._conj_id_factory)
-        # Stores the set of conjuntion IDs used for each unique tuple
-        # (sg_id, remote_sg_id, direction, ethertype). Each tuple can have up
-        # to 8 conjuntion IDs (see ConjIPFlowManager.add()).
+        # Stores the set of conjunction IDs used for each unique tuple
+        # (sg_id, remote_id, direction, ethertype). Each tuple
+        # can have up to 8 conjunction IDs (see ConjIPFlowManager.add()).
         self.id_map_group = collections.defaultdict(set)
         self.id_free = collections.deque()
-        self.max_id = 0
+        self._max_id = self._init_max_id(int_br)
+
+    def _init_max_id(self, int_br):
+        """Read the maximum conjunction ID number in the integration bridge
+
+        This method will dump all integration bridge flows, parse them and
+        return the maximum conjunction ID number. By default, "int_br" is a
+        ``OVSAgentBridge`` instance, using "os-ken" library to access to the OF
+        rules.
+        If not, "int_br" will default to a ``OVSBridge`` instance. The CLI
+        command "ovs-ofctl" will be used instead.
+
+        :param int_br: ``OVSAgentBridge`` or ``OVSBridge`` instance.
+        :returns: The maximum conjunction ID number in the integration bridge
+        """
+        conj_id_max = 0
+        try:
+            for flow in itertools.chain(
+                    *[int_br.dump_flows(table)
+                      for table in ovs_consts.OVS_FIREWALL_TABLES]):
+                conj_id_max = max(conj_id_max, flow.match.get('conj_id', 0))
+        except AttributeError:  # br_int is a ``OVSBridge`` instance.
+            flows_iter = itertools.chain(
+                *[int_br.dump_flows_for_table(table)
+                  for table in ovs_consts.OVS_FIREWALL_TABLES])
+            conj_ids = CONJ_ID_REGEX.findall(" | ".join(flows_iter))
+            try:
+                conj_id_max = max([int(conj_id) for conj_id in conj_ids])
+            except ValueError:
+                conj_id_max = 0
+
+        max_id = conj_id_max - conj_id_max % self.CONJ_ID_BLOCK_SIZE
+        return self._next_max_id(max_id)
+
+    def _next_max_id(self, max_id):
+        max_id += self.CONJ_ID_BLOCK_SIZE
+        if max_id >= self.MAX_CONJ_ID:
+            max_id = 0
+        return max_id
 
     def _conj_id_factory(self):
         # If there is any freed ID, use one.
         if self.id_free:
             return self.id_free.popleft()
         # Allocate new one. It must be divisible by 8. (See the next function.)
-        self.max_id += 8
-        return self.max_id
+        self._max_id = self._next_max_id(self._max_id)
+        return self._max_id
 
-    def get_conj_id(self, sg_id, remote_sg_id, direction, ethertype):
+    def get_conj_id(self, sg_id, remote_id, direction, ethertype):
         """Return a conjunction ID specified by the arguments.
-        Allocate one if necessary.  The returned ID is divisible by 8,
-        as there are 4 priority levels (see rules.flow_priority_offset)
-        and 2 conjunction IDs are needed per priority.
+        Allocate one if necessary.  The returned ID is divisible by 8
+        (CONJ_ID_BLOCK_SIZE), as there are 4 priority levels
+        (see rules.flow_priority_offset) and 2 conjunction IDs are needed per
+        priority.
         """
         if direction not in [lib_const.EGRESS_DIRECTION,
                              lib_const.INGRESS_DIRECTION]:
@@ -296,7 +340,7 @@ class ConjIdMap(object):
         if ethertype not in [lib_const.IPv4, lib_const.IPv6]:
             raise ValueError(_("Invalid ethertype '%s'") % ethertype)
 
-        return self.id_map[(sg_id, remote_sg_id, direction, ethertype)]
+        return self.id_map[(sg_id, remote_id, direction, ethertype)]
 
     def delete_sg(self, sg_id):
         """Free all conj_ids associated with the sg_id and
@@ -337,7 +381,7 @@ class ConjIPFlowManager(object):
     """
 
     def __init__(self, driver):
-        self.conj_id_map = ConjIdMap()
+        self.conj_id_map = ConjIdMap(driver.int_br.br)
         self.driver = driver
         # The following two are dict of dicts and are indexed like:
         #     self.x[vlan_tag][(direction, ethertype)]
@@ -345,23 +389,23 @@ class ConjIPFlowManager(object):
         self.flow_state = collections.defaultdict(
             lambda: collections.defaultdict(dict))
 
-    def _build_addr_conj_id_map(self, ethertype, sg_conj_id_map):
+    def _build_addr_conj_id_map(self, ethertype, sg_ag_conj_id_map):
         """Build a map of addr -> list of conj_ids."""
         addr_to_conj = collections.defaultdict(list)
-        for remote_id, conj_id_set in sg_conj_id_map.items():
+        for remote_id, conj_id_set in sg_ag_conj_id_map.items():
             remote_group = self.driver.sg_port_map.get_sg(remote_id)
             if not remote_group or not remote_group.members:
-                LOG.debug('No member for SG %s', remote_id)
+                LOG.debug('No member for security group or'
+                          'address group %s', remote_id)
                 continue
             for addr in remote_group.get_ethertype_filtered_addresses(
                     ethertype):
                 addr_to_conj[addr].extend(conj_id_set)
-
         return addr_to_conj
 
     def _update_flows_for_vlan_subr(self, direction, ethertype, vlan_tag,
                                     flow_state, addr_to_conj,
-                                    conj_id_to_remove):
+                                    conj_id_to_remove, ofport):
         """Do the actual flow updates for given direction and ethertype."""
         conj_id_to_remove = conj_id_to_remove or []
         # Delete any current flow related to any deleted IP address, before
@@ -369,57 +413,81 @@ class ConjIPFlowManager(object):
         self.driver.delete_flows_for_flow_state(
             flow_state, addr_to_conj, direction, ethertype, vlan_tag)
         for conj_id_set in conj_id_to_remove:
-            # Remove any remaining flow with remote SG ID conj_id_to_remove
+            # Remove any remaining flow with remote SG/AG ID conj_id_to_remove
             for current_ip, conj_ids in flow_state.items():
                 conj_ids_to_remove = conj_id_set & set(conj_ids)
                 self.driver.delete_flow_for_ip(
                     current_ip, direction, ethertype, vlan_tag,
                     conj_ids_to_remove)
 
+        # NOTE(hangyang): Handle add/delete overlapped IPs among
+        # remote security groups and remote address groups
+        removed_ips = set([str(netaddr.IPNetwork(addr[0]).cidr) for addr in (
+                set(flow_state.keys()) - set(addr_to_conj.keys()))])
+        ip_to_conj = collections.defaultdict(set)
         for addr, conj_ids in addr_to_conj.items():
+            # Addresses from remote security groups have mac addresses,
+            # others from remote address groups have not.
+            ip_to_conj[str(netaddr.IPNetwork(addr[0]).cidr)].update(conj_ids)
+
+        for addr in addr_to_conj.keys():
+            ip_cidr = str(netaddr.IPNetwork(addr[0]).cidr)
+            # When the overlapped IP in remote security group and remote
+            # address group have different conjunction ids but with the
+            # same priority offset, we need to combine the conj_ids together
+            # before create flows otherwise flows will be overridden in the
+            # creation sequence.
+            conj_ids = list(ip_to_conj[ip_cidr])
             conj_ids.sort()
-            if flow_state.get(addr) == conj_ids:
+            if flow_state.get(addr) == conj_ids and ip_cidr not in removed_ips:
+                # When there are IP overlaps among remote security groups
+                # and remote address groups, removal of the overlapped ips
+                # from one remote group will also delete the flows for the
+                # other groups because the non-strict delete method cannot
+                # match flow priority or actions for different conjunction
+                # ids, therefore we need to recreate the affected flows.
                 continue
             for flow in rules.create_flows_for_ip_address(
                     addr, direction, ethertype, vlan_tag, conj_ids):
-                self.driver._add_flow(**flow)
+                self.driver._add_flow(flow_group_id=ofport, **flow)
 
-    def update_flows_for_vlan(self, vlan_tag, conj_id_to_remove=None):
+    def update_flows_for_vlan(self, vlan_tag, ofport, conj_id_to_remove=None):
         """Install action=conjunction(conj_id, 1/2) flows,
-        which depend on IP addresses of remote_group_id.
+        which depend on IP addresses of remote_group_id or
+        remote_address_group_id.
         """
-        for (direction, ethertype), sg_conj_id_map in (
+        for (direction, ethertype), sg_ag_conj_id_map in (
                 self.conj_ids[vlan_tag].items()):
             # TODO(toshii): optimize when remote_groups have
             # no address overlaps.
             addr_to_conj = self._build_addr_conj_id_map(
-                ethertype, sg_conj_id_map)
+                ethertype, sg_ag_conj_id_map)
             self._update_flows_for_vlan_subr(
                 direction, ethertype, vlan_tag,
                 self.flow_state[vlan_tag][(direction, ethertype)],
-                addr_to_conj, conj_id_to_remove)
+                addr_to_conj, conj_id_to_remove, ofport)
             self.flow_state[vlan_tag][(direction, ethertype)] = addr_to_conj
 
-    def add(self, vlan_tag, sg_id, remote_sg_id, direction, ethertype,
+    def add(self, vlan_tag, sg_id, remote_id, direction, ethertype,
             priority_offset):
         """Get conj_id specified by the arguments
         and notify the manager that
-        (remote_sg_id, direction, ethertype, conj_id) flows need to be
-        populated on the vlan_tag network.
+        (remote_id, direction, ethertype, conj_id) flows need
+        to be populated on the vlan_tag network.
 
         A caller must call update_flows_for_vlan to have the change in effect.
 
         """
         conj_id = self.conj_id_map.get_conj_id(
-            sg_id, remote_sg_id, direction, ethertype) + priority_offset * 2
+            sg_id, remote_id, direction, ethertype) + priority_offset * 2
 
         if (direction, ethertype) not in self.conj_ids[vlan_tag]:
             self.conj_ids[vlan_tag][(direction, ethertype)] = (
                 collections.defaultdict(set))
-        self.conj_ids[vlan_tag][(direction, ethertype)][remote_sg_id].add(
+        self.conj_ids[vlan_tag][(direction, ethertype)][remote_id].add(
             conj_id)
 
-        conj_id_tuple = (sg_id, remote_sg_id, direction, ethertype)
+        conj_id_tuple = (sg_id, remote_id, direction, ethertype)
         self.conj_id_map.id_map_group[conj_id_tuple].add(conj_id)
         return conj_id
 
@@ -453,7 +521,7 @@ class ConjIPFlowManager(object):
                         update = True
 
             if update:
-                self.update_flows_for_vlan(vlan_tag,
+                self.update_flows_for_vlan(vlan_tag, None,
                                            conj_id_to_remove=conj_id_to_remove)
 
 
@@ -526,7 +594,18 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         for f in rules.create_accept_flows(flow):
             self._add_flow(**f)
 
-    def _add_flow(self, **kwargs):
+    def _add_flow(self, flow_group_id=None, **kwargs):
+        """Add a new flow.
+
+        Most of the port related flows will have the parameters "reg_port" or
+        "in_port". If no "flow_group_id" is defined, "in_port" or "reg_port"
+        will be used instead (those parameters store the port "ofport"). The
+        flow group ID will be used to commit all flows related to a port in
+        the same transaction (for deferred OVS bridge implementation only).
+        """
+        flow_group_id = (flow_group_id or
+                         kwargs.get('in_port') or
+                         kwargs.get('reg_port'))
         dl_type = kwargs.get('dl_type')
         create_reg_numbers(kwargs)
         if isinstance(dl_type, int):
@@ -534,7 +613,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         if self._update_cookie:
             kwargs['cookie'] = self._update_cookie
         if self._deferred:
-            self.int_br.add_flow(**kwargs)
+            self.int_br.add_flow(flow_group_id=flow_group_id, **kwargs)
         else:
             self.int_br.br.add_flow(**kwargs)
 
@@ -546,14 +625,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         else:
             self.int_br.br.delete_flows(**kwargs)
 
-    def _strict_delete_flow(self, **kwargs):
-        """Delete given flow right away even if bridge is deferred.
-
-        Delete command will use strict delete.
-        """
-        create_reg_numbers(kwargs)
-        self.int_br.br.delete_flows(strict=True, **kwargs)
-
     @staticmethod
     def initialize_bridge(int_br):
         int_br.add_protocols(*OVSFirewallDriver.REQUIRED_PROTOCOLS)
@@ -735,13 +806,6 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def _update_flows_for_port(self, of_port, old_of_port):
         with self.update_cookie_context():
             self._set_port_filters(of_port)
-        # Flush the flows caused by changes made to deferred bridge. The reason
-        # is that following delete_all_port_flows() call uses --strict
-        # parameter that cannot be combined with other non-strict rules, hence
-        # all parameters with --strict are applied right away. In order to
-        # avoid applying delete rules with --strict *before*
-        # _set_port_filters() we dump currently cached flows here.
-        self.int_br.apply_flows()
         self.delete_all_port_flows(old_of_port)
         # Rewrite update cookie with default cookie
         self._set_port_filters(of_port)
@@ -835,6 +899,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             actions += 'strip_vlan,resubmit(,{:d})'.format(
                     ovs_consts.BASE_INGRESS_TABLE)
             self._add_flow(
+                flow_group_id=ofport,
                 table=ovs_consts.TRANSIENT_TABLE,
                 priority=90,
                 dl_dst=mac,
@@ -846,6 +911,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             actions += 'resubmit(,{:d})'.format(
                     ovs_consts.BASE_INGRESS_TABLE)
             self._add_flow(
+                flow_group_id=ofport,
                 table=ovs_consts.TRANSIENT_TABLE,
                 priority=90,
                 dl_dst=mac,
@@ -854,15 +920,13 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
     def delete_physical_direct_flow(self, mac, segment_id):
         if segment_id:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac,
-                                     dl_vlan=segment_id)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac,
+                               dl_vlan=segment_id)
         else:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac,
-                                     vlan_tci=ovs_consts.FLAT_VLAN_TCI)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac,
+                               vlan_tci=ovs_consts.FLAT_VLAN_TCI)
 
     def initialize_port_flows(self, port):
         """Set base flows for port
@@ -892,6 +956,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
                 port.vlan_tag, port.network_type)
 
             self._add_flow(
+                flow_group_id=port.ofport,
                 table=ovs_consts.TRANSIENT_TABLE,
                 priority=90,
                 dl_dst=mac_addr,
@@ -1146,6 +1211,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
         # and if not, accept it
         for mac_addr in port.all_allowed_macs:
             self._add_flow(
+                flow_group_id=port.ofport,
                 table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                 priority=100,
                 dl_dst=mac_addr,
@@ -1188,6 +1254,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
         # Prevent flood for accepted egress traffic
         self._add_flow(
+            flow_group_id=dst_port,
             table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
             priority=12,
             dl_dst=mac,
@@ -1220,6 +1287,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
             if patch_ofport is not ovs_lib.INVALID_OFPORT:
                 self._add_flow(
+                    flow_group_id=dst_port,
                     table=ovs_consts.ACCEPTED_EGRESS_TRAFFIC_NORMAL_TABLE,
                     priority=10,
                     dl_src=mac,
@@ -1244,6 +1312,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def _initialize_tracked_egress(self, port):
         # Drop invalid packets
         self._add_flow(
+            flow_group_id=port.ofport,
             table=ovs_consts.RULES_EGRESS_TABLE,
             priority=50,
             ct_state=ovsfw_consts.OF_STATE_INVALID,
@@ -1293,7 +1362,12 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             )
 
     def _initialize_ingress_ipv6_icmp(self, port):
-        for icmp_type in firewall.ICMPV6_ALLOWED_INGRESS_TYPES:
+        # NOTE(ralonsoh): "ICMPV6_TYPE_RA" was removed from
+        # "ICMPV6_ALLOWED_INGRESS_TYPES" because of a bug in the iptables
+        # firewall. This rule was added in "_add_ingress_ra_rule". However,
+        # the OVS firewall does not use port["security_group_rules"].
+        for icmp_type in (firewall.ICMPV6_ALLOWED_INGRESS_TYPES +
+                          (lib_const.ICMPV6_TYPE_RA, )):
             self._add_flow(
                 table=ovs_consts.BASE_INGRESS_TABLE,
                 priority=100,
@@ -1354,6 +1428,7 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def _initialize_tracked_ingress(self, port):
         # Drop invalid packets
         self._add_flow(
+            flow_group_id=port.ofport,
             table=ovs_consts.RULES_INGRESS_TABLE,
             priority=50,
             ct_state=ovsfw_consts.OF_STATE_INVALID,
@@ -1427,20 +1502,28 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             protocol = rule.get('protocol')
             priority_offset = rules.flow_priority_offset(rule)
 
+            if rule.get('remote_group_id'):
+                remote_type = 'security group'
+                remote_id = rule.get('remote_group_id')
+            else:
+                remote_type = 'address group'
+                remote_id = rule.get('remote_address_group_id')
             conj_id = self.conj_ip_manager.add(port.vlan_tag, sec_group_id,
-                                               rule['remote_group_id'],
+                                               remote_id,
                                                direction, ethertype,
                                                priority_offset)
             LOG.debug("Created conjunction %(conj_id)s for SG %(sg_id)s "
-                      "referencing remote SG ID %(remote_sg_id)s on port "
-                      "%(port_id)s.",
+                      "referencing remote %(remote_type)s ID %(remote_id)s "
+                      "on port %(port_id)s.",
                       {'conj_id': conj_id,
                        'sg_id': sec_group_id,
-                       'remote_sg_id': rule['remote_group_id'],
+                       'remote_type': remote_type,
+                       'remote_id': remote_id,
                        'port_id': port.id})
 
             rule1 = rule.copy()
-            del rule1['remote_group_id']
+            rule1.pop('remote_group_id', None)
+            rule1.pop('remote_address_group_id', None)
             port_rules_key = (direction, ethertype, protocol)
             port_rules[port_rules_key].append((rule1, conj_id))
 
@@ -1492,7 +1575,8 @@ class OVSFirewallDriver(firewall.FirewallDriver):
 
         self._add_non_ip_conj_flows(port)
 
-        self.conj_ip_manager.update_flows_for_vlan(port.vlan_tag)
+        self.conj_ip_manager.update_flows_for_vlan(port.vlan_tag,
+                                                   port.ofport)
 
     def _create_rules_generator_for_port(self, port):
         for sec_group in port.sec_groups:
@@ -1507,19 +1591,17 @@ class OVSFirewallDriver(firewall.FirewallDriver):
     def delete_all_port_flows(self, port):
         """Delete all flows for given port"""
         for mac_addr in port.all_allowed_macs:
-            self._strict_delete_flow(priority=90,
-                                     table=ovs_consts.TRANSIENT_TABLE,
-                                     dl_dst=mac_addr,
-                                     dl_vlan=port.vlan_tag)
+            self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                               dl_dst=mac_addr,
+                               dl_vlan=port.vlan_tag)
             self.delete_physical_direct_flow(mac_addr, port.segment_id)
             self._delete_flows(table=ovs_consts.ACCEPT_OR_INGRESS_TABLE,
                                dl_dst=mac_addr, reg_net=port.vlan_tag)
 
         self.delete_accepted_egress_direct_flow(
             port.mac, port.vlan_tag)
-        self._strict_delete_flow(priority=100,
-                                 table=ovs_consts.TRANSIENT_TABLE,
-                                 in_port=port.ofport)
+        self._delete_flows(table=ovs_consts.TRANSIENT_TABLE,
+                           in_port=port.ofport)
         self._delete_flows(reg_port=port.ofport)
 
     def delete_flows_for_flow_state(
@@ -1554,4 +1636,4 @@ class OVSFirewallDriver(firewall.FirewallDriver):
             # will not match with the ip flow's cookie so OVS won't actually
             # delete the flow
             flow['cookie'] = ovs_lib.COOKIE_ANY
-            self._delete_flows(deferred=False, **flow)
+            self._delete_flows(**flow)
