@@ -15,15 +15,15 @@
 import collections
 import functools
 import re
+import threading
+import uuid
 
 from neutron_lib import constants as n_const
 from oslo_concurrency import lockutils
 from oslo_log import log
 from oslo_utils import netutils
-from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event as row_event
 from ovsdbapp.backend.ovs_idl import vlog
-import tenacity
 
 from neutron.agent.linux import external_process
 from neutron.agent.linux import ip_lib
@@ -48,6 +48,8 @@ OVN_VIF_PORT_TYPES = ("", "external", )
 MetadataPortInfo = collections.namedtuple('MetadataPortInfo', ['mac',
                                                                'ip_addresses'])
 
+OVN_METADATA_UUID_NAMESPACE = uuid.UUID('d34bf9f6-da32-4871-9af8-15a4626b41ab')
+
 
 def _sync_lock(f):
     """Decorator to block all operations for a global sync call."""
@@ -200,12 +202,30 @@ class MetadataAgent(object):
         self._process_monitor = external_process.ProcessMonitor(
             config=self.conf,
             resource_type='metadata')
+        self._sb_idl = None
+        self._post_fork_event = threading.Event()
+
+    @property
+    def sb_idl(self):
+        if not self._sb_idl:
+            self._post_fork_event.wait()
+        return self._sb_idl
+
+    @sb_idl.setter
+    def sb_idl(self, val):
+        self._sb_idl = val
 
     def _load_config(self):
         self.chassis = self._get_own_chassis_name()
+        try:
+            self.chassis_id = uuid.UUID(self.chassis)
+        except ValueError:
+            # OVS system-id could be a non UUID formatted string.
+            self.chassis_id = uuid.uuid5(OVN_METADATA_UUID_NAMESPACE,
+                                         self.chassis)
         self.ovn_bridge = self._get_ovn_bridge()
-        LOG.debug("Loaded chassis %s and ovn bridge %s.",
-                  self.chassis, self.ovn_bridge)
+        LOG.debug("Loaded chassis name %s (UUID: %s) and ovn bridge %s.",
+                  self.chassis, self.chassis_id, self.ovn_bridge)
 
     @_sync_lock
     def resync(self):
@@ -237,6 +257,7 @@ class MetadataAgent(object):
         # Chassis table.
         # Open the connection to OVN SB database.
         self.has_chassis_private = False
+        self._post_fork_event.clear()
         try:
             self.sb_idl = ovsdb.MetadataAgentOvnSbIdl(
                 chassis=self.chassis, tables=tables + ('Chassis_Private', ),
@@ -247,6 +268,9 @@ class MetadataAgent(object):
                 chassis=self.chassis, tables=tables,
                 events=events + (ChassisCreateEvent(self), )).start()
 
+        # Now IDL connections can be safely used.
+        self._post_fork_event.set()
+
         # Do the initial sync.
         self.sync()
 
@@ -255,16 +279,14 @@ class MetadataAgent(object):
 
         proxy.wait()
 
-    @tenacity.retry(
-        wait=tenacity.wait_exponential(
-            max=config.get_ovn_ovsdb_retry_max_interval()),
-        reraise=True)
+    @ovn_utils.retry()
     def register_metadata_agent(self):
         # NOTE(lucasagomes): db_add() will not overwrite the UUID if
         # it's already set.
         table = ('Chassis_Private' if self.has_chassis_private else 'Chassis')
-        ext_ids = {
-            ovn_const.OVN_AGENT_METADATA_ID_KEY: uuidutils.generate_uuid()}
+        # Generate unique, but consistent metadata id for chassis name
+        agent_id = uuid.uuid5(self.chassis_id, 'metadata_agent')
+        ext_ids = {ovn_const.OVN_AGENT_METADATA_ID_KEY: str(agent_id)}
         self.sb_idl.db_add(table, self.chassis, 'external_ids',
                            ext_ids).execute(check_error=True)
 
@@ -327,20 +349,13 @@ class MetadataAgent(object):
     def _vif_ports(self, ports):
         return (p for p in ports if p.type in OVN_VIF_PORT_TYPES)
 
-    def teardown_datapath(self, datapath, net_name=None):
+    def teardown_datapath(self, net_name):
         """Unprovision this datapath to stop serving metadata.
 
         This function will shutdown metadata proxy if it's running and delete
         the VETH pair, the OVS port and the namespace.
         """
-        self.update_chassis_metadata_networks(datapath, remove=True)
-
-        # TODO(dalvarez): Remove this in Y cycle when we are sure that all
-        # namespaces will be created with the Neutron network UUID and not
-        # anymore with the OVN datapath UUID.
-        dp = net_name or datapath
-
-        namespace = self._get_namespace_name(dp)
+        namespace = self._get_namespace_name(net_name)
         ip = ip_lib.IPWrapper(namespace)
         # If the namespace doesn't exist, return
         if not ip.netns.exists(namespace):
@@ -350,9 +365,9 @@ class MetadataAgent(object):
                  namespace)
 
         metadata_driver.MetadataDriver.destroy_monitored_metadata_proxy(
-            self._process_monitor, dp, self.conf, namespace)
+            self._process_monitor, net_name, self.conf, namespace)
 
-        veth_name = self._get_veth_name(dp)
+        veth_name = self._get_veth_name(net_name)
         self.ovs_idl.del_port(veth_name[0]).execute()
         if ip_lib.device_exists(veth_name[0]):
             ip_lib.IPWrapper().del_veth(veth_name[0])
@@ -376,7 +391,7 @@ class MetadataAgent(object):
         if datapath_ports:
             self.provision_datapath(datapath, net_name)
         else:
-            self.teardown_datapath(datapath, net_name)
+            self.teardown_datapath(net_name)
 
     def _ensure_datapath_checksum(self, namespace):
         """Ensure the correct checksum in the metadata packets in DPDK bridges
@@ -407,7 +422,7 @@ class MetadataAgent(object):
 
         :return: The metadata namespace name of this datapath
         """
-        LOG.debug("Provisioning metadata for network %s", net_name)
+        LOG.info("Provisioning metadata for network %s", net_name)
         port = self.sb_idl.get_metadata_port_network(datapath)
         # If there's no metadata port or it doesn't have a MAC or IP
         # addresses, then tear the namespace down if needed. This might happen
@@ -418,7 +433,7 @@ class MetadataAgent(object):
             LOG.debug("There is no metadata port for network %s or it has no "
                       "MAC or IP addresses configured, tearing the namespace "
                       "down if needed", net_name)
-            self.teardown_datapath(datapath, net_name)
+            self.teardown_datapath(net_name)
             return
 
         # First entry of the mac field must be the MAC address.
@@ -429,7 +444,7 @@ class MetadataAgent(object):
             LOG.error("Metadata port for network %s doesn't have a MAC "
                       "address, tearing the namespace down if needed",
                       net_name)
-            self.teardown_datapath(datapath)
+            self.teardown_datapath(net_name)
             return
 
         mac = match.group()
@@ -513,7 +528,6 @@ class MetadataAgent(object):
             self.conf, bind_address=n_const.METADATA_V4_IP,
             network_id=net_name)
 
-        self.update_chassis_metadata_networks(net_name)
         return namespace
 
     def ensure_all_networks_provisioned(self):
@@ -539,30 +553,3 @@ class MetadataAgent(object):
                 namespaces.append(netns)
 
         return namespaces
-
-    # NOTE(lucasagomes): Even tho the metadata agent is a multi-process
-    # application, there's only one Southbound database IDL instance in
-    # the agent which handles the OVSDB events therefore we do not need
-    # the external=True parameter in the @synchronized decorator.
-    @lockutils.synchronized(CHASSIS_METADATA_LOCK)
-    def update_chassis_metadata_networks(self, datapath, remove=False):
-        """Update metadata networks hosted in this chassis.
-
-        Add or remove a datapath from the list of current datapaths that
-        we're currently serving metadata.
-        """
-        current_dps = self.sb_idl.get_chassis_metadata_networks(self.chassis)
-        updated = False
-        if remove:
-            if datapath in current_dps:
-                current_dps.remove(datapath)
-                updated = True
-        else:
-            if datapath not in current_dps:
-                current_dps.append(datapath)
-                updated = True
-
-        if updated:
-            with self.sb_idl.create_transaction(check_error=True) as txn:
-                txn.add(self.sb_idl.set_chassis_metadata_networks(
-                    self.chassis, current_dps))
