@@ -143,11 +143,11 @@ class DhcpAgentNotifyAPI(object):
                         network['id'])
         return new_agents + existing_agents
 
-    def _get_enabled_agents(self, context, network, agents, method, payload):
+    def _get_enabled_agents(
+            self, context, network_id, network, agents, method, payload):
         """Get the list of agents who can provide services."""
         if not agents:
             return []
-        network_id = network['id']
         enabled_agents = agents
         if not cfg.CONF.enable_services_on_agents_with_admin_state_down:
             enabled_agents = [x for x in agents if x.admin_state_up]
@@ -165,6 +165,9 @@ class DhcpAgentNotifyAPI(object):
         if not enabled_agents:
             num_ports = self.plugin.get_ports_count(
                 context, {'network_id': [network_id]})
+            if not network:
+                admin_ctx = context if context.is_admin else context.elevated()
+                network = self.plugin.get_network(admin_ctx, network_id)
             notification_required = (
                 num_ports > 0 and len(network['subnets']) >= 1)
             if notification_required:
@@ -179,7 +182,8 @@ class DhcpAgentNotifyAPI(object):
     def _is_reserved_dhcp_port(self, port):
         return port.get('device_id') == constants.DEVICE_ID_RESERVED_DHCP_PORT
 
-    def _notify_agents(self, context, method, payload, network_id):
+    def _notify_agents(
+            self, context, method, payload, network_id, network=None):
         """Notify all the agents that are hosting the network."""
         payload['priority'] = METHOD_PRIORITY_MAP.get(method)
         # fanout is required as we do not know who is "listening"
@@ -194,33 +198,37 @@ class DhcpAgentNotifyAPI(object):
         if fanout_required:
             self._fanout_message(context, method, payload)
         elif cast_required:
-            admin_ctx = (context if context.is_admin else context.elevated())
-            network = self.plugin.get_network(admin_ctx, network_id)
+            candidate_hosts = None
             if 'subnet' in payload and payload['subnet'].get('segment_id'):
                 # if segment_id exists then the segment service plugin
                 # must be loaded
                 segment_plugin = directory.get_plugin('segments')
                 segment = segment_plugin.get_segment(
                     context, payload['subnet']['segment_id'])
-                network['candidate_hosts'] = segment['hosts']
+                candidate_hosts = segment['hosts']
 
             agents = self.plugin.get_dhcp_agents_hosting_networks(
-                context, [network_id], hosts=network.get('candidate_hosts'))
+                context, [network_id], hosts=candidate_hosts)
             # schedule the network first, if needed
             schedule_required = (
                 method == 'subnet_create_end' or
                 method == 'port_create_end' and
                 not self._is_reserved_dhcp_port(payload['port']))
             if schedule_required:
+                admin_ctx = context if context.is_admin else context.elevated()
+                network = network or self.plugin.get_network(
+                    admin_ctx, network_id)
+                if candidate_hosts:
+                    network['candidate_hosts'] = candidate_hosts
                 agents = self._schedule_network(admin_ctx, network, agents)
             if not agents:
                 LOG.debug("Network %s is not hosted by any dhcp agent",
                           network_id)
                 return
             enabled_agents = self._get_enabled_agents(
-                context, network, agents, method, payload)
+                context, network_id, network, agents, method, payload)
 
-            if method == 'port_create_end':
+            if method == 'port_create_end' and enabled_agents:
                 high_agent = enabled_agents.pop(
                     random.randint(0, len(enabled_agents) - 1))
                 self._notify_high_priority_agent(
@@ -260,20 +268,22 @@ class DhcpAgentNotifyAPI(object):
                            {'admin_state_up': admin_state_up}, host)
 
     def _after_router_interface_created(self, resource, event, trigger,
-                                        **kwargs):
-        self._notify_agents(kwargs['context'], 'port_create_end',
-                            {'port': kwargs['port']},
-                            kwargs['port']['network_id'])
+                                        payload=None):
+        port = payload.metadata.get('port')
+        self._notify_agents(payload.context, 'port_create_end',
+                            {'port': port},
+                            port['network_id'])
 
     def _after_router_interface_deleted(self, resource, event, trigger,
-                                        **kwargs):
-        self._notify_agents(kwargs['context'], 'port_delete_end',
-                            {'port_id': kwargs['port']['id'],
-                             'fixed_ips': kwargs['port']['fixed_ips']},
-                            kwargs['port']['network_id'])
+                                        payload=None):
+        port = payload.metadata.get('port')
+        self._notify_agents(payload.context, 'port_delete_end',
+                            {'port_id': port['id'],
+                             'network_id': port['network_id']},
+                            port['network_id'])
 
     def _native_event_send_dhcp_notification(self, resource, event, trigger,
-                                             context, **kwargs):
+                                             payload):
         action = event.replace('after_', '')
         # we unsubscribe the _send_dhcp_notification method now that we know
         # the loaded core plugin emits native resource events
@@ -286,29 +296,25 @@ class DhcpAgentNotifyAPI(object):
                 registry.unsubscribe_by_resource(self._send_dhcp_notification,
                                                  resource)
         method_name = '.'.join((resource, action, 'end'))
-        payload = kwargs[resource]
-        data = {resource: payload}
-        if resource == resources.PORT:
-            if self._only_status_changed(kwargs.get('original_port'),
-                                         kwargs.get('port')):
+        data = {resource: payload.latest_state}
+        if resource == resources.PORT and event == events.AFTER_UPDATE:
+            if not self._notification_is_needed(payload.states[0],
+                                                payload.latest_state):
                 # don't waste time updating the DHCP agent for status updates
                 return
-        self.notify(context, data, method_name)
+        self.notify(payload.context, data, method_name)
 
-    def _only_status_changed(self, orig, new):
-        # a status change will manifest as a bumped revision number, a new
-        # updated_at timestamp, and a new status. If that's all that changed,
-        # return True, else False
+    def _notification_is_needed(self, orig, new):
+        # notification to the DHCP agent should be send only if DHCP related
+        # attributes of the port were changed, like: fixed_ips, mac_address,
+        # extra_dhcp_opts, dns_name, dns_assignment or dns_domain.
+        # Otherwise there is no need to waste DHCP agent's time for doing
+        # updates.
         if not orig or not new:
-            return False
-        if set(orig.keys()) != set(new.keys()):
-            return False
-        for k in orig.keys():
-            if k in ('status', 'updated_at', 'revision_number'):
-                continue
-            if orig[k] != new[k]:
-                return False
-        return True
+            return True
+        return any(orig.get(k) != new.get(k) for k in (
+            'fixed_ips', 'mac_address', 'extra_dhcp_opts',
+            'dns_name', 'dns_assignment', 'dns_domain'))
 
     def _send_dhcp_notification(self, resource, event, trigger, payload=None):
         action = payload.action.split('_')[0]
@@ -346,6 +352,8 @@ class DhcpAgentNotifyAPI(object):
                     payload['network_id'] = network_id
                 if obj_type == 'port':
                     payload['fixed_ips'] = obj_value['fixed_ips']
-                self._notify_agents(context, method_name, payload, network_id)
+                self._notify_agents(context, method_name, payload, network_id,
+                                    obj_value.get('network'))
         else:
-            self._notify_agents(context, method_name, data, network_id)
+            self._notify_agents(context, method_name, data, network_id,
+                                obj_value.get('network'))
