@@ -13,7 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import distutils
+import enum
 import re
 import shutil
 import tempfile
@@ -21,6 +21,7 @@ import tempfile
 import netaddr
 from neutron_lib import constants as n_consts
 from neutron_lib import exceptions
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 from neutron_lib.utils import helpers
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -36,8 +37,6 @@ from neutron.agent.linux import keepalived
 from neutron.agent.linux import utils as agent_utils
 from neutron.common import utils as common_utils
 from neutron.conf.agent.l3 import config as l3_config
-from neutron.plugins.ml2.drivers.openvswitch.agent.common \
-    import constants as ovs_const
 from neutron.privileged.agent.linux import dhcp as priv_dhcp
 
 LOG = logging.getLogger(__name__)
@@ -49,6 +48,37 @@ DNSMASQ_VERSION_HOST_ADDR6_LIST = '2.81'
 DIRECT_PORT_QOS_MIN_OVS_VERSION = '2.11'
 MINIMUM_DIBBLER_VERSION = '1.0.1'
 CONNTRACK_GRE_MODULE = 'nf_conntrack_proto_gre'
+OVN_NB_DB_SCHEMA_PORT_GROUP = '5.11'
+OVN_NB_DB_SCHEMA_STATELESS_NAT = '5.17'
+OVN_SB_DB_SCHEMA_VIRTUAL_PORT = '2.5'
+
+
+class OVNCheckType(enum.Enum):
+    nb_version = 0
+    nb_db_schema = 1
+    sb_version = 2
+    sb_db_schema = 3
+
+
+def _get_ovn_version(check_type):
+    if check_type in (OVNCheckType.nb_version, OVNCheckType.nb_db_schema):
+        cmd = ['ovn-nbctl', '--version']
+    elif check_type in (OVNCheckType.nb_version, OVNCheckType.nb_db_schema):
+        cmd = ['ovn-sbctl', '--version']
+    else:
+        raise RuntimeError
+
+    out = agent_utils.execute(cmd)
+    if check_type == OVNCheckType.nb_version:
+        matched_line = re.search(r"ovn-nbctl.*", out)
+    elif check_type == OVNCheckType.sb_version:
+        matched_line = re.search(r"ovn-sbctl.*", out)
+    else:
+        matched_line = re.search(r"DB Schema.*", out)
+
+    matched_version = re.search(r"(\d+\.\d+)", matched_line.group(0))
+    return versionutils.convert_version_to_tuple(matched_version.group(1) if
+                                                 matched_version else '0.0')
 
 
 def ovs_vxlan_supported(from_ip='192.0.2.1', to_ip='192.0.2.2'):
@@ -77,8 +107,10 @@ def ovs_geneve_supported(from_ip='192.0.2.3', to_ip='192.0.2.4'):
 
 def iproute2_vxlan_supported():
     ip = ip_lib.IPWrapper()
+    name_dummy = common_utils.get_rand_device_name(prefix='vxlantest-')
+    ip.add_dummy(name_dummy)
     name = common_utils.get_rand_device_name(prefix='vxlantest-')
-    port = ip.add_vxlan(name, 3000)
+    port = ip.add_vxlan(name, 3000, name_dummy)
     ip.del_veth(name)
     return name == port.name
 
@@ -113,7 +145,8 @@ def ofctl_arg_supported(cmd, **kwargs):
                      ovs_lib._build_flow_expr_str(kwargs, cmd.split('-')[0],
                                                   False)]
         try:
-            agent_utils.execute(full_args, run_as_root=True)
+            agent_utils.execute(full_args, run_as_root=True,
+                                privsep_exec=True)
         except RuntimeError as e:
             LOG.debug("Exception while checking supported feature via "
                       "command %s. Exception: %s", full_args, e)
@@ -209,11 +242,12 @@ def dnsmasq_version_supported():
         env = {'LC_ALL': 'C'}
         out = agent_utils.execute(cmd, addl_env=env)
         m = re.search(r"version (\d+\.\d+)", out)
-        ver = distutils.version.StrictVersion(m.group(1) if m else '0.0')
-        if ver < distutils.version.StrictVersion(MINIMUM_DNSMASQ_VERSION):
+        ver = versionutils.convert_version_to_tuple(m.group(1) if m else '0.0')
+        if ver < versionutils.convert_version_to_tuple(
+                MINIMUM_DNSMASQ_VERSION):
             return False
         if (cfg.CONF.dnsmasq_enable_addr6_list is True and
-                ver < distutils.version.StrictVersion(
+                ver < versionutils.convert_version_to_tuple(
                     DNSMASQ_VERSION_HOST_ADDR6_LIST)):
             LOG.warning('Support for multiple IPv6 addresses in host '
                         'entries was introduced in dnsmasq version '
@@ -240,7 +274,7 @@ def ovs_qos_direct_port_supported():
         ver = versionutils.convert_version_to_tuple(matched_version.group(1) if
                                                     matched_version else '0.0')
         minver = versionutils.convert_version_to_tuple(
-                 DIRECT_PORT_QOS_MIN_OVS_VERSION)
+            DIRECT_PORT_QOS_MIN_OVS_VERSION)
         if ver < minver:
             return False
     except (OSError, RuntimeError, ValueError) as e:
@@ -550,3 +584,45 @@ def min_tx_rate_support():
                   devices_without_support)
         return False
     return True
+
+
+def ovn_nb_db_schema_port_group_supported():
+    try:
+        ver = _get_ovn_version(OVNCheckType.nb_db_schema)
+        minver = versionutils.convert_version_to_tuple(
+            OVN_NB_DB_SCHEMA_PORT_GROUP)
+        if ver < minver:
+            return False
+    except (OSError, RuntimeError, ValueError) as e:
+        LOG.debug('Exception while checking OVN DB schema version. '
+                  'Exception: %s', e)
+        return False
+    return True
+
+
+def ovn_nb_db_schema_stateless_nat_supported():
+    try:
+        ver = _get_ovn_version(OVNCheckType.nb_db_schema)
+        minver = versionutils.convert_version_to_tuple(
+            OVN_NB_DB_SCHEMA_STATELESS_NAT)
+        if ver < minver:
+            return False
+    except (OSError, RuntimeError, ValueError) as e:
+        LOG.debug('Exception while checking OVN DB schema version. '
+                  'Exception: %s', e)
+        return False
+    return True
+
+
+def ovn_sb_db_schema_virtual_port_supported():
+    try:
+        ver = _get_ovn_version(OVNCheckType.sb_db_schema)
+        minver = versionutils.convert_version_to_tuple(
+            OVN_SB_DB_SCHEMA_VIRTUAL_PORT)
+        if ver < minver:
+            return False
+    except (OSError, RuntimeError, ValueError) as e:
+        LOG.debug('Exception while checking OVN DB schema version. '
+                  'Exception: %s', e)
+        return False
+    return True
