@@ -11,12 +11,11 @@
 #    under the License.
 
 import collections
+import copy
 import inspect
 import os
-import re
 
 import netaddr
-from neutron_lib.api.definitions import availability_zone as az_def
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib.api.definitions import l3
@@ -30,16 +29,19 @@ from neutron_lib.plugins import directory
 from neutron_lib.utils import net as n_utils
 from oslo_config import cfg
 from oslo_log import log
+from oslo_serialization import jsonutils
 from oslo_utils import netutils
 from oslo_utils import strutils
-from ovs.db import idl
-from ovsdbapp.backend.ovs_idl import connection
-from ovsdbapp.backend.ovs_idl import idlutils
 from ovsdbapp import constants as ovsdbapp_const
+import tenacity
 
 from neutron._i18n import _
 from neutron.common.ovn import constants
 from neutron.common.ovn import exceptions as ovn_exc
+from neutron.common import utils as common_utils
+from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
+from neutron.db import models_v2
+from neutron.objects import ports as ports_obj
 
 LOG = log.getLogger(__name__)
 
@@ -60,7 +62,7 @@ def ovn_name(id):
     # is a UUID. If so then there will be no matches.
     # We prefix the UUID to enable us to use the Neutron UUID when
     # updating, deleting etc.
-    return 'neutron-%s' % id
+    return "%s%s" % (constants.OVN_NAME_PREFIX, id)
 
 
 def ovn_lrouter_port_name(id):
@@ -74,6 +76,12 @@ def ovn_lrouter_port_name(id):
     return constants.LRP_PREFIX + '%s' % id
 
 
+def ovn_cr_lrouter_port_name(_id):
+    # The name of the OVN chassisredirect lrouter port entry will be
+    # cr-lrp-<UUID>
+    return 'cr-lrp-%s' % _id
+
+
 def ovn_provnet_port_name(network_id):
     # The name of OVN lswitch provider network port entry will be
     # provnet-<Network-UUID>. The port is created for network having
@@ -130,6 +138,12 @@ def validate_port_extra_dhcp_opts(port):
     :param port: A neutron port.
     :returns: A PortExtraDHCPValidation object.
     """
+    # Get the right option mappings according to the port's vnic_type
+    vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
+    mapping = constants.SUPPORTED_DHCP_OPTS_MAPPING
+    if vnic_type == portbindings.VNIC_BAREMETAL:
+        mapping = constants.SUPPORTED_BM_DHCP_OPTS_MAPPING
+
     invalid = {const.IP_VERSION_4: [], const.IP_VERSION_6: []}
     failed = False
     for edo in port.get(edo_ext.EXTRADHCPOPTS, []):
@@ -142,7 +156,7 @@ def validate_port_extra_dhcp_opts(port):
             failed = False
             break
 
-        if opt_name not in constants.SUPPORTED_DHCP_OPTS_MAPPING[ip_version]:
+        if opt_name not in mapping[ip_version]:
             invalid[ip_version].append(opt_name)
             failed = True
 
@@ -152,15 +166,26 @@ def validate_port_extra_dhcp_opts(port):
         invalid_ipv6=invalid[const.IP_VERSION_6] if failed else [])
 
 
+def is_dhcp_option_quoted(opt_value):
+    return opt_value.startswith('"') and opt_value.endswith('"')
+
+
 def get_lsp_dhcp_opts(port, ip_version):
     # Get dhcp options from Neutron port, for setting DHCP_Options row
     # in OVN.
     lsp_dhcp_disabled = False
     lsp_dhcp_opts = {}
+    vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
+    is_baremetal = vnic_type == portbindings.VNIC_BAREMETAL
+
     if is_network_device_port(port):
         lsp_dhcp_disabled = True
+    elif is_baremetal and ovn_conf.is_ovn_dhcp_disabled_for_baremetal():
+        lsp_dhcp_disabled = True
     else:
-        mapping = constants.SUPPORTED_DHCP_OPTS_MAPPING[ip_version]
+        mapping = (constants.SUPPORTED_BM_DHCP_OPTS_MAPPING[ip_version]
+                   if is_baremetal else
+                   constants.SUPPORTED_DHCP_OPTS_MAPPING[ip_version])
         for edo in port.get(edo_ext.EXTRADHCPOPTS, []):
             if edo['ip_version'] != ip_version:
                 continue
@@ -180,6 +205,9 @@ def get_lsp_dhcp_opts(port, ip_version):
                 continue
 
             opt = mapping[edo['opt_name']]
+            if (opt in constants.OVN_STR_TYPE_DHCP_OPTS and
+                    not is_dhcp_option_quoted(edo['opt_value'])):
+                edo['opt_value'] = '"%s"' % edo['opt_value']
             lsp_dhcp_opts[opt] = edo['opt_value']
 
     return (lsp_dhcp_disabled, lsp_dhcp_opts)
@@ -222,34 +250,77 @@ def validate_and_get_data_from_binding_profile(port):
         return {}
     param_set = {}
     param_dict = {}
-    for param_set in constants.OVN_PORT_BINDING_PROFILE_PARAMS:
+    vnic_type = port.get(portbindings.VNIC_TYPE, portbindings.VNIC_NORMAL)
+
+    # A port's capabilities is listed as part of the binding profile, but we
+    # treat it separately and do not want it to be included in the generic
+    # validation.
+    binding_profile = copy.deepcopy(port[constants.OVN_PORT_BINDING_PROFILE])
+    capabilities = binding_profile.pop(constants.PORT_CAP_PARAM, [])
+    if not isinstance(capabilities, list):
+        msg = _('Invalid binding:profile. %s must be of type list.'
+                ) % constants.PORT_CAP_PARAM
+        raise n_exc.InvalidInput(error_message=msg)
+
+    # Note that only the keys mentioned in each parameter set, as defined in
+    # constants.OVN_PORT_BINDING_PROFILE_PARAMS, will be evaluated.
+    #
+    # Any surplus keys provided by Nova will be ignored and pruned from the
+    # Dict returned by this function.
+    for pbp_param_set in constants.OVN_PORT_BINDING_PROFILE_PARAMS:
+        if pbp_param_set.vnic_type:
+            if pbp_param_set.vnic_type != vnic_type:
+                continue
+            if capabilities and pbp_param_set.capability not in capabilities:
+                continue
+        param_set = pbp_param_set.param_set
         param_keys = param_set.keys()
         for param_key in param_keys:
             try:
-                param_dict[param_key] = (port[
-                    constants.OVN_PORT_BINDING_PROFILE][param_key])
+                param_dict[param_key] = binding_profile[param_key]
             except KeyError:
                 pass
         if len(param_dict) == 0:
             continue
-        if len(param_dict) != len(param_keys):
+        if param_keys - binding_profile.keys():
             msg = _('Invalid binding:profile. %s are all '
                     'required.') % param_keys
             raise n_exc.InvalidInput(error_message=msg)
-        if (len(port[constants.OVN_PORT_BINDING_PROFILE]) != len(
-                param_keys)):
-            msg = _('Invalid binding:profile. too many parameters')
-            raise n_exc.InvalidInput(error_message=msg)
         break
 
     if not param_dict:
         return {}
 
-    for param_key, param_type in param_set.items():
-        if param_type is None:
+    # With this example param_set:
+    #
+    # param_set = {
+    #     'do_not_check_this_key': None,
+    #     'pci_slot': [str],
+    #     'physical_network': [str, type(None)]
+    # }
+    #
+    # We confirm that each binding_profile key is of one of the listed types,
+    # allowing validation of polymorphic entries.
+    #
+    # 'physical_network' is polymorphic because:  When a VNIC_REMOTE_MANAGED or
+    # VNIC_DIRECT with PORT_CAP_SWITCHDEV capability port is attached to a
+    # project network backed by an overlay (tunneled) network the value will be
+    # 'None'.  For the case of ports attached to a project network backed by
+    # VLAN the value will be of type ``str``.  This comes from Nova and is
+    # provided in the ``physical_network`` tag in the Nova PCI Passthrough
+    # configuration.
+    #
+    # In the above example the type of the value behind 'do_not_check_this_key'
+    # will not be checked, 'pci_slot' must be ``str``, 'physical_network  must
+    # be either ``str`` or ``NoneType``.
+    for param_key, param_types in param_set.items():
+        if param_types is None:
             continue
         param_value = param_dict[param_key]
-        if not isinstance(param_value, param_type):
+        for param_type in param_types:
+            if isinstance(param_value, param_type):
+                break
+        else:
             msg = _('Invalid binding:profile. %(key)s %(value)s '
                     'value invalid type') % {'key': param_key,
                                              'value': param_value}
@@ -299,12 +370,12 @@ def get_revision_number(resource, resource_type):
 
 
 def remove_macs_from_lsp_addresses(addresses):
-    """Remove the mac addreses from the Logical_Switch_Port addresses column.
+    """Remove the mac addresses from the Logical_Switch_Port addresses column.
 
     :param addresses: The list of addresses from the Logical_Switch_Port.
         Example: ["80:fa:5b:06:72:b7 158.36.44.22",
                   "ff:ff:ff:ff:ff:ff 10.0.0.2"]
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     ip_list = []
     for addr in addresses:
@@ -318,7 +389,7 @@ def get_allowed_address_pairs_ip_addresses(port):
     """Return a list of IP addresses from port's allowed_address_pairs.
 
     :param port: A neutron port
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     return [x['ip_address'] for x in port.get('allowed_address_pairs', [])
             if 'ip_address' in x]
@@ -331,7 +402,7 @@ def get_allowed_address_pairs_ip_addresses_from_ovn_port(ovn_port):
     allowed_address_pairs column using the data in the OVN port.
 
     :param ovn_port: A OVN port
-    :returns: A list of IP addesses (v4 and v6)
+    :returns: A list of IP addresses (v4 and v6)
     """
     addresses = remove_macs_from_lsp_addresses(ovn_port.addresses)
     port_security = remove_macs_from_lsp_addresses(ovn_port.port_security)
@@ -352,6 +423,13 @@ def get_ovn_port_addresses(ovn_port):
     return list(set(addresses + port_security))
 
 
+def get_virtual_port_parents(nb_idl, virtual_ip, network_id, port_id):
+    ls = nb_idl.ls_get(ovn_name(network_id)).execute(check_error=True)
+    return [lsp.name for lsp in ls.ports
+            if lsp.name != port_id and
+            virtual_ip in get_ovn_port_addresses(lsp)]
+
+
 def sort_ips_by_version(addresses):
     ip_map = {'ip4': [], 'ip6': []}
     for addr in addresses:
@@ -365,15 +443,10 @@ def is_lsp_router_port(port):
 
 
 def get_lrouter_ext_gw_static_route(ovn_router):
-    # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
-    # is tagged.
-    try:
-        return [route for route in getattr(ovn_router, 'static_routes', []) if
-                strutils.bool_from_string(getattr(
-                    route, 'external_ids', {}).get(
-                        constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
-    except KeyError:
-        pass
+    return [route for route in getattr(ovn_router, 'static_routes', []) if
+            strutils.bool_from_string(getattr(
+                route, 'external_ids', {}).get(
+                    constants.OVN_ROUTER_IS_EXT_GW, 'false'))]
 
 
 def get_lrouter_snats(ovn_router):
@@ -382,19 +455,14 @@ def get_lrouter_snats(ovn_router):
 
 def get_lrouter_non_gw_routes(ovn_router):
     routes = []
-    # TODO(lucasagomes): Remove the try...except block after OVS 2.8.2
-    # is tagged.
-    try:
-        for route in getattr(ovn_router, 'static_routes', []):
-            external_ids = getattr(route, 'external_ids', {})
-            if strutils.bool_from_string(
-                    external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
-                continue
+    for route in getattr(ovn_router, 'static_routes', []):
+        external_ids = getattr(route, 'external_ids', {})
+        if strutils.bool_from_string(
+                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
+            continue
 
-            routes.append({'destination': route.ip_prefix,
-                           'nexthop': route.nexthop})
-    except KeyError:
-        pass
+        routes.append({'destination': route.ip_prefix,
+                       'nexthop': route.nexthop})
     return routes
 
 
@@ -413,27 +481,40 @@ def get_system_dns_resolvers(resolver_file=DNS_RESOLVER_FILE):
                 continue
 
             line = line.split('nameserver')[1].strip()
-            ipv4 = re.search(r'^(?:[0-9]{1,3}\.){3}[0-9]{1,3}', line)
-            if ipv4:
-                resolvers.append(ipv4.group(0))
+            valid_ip = (netutils.is_valid_ipv4(line, strict=True) or
+                        netutils.is_valid_ipv6(line))
+            if valid_ip:
+                resolvers.append(line)
+
     return resolvers
 
 
+def get_dhcp_dns_servers(subnet, ip_version=const.IP_VERSION_4):
+    """Retrieve the DHCP option DNS servers
+
+    The DHCP should not announce any DNS resolver at all on the subnet if any
+    configured DNS server is "0.0.0.0" (IPv4) or "::" (IPv6).
+    https://docs.openstack.org/neutron/latest/admin/config-dns-res.html
+    """
+    def filter_ips(ips, ip_version=const.IP_VERSION_4):
+        return [ip for ip in ips
+                if netaddr.IPAddress(ip).version == ip_version]
+
+    dns_servers = (subnet.get('dns_nameservers') or
+                   filter_ips(ovn_conf.get_dns_servers(), ip_version) or
+                   filter_ips(get_system_dns_resolvers(), ip_version))
+
+    if common_utils.is_dns_servers_any_address(dns_servers, ip_version):
+        return []
+
+    return dns_servers
+
+
 def get_port_subnet_ids(port):
     fixed_ips = list(port['fixed_ips'])
     return [f['subnet_id'] for f in fixed_ips]
 
 
-def get_ovsdb_connection(connection_string, schema, timeout, tables=None):
-    helper = idlutils.get_schema_helper(connection_string, schema)
-    if tables:
-        for table in tables:
-            helper.register_table(table)
-    else:
-        helper.register_all()
-    return connection.Connection(idl.Idl(connection_string, helper), timeout)
-
-
 def get_method_class(method):
     if not inspect.ismethod(method):
         return
@@ -446,7 +527,8 @@ def ovn_metadata_name(id_):
 
 
 def is_gateway_chassis_invalid(chassis_name, gw_chassis,
-                               physnet, chassis_physnets):
+                               physnet, chassis_physnets,
+                               az_hints, chassis_with_azs):
     """Check if gateway chassis is invalid
 
     @param    chassis_name: gateway chassis name
@@ -457,6 +539,10 @@ def is_gateway_chassis_invalid(chassis_name, gw_chassis,
     @type     physnet: string
     @param    chassis_physnets: Dictionary linking chassis with their physnets
     @type     chassis_physnets: {}
+    @param   az_hints: available zone hints associated to chassis_name
+    @type     az_hints: []
+    @param    chassis_with_azs: Dictionary linking chassis with their azs
+    @type     chassis_with_azs: {}
     @return   Boolean
     """
 
@@ -468,6 +554,9 @@ def is_gateway_chassis_invalid(chassis_name, gw_chassis,
         return True
     elif gw_chassis and chassis_name not in gw_chassis:
         return True
+    elif az_hints and not set(az_hints) & set(chassis_with_azs.get(
+            chassis_name, [])):
+        return True
     return False
 
 
@@ -475,17 +564,6 @@ def is_provider_network(network):
     return network.get(external_net.EXTERNAL, False)
 
 
-def is_neutron_dhcp_agent_port(port):
-    """Check if the given DHCP port belongs to Neutron DHCP agents
-
-    The DHCP ports with the device_id equals to 'reserved_dhcp_port'
-    or starting with the word 'dhcp' belongs to the Neutron DHCP agents.
-    """
-    return (port['device_owner'] == const.DEVICE_OWNER_DHCP and
-            (port['device_id'] == const.DEVICE_ID_RESERVED_DHCP_PORT or
-             port['device_id'].startswith('dhcp')))
-
-
 def compute_address_pairs_diff(ovn_port, neutron_port):
     """Compute the differences in the allowed_address_pairs field."""
     ovn_ap = get_allowed_address_pairs_ip_addresses_from_ovn_port(
@@ -509,7 +587,7 @@ def is_gateway_chassis(chassis):
 
 def get_port_capabilities(port):
     """Return a list of port's capabilities"""
-    return port.get(portbindings.PROFILE, {}).get('capabilities', [])
+    return port.get(portbindings.PROFILE, {}).get(constants.PORT_CAP_PARAM, [])
 
 
 def get_port_id_from_gwc_row(row):
@@ -526,14 +604,9 @@ def get_port_id_from_gwc_row(row):
     return constants.RE_PORT_FROM_GWC.search(row.name).group(2)
 
 
-def get_az_hints(resource):
-    """Return the availability zone hints from a given resource."""
-    return (resource.get(az_def.AZ_HINTS) or CONF.default_availability_zones)
-
-
 def get_chassis_availability_zones(chassis):
     """Return a list of availability zones from a given OVN Chassis."""
-    azs = []
+    azs = set()
     if not chassis:
         return azs
 
@@ -542,11 +615,42 @@ def get_chassis_availability_zones(chassis):
         if not opt.startswith(opt_key):
             continue
         values = opt.split('=')[1]
-        azs = [az.strip() for az in values.split(':') if az.strip()]
+        azs = {az.strip() for az in values.split(':') if az.strip()}
         break
     return azs
 
 
+def get_chassis_in_azs(chassis_list, az_list):
+    """Return a set of Chassis that belongs to the AZs.
+
+    Given a list of Chassis and a list of availability zones (AZs),
+    return a set of Chassis that belongs to one or more AZs.
+
+    :param chassis_list: A list of Chassis objects
+    :param az_list: A list of availability zones
+    :returns: A set of Chassis names
+    """
+    chassis = set()
+    for ch in chassis_list:
+        chassis_azs = get_chassis_availability_zones(ch)
+        if chassis_azs.intersection(az_list):
+            chassis.add(ch.name)
+    return chassis
+
+
+def get_gateway_chassis_without_azs(chassis_list):
+    """Return a set of Chassis that does not belong to any AZs.
+
+    Filter a list of Chassis and return only the Chassis that does not
+    belong to any availability zones.
+
+    :param chassis_list: A list of Chassis objects
+    :returns: A set of Chassis names
+    """
+    return {ch.name for ch in chassis_list if is_gateway_chassis(ch) and not
+            get_chassis_availability_zones(ch)}
+
+
 def parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs):
     """Return a dictionary compatible with port forwarding from OVN lb."""
     result = {}
@@ -568,3 +672,44 @@ def parse_ovn_lb_port_forwarding(ovn_rtr_lb_pfs):
 
 def get_network_name_from_datapath(datapath):
     return datapath.external_ids['name'].replace('neutron-', '')
+
+
+def is_port_external(port):
+    # This port is represented in OVN DB as lsp.type=external
+    capabilities = []
+    vnic_type = portbindings.VNIC_NORMAL
+
+    if isinstance(port, dict):
+        capabilities = get_port_capabilities(port)
+        vnic_type = port.get(portbindings.VNIC_TYPE,
+                             portbindings.VNIC_NORMAL)
+    else:
+        if isinstance(port, models_v2.Port):
+            bindings = port.port_bindings
+        elif isinstance(port, ports_obj.Port):
+            bindings = port.bindings
+        else:  # What else could be "port"?
+            bindings = []
+
+        if bindings:
+            profile = bindings[0].get('profile')
+            if profile:
+                # DB object, not OVO, stores the dict in JSON.
+                profile = (jsonutils.loads(profile) if isinstance(profile, str)
+                           else profile)
+                capabilities = profile.get(constants.PORT_CAP_PARAM, [])
+            vnic_type = bindings[0].get('vnic_type', portbindings.VNIC_NORMAL)
+
+    return (vnic_type in constants.EXTERNAL_PORT_TYPES and
+            constants.PORT_CAP_SWITCHDEV not in capabilities)
+
+
+def retry(max_=None):
+    def inner(func):
+        def wrapper(*args, **kwargs):
+            local_max = max_ or ovn_conf.get_ovn_ovsdb_retry_max_interval()
+            return tenacity.retry(
+                wait=tenacity.wait_exponential(max=local_max),
+                reraise=True)(func)(*args, **kwargs)
+        return wrapper
+    return inner
