@@ -34,8 +34,11 @@ import uuid
 import eventlet
 from eventlet.green import subprocess
 import netaddr
+from neutron_lib.api.definitions import availability_zone as az_def
 from neutron_lib import constants as n_const
+from neutron_lib import context as n_context
 from neutron_lib.db import api as db_api
+from neutron_lib.services.qos import constants as qos_consts
 from neutron_lib.services.trunk import constants as trunk_constants
 from neutron_lib.utils import helpers
 from oslo_config import cfg
@@ -46,6 +49,10 @@ from oslo_utils import timeutils
 from oslo_utils import uuidutils
 from osprofiler import profiler
 import pkg_resources
+from sqlalchemy.dialects.mysql import dialect as mysql_dialect
+from sqlalchemy.dialects.postgresql import dialect as postgresql_dialect
+from sqlalchemy.dialects.sqlite import dialect as sqlite_dialect
+from sqlalchemy.sql.expression import func as sql_func
 
 import neutron
 from neutron._i18n import _
@@ -130,13 +137,22 @@ def log_opt_values(log):
     cfg.CONF.log_opt_values(log, logging.DEBUG)
 
 
-def get_dhcp_agent_device_id(network_id, host):
+def get_dhcp_agent_device_id(network_id, host, segmentation_id=None):
     # Split host so as to always use only the hostname and
     # not the domain name. This will guarantee consistency
     # whether a local hostname or an fqdn is passed in.
     local_hostname = host.split('.')[0]
     host_uuid = uuid.uuid5(uuid.NAMESPACE_DNS, str(local_hostname))
-    return 'dhcp%s-%s' % (host_uuid, network_id)
+    if not segmentation_id:
+        return 'dhcp%s-%s' % (host_uuid, network_id)
+    return 'dhcp%s-%s-%s' % (host_uuid, network_id, segmentation_id)
+
+
+def is_dns_servers_any_address(dns_servers, ip_version):
+    """Checks if DNS server list matches the IP any address '0.0.0.0'/'::'"""
+    ip_any = netaddr.IPNetwork(n_const.IP_ANY[ip_version]).ip
+    return (len(dns_servers) == 1 and
+            netaddr.IPNetwork(dns_servers[0]).ip == ip_any)
 
 
 class exception_logger(object):
@@ -219,7 +235,7 @@ def is_fip_serviced(device_owner):
 def ip_to_cidr(ip, prefix=None):
     """Convert an ip with no prefix to cidr notation
 
-    :param ip: An ipv4 or ipv6 address.  Convertable to netaddr.IPNetwork.
+    :param ip: An ipv4 or ipv6 address.  Convertible to netaddr.IPNetwork.
     :param prefix: Optional prefix.  If None, the default 32 will be used for
         ipv4 and 128 for ipv6.
     """
@@ -308,7 +324,7 @@ def ip_version_from_int(ip_version_int):
 
 
 def get_network_length(ip_version):
-    """Returns the network length depeding on the IP version"""
+    """Returns the network length depending on the IP version"""
     return (n_const.IPv4_BITS if ip_version == n_const.IP_VERSION_4
             else n_const.IPv6_BITS)
 
@@ -670,15 +686,22 @@ def transaction_guard(f):
     If you receive this error, you must alter your code to handle the fact that
     the thing you are calling can have side effects so using transactions to
     undo on failures is not possible.
+
+    This method can be called from a class method or a static method. "inner"
+    should consider if "self" is passed or not. The next mandatory parameter is
+    the context ``neutron_lib.context.Context``.
     """
     @functools.wraps(f)
-    def inner(self, context, *args, **kwargs):
+    def inner(*args, **kwargs):
+        context = (args[0] if issubclass(type(args[0]),
+                                         n_context.ContextBaseWithSession) else
+                   args[1])
         # FIXME(kevinbenton): get rid of all uses of this flag
-        if (context.session.is_active and
+        if (db_api.is_session_active(context.session) and
                 getattr(context, 'GUARD_TRANSACTION', True)):
             raise RuntimeError(_("Method %s cannot be called within a "
                                  "transaction.") % f)
-        return f(self, context, *args, **kwargs)
+        return f(*args, **kwargs)
     return inner
 
 
@@ -967,6 +990,25 @@ def with_metaclass(meta, *bases):
     return metaclass('temporary_class', None, {})
 
 
+def get_sql_random_method(sql_dialect_name):
+    """Return the SQL random method supported depending on the dialect."""
+    # NOTE(ralonsoh): this method is a good candidate to be implemented in
+    # oslo.db.
+    # https://www.postgresql.org/docs/8.2/functions-math.html
+    # https://www.sqlite.org/c3ref/randomness.html
+    if sql_dialect_name in (postgresql_dialect.name, sqlite_dialect.name):
+        return sql_func.random
+    # https://dev.mysql.com/doc/refman/8.0/en/mathematical-functions.html
+    elif sql_dialect_name == mysql_dialect.name:
+        return sql_func.rand
+
+
+def get_az_hints(resource):
+    """Return the availability zone hints from a given resource."""
+    return (sorted(resource.get(az_def.AZ_HINTS, [])) or
+            cfg.CONF.default_availability_zones)
+
+
 def skip_exceptions(exceptions):
     """Decorator to catch and hide any provided exception in the argument"""
 
@@ -987,3 +1029,15 @@ def skip_exceptions(exceptions):
                         ctx.reraise = False
         return wrapper
     return decorator
+
+
+def effective_qos_policy_id(resource):
+    """Return the resource effective QoS policy
+
+    If the resource does not have any QoS policy reference, returns the
+    QoS policy inherited from the network (if exists).
+    :param resource: [dict] resource with QoS policies
+    :return: [str] resource QoS policy ID or network QoS policy ID
+    """
+    return (resource.get(qos_consts.QOS_POLICY_ID) or
+            resource.get(qos_consts.QOS_NETWORK_POLICY_ID))
