@@ -10,22 +10,32 @@
 #  License for the specific language governing permissions and limitations
 #  under the License.
 
+from oslo_log import versionutils
 from oslo_policy import policy
 
 from neutron.conf.policies import base
 
+DEPRECATION_REASON = (
+    "The Availability Zone API now supports project scope and default roles.")
+
 
 rules = [
     policy.DocumentedRuleDefault(
-        'get_availability_zone',
-        base.RULE_ANY,
-        'List availability zones',
-        [
+        name='get_availability_zone',
+        check_str=base.ADMIN,
+        description='List availability zones',
+        operations=[
             {
                 'method': 'GET',
                 'path': '/availability_zones',
             },
-        ]
+        ],
+        scope_types=['project'],
+        deprecated_rule=policy.DeprecatedRule(
+            name='get_availability_zone',
+            check_str=base.RULE_ANY,
+            deprecated_reason=DEPRECATION_REASON,
+            deprecated_since=versionutils.deprecated.WALLABY)
     ),
 ]
 
