@@ -20,6 +20,7 @@ from eventlet import greenthread
 from neutron_lib.agent import constants as agent_consts
 from neutron_lib.api import converters
 from neutron_lib.api.definitions import agent as agent_apidef
+from neutron_lib.api.definitions import availability_zone_filter as azfil_ext
 from neutron_lib.api import extensions
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -27,7 +28,6 @@ from neutron_lib.callbacks import resources
 from neutron_lib import constants
 from neutron_lib import context
 from neutron_lib.db import api as db_api
-from neutron_lib.db import model_query
 from neutron_lib.db import utils as db_utils
 from neutron_lib.exceptions import agent as agent_exc
 from neutron_lib.exceptions import availability_zone as az_exc
@@ -42,8 +42,6 @@ from oslo_utils import timeutils
 from neutron.agent.common import utils
 from neutron.api.rpc.callbacks import version_manager
 from neutron.conf.agent.database import agents_db
-from neutron.db.models import agent as agent_model
-from neutron.extensions import _availability_zone_filter_lib as azfil_ext
 from neutron.extensions import agent as ext_agent
 from neutron.extensions import availability_zone as az_ext
 from neutron.objects import agent as agent_obj
@@ -79,7 +77,7 @@ def get_availability_zones_by_agent_type(context, agent_type,
                                          availability_zones):
     """Get list of availability zones based on agent type"""
 
-    agents = agent_obj.Agent._get_agents_by_availability_zones_and_agent_type(
+    agents = agent_obj.Agent.get_agents_by_availability_zones_and_agent_type(
         context, agent_type=agent_type, availability_zones=availability_zones)
     return set(agent.availability_zone for agent in agents)
 
@@ -282,15 +280,6 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
             agent.update()
         return self._make_agent_dict(agent)
 
-    @db_api.retry_if_session_inactive()
-    def get_agents_db(self, context, filters=None):
-        # TODO(annp): keep this method for backward compatibility,
-        #             will need to clean it up later
-        query = model_query.get_collection_query(context,
-                                                 agent_model.Agent,
-                                                 filters=filters)
-        return query.all()
-
     @db_api.retry_if_session_inactive()
     def get_agent_objects(self, context, filters=None):
         filters = filters or {}
@@ -397,6 +386,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 agent = self._get_agent_by_type_and_host(
                     context, agent_state['agent_type'], agent_state['host'])
                 agent_state_orig = copy.deepcopy(agent_state)
+                agent_state_previous = copy.deepcopy(agent)
                 if not agent.is_active:
                     status = agent_consts.AGENT_REVIVED
                     if 'resource_versions' not in agent_state:
@@ -417,6 +407,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                 event_type = events.AFTER_UPDATE
             except agent_exc.AgentNotFoundByTypeHost:
                 agent_state_orig = None
+                agent_state_previous = None
                 greenthread.sleep(0)
                 res['created_at'] = current_time
                 res['started_at'] = current_time
@@ -441,7 +432,7 @@ class AgentDbMixin(ext_agent.AgentPluginBase, AgentAvailabilityZoneMixin):
                                  'plugin': self,
                                  'status': status
                              },
-                             states=(agent_state_orig, ),
+                             states=(agent_state_orig, agent_state_previous),
                              desired_state=agent_state,
                              resource_id=agent.id
                          ))
