@@ -143,6 +143,11 @@ class DbBasePluginCommon(object):
         allocated.create()
 
     def _make_subnet_dict(self, subnet, fields=None, context=None):
+        if isinstance(subnet, subnet_obj.Subnet):
+            standard_attr_id = subnet.db_obj.standard_attr.id
+        else:
+            standard_attr_id = subnet.standard_attr.id
+
         res = {'id': subnet['id'],
                'name': subnet['name'],
                'tenant_id': subnet['tenant_id'],
@@ -152,9 +157,10 @@ class DbBasePluginCommon(object):
                'enable_dhcp': subnet['enable_dhcp'],
                'ipv6_ra_mode': subnet['ipv6_ra_mode'],
                'ipv6_address_mode': subnet['ipv6_address_mode'],
+               'standard_attr_id': standard_attr_id,
                }
         res['gateway_ip'] = str(
-                subnet['gateway_ip']) if subnet['gateway_ip'] else None
+            subnet['gateway_ip']) if subnet['gateway_ip'] is not None else None
         # TODO(korzen) this method can get subnet as DB object or Subnet OVO,
         # so temporary workaround will be to fill in the fields in separate
         # ways. After converting all code pieces to use Subnet OVO, the latter
@@ -219,6 +225,13 @@ class DbBasePluginCommon(object):
                         process_extensions=True,
                         with_fixed_ips=True,
                         bulk=False):
+        if isinstance(port, port_obj.Port):
+            port_data = port.db_obj
+            standard_attr_id = port.db_obj.standard_attr.id
+        else:
+            port_data = port
+            standard_attr_id = port.standard_attr.id
+
         mac = port["mac_address"]
         if isinstance(mac, netaddr.EUI):
             mac.dialect = netaddr.mac_unix_expanded
@@ -230,7 +243,9 @@ class DbBasePluginCommon(object):
                "admin_state_up": port["admin_state_up"],
                "status": port["status"],
                "device_id": port["device_id"],
-               "device_owner": port["device_owner"]}
+               "device_owner": port["device_owner"],
+               'standard_attr_id': standard_attr_id,
+               }
         if with_fixed_ips:
             res["fixed_ips"] = [
                 {'subnet_id': ip["subnet_id"],
@@ -238,15 +253,13 @@ class DbBasePluginCommon(object):
                      ip["ip_address"])} for ip in port["fixed_ips"]]
         # Call auxiliary extend functions, if any
         if process_extensions:
-            port_data = port
-            if isinstance(port, port_obj.Port):
-                port_data = port.db_obj
             res['bulk'] = bulk
             resource_extend.apply_funcs(
                 port_def.COLLECTION_NAME, res, port_data)
             res.pop('bulk')
         return db_utils.resource_fields(res, fields)
 
+    @db_api.CONTEXT_READER
     def _get_network(self, context, id):
         try:
             network = model_query.get_by_id(context, models_v2.Network, id)
@@ -254,6 +267,7 @@ class DbBasePluginCommon(object):
             raise exceptions.NetworkNotFound(net_id=id)
         return network
 
+    @db_api.CONTEXT_READER
     def _network_exists(self, context, network_id):
         query = model_query.query_with_hooks(
             context, models_v2.Network, field='id')
@@ -272,9 +286,11 @@ class DbBasePluginCommon(object):
             raise exceptions.SubnetPoolNotFound(subnetpool_id=id)
         return subnetpool
 
-    def _get_port(self, context, id):
+    @db_api.CONTEXT_READER
+    def _get_port(self, context, id, lazy_fields=None):
         try:
-            port = model_query.get_by_id(context, models_v2.Port, id)
+            port = model_query.get_by_id(context, models_v2.Port, id,
+                                         lazy_fields=lazy_fields)
         except exc.NoResultFound:
             raise exceptions.PortNotFound(port_id=id)
         return port
@@ -318,7 +334,8 @@ class DbBasePluginCommon(object):
                'mtu': network.get('mtu', constants.DEFAULT_NETWORK_MTU),
                'status': network['status'],
                'subnets': [subnet['id']
-                           for subnet in network['subnets']]}
+                           for subnet in network['subnets']],
+               'standard_attr_id': network.standard_attr.id}
         res['shared'] = self._is_network_shared(context, network.rbac_entries)
         # Call auxiliary extend functions, if any
         if process_extensions:
@@ -331,7 +348,7 @@ class DbBasePluginCommon(object):
         matches = ('*',) + ((context.tenant_id,) if context else ())
         for entry in rbac_entries:
             if (entry.action == 'access_as_shared' and
-                    entry.target_tenant in matches):
+                    entry.target_project in matches):
                 return True
         return False
 
