@@ -16,8 +16,10 @@
 import functools
 
 import netaddr
+from neutron_lib.api import converters
 from neutron_lib.api.definitions import external_net as extnet_def
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
+from neutron_lib.api.definitions import port as port_def
 from neutron_lib.api.definitions import portbindings as portbindings_def
 from neutron_lib.api.definitions import subnetpool as subnetpool_def
 from neutron_lib.api import validators
@@ -41,6 +43,7 @@ from oslo_db import exception as os_db_exc
 from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import uuidutils
+from sqlalchemy import and_
 from sqlalchemy import exc as sql_exc
 from sqlalchemy import func
 from sqlalchemy import not_
@@ -110,10 +113,9 @@ def _update_subnetpool_dict(orig_pool, new_pool):
 def _port_filter_hook(context, original_model, conditions):
     # Apply the port filter only in non-admin and non-advsvc context
     if ndb_utils.model_query_scope_is_project(context, original_model):
-        conditions |= (models_v2.Port.network_id.in_(
-            context.session.query(models_v2.Network.id).
-            filter(context.project_id == models_v2.Network.project_id).
-            subquery()))
+        conditions |= and_(
+            models_v2.Port.network_id == models_v2.Network.id,
+            models_v2.Network.project_id == context.project_id)
     return conditions
 
 
@@ -216,49 +218,51 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 raise exc.InvalidInput(error_message=msg)
 
         tenant_to_check = None
-        self_sharing = policy['target_tenant'] == net['tenant_id']
+        self_sharing = policy['target_project'] == net['tenant_id']
         if self_sharing:
             return
         if event == events.BEFORE_UPDATE:
-            new_tenant = payload.request_body['target_tenant']
-            if policy['target_tenant'] != new_tenant:
-                tenant_to_check = policy['target_tenant']
+            new_tenant = payload.request_body['target_project']
+            if policy['target_project'] != new_tenant:
+                tenant_to_check = policy['target_project']
 
         if event == events.BEFORE_DELETE:
-            tenant_to_check = policy['target_tenant']
+            tenant_to_check = policy['target_project']
 
         if tenant_to_check:
-            self.ensure_no_tenant_ports_on_network(net['id'], net['tenant_id'],
-                                                   tenant_to_check)
+            self.ensure_no_tenant_ports_on_network(
+                context, net['id'], net['tenant_id'], tenant_to_check)
 
-    def ensure_no_tenant_ports_on_network(self, network_id, net_tenant_id,
-                                          tenant_id):
-        ctx_admin = ctx.get_admin_context()
-        ports = model_query.query_with_hooks(ctx_admin, models_v2.Port).filter(
-            models_v2.Port.network_id == network_id)
-        if tenant_id == '*':
-            # for the wildcard we need to get all of the rbac entries to
-            # see if any allow the remaining ports on the network.
-            # any port with another RBAC entry covering it or one belonging to
-            # the same tenant as the network owner is ok
-            other_rbac_objs = network_obj.NetworkRBAC.get_objects(
-                ctx_admin, object_id=network_id, action='access_as_shared')
-            allowed_tenants = [rbac['target_tenant'] for rbac
-                               in other_rbac_objs
-                               if rbac.target_tenant != tenant_id]
-            allowed_tenants.append(net_tenant_id)
-            ports = ports.filter(
-                ~models_v2.Port.tenant_id.in_(allowed_tenants))
-        else:
-            # if there is a wildcard rule, we can return early because it
-            # allows any ports
-            if network_obj.NetworkRBAC.get_object(
-                    ctx_admin, object_id=network_id, action='access_as_shared',
-                    target_tenant='*'):
-                return
-            ports = ports.filter(models_v2.Port.project_id == tenant_id)
-        if ports.count():
-            raise exc.InvalidSharedSetting(network=network_id)
+    def ensure_no_tenant_ports_on_network(self, context, network_id,
+                                          net_tenant_id, tenant_id):
+        elevated = context.elevated()
+        with db_api.CONTEXT_READER.using(elevated):
+            ports = model_query.query_with_hooks(
+                elevated, models_v2.Port).filter(
+                models_v2.Port.network_id == network_id)
+            if tenant_id == '*':
+                # for the wildcard we need to get all of the rbac entries to
+                # see if any allow the remaining ports on the network.
+                # any port with another RBAC entry covering it or one belonging
+                # to the same tenant as the network owner is ok
+                other_rbac_objs = network_obj.NetworkRBAC.get_objects(
+                    elevated, object_id=network_id, action='access_as_shared')
+                allowed_tenants = [rbac['target_project'] for rbac
+                                   in other_rbac_objs
+                                   if rbac.target_project != tenant_id]
+                allowed_tenants.append(net_tenant_id)
+                ports = ports.filter(
+                    ~models_v2.Port.tenant_id.in_(allowed_tenants))
+            else:
+                # if there is a wildcard rule, we can return early because it
+                # allows any ports
+                if network_obj.NetworkRBAC.get_object(
+                        elevated, object_id=network_id,
+                        action='access_as_shared', target_project='*'):
+                    return
+                ports = ports.filter(models_v2.Port.project_id == tenant_id)
+            if ports.count():
+                raise exc.InvalidSharedSetting(network=network_id)
 
     def set_ipam_backend(self):
         self.ipam = ipam_pluggable_backend.IpamPluggableBackend()
@@ -301,8 +305,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         ctx_admin = ctx.get_admin_context()
         other_rbac_objs = network_obj.NetworkRBAC.get_objects(
             ctx_admin, object_id=network.id, action='access_as_shared')
-        allowed_projects = {rbac['target_tenant'] for rbac in other_rbac_objs
-                            if rbac.target_tenant != '*'}
+        allowed_projects = {rbac['target_project'] for rbac in other_rbac_objs
+                            if rbac.target_project != '*'}
         allowed_projects.add(network.project_id)
         if project_ids - allowed_projects:
             raise exc.InvalidSharedSetting(network=network.name)
@@ -399,8 +403,10 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     def create_network_db(self, context, network):
         # single request processing
         n = network['network']
+        # TODO(ralonsoh): "tenant_id" reference should be removed.
+        project_id = n.get('project_id') or n['tenant_id']
         with db_api.CONTEXT_WRITER.using(context):
-            args = {'tenant_id': n['tenant_id'],
+            args = {'tenant_id': project_id,
                     'id': n.get('id') or uuidutils.generate_uuid(),
                     'name': n['name'],
                     'mtu': n.get('mtu', constants.DEFAULT_NETWORK_MTU),
@@ -413,7 +419,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 np_rbac_args = {'project_id': network.project_id,
                                 'object_id': network.id,
                                 'action': 'access_as_shared',
-                                'target_tenant': '*'}
+                                'target_project': '*'}
                 np_rbac_obj = network_obj.NetworkRBAC(context, **np_rbac_args)
                 np_rbac_obj.create()
         return network
@@ -422,7 +428,8 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     def update_network(self, context, id, network, db_network=None):
         n = network['network']
         # we dont't use DB objects not belonging to the current active session
-        db_network = db_network if context.session.is_active else None
+        db_network = (db_network if db_api.is_session_active(context.session)
+                      else None)
         with db_api.CONTEXT_WRITER.using(context):
             network = db_network or self._get_network(context, id)
             # validate 'shared' parameter
@@ -430,7 +437,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 entry = None
                 for item in network.rbac_entries:
                     if (item.action == 'access_as_shared' and
-                            item.target_tenant == '*'):
+                            item.target_project == '*'):
                         entry = item
                         break
                 setattr(network, 'shared', bool(entry))
@@ -440,14 +447,14 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     np_rbac_args = {'project_id': network.project_id,
                                     'object_id': network.id,
                                     'action': 'access_as_shared',
-                                    'target_tenant': '*'}
+                                    'target_project': '*'}
                     np_rbac_obj = network_obj.NetworkRBAC(context,
                                                           **np_rbac_args)
                     np_rbac_obj.create()
                 elif not update_shared and entry:
                     network_obj.NetworkRBAC.delete_objects(
                         context, object_id=network.id,
-                        action='access_as_shared', target_tenant='*')
+                        action='access_as_shared', target_project='*')
 
                 # TODO(ihrachys) Below can be removed when we make sqlalchemy
                 # event listeners in neutron_lib/db/api.py to refresh expired
@@ -472,15 +479,18 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             ~models_v2.Port.device_owner.in_(
                 _constants.AUTO_DELETE_PORT_OWNERS))
         if non_auto_ports.count():
-            raise exc.NetworkInUse(net_id=net_id)
+            ports = [port.id for port in non_auto_ports.all()]
+            reason = _("There are one or more ports still in use on the "
+                       "network, id for these ports is: %s" % ",".join(ports))
+            raise exc.NetworkInUse(net_id=net_id, reason=reason)
 
     @db_api.retry_if_session_inactive()
     def delete_network(self, context, id):
         registry.publish(resources.NETWORK, events.BEFORE_DELETE, self,
                          payload=events.DBEventPayload(
                              context, resource_id=id))
-        self._ensure_network_not_in_use(context, id)
         with db_api.CONTEXT_READER.using(context):
+            self._ensure_network_not_in_use(context, id)
             auto_delete_port_ids = [p.id for p in context.session.query(
                 models_v2.Port.id).filter_by(network_id=id).filter(
                 models_v2.Port.device_owner.in_(
@@ -499,24 +509,29 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             # cleanup if a network-owned port snuck in without failing
             for subnet in subnets:
                 self._delete_subnet(context, subnet)
-                # TODO(ralonsoh): use payloads
-                registry.notify(resources.SUBNET, events.AFTER_DELETE,
-                                self, context=context, subnet=subnet.to_dict(),
-                                for_net_delete=True)
+                registry.publish(
+                    resources.SUBNET, events.AFTER_DELETE, self,
+                    payload=events.DBEventPayload(
+                        context,
+                        resource_id=subnet['id'],
+                        metadata={'for_net_delete': True},
+                        states=(subnet.to_dict(),)))
             with db_api.CONTEXT_WRITER.using(context):
                 network_db = self._get_network(context, id)
                 network = self._make_network_dict(network_db, context=context)
-                registry.notify(resources.NETWORK, events.PRECOMMIT_DELETE,
-                                self, context=context, network_id=id,
-                                network=network)
+                registry.publish(resources.NETWORK, events.PRECOMMIT_DELETE,
+                                 self, payload=events.DBEventPayload(
+                                     context, resource_id=id,
+                                     states=(network,)))
                 # We expire network_db here because precommit deletion
                 # might have left the relationship stale, for example,
                 # if we deleted a segment.
                 context.session.expire(network_db)
                 network_db = self._get_network(context, id)
                 context.session.delete(network_db)
-        registry.notify(resources.NETWORK, events.AFTER_DELETE,
-                        self, context=context, network=network)
+        registry.publish(resources.NETWORK, events.AFTER_DELETE,
+                         self, payload=events.DBEventPayload(
+                             context, resource_id=id, states=(network,)))
 
     @db_api.retry_if_session_inactive()
     def get_network(self, context, id, fields=None):
@@ -554,8 +569,9 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
 
     @db_api.retry_if_session_inactive()
     def get_networks_count(self, context, filters=None):
-        return model_query.get_collection_count(context, models_v2.Network,
-                                                filters=filters)
+        return model_query.get_collection_count(
+            context, models_v2.Network, filters=filters,
+            query_field=models_v2.Network.id.key)
 
     @db_api.retry_if_session_inactive()
     def create_subnet_bulk(self, context, subnets):
@@ -628,20 +644,19 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             # NOTE(salv-orlando): There is slight chance of a race, when
             # a subnet-update and a router-interface-add operation are
             # executed concurrently
-            if cur_subnet and not ipv6_utils.is_ipv6_pd_enabled(s):
-                with db_api.CONTEXT_READER.using(context):
-                    # TODO(electrocucaracha): Look a solution for Join in OVO
-                    ipal = models_v2.IPAllocation
-                    alloc_qry = context.session.query(ipal.port_id)
-                    alloc_qry = alloc_qry.join("port", "routerport")
-                    gateway_ip = str(cur_subnet['gateway_ip'])
-                    allocated = alloc_qry.filter(
-                        ipal.ip_address == gateway_ip,
-                        ipal.subnet_id == cur_subnet['id']).first()
-                if allocated and allocated.port_id:
+            s_gateway_ip = netaddr.IPAddress(s['gateway_ip'])
+            if (cur_subnet and
+                    s_gateway_ip != cur_subnet['gateway_ip'] and
+                    not ipv6_utils.is_ipv6_pd_enabled(s)):
+                gateway_ip = str(cur_subnet['gateway_ip'])
+                alloc = port_obj.IPAllocation.get_alloc_routerports(
+                    context, cur_subnet['id'], gateway_ip=gateway_ip,
+                    first=True)
+
+                if alloc and alloc.port_id:
                     raise exc.GatewayIpInUse(
                         ip_address=gateway_ip,
-                        port_id=allocated.port_id)
+                        port_id=alloc.port_id)
 
         if validators.is_attr_set(s.get('dns_nameservers')):
             if len(s['dns_nameservers']) > cfg.CONF.max_dns_nameservers:
@@ -735,7 +750,7 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         external_gateway_info['external_fixed_ips'].append(
                                      {'subnet_id': subnet['id']})
         info = {'router': {'external_gateway_info': external_gateway_info}}
-        l3plugin.update_router(context, router_id, info)
+        l3plugin.update_router(ctx_admin, router_id, info)
 
     @db_api.retry_if_session_inactive()
     def _create_subnet_postcommit(self, context, result,
@@ -942,10 +957,12 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             if gateway_ip:
                 self.ipam.validate_gw_out_of_pools(gateway_ip, pools)
 
-        kwargs = {'context': context, 'original_subnet': orig,
-                  'request': s}
-        registry.notify(resources.SUBNET, events.BEFORE_UPDATE,
-                        self, **kwargs)
+        registry.publish(
+            resources.SUBNET, events.BEFORE_UPDATE, self,
+            payload=events.DBEventPayload(
+                context,
+                resource_id=id,
+                states=(orig, s,)))
 
         with db_api.CONTEXT_WRITER.using(context):
             subnet, changes = self.ipam.update_db_subnet(
@@ -996,10 +1013,13 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
             if routers:
                 self.l3_rpc_notifier.routers_updated(context, routers)
 
-        kwargs = {'context': context, 'subnet': result,
-                  'original_subnet': orig}
-        registry.notify(resources.SUBNET, events.AFTER_UPDATE, self,
-                        **kwargs)
+        registry.publish(
+            resources.SUBNET, events.AFTER_UPDATE, self,
+            payload=events.DBEventPayload(
+                context,
+                resource_id=result['id'],
+                states=(orig, result,)))
+
         return result
 
     def _subnet_get_user_allocation(self, context, subnet_id):
@@ -1053,15 +1073,21 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 payload=events.DBEventPayload(context, resource_id=subnet.id))
             self._remove_subnet_ip_allocations_from_ports(context, subnet)
             self._delete_subnet(context, subnet)
-        registry.notify(resources.SUBNET, events.AFTER_DELETE,
-                        self, context=context, subnet=subnet.to_dict())
+        registry.publish(
+            resources.SUBNET, events.AFTER_DELETE, self,
+            payload=events.DBEventPayload(
+                context,
+                resource_id=id,
+                states=(subnet.to_dict(),)))
 
     def _delete_subnet(self, context, subnet):
         with db_api.exc_to_retry(sql_exc.IntegrityError), \
                 db_api.CONTEXT_WRITER.using(context):
-            registry.notify(resources.SUBNET, events.PRECOMMIT_DELETE,
-                            self, context=context, subnet_id=subnet.id,
-                            subnet_obj=subnet)
+            registry.publish(
+                resources.SUBNET, events.PRECOMMIT_DELETE, self,
+                payload=events.DBEventPayload(context,
+                                              resource_id=subnet.id,
+                                              states=(subnet,)))
             subnet.delete()
             # Delete related ipam subnet manually,
             # since there is no FK relationship
@@ -1411,12 +1437,14 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         p = port['port']
         port_id = p.get('id') or uuidutils.generate_uuid()
         network_id = p['network_id']
+        # TODO(ralonsoh): "tenant_id" reference should be removed.
+        project_id = p.get('project_id') or p['tenant_id']
         if p.get('device_owner'):
             self._enforce_device_owner_not_router_intf_or_device_id(
                 context, p.get('device_owner'), p.get('device_id'),
-                p['tenant_id'])
+                project_id)
 
-        port_data = dict(tenant_id=p['tenant_id'],
+        port_data = dict(tenant_id=project_id,
                          name=p['name'],
                          id=port_id,
                          network_id=network_id,
@@ -1463,14 +1491,17 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 context, current_owner, current_device_id,
                 db_port['tenant_id'])
 
-        if new_mac and new_mac != db_port['mac_address']:
+        if (new_mac and
+                new_mac != converters.convert_to_sanitized_mac_address(
+                    db_port['mac_address'])):
             self._check_mac_addr_update(context, db_port,
                                         new_mac, current_owner)
 
     @db_api.retry_if_session_inactive()
     def update_port(self, context, id, port, db_port=None):
         # we dont't use DB objects not belonging to the current active session
-        db_port = db_port if context.session.is_active else None
+        db_port = (db_port if db_api.is_session_active(context.session)
+                   else None)
         new_port = port['port']
 
         with db_api.CONTEXT_WRITER.using(context):
@@ -1495,21 +1526,14 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                     old_port_db=db_port,
                     old_port=self._make_port_dict(db_port),
                     new_port=new_port)
-            except ipam_exc.IpAddressAllocationNotFound as e:
-                # If a port update and a subnet delete interleave, there is a
-                # chance that the IPAM update operation raises this exception.
-                # Rather than throwing that up to the user under some sort of
-                # conflict, bubble up a retry instead that should bring things
-                # back to sanity.
-                raise os_db_exc.RetryRequest(e)
             except ipam_exc.IPAddressChangeNotAllowed as e:
                 raise exc.BadRequest(resource='ports', msg=e)
         return self._make_port_dict(db_port)
 
     @db_api.retry_if_session_inactive()
-    def delete_port(self, context, id):
+    def delete_port(self, context, id, port=None):
         with db_api.CONTEXT_WRITER.using(context):
-            self.ipam.delete_port(context, id)
+            self.ipam.delete_port(context, id, port)
 
     def delete_ports_by_device_id(self, context, device_id, network_id=None):
         with db_api.CONTEXT_READER.using(context):
@@ -1531,10 +1555,13 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
     @db_api.retry_if_session_inactive()
     @db_api.CONTEXT_READER
     def get_port(self, context, id, fields=None):
-        port = self._get_port(context, id)
+        lazy_fields = [models_v2.Port.port_forwardings,
+                       models_v2.Port.distributed_port_binding]
+        port = self._get_port(context, id, lazy_fields=lazy_fields)
         return self._make_port_dict(port, fields)
 
-    def _get_ports_query(self, context, filters=None, *args, **kwargs):
+    def _get_ports_query(self, context, filters=None, lazy_fields=None,
+                         *args, **kwargs):
         Port = models_v2.Port
         IPAllocation = models_v2.IPAllocation
 
@@ -1545,15 +1572,17 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
         vif_type = filters.pop(portbindings_def.VIF_TYPE, None)
         query = model_query.get_collection_query(context, Port,
                                                  filters=filters,
+                                                 lazy_fields=lazy_fields,
                                                  *args, **kwargs)
         ip_addresses = fixed_ips.get('ip_address')
         subnet_ids = fixed_ips.get('subnet_id')
         if vif_type is not None:
             query = query.filter(Port.port_bindings.any(vif_type=vif_type))
         if mac_address:
-            lowered_macs = [x.lower() for x in mac_address]
-            query = query.filter(func.lower(Port.mac_address).in_(
-                                                    lowered_macs))
+            sanitized_macs = [converters.convert_to_sanitized_mac_address(x)
+                              for x in mac_address]
+            query = query.filter(
+                func.lower(Port.mac_address).in_(sanitized_macs))
         if ip_addresses:
             query = query.filter(
                 Port.fixed_ips.any(IPAllocation.ip_address.in_(ip_addresses)))
@@ -1562,26 +1591,31 @@ class NeutronDbPluginV2(db_base_plugin_common.DbBasePluginCommon,
                 Port.fixed_ips.any(IPAllocation.subnet_id.in_(subnet_ids)))
         if limit:
             query = query.limit(limit)
+        query = query.distinct()
         return query
 
     @db_api.retry_if_session_inactive()
+    @db_api.CONTEXT_READER
     def get_ports(self, context, filters=None, fields=None,
                   sorts=None, limit=None, marker=None,
                   page_reverse=False):
         marker_obj = ndb_utils.get_marker_obj(self, context, 'port',
                                               limit, marker)
+        lazy_fields = [models_v2.Port.port_forwardings,
+                       models_v2.Port.distributed_port_binding]
         query = self._get_ports_query(context, filters=filters,
                                       sorts=sorts, limit=limit,
                                       marker_obj=marker_obj,
-                                      page_reverse=page_reverse)
+                                      page_reverse=page_reverse,
+                                      lazy_fields=lazy_fields)
         items = [self._make_port_dict(c, fields, bulk=True) for c in query]
-        # TODO(obondarev): use neutron_lib constant
-        resource_extend.apply_funcs('ports_bulk', items, None)
+        resource_extend.apply_funcs(port_def.COLLECTION_NAME_BULK, items, None)
         if limit and page_reverse:
             items.reverse()
         return items
 
     @db_api.retry_if_session_inactive()
+    @db_api.CONTEXT_READER
     def get_ports_count(self, context, filters=None):
         return self._get_ports_query(context, filters).count()
 
