@@ -20,6 +20,7 @@ import itertools
 import netaddr
 from neutron_lib.api.definitions import ip_allocation as ipalloc_apidef
 from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import segment
 from neutron_lib.api import validators
 from neutron_lib import constants as const
 from neutron_lib.db import api as db_api
@@ -27,7 +28,6 @@ from neutron_lib.db import utils as db_utils
 from neutron_lib import exceptions as exc
 from neutron_lib.exceptions import address_scope as addr_scope_exc
 from neutron_lib.utils import net as net_utils
-from oslo_config import cfg
 from oslo_log import log as logging
 from sqlalchemy.orm import exc as orm_exc
 
@@ -35,7 +35,6 @@ from neutron._i18n import _
 from neutron.common import ipv6_utils
 from neutron.db import db_base_plugin_common
 from neutron.db import models_v2
-from neutron.extensions import segment
 from neutron.ipam import exceptions as ipam_exceptions
 from neutron.ipam import utils as ipam_utils
 from neutron.objects import address_scope as addr_scope_obj
@@ -58,9 +57,6 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         if subnet.get('gateway_ip') is const.ATTR_NOT_SPECIFIED:
             if subnet.get('ip_version') == const.IP_VERSION_6:
                 gateway_ip = netaddr.IPNetwork(cidr_net).network
-                pd_net = netaddr.IPNetwork(const.PROVISIONAL_IPV6_PD_PREFIX)
-                if gateway_ip == pd_net.network:
-                    return
             else:
                 gateway_ip = netaddr.IPNetwork(cidr_net).network + 1
             return str(gateway_ip)
@@ -241,18 +237,14 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
             if cidr.prefixlen == 0:
                 err_msg = _("0 is not allowed as CIDR prefix length")
                 raise exc.InvalidInput(error_message=err_msg)
-        if cfg.CONF.allow_overlapping_ips:
-            subnet_list = [{'id': s.id, 'cidr': s.cidr}
-                           for s in network.subnets]
-        else:
-            subnet_list = subnet_obj.Subnet.get_subnet_cidrs(context)
+        subnet_list = [{'id': s.id, 'cidr': s.cidr} for s in network.subnets]
         for subnet in subnet_list:
             if ((netaddr.IPSet([subnet['cidr']]) & new_subnet_ipset) and
                     str(subnet['cidr']) != const.PROVISIONAL_IPV6_PD_PREFIX):
                 # don't give out details of the overlapping subnet
-                err_msg = ("Requested subnet with cidr: %(cidr)s for "
+                err_msg = (_("Requested subnet with cidr: %(cidr)s for "
                            "network: %(network_id)s overlaps with another "
-                           "subnet" %
+                           "subnet") %
                            {'cidr': new_subnet_cidr,
                             'network_id': network.id})
                 LOG.info("Validation for CIDR: %(new_cidr)s failed - "
@@ -310,9 +302,17 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         Finally, verify that each range fall within the subnet's CIDR.
         """
         subnet = netaddr.IPNetwork(subnet_cidr)
-        subnet_first_ip = netaddr.IPAddress(subnet.first + 1)
-        # last address is broadcast in v4
-        subnet_last_ip = netaddr.IPAddress(subnet.last - (subnet.version == 4))
+        if subnet.version == const.IP_VERSION_4:
+            if subnet.prefixlen <= 30:
+                subnet_first_ip = netaddr.IPAddress(subnet.first + 1)
+                # last address is broadcast in v4
+                subnet_last_ip = netaddr.IPAddress(subnet.last - 1)
+            else:
+                subnet_first_ip = netaddr.IPAddress(subnet.first)
+                subnet_last_ip = netaddr.IPAddress(subnet.last)
+        else:  # IPv6 case
+            subnet_first_ip = netaddr.IPAddress(subnet.first + 1)
+            subnet_last_ip = netaddr.IPAddress(subnet.last)
 
         LOG.debug("Performing IP validity checks on allocation pools")
         ip_sets = []
@@ -355,9 +355,24 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
                         subnet_cidr=subnet_cidr)
 
     def _validate_segment(self, context, network_id, segment_id, action=None,
-                          old_segment_id=None):
-        segments = subnet_obj.Subnet.get_values(
-            context, 'segment_id', network_id=network_id)
+                          old_segment_id=None, requested_service_types=None,
+                          subnet_id=None):
+        # NOTE(zigo): If we're creating a network:routed subnet (here written
+        # as: const.DEVICE_OWNER_ROUTED), then the created subnet must be
+        # removed from the segment list, otherwise its segment ID will be
+        # returned as None, and SubnetsNotAllAssociatedWithSegments will be
+        # raised.
+        if (action == 'create' and requested_service_types and
+                const.DEVICE_OWNER_ROUTED in requested_service_types):
+            to_create_subnet_id = subnet_id
+        else:
+            to_create_subnet_id = None
+
+        segments = subnet_obj.Subnet.get_subnet_segment_ids(
+                    context, network_id,
+                    ignored_service_type=const.DEVICE_OWNER_ROUTED,
+                    subnet_id=to_create_subnet_id)
+
         associated_segments = set(segments)
         if None in associated_segments and len(associated_segments) > 1:
             raise segment_exc.SubnetsNotAllAssociatedWithSegments(
@@ -458,9 +473,9 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         if device_owner in const.ROUTER_INTERFACE_OWNERS_SNAT:
             return True
 
-        subnet_obj = self._get_subnet_object(context, subnet_id)
-        return not (ipv6_utils.is_auto_address_subnet(subnet_obj) and
-                    not ipv6_utils.is_ipv6_pd_enabled(subnet_obj))
+        _subnet_obj = self._get_subnet_object(context, subnet_id)
+        return not (ipv6_utils.is_auto_address_subnet(_subnet_obj) and
+                    not ipv6_utils.is_ipv6_pd_enabled(_subnet_obj))
 
     def _get_changed_ips_for_port(self, context, original_ips,
                                   new_ips, device_owner):
@@ -576,9 +591,8 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         if segment_id:
             # TODO(slaweq): integrate check if segment exists in
             # self._validate_segment() method
-            segment = network_obj.NetworkSegment.get_object(context,
-                                                            id=segment_id)
-            if not segment:
+            if not network_obj.NetworkSegment.get_object(context,
+                                                         id=segment_id):
                 raise segment_exc.SegmentNotFound(segment_id=segment_id)
 
         subnet = subnet_obj.Subnet(context, **subnet_args)
@@ -586,7 +600,10 @@ class IpamBackendMixin(db_base_plugin_common.DbBasePluginCommon):
         # TODO(slaweq): when check is segment exists will be integrated in
         # self._validate_segment() method, it should be moved to be done before
         # subnet object is created
-        self._validate_segment(context, network['id'], segment_id)
+        self._validate_segment(context, network['id'], segment_id,
+                               action='create',
+                               requested_service_types=service_types,
+                               subnet_id=subnet.id)
 
         # NOTE(changzhi) Store DNS nameservers with order into DB one
         # by one when create subnet with DNS nameservers
