@@ -18,6 +18,7 @@ from neutron_lib.api.definitions import external_net as extnet_apidef
 from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import portbindings_extended
+from neutron_lib.api.definitions import router_admin_state_down_before_update
 from neutron_lib.api import validators
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import exceptions
@@ -47,7 +48,6 @@ from neutron.db import l3_attrs_db
 from neutron.db import l3_db
 from neutron.db.models import allowed_address_pair as aap_models
 from neutron.db import models_v2
-from neutron.extensions import _admin_state_down_before_update_lib
 from neutron.ipam import utils as ipam_utils
 from neutron.objects import agent as ag_obj
 from neutron.objects import base as base_obj
@@ -65,7 +65,7 @@ def is_admin_state_down_necessary():
     global _IS_ADMIN_STATE_DOWN_NECESSARY
     if _IS_ADMIN_STATE_DOWN_NECESSARY is None:
         _IS_ADMIN_STATE_DOWN_NECESSARY = \
-            _admin_state_down_before_update_lib.ALIAS in (extensions.
+            router_admin_state_down_before_update.ALIAS in (extensions.
                     PluginAwareExtensionManager.get_instance().extensions)
     return _IS_ADMIN_STATE_DOWN_NECESSARY
 
@@ -86,7 +86,7 @@ def is_port_bound(port):
 class DVRResourceOperationHandler(object):
     """Contains callbacks for DVR operations.
 
-    This can be implemented as a mixin or can be intantiated as a stand-alone
+    This can be implemented as a mixin or can be instantiated as a stand-alone
     object. Either way, it will subscribe itself to the relevant L3 events and
     use the plugin directory to find the L3 plugin to make calls to it as
     necessary.
@@ -101,9 +101,11 @@ class DVRResourceOperationHandler(object):
 
     @registry.receives(resources.ROUTER, [events.PRECOMMIT_CREATE],
                        priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
-    def _set_distributed_flag(self, resource, event, trigger, context,
-                              router, router_db, **kwargs):
+    def _set_distributed_flag(self, resource, event, trigger, payload):
         """Event handler to set distributed flag on creation."""
+        context = payload.context
+        router = payload.latest_state
+        router_db = payload.metadata['router_db']
         dist = is_distributed_router(router)
         router['distributed'] = dist
         self.l3plugin.set_extra_attr_value(context, router_db, 'distributed',
@@ -148,9 +150,12 @@ class DVRResourceOperationHandler(object):
         # Notify advanced services of the imminent state transition
         # for the router.
         try:
-            kwargs = {'context': context, 'router': router_db}
-            registry.notify(
-                resources.ROUTER, events.BEFORE_UPDATE, self, **kwargs)
+            registry.publish(
+                resources.ROUTER, events.BEFORE_UPDATE, self,
+                payload=events.DBEventPayload(
+                    context,
+                    resource_id=router_db['id'],
+                    states=(old_router, router_db,)))
         except exceptions.CallbackFailure as e:
             # NOTE(armax): preserve old check's behavior
             if len(e.errors) == 1:
@@ -204,11 +209,12 @@ class DVRResourceOperationHandler(object):
     @registry.receives(resources.ROUTER, [events.AFTER_UPDATE],
                        priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _delete_distributed_port_bindings_after_change(self, resource, event,
-                                                       trigger, context,
-                                                       router_id, router,
-                                                       request_attrs,
-                                                       router_db, **kwargs):
-        old_router = kwargs['old_router']
+                                                       trigger, payload):
+        context = payload.context
+        router = payload.latest_state
+        router_db = payload.metadata['router_db']
+        old_router = payload.states[0] if (len(payload.states) > 1) else None
+
         if (old_router and old_router['distributed'] and not
                 router['distributed']):
             self._core_plugin.delete_distributed_port_bindings_by_router_id(
@@ -217,12 +223,15 @@ class DVRResourceOperationHandler(object):
     @registry.receives(resources.ROUTER, [events.AFTER_UPDATE],
                        priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _delete_snat_interfaces_after_change(self, resource, event, trigger,
-                                             context, router_id, router,
-                                             request_attrs, router_db,
-                                             **kwargs):
+                                             payload):
+        context = payload.context
+        router = payload.latest_state
+        router_db = payload.metadata['router_db']
+
         if (router.get(l3_apidef.EXTERNAL_GW_INFO) and
                 not router['distributed']):
-            old_router = kwargs['old_router']
+            old_router = (payload.states[0] if (len(payload.states) > 1)
+                          else None)
             if old_router and old_router['distributed']:
                 self.delete_csnat_router_interface_ports(
                     context.elevated(), router_db)
@@ -231,9 +240,11 @@ class DVRResourceOperationHandler(object):
                        [events.AFTER_CREATE, events.AFTER_UPDATE],
                        priority_group.PRIORITY_ROUTER_EXTENDED_ATTRIBUTE)
     def _create_snat_interfaces_after_change(self, resource, event, trigger,
-                                             context, router_id, router,
-                                             request_attrs, router_db,
-                                             **kwargs):
+                                             payload):
+        context = payload.context
+        router = payload.latest_state
+        router_db = payload.metadata['router_db']
+
         if (not router.get(l3_apidef.EXTERNAL_GW_INFO) or
                 not router['distributed']):
             # we don't care if it's not distributed or not attached to an
@@ -242,7 +253,8 @@ class DVRResourceOperationHandler(object):
         if event == events.AFTER_UPDATE:
             # after an update, we check to see if it was a migration or a
             # gateway attachment
-            old_router = kwargs['old_router']
+            old_router = (payload.states[0] if (len(payload.states) > 1)
+                          else None)
             do_create = (not old_router['distributed'] or
                          not old_router.get(l3_apidef.EXTERNAL_GW_INFO))
             if not do_create:
@@ -397,6 +409,7 @@ class DVRResourceOperationHandler(object):
         except n_exc.ObjectNotFound:
             pass
 
+    @db_api.CONTEXT_WRITER
     def delete_floatingip_agent_gateway_port(self, context, host_id,
                                              ext_net_id):
         """Function to delete FIP gateway port with given ext_net_id."""
@@ -415,13 +428,15 @@ class DVRResourceOperationHandler(object):
 
     @registry.receives(resources.NETWORK, [events.AFTER_DELETE])
     def delete_fip_namespaces_for_ext_net(self, rtype, event, trigger,
-                                          context, network, **kwargs):
+                                          payload=None):
+        network = payload.latest_state
         if network.get(extnet_apidef.EXTERNAL):
             # Send the information to all the L3 Agent hosts
             # to clean up the fip namespace as it is no longer required.
             self.l3plugin.l3_rpc_notifier.delete_fipnamespace_for_ext_net(
-                context, network['id'])
+                payload.context, payload.resource_id)
 
+    @db_api.CONTEXT_READER
     def _get_ports_for_allowed_address_pair_ip(self, context, network_id,
                                                fixed_ip):
         """Return all active ports associated with the allowed_addr_pair ip."""
@@ -433,24 +448,24 @@ class DVRResourceOperationHandler(object):
                 models_v2.Port.admin_state_up == True)  # noqa
         return query.all()
 
-    @registry.receives(resources.FLOATING_IP, [events.AFTER_UPDATE])
-    def _create_dvr_floating_gw_port(self, resource, event, trigger, context,
-                                     router_id, fixed_port_id, floating_ip_id,
-                                     floating_network_id, fixed_ip_address,
-                                     association_event, **kwargs):
+    @registry.receives(resources.FLOATING_IP, [events.AFTER_CREATE,
+                                               events.AFTER_UPDATE])
+    def _create_dvr_floating_gw_port(self, rtype, event, trigger, payload):
         """Create floating agent gw port for DVR.
 
         Floating IP Agent gateway port will be created when a
         floatingIP association happens.
         """
-        if association_event and router_id:
+        fip = payload.latest_state
+        context = payload.context
+        if payload.metadata['association_event'] and fip['router_id']:
             admin_ctx = context.elevated()
-            router_dict = self.get_router(admin_ctx, router_id)
+            router_dict = self.get_router(admin_ctx, fip['router_id'])
             # Check if distributed router and then create the
             # FloatingIP agent gateway port
             if router_dict.get('distributed'):
-                hostid = self._get_dvr_service_port_hostid(context,
-                                                           fixed_port_id)
+                hostid = self._get_dvr_service_port_hostid(
+                    context, fip['port_id'])
                 if hostid:
                     # FIXME (Swami): This FIP Agent Gateway port should be
                     # created only once and there should not be a duplicate
@@ -459,7 +474,7 @@ class DVRResourceOperationHandler(object):
                     # existing flow.
                     fip_agent_port = (
                         self.create_fip_agent_gw_port_if_not_exists(
-                            admin_ctx, floating_network_id, hostid))
+                            admin_ctx, fip['floating_network_id'], hostid))
                     LOG.debug("FIP Agent gateway port: %s", fip_agent_port)
                 else:
                     # If not hostid check if the fixed ip provided has to
@@ -467,7 +482,7 @@ class DVRResourceOperationHandler(object):
                     # port. Get the port_dict, inherit the service port host
                     # and device owner(if it does not exist).
                     port = self._core_plugin.get_port(
-                        admin_ctx, fixed_port_id)
+                        admin_ctx, fip['port_id'])
                     allowed_device_owners = (
                         n_utils.get_dvr_allowed_address_pair_device_owners())
                     # NOTE: We just need to deal with ports that do not
@@ -479,7 +494,7 @@ class DVRResourceOperationHandler(object):
                         addr_pair_active_service_port_list = (
                             self._get_ports_for_allowed_address_pair_ip(
                                 admin_ctx, port['network_id'],
-                                fixed_ip_address))
+                                fip['fixed_ip_address']))
                         if not addr_pair_active_service_port_list:
                             return
                         self._inherit_service_port_and_arp_update(
@@ -494,30 +509,34 @@ class DVRResourceOperationHandler(object):
             self.update_arp_entry_for_dvr_service_port(context,
                                                        service_port_dict)
 
-    @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_CREATE])
     @db_api.retry_if_session_inactive()
-    def _add_csnat_on_interface_create(self, resource, event, trigger,
-                                       context, router_db, port, **kwargs):
-        """Event handler to for csnat port creation on interface creation."""
-        if not router_db.extra_attributes.distributed or not router_db.gw_port:
+    def _retry_add_csnat_on_interface_create(self, context, payload):
+        router_db = payload.latest_state
+        if not router_db.extra_attributes.distributed or not \
+                router_db.gw_port:
             return
-        admin_context = context.elevated()
+        admin_context = payload.context.elevated()
+        port = payload.metadata.get('port')
         self._add_csnat_router_interface_port(
-            admin_context, router_db, port['network_id'],
+            admin_context, payload.latest_state,
+            port['network_id'],
             [{'subnet_id': port['fixed_ips'][-1]['subnet_id']}])
 
-    @registry.receives(resources.ROUTER_INTERFACE, [events.AFTER_CREATE])
+    @registry.receives(resources.ROUTER_INTERFACE, [events.BEFORE_CREATE])
+    def _add_csnat_on_interface_create(self, resource, event, trigger,
+                                       payload=None):
+        """Event handler to for csnat port creation on interface creation."""
+        return self._retry_add_csnat_on_interface_create(
+            payload.context, payload)
+
     @db_api.retry_if_session_inactive()
-    def _update_snat_v6_addrs_after_intf_update(self, resource, event, trigger,
-                                                context, subnets, port,
-                                                router_id, new_interface,
-                                                **kwargs):
-        if new_interface:
+    def _retry_update_snat_v6_addrs_after_intf_update(self, context, payload):
+        if payload.metadata.get('new_interface'):
             # _add_csnat_on_interface_create handler deals with new ports
             return
         # if not a new interface, the interface was added to a new subnet,
         # which is the first in this list
-        subnet = subnets[0]
+        subnet = payload.metadata.get('subnets')[0]
         if not subnet or subnet['ip_version'] != 6:
             return
         # NOTE: For IPv6 additional subnets added to the same
@@ -525,8 +544,8 @@ class DVRResourceOperationHandler(object):
         # IPv6 subnet
         # Add new prefix to an existing ipv6 csnat port with the
         # same network id if one exists
-        admin_ctx = context.elevated()
-        router = self.l3plugin._get_router(admin_ctx, router_id)
+        admin_ctx = payload.context.elevated()
+        router = self.l3plugin._get_router(admin_ctx, payload.resource_id)
         cs_port = self._find_v6_router_port_by_network_and_device_owner(
             router, subnet['network_id'], const.DEVICE_OWNER_ROUTER_SNAT)
         if not cs_port:
@@ -534,9 +553,11 @@ class DVRResourceOperationHandler(object):
         new_fixed_ip = {'subnet_id': subnet['id']}
         fixed_ips = list(cs_port['fixed_ips'])
         fixed_ips.append(new_fixed_ip)
+        port = payload.metadata.get('port')
         try:
             updated_port = self._core_plugin.update_port(
-                admin_ctx, cs_port['id'], {'port': {'fixed_ips': fixed_ips}})
+                admin_ctx, cs_port['id'],
+                {'port': {'fixed_ips': fixed_ips}})
         except Exception:
             with excutils.save_and_reraise_exception():
                 # we need to try to undo the updated router
@@ -554,10 +575,11 @@ class DVRResourceOperationHandler(object):
                     # future with a compare-and-swap style update
                     # using the revision number of the port.
                     p = self._core_plugin.get_port(admin_ctx, port['id'])
-                    rollback_fixed_ips = [ip for ip in p['fixed_ips']
-                                          if ip['subnet_id'] != subnet['id']]
-                    upd = {'port': {'fixed_ips': rollback_fixed_ips}}
-                    self._core_plugin.update_port(admin_ctx, port['id'], upd)
+                    fixed_ips = [ip for ip in p['fixed_ips']
+                                 if ip['subnet_id'] != subnet['id']]
+                    upd = {'port': {'fixed_ips': fixed_ips}}
+                    self._core_plugin.update_port(
+                        admin_ctx, port['id'], upd)
                 try:
                     revert()
                 except Exception:
@@ -566,8 +588,14 @@ class DVRResourceOperationHandler(object):
                                   port['id'])
         LOG.debug("CSNAT port updated for IPv6 subnet: %s", updated_port)
 
-    def _find_v6_router_port_by_network_and_device_owner(self, router, net_id,
-                                                         device_owner):
+    @registry.receives(resources.ROUTER_INTERFACE, [events.AFTER_CREATE])
+    def _update_snat_v6_addrs_after_intf_update(self, resource, event, trigger,
+                                                payload=None):
+        return self._retry_update_snat_v6_addrs_after_intf_update(
+            payload.context, payload)
+
+    def _find_v6_router_port_by_network_and_device_owner(
+            self, router, net_id, device_owner):
         for port in router.attached_ports:
             p = port['port']
             if (p['network_id'] == net_id and
@@ -632,13 +660,11 @@ class DVRResourceOperationHandler(object):
         self.related_dvr_router_routers[cache_key] = (
             existing_routers | other_routers)
 
-    @registry.receives(resources.ROUTER_INTERFACE, [events.AFTER_DELETE])
     @db_api.retry_if_session_inactive()
-    def _cleanup_after_interface_removal(self, resource, event, trigger,
-                                         context, port, interface_info,
-                                         router_id, **kwargs):
-        """Handler to cleanup distributed resources after intf removal."""
-        router = self.l3plugin._get_router(context, router_id)
+    def _retry_cleanup_after_interface_removal(self, context, payload):
+        context = payload.context
+        port = payload.metadata.get('port')
+        router = self.l3plugin._get_router(context, payload.resource_id)
         if not router.extra_attributes.distributed:
             return
 
@@ -646,29 +672,31 @@ class DVRResourceOperationHandler(object):
         # the removed port's subnets and then subtract out any hosts still
         # hosting the router for the remaining interfaces
         router_hosts_for_removed = self.l3plugin._get_dvr_hosts_for_subnets(
-            context, subnet_ids={ip['subnet_id'] for ip in port['fixed_ips']})
+            context,
+            subnet_ids={ip['subnet_id'] for ip in port['fixed_ips']})
         router_hosts_after = self.l3plugin._get_dvr_hosts_for_router(
-            context, router_id)
+            context, payload.resource_id)
         removed_hosts = set(router_hosts_for_removed) - set(router_hosts_after)
         if removed_hosts:
             # Get hosts where this router is placed as "related" to other dvr
             # routers and don't remove it from such hosts
-            related_hosts = self._get_other_dvr_hosts(context, router_id)
+            related_hosts = self._get_other_dvr_hosts(context,
+                                                      payload.resource_id)
             agents = self.l3plugin.get_l3_agents(
                 context, filters={'host': removed_hosts})
             bindings = rb_obj.RouterL3AgentBinding.get_objects(
-                context, router_id=router_id)
+                context, router_id=payload.resource_id)
             snat_binding = bindings.pop() if bindings else None
             connected_dvr_routers = set(
                 self.l3plugin._get_other_dvr_router_ids_connected_router(
-                    context, router_id))
+                    context, payload.resource_id))
             for agent in agents:
                 is_this_snat_agent = (
                     snat_binding and snat_binding.l3_agent_id == agent['id'])
                 if (not is_this_snat_agent and
                         agent['host'] not in related_hosts):
                     self.l3plugin.l3_rpc_notifier.router_removed_from_agent(
-                        context, router_id, agent['host'])
+                        context, payload.resource_id, agent['host'])
                     for connected_router_id in connected_dvr_routers:
                         connected_router_hosts = set(
                             self.l3plugin._get_dvr_hosts_for_router(
@@ -677,17 +705,18 @@ class DVRResourceOperationHandler(object):
                             self._get_other_dvr_hosts(
                                 context, connected_router_id))
                         if agent['host'] not in connected_router_hosts:
-                            self.l3plugin.l3_rpc_notifier.\
+                            self.l3plugin.l3_rpc_notifier. \
                                 router_removed_from_agent(
                                     context, connected_router_id,
                                     agent['host'])
         # if subnet_id not in interface_info, request was to remove by port
+        interface_info = payload.metadata.get('interface_info')
         sub_id = (interface_info.get('subnet_id') or
                   port['fixed_ips'][0]['subnet_id'])
         self._cleanup_related_hosts_after_interface_removal(
-            context, router_id, sub_id)
+            context, payload.resource_id, sub_id)
         self._cleanup_related_routers_after_interface_removal(
-            context, router_id, sub_id)
+            context, payload.resource_id, sub_id)
         is_multiple_prefix_csport = (
             self._check_for_multiprefix_csnat_port_and_update(
                 context, router, port['network_id'], sub_id))
@@ -696,6 +725,13 @@ class DVRResourceOperationHandler(object):
             self.delete_csnat_router_interface_ports(
                 context.elevated(), router, subnet_id=sub_id)
 
+    @registry.receives(resources.ROUTER_INTERFACE, [events.AFTER_DELETE])
+    def _cleanup_after_interface_removal(self, resource, event, trigger,
+                                         payload=None):
+        """Handler to cleanup distributed resources after intf removal."""
+        return self._retry_cleanup_after_interface_removal(
+            payload.context, payload)
+
     def _cleanup_related_hosts_after_interface_removal(
             self, context, router_id, subnet_id):
         router_hosts = self.l3plugin._get_dvr_hosts_for_router(
@@ -1393,8 +1429,8 @@ class L3_NAT_with_dvr_db_mixin(_DVRAgentInterfaceMixin,
         return False
 
     def get_ports_under_dvr_connected_subnet(self, context, subnet_id):
-        query = dvr_mac_db.get_ports_query_by_subnet_and_ip(context, subnet_id)
-        ports = [p for p in query.all() if is_port_bound(p)]
+        ports = dvr_mac_db.get_ports_query_by_subnet_and_ip(context, subnet_id)
+        ports = [p for p in ports if is_port_bound(p)]
         # TODO(slaweq): if there would be way to pass to neutron-lib only
         # list of extensions which actually should be processed, than setting
         # process_extensions=True below could avoid that second loop and
