@@ -19,6 +19,7 @@ from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
+from neutron_lib.db import api as db_api
 from neutron_lib.exceptions import l3 as l3_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
@@ -270,7 +271,8 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
     def _check_for_rtr_serviceable_ports(
             self, admin_context, router_id, port_host):
         subnet_ids = self.get_subnet_ids_on_router(admin_context,
-                                                   router_id)
+                                                   router_id,
+                                                   keep_gateway_port=False)
         return self._check_dvr_serviceable_ports_on_host(
             admin_context, port_host, subnet_ids)
 
@@ -282,13 +284,9 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         int_ports = self._core_plugin.get_ports(
             context.elevated(), filters=filter_rtr)
         for port in int_ports:
-            dvr_binding = (ml2_db.
-                           get_distributed_port_binding_by_host(
-                               context, port['id'], port_host))
-            if dvr_binding:
-                # unbind this port from router
-                dvr_binding['router_id'] = None
-                dvr_binding.update(dvr_binding)
+            # unbind this port from router
+            ml2_db.update_distributed_port_binding_by_host(
+                context, port['id'], port_host, None)
 
     def _get_active_l3_agent_routers_sync_data(self, context, host, agent,
                                                router_ids):
@@ -347,8 +345,9 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
                   {'router_id': router_id, 'dvr_hosts': dvr_hosts})
         return dvr_hosts
 
+    @db_api.CONTEXT_READER
     def _get_dvr_hosts_for_subnets(self, context, subnet_ids):
-        """Get a list of hosts with DVR servicable ports on subnet_ids."""
+        """Get a list of hosts with DVR serviceable ports on subnet_ids."""
         host_dvr_dhcp = cfg.CONF.host_dvr_for_dhcp
         Binding = ml2_models.PortBinding
         Port = models_v2.Port
@@ -366,6 +365,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         hosts = [item[0] for item in query if item[0] != '']
         return hosts
 
+    @db_api.CONTEXT_READER
     def _get_dvr_subnet_ids_on_host_query(self, context, host):
         host_dvr_dhcp = cfg.CONF.host_dvr_for_dhcp
         query = context.session.query(
@@ -381,6 +381,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         query = query.filter(owner_filter)
         return query
 
+    @db_api.CONTEXT_READER
     def _get_dvr_router_ids_for_host(self, context, host):
         subnet_ids_on_host_query = self._get_dvr_subnet_ids_on_host_query(
             context, host)
@@ -394,6 +395,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         LOG.debug('DVR routers on host %s: %s', host, router_ids)
         return router_ids
 
+    @db_api.CONTEXT_READER
     def _get_other_dvr_router_ids_connected_router(self, context, router_id):
         # TODO(slaweq): move this method to RouterPort OVO object
         subnet_ids = self.get_subnet_ids_on_router(context, router_id)
@@ -465,6 +467,7 @@ class L3_DVRsch_db_mixin(l3agent_sch_db.L3AgentSchedulerDbMixin):
         return list(result_set)
 
     @log_helpers.log_method_call
+    @db_api.CONTEXT_READER
     def _check_dvr_serviceable_ports_on_host(self, context, host, subnet_ids):
         """Check for existence of dvr serviceable ports on host
 
@@ -531,25 +534,26 @@ def _dvr_handle_unbound_allowed_addr_pair_del(
             context, port, fixed_ips_to_delete=aa_fixed_ips)
 
 
-def _notify_l3_agent_new_port(resource, event, trigger, **kwargs):
+def _notify_l3_agent_new_port(resource, event, trigger, payload=None):
     LOG.debug('Received %(resource)s %(event)s', {
         'resource': resource,
         'event': event})
-    port = kwargs.get('port')
+    port = payload.latest_state
     if not port:
         return
 
     if n_utils.is_dvr_serviced(port['device_owner']):
         l3plugin = directory.get_plugin(plugin_constants.L3)
-        context = kwargs['context']
+        context = payload.context
         l3plugin.dvr_handle_new_service_port(context, port)
         l3plugin.update_arp_entry_for_dvr_service_port(context, port)
 
 
-def _notify_port_delete(event, resource, trigger, **kwargs):
-    context = kwargs['context']
-    port = kwargs['port']
-    get_related_hosts_info = kwargs.get("get_related_hosts_info", True)
+def _notify_port_delete(event, resource, trigger, payload):
+    context = payload.context
+    port = payload.latest_state
+    get_related_hosts_info = payload.metadata.get(
+                                 "get_related_hosts_info", True)
     l3plugin = directory.get_plugin(plugin_constants.L3)
     if port:
         port_host = port.get(portbindings.HOST_ID)
@@ -566,9 +570,9 @@ def _notify_port_delete(event, resource, trigger, **kwargs):
             context, info['router_id'], info['host'])
 
 
-def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
-    new_port = kwargs.get('port')
-    original_port = kwargs.get('original_port')
+def _notify_l3_agent_port_update(resource, event, trigger, payload):
+    new_port = payload.latest_state
+    original_port = payload.states[0]
 
     is_fixed_ips_changed = n_utils.port_ip_changed(new_port, original_port)
 
@@ -581,7 +585,7 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
 
     if new_port and original_port:
         l3plugin = directory.get_plugin(plugin_constants.L3)
-        context = kwargs['context']
+        context = payload.context
         new_port_host = new_port.get(portbindings.HOST_ID)
         original_port_host = original_port.get(portbindings.HOST_ID)
         is_new_port_binding_changed = (
@@ -607,14 +611,13 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
                 original_port,
                 get_related_hosts_info=False)
             if removed_routers:
-                removed_router_args = {
-                    'context': context,
-                    'port': original_port,
-                    'removed_routers': removed_routers,
-                    'get_related_hosts_info': False,
-                }
                 _notify_port_delete(
-                    event, resource, trigger, **removed_router_args)
+                    event, resource, trigger,
+                    payload=events.DBEventPayload(
+                        context,
+                        metadata={'removed_routers': removed_routers,
+                                  'get_related_hosts_info': False},
+                        states=(original_port,)))
 
             def _should_notify_on_fip_update():
                 if not fip_router_id:
@@ -670,7 +673,7 @@ def _notify_l3_agent_port_update(resource, event, trigger, **kwargs):
                         l3plugin, context, original_port, address_pair)
                 return
 
-        if kwargs.get('mac_address_updated') or is_fixed_ips_changed:
+        if payload.metadata.get('mac_address_updated') or is_fixed_ips_changed:
             l3plugin.update_arp_entry_for_dvr_service_port(
                 context, new_port)
 
