@@ -36,6 +36,7 @@ from sqlalchemy.orm import scoped_session
 
 from neutron._i18n import _
 from neutron.common import _constants as const
+from neutron.db import address_group_db as ag_db
 from neutron.db.models import securitygroup as sg_models
 from neutron.db import rbac_db_mixin as rbac_mixin
 from neutron.extensions import securitygroup as ext_sg
@@ -62,16 +63,13 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         return self._create_bulk('security_group', context,
                                  security_groups)
 
-    def _registry_notify(self, res, event, id=None, exc_cls=None, **kwargs):
+    def _registry_publish(self, res, event, id=None, exc_cls=None,
+                          payload=None):
         # NOTE(armax): a callback exception here will prevent the request
         # from being processed. This is a hook point for backend's validation;
         # we raise to propagate the reason for the failure.
         try:
-            if 'payload' in kwargs:
-                # TODO(boden): remove shim once all callbacks use payloads
-                registry.publish(res, event, self, payload=kwargs['payload'])
-            else:
-                registry.notify(res, event, self, **kwargs)
+            registry.publish(res, event, self, payload=payload)
         except exceptions.CallbackFailure as e:
             if exc_cls:
                 reason = (_('cannot perform %(event)s due to %(reason)s') %
@@ -86,17 +84,13 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         a given tenant if it does not exist.
         """
         s = security_group['security_group']
-        kwargs = {
-            'context': context,
-            'security_group': s,
-            'is_default': default_sg,
-        }
-        self._registry_notify(resources.SECURITY_GROUP, events.BEFORE_CREATE,
-                              exc_cls=ext_sg.SecurityGroupConflict,
-                              payload=events.DBEventPayload(
-                                  context, metadata={'is_default': default_sg},
-                                  request_body=security_group,
-                                  desired_state=s))
+        self._registry_publish(resources.SECURITY_GROUP, events.BEFORE_CREATE,
+                               exc_cls=ext_sg.SecurityGroupConflict,
+                               payload=events.DBEventPayload(
+                                   context,
+                                   metadata={'is_default': default_sg},
+                                   request_body=security_group,
+                                   desired_state=s))
 
         tenant_id = s['tenant_id']
         stateful = s.get('stateful', True)
@@ -110,18 +104,17 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 return self.get_security_group(context, existing_def_sg_id)
 
         with db_api.CONTEXT_WRITER.using(context):
+            delta = len(ext_sg.sg_supported_ethertypes)
+            delta = delta * 2 if default_sg else delta
+            quota.QUOTAS.quota_limit_check(context, tenant_id,
+                                           security_group_rule=delta)
+
             sg = sg_obj.SecurityGroup(
                 context, id=s.get('id') or uuidutils.generate_uuid(),
                 description=s['description'], project_id=tenant_id,
                 name=s['name'], is_default=default_sg, stateful=stateful)
             sg.create()
 
-            delta = len(ext_sg.sg_supported_ethertypes)
-            delta = delta * 2 if default_sg else delta
-            reservation = quota.QUOTAS.make_reservation(
-                context, tenant_id, {'security_group_rule': delta},
-                self)
-
             for ethertype in ext_sg.sg_supported_ethertypes:
                 if default_sg:
                     # Allow intercommunication
@@ -141,20 +134,32 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 sg.rules.append(egress_rule)
             sg.obj_reset_changes(['rules'])
 
-            quota.QUOTAS.commit_reservation(context,
-                                            reservation.reservation_id)
-
             # fetch sg from db to load the sg rules with sg model.
-            sg = sg_obj.SecurityGroup.get_object(context, id=sg.id)
+            # NOTE(slaweq): With new system/project scopes it may happen that
+            # project admin will try to list security groups for different
+            # project and during that call Neutron will ensure that default
+            # security group is created. In such case elevated context needs to
+            # be used here otherwise, SG will not be found and error 500 will
+            # be returned through the API
+            get_context = context.elevated() if default_sg else context
+            sg = sg_obj.SecurityGroup.get_object(get_context, id=sg.id)
             secgroup_dict = self._make_security_group_dict(sg)
-            kwargs['security_group'] = secgroup_dict
-            self._registry_notify(resources.SECURITY_GROUP,
-                                  events.PRECOMMIT_CREATE,
-                                  exc_cls=ext_sg.SecurityGroupConflict,
-                                  **kwargs)
+            self._registry_publish(resources.SECURITY_GROUP,
+                                   events.PRECOMMIT_CREATE,
+                                   exc_cls=ext_sg.SecurityGroupConflict,
+                                   payload=events.DBEventPayload(
+                                       context,
+                                       resource_id=sg.id,
+                                       metadata={'is_default': default_sg},
+                                       states=(secgroup_dict,)))
+
+        registry.publish(resources.SECURITY_GROUP, events.AFTER_CREATE,
+                         self, payload=events.DBEventPayload(
+                             context,
+                             resource_id=secgroup_dict['id'],
+                             metadata={'is_default': default_sg},
+                             states=(secgroup_dict,)))
 
-        registry.notify(resources.SECURITY_GROUP, events.AFTER_CREATE, self,
-                        **kwargs)
         return secgroup_dict
 
     @db_api.retry_if_session_inactive()
@@ -247,37 +252,35 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
             if sg['name'] == 'default' and not context.is_admin:
                 raise ext_sg.SecurityGroupCannotRemoveDefault()
-        kwargs = {
-            'context': context,
-            'security_group_id': id,
-            'security_group': sg,
-        }
-        self._registry_notify(resources.SECURITY_GROUP,
-                              events.BEFORE_DELETE,
-                              exc_cls=ext_sg.SecurityGroupInUse, id=id,
-                              payload=events.DBEventPayload(
-                                  context, states=(sg,), resource_id=id))
+
+        self._registry_publish(resources.SECURITY_GROUP,
+                               events.BEFORE_DELETE,
+                               exc_cls=ext_sg.SecurityGroupInUse, id=id,
+                               payload=events.DBEventPayload(
+                                   context, states=(sg,), resource_id=id))
 
         with db_api.CONTEXT_WRITER.using(context):
             # pass security_group_rule_ids to ensure
             # consistency with deleted rules
-            # get security_group_bindings and security_group one more time
-            # so that they will be attached for session where sg will be
-            # deleted
-            ports = self._get_port_security_group_bindings(context, filters)
             sg = self._get_security_group(context, id)
-            kwargs['security_group_rule_ids'] = [r['id'] for r in sg.rules]
-            kwargs['security_group'] = self._make_security_group_dict(sg)
-            self._registry_notify(resources.SECURITY_GROUP,
-                                  events.PRECOMMIT_DELETE,
-                                  exc_cls=ext_sg.SecurityGroupInUse, id=id,
-                                  **kwargs)
+            sgr_ids = [r['id'] for r in sg.rules]
+            sec_group = self._make_security_group_dict(sg)
+            self._registry_publish(resources.SECURITY_GROUP,
+                                   events.PRECOMMIT_DELETE,
+                                   exc_cls=ext_sg.SecurityGroupInUse,
+                                   payload=events.DBEventPayload(
+                                       context, resource_id=id,
+                                       states=(sec_group,),
+                                       metadata={
+                                           'security_group_rule_ids': sgr_ids
+                                       }))
             sg.delete()
 
-        kwargs.pop('security_group')
-        kwargs['name'] = sg['name']
-        registry.notify(resources.SECURITY_GROUP, events.AFTER_DELETE,
-                        self, **kwargs)
+        registry.publish(resources.SECURITY_GROUP, events.AFTER_DELETE,
+                         self, payload=events.DBEventPayload(
+                             context, resource_id=id, states=(sec_group,),
+                             metadata={'security_group_rule_ids': sgr_ids,
+                                       'name': sg['name']}))
 
     @db_api.retry_if_session_inactive()
     def update_security_group(self, context, id, security_group):
@@ -293,34 +296,34 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                     if ports:
                         raise ext_sg.SecurityGroupInUse(id=id)
 
-        kwargs = {
-            'context': context,
-            'security_group_id': id,
-            'security_group': s,
-        }
-        self._registry_notify(resources.SECURITY_GROUP, events.BEFORE_UPDATE,
-                              exc_cls=ext_sg.SecurityGroupConflict, **kwargs)
+        self._registry_publish(resources.SECURITY_GROUP, events.BEFORE_UPDATE,
+                               exc_cls=ext_sg.SecurityGroupConflict,
+                               payload=events.DBEventPayload(
+                                   context, resource_id=id, states=(s,)))
 
         with db_api.CONTEXT_WRITER.using(context):
             sg = self._get_security_group(context, id)
             if sg.name == 'default' and 'name' in s:
                 raise ext_sg.SecurityGroupCannotUpdateDefault()
             sg_dict = self._make_security_group_dict(sg)
-            kwargs['original_security_group'] = sg_dict
+            original_security_group = sg_dict
             sg.update_fields(s)
             sg.update()
             sg_dict = self._make_security_group_dict(sg)
-            kwargs['security_group'] = sg_dict
-            self._registry_notify(
+            self._registry_publish(
                     resources.SECURITY_GROUP,
                     events.PRECOMMIT_UPDATE,
                     exc_cls=ext_sg.SecurityGroupConflict,
                     payload=events.DBEventPayload(
                         context, request_body=s,
-                        states=(kwargs['original_security_group'],),
+                        states=(original_security_group,),
                         resource_id=id, desired_state=sg_dict))
-        registry.notify(resources.SECURITY_GROUP, events.AFTER_UPDATE, self,
-                        **kwargs)
+        registry.publish(resources.SECURITY_GROUP, events.AFTER_UPDATE, self,
+                         payload=events.DBEventPayload(
+                             context, request_body=s,
+                             states=(original_security_group, sg_dict),
+                             resource_id=id))
+
         return sg_dict
 
     def _make_security_group_dict(self, security_group, fields=None):
@@ -328,7 +331,10 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                'name': security_group['name'],
                'stateful': security_group['stateful'],
                'tenant_id': security_group['tenant_id'],
-               'description': security_group['description']}
+               'description': security_group['description'],
+               'standard_attr_id': security_group.db_obj.standard_attr.id,
+               'shared': security_group['shared'],
+               }
         if security_group.rules:
             res['security_group_rules'] = [
                 self._make_security_group_rule_dict(r.db_obj)
@@ -346,13 +352,13 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                'security_group_id': security_group['security_group_id']}
         return db_utils.resource_fields(res, fields)
 
-    @db_api.retry_if_session_inactive()
     def _create_port_security_group_binding(self, context, port_id,
                                             security_group_id):
-        with db_api.CONTEXT_WRITER.using(context):
-            db = sg_models.SecurityGroupPortBinding(
-                port_id=port_id, security_group_id=security_group_id)
-            context.session.add(db)
+        # This method must be called from inside an active DB writer
+        # transaction.
+        db = sg_models.SecurityGroupPortBinding(
+            port_id=port_id, security_group_id=security_group_id)
+        context.session.add(db)
 
     def _get_port_security_group_bindings(self, context,
                                           filters=None, fields=None):
@@ -391,17 +397,24 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                     context, rule_dict, validate=False)
                 ret.append(res_rule_dict)
         for rdict in ret:
-            registry.notify(
-                resources.SECURITY_GROUP_RULE, events.AFTER_CREATE, self,
-                context=context, security_group_rule=rdict)
+            registry.publish(resources.SECURITY_GROUP_RULE,
+                             events.AFTER_CREATE,
+                             self,
+                             payload=events.DBEventPayload(
+                                 context,
+                                 resource_id=rdict['id'],
+                                 states=(rdict,)))
         return ret
 
     @db_api.retry_if_session_inactive()
     def create_security_group_rule(self, context, security_group_rule):
         res = self._create_security_group_rule(context, security_group_rule)
-        registry.notify(
-            resources.SECURITY_GROUP_RULE, events.AFTER_CREATE, self,
-            context=context, security_group_rule=res)
+        registry.publish(resources.SECURITY_GROUP_RULE, events.AFTER_CREATE,
+                         self, payload=events.DBEventPayload(
+                                   context,
+                                   resource_id=res['id'],
+                                   states=(res,)))
+
         return res
 
     def _create_security_group_rule(self, context, security_group_rule,
@@ -425,6 +438,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             'security_group_id': rule_dict['security_group_id'],
             'direction': rule_dict['direction'],
             'remote_group_id': rule_dict.get('remote_group_id'),
+            'remote_address_group_id': rule_dict.get(
+                'remote_address_group_id'),
             'ethertype': rule_dict['ethertype'],
             'protocol': protocol,
             'remote_ip_prefix': remote_ip_prefix,
@@ -439,13 +454,14 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         if port_range_max is not None:
             args['port_range_max'] = port_range_max
 
-        kwargs = {
-            'context': context,
-            'security_group_rule': args
-        }
-        self._registry_notify(resources.SECURITY_GROUP_RULE,
-                              events.BEFORE_CREATE,
-                              exc_cls=ext_sg.SecurityGroupConflict, **kwargs)
+        self._registry_publish(
+            resources.SECURITY_GROUP_RULE,
+            events.BEFORE_CREATE,
+            exc_cls=ext_sg.SecurityGroupConflict,
+            payload=events.DBEventPayload(
+                context, resource_id=args['id'],
+                states=(args,)))
+
         with db_api.CONTEXT_WRITER.using(context):
             if validate:
                 self._check_for_duplicate_rules(context, sg_id,
@@ -458,11 +474,14 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             sg_rule = sg_obj.SecurityGroupRule.get_object(context,
                                                           id=sg_rule.id)
             res_rule_dict = self._make_security_group_rule_dict(sg_rule.db_obj)
-            kwargs['security_group_rule'] = res_rule_dict
-            self._registry_notify(
+            self._registry_publish(
                 resources.SECURITY_GROUP_RULE,
                 events.PRECOMMIT_CREATE,
-                exc_cls=ext_sg.SecurityGroupConflict, **kwargs)
+                exc_cls=ext_sg.SecurityGroupConflict,
+                payload=events.DBEventPayload(
+                    context, resource_id=res_rule_dict['id'],
+                    states=(res_rule_dict,)))
+
         return res_rule_dict
 
     def _get_ip_proto_number(self, protocol):
@@ -620,8 +639,12 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         self._validate_ip_prefix(rule)
         self._validate_ethertype_and_protocol(rule)
 
-        if rule['remote_ip_prefix'] and rule['remote_group_id']:
-            raise ext_sg.SecurityGroupRemoteGroupAndRemoteIpPrefix()
+        remote = None
+        for key in ['remote_ip_prefix', 'remote_group_id',
+                    'remote_address_group_id']:
+            if remote and rule.get(key):
+                raise ext_sg.SecurityGroupMultipleRemoteEntites()
+            remote = rule.get(key) or remote
 
         remote_group_id = rule['remote_group_id']
         # Check that remote_group_id exists for tenant
@@ -629,8 +652,14 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             self._check_security_group(context, remote_group_id,
                                        tenant_id=rule['tenant_id'])
 
-        security_group_id = rule['security_group_id']
+        remote_address_group_id = rule['remote_address_group_id']
+        # Check that remote_address_group_id exists for project
+        if remote_address_group_id:
+            ag_db.AddressGroupDbMixin.check_address_group(
+                context, remote_address_group_id,
+                project_id=rule['project_id'])
 
+        security_group_id = rule['security_group_id']
         # Confirm that the tenant has permission
         # to add rules to this security group.
         self._check_security_group(context, security_group_id,
@@ -662,12 +691,27 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                'port_range_min': security_group_rule['port_range_min'],
                'port_range_max': security_group_rule['port_range_max'],
                'remote_ip_prefix': security_group_rule['remote_ip_prefix'],
-               'remote_group_id': security_group_rule['remote_group_id']}
+               'remote_address_group_id': security_group_rule[
+                   'remote_address_group_id'],
+               'normalized_cidr': self._get_normalized_cidr_from_rule(
+                   security_group_rule),
+               'remote_group_id': security_group_rule['remote_group_id'],
+               'standard_attr_id': security_group_rule.standard_attr.id,
+               }
 
         resource_extend.apply_funcs(ext_sg.SECURITYGROUPRULES, res,
                                     security_group_rule)
         return db_utils.resource_fields(res, fields)
 
+    @staticmethod
+    def _get_normalized_cidr_from_rule(rule):
+        normalized_cidr = None
+        remote_ip_prefix = rule.get('remote_ip_prefix')
+        if remote_ip_prefix:
+            normalized_cidr = str(
+                net.AuthenticIPNetwork(remote_ip_prefix).cidr)
+        return normalized_cidr
+
     def _rule_to_key(self, rule):
         def _normalize_rule_value(key, value):
             # This string is used as a placeholder for str(None), but shorter.
@@ -691,6 +735,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             'port_range_min',
             'protocol',
             'remote_group_id',
+            'remote_address_group_id',
             'remote_ip_prefix',
             'security_group_id'
         ]
@@ -736,6 +781,17 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                 raise ext_sg.SecurityGroupRuleParameterConflict(
                     ethertype=rule['ethertype'], cidr=input_prefix)
 
+    @db_api.retry_if_session_inactive()
+    def get_security_group_rules_count(self, context, filters=None):
+        filters = filters if filters else {}
+        if not filters and context.project_id and not context.is_admin:
+            rule_ids = sg_obj.SecurityGroupRule.get_security_group_rule_ids(
+                context.project_id)
+            filters = {'id': rule_ids}
+
+        return sg_obj.SecurityGroupRule.count(context_lib.get_admin_context(),
+                                              **filters)
+
     @db_api.retry_if_session_inactive()
     def get_security_group_rules(self, context, filters=None, fields=None,
                                  sorts=None, limit=None, marker=None,
@@ -763,10 +819,10 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
         # be returned
         rule_objs = sg_obj.SecurityGroupRule.get_objects(
             context_lib.get_admin_context(), _pager=pager,
-            validate_filters=False, **filters
+            validate_filters=False, return_db_obj=True, **filters
         )
         return [
-            self._make_security_group_rule_dict(obj.db_obj, fields)
+            self._make_security_group_rule_dict(obj, fields)
             for obj in rule_objs
         ]
 
@@ -789,26 +845,32 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
     @db_api.retry_if_session_inactive()
     def delete_security_group_rule(self, context, id):
-        kwargs = {
-            'context': context,
-            'security_group_rule_id': id
-        }
-        self._registry_notify(resources.SECURITY_GROUP_RULE,
-                              events.BEFORE_DELETE, id=id,
-                              exc_cls=ext_sg.SecurityGroupRuleInUse, **kwargs)
+        self._registry_publish(resources.SECURITY_GROUP_RULE,
+                               events.BEFORE_DELETE,
+                               exc_cls=ext_sg.SecurityGroupRuleInUse,
+                               payload=events.DBEventPayload(
+                                   context, resource_id=id,))
 
         with db_api.CONTEXT_WRITER.using(context):
             sgr = self._get_security_group_rule(context, id)
-            kwargs['security_group_id'] = sgr['security_group_id']
-            self._registry_notify(resources.SECURITY_GROUP_RULE,
-                                  events.PRECOMMIT_DELETE,
-                                  exc_cls=ext_sg.SecurityGroupRuleInUse, id=id,
-                                  **kwargs)
+            self._registry_publish(
+                resources.SECURITY_GROUP_RULE,
+                events.PRECOMMIT_DELETE,
+                exc_cls=ext_sg.SecurityGroupRuleInUse,
+                payload=events.DBEventPayload(
+                    context,
+                    resource_id=id,
+                    metadata={'security_group_id': sgr['security_group_id']}))
             sgr.delete()
 
-        registry.notify(
-            resources.SECURITY_GROUP_RULE, events.AFTER_DELETE, self,
-            **kwargs)
+        registry.publish(
+            resources.SECURITY_GROUP_RULE,
+            events.AFTER_DELETE,
+            self,
+            payload=events.DBEventPayload(
+                context,
+                resource_id=id,
+                metadata={'security_group_id': sgr['security_group_id']}))
 
     @staticmethod
     @resource_extend.extends([port_def.COLLECTION_NAME])
@@ -826,6 +888,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
 
     def _process_port_create_security_group(self, context, port,
                                             security_groups):
+        # This method must be called from inside an active DB writer
+        # transaction.
         self._validate_sgs_for_port(security_groups)
         if validators.is_attr_set(security_groups):
             for sg in security_groups:
@@ -837,8 +901,11 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                                        security_groups else [])
 
     def _get_default_sg_id(self, context, tenant_id):
+        # NOTE(slaweq): With new system/project scopes it may happen that
+        # project admin will try to find default SG for different
+        # project. In such case elevated context needs to be used.
         default_group = sg_obj.DefaultSecurityGroup.get_object(
-            context,
+            context.elevated(),
             project_id=tenant_id,
         )
         if default_group:
@@ -848,19 +915,23 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
                                         events.BEFORE_UPDATE])
     @registry.receives(resources.NETWORK, [events.BEFORE_CREATE])
     def _ensure_default_security_group_handler(self, resource, event, trigger,
-                                               context, **kwargs):
-        if event == events.BEFORE_UPDATE:
-            tenant_id = kwargs['original_' + resource]['tenant_id']
-        else:
-            tenant_id = kwargs[resource]['tenant_id']
-        if tenant_id:
-            self._ensure_default_security_group(context, tenant_id)
+                                               payload):
+        _state = (payload.states[0] if event == events.BEFORE_UPDATE else
+                  payload.latest_state)
+        # TODO(ralonsoh): "tenant_id" reference should be removed.
+        project_id = _state.get('project_id') or _state['tenant_id']
+        if project_id:
+            self._ensure_default_security_group(payload.context, project_id)
 
     def _ensure_default_security_group(self, context, tenant_id):
         """Create a default security group if one doesn't exist.
 
         :returns: the default security group id for given tenant.
         """
+        # Do not allow a tenant to create a default SG for another one.
+        # See Bug 1987410.
+        if tenant_id != context.tenant_id and not context.is_admin:
+            return
         if not extensions.is_extension_supported(self, 'security-group'):
             return
         default_group_id = self._get_default_sg_id(context, tenant_id)
@@ -900,7 +971,7 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             g.id for g in sg_objs
             if (context.is_admin or not tenant_id or
                 g.tenant_id == tenant_id or
-                sg_obj.SecurityGroup.is_shared_with_tenant(
+                sg_obj.SecurityGroup.is_shared_with_project(
                     context, g.id, tenant_id))
         )
 
@@ -918,7 +989,8 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             return
         port_sg = port.get(ext_sg.SECURITYGROUPS)
         if port_sg is None or not validators.is_attr_set(port_sg):
-            port_project = port.get('tenant_id')
+            # TODO(ralonsoh): "tenant_id" reference should be removed.
+            port_project = port.get('project_id') or port.get('tenant_id')
             default_sg = self._ensure_default_security_group(context,
                                                              port_project)
             if default_sg:
@@ -946,13 +1018,14 @@ class SecurityGroupDbMixin(ext_sg.SecurityGroupPluginBase,
             return True
         return False
 
-    def update_security_group_on_port(self, context, id, port,
-                                      original_port, updated_port):
+    def _update_security_group_on_port(self, context, id, port,
+                                       original_port, updated_port):
         """Update security groups on port.
 
         This method returns a flag which indicates request notification
         is required and does not perform notification itself.
         It is because another changes for the port may require notification.
+        This method must be called from inside an active DB writer transaction.
         """
         need_notify = False
         port_updates = port['port']
