@@ -15,22 +15,6 @@
 
 from neutron_lib.api.definitions import provider_net
 from neutron_lib.api import extensions
-from neutron_lib.api import validators
-from neutron_lib import exceptions as n_exc
-
-from neutron._i18n import _
-
-
-def _raise_if_updates_provider_attributes(attrs):
-    """Raise exception if provider attributes are present.
-
-    This method is used for plugins that do not support
-    updating provider networks.
-    """
-    if any(validators.is_attr_set(attrs.get(a))
-           for a in provider_net.ATTRIBUTES):
-        msg = _("Plugin does not support updating provider attributes")
-        raise n_exc.InvalidInput(error_message=msg)
 
 
 class Providernet(extensions.APIExtensionDescriptor):
