@@ -11,14 +11,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-from neutron.api import extensions
-from neutron.db import standard_attr
+from neutron_lib.api import extensions
+
+from neutron.extensions import stdattrs_common
 
 
 REVISION = 'revision_number'
 REVISION_BODY = {
     REVISION: {'allow_post': False, 'allow_put': False,
-               'is_visible': True, 'default': None},
+               'is_visible': True, 'default': None, 'is_filter': True},
 }
 
 
@@ -45,5 +46,4 @@ class Revisions(extensions.ExtensionDescriptor):
     def get_extended_resources(self, version):
         if version != "2.0":
             return {}
-        rs_map = standard_attr.get_standard_attr_resource_model_map()
-        return {resource: REVISION_BODY for resource in rs_map}
+        return stdattrs_common.stdattrs_extended_resources(REVISION_BODY)
