@@ -13,9 +13,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from neutron_lib import constants as const
 from oslo_utils import uuidutils
 
-from neutron.ipam.drivers.neutrondb_ipam import db_models
+from neutron.objects import ipam as ipam_objs
 
 # Database operations for Neutron's DB-backed IPAM driver
 
@@ -23,9 +24,11 @@ from neutron.ipam.drivers.neutrondb_ipam import db_models
 class IpamSubnetManager(object):
 
     @classmethod
-    def load_by_neutron_subnet_id(cls, session, neutron_subnet_id):
-        return session.query(db_models.IpamSubnet).filter_by(
-            neutron_subnet_id=neutron_subnet_id).first()
+    def load_by_neutron_subnet_id(cls, context, neutron_subnet_id):
+
+        objs = ipam_objs.IpamSubnet.get_objects(
+                context, neutron_subnet_id=neutron_subnet_id)
+        return objs.pop() if objs else None
 
     def __init__(self, ipam_subnet_id, neutron_subnet_id):
         self._ipam_subnet_id = ipam_subnet_id
@@ -35,37 +38,36 @@ class IpamSubnetManager(object):
     def neutron_id(self):
         return self._neutron_subnet_id
 
-    def create(self, session):
+    def create(self, context):
         """Create database models for an IPAM subnet.
 
         This method creates a subnet resource for the IPAM driver and
         associates it with its neutron identifier, if specified.
 
-        :param session: database sesssion.
-        :returns: the idenfier of created IPAM subnet
+        :param context: neutron api request context
+        :returns: the identifier of created IPAM subnet
         """
         if not self._ipam_subnet_id:
             self._ipam_subnet_id = uuidutils.generate_uuid()
-        ipam_subnet = db_models.IpamSubnet(
-            id=self._ipam_subnet_id,
-            neutron_subnet_id=self._neutron_subnet_id)
-        session.add(ipam_subnet)
+        ipam_objs.IpamSubnet(
+            context, id=self._ipam_subnet_id,
+            neutron_subnet_id=self._neutron_subnet_id).create()
         return self._ipam_subnet_id
 
     @classmethod
-    def delete(cls, session, neutron_subnet_id):
+    def delete(cls, context, neutron_subnet_id):
         """Delete IPAM subnet.
 
         IPAM subnet no longer has foreign key to neutron subnet,
         so need to perform delete manually
 
-        :param session: database sesssion
+        :param context: neutron api request context
         :param neutron_subnet_id: neutron subnet id associated with ipam subnet
         """
-        return session.query(db_models.IpamSubnet).filter_by(
-            neutron_subnet_id=neutron_subnet_id).delete()
+        return ipam_objs.IpamSubnet.delete_objects(
+             context, neutron_subnet_id=neutron_subnet_id)
 
-    def create_pool(self, session, pool_start, pool_end):
+    def create_pool(self, context, pool_start, pool_end):
         """Create an allocation pool for the subnet.
 
         This method does not perform any validation on parameters; it simply
@@ -75,71 +77,64 @@ class IpamSubnetManager(object):
         :param pool_end: string expressing the end of the pool
         :return: the newly created pool object.
         """
-        ip_pool = db_models.IpamAllocationPool(
-            ipam_subnet_id=self._ipam_subnet_id,
-            first_ip=pool_start,
+        ip_pool_obj = ipam_objs.IpamAllocationPool(
+            context, ipam_subnet_id=self._ipam_subnet_id, first_ip=pool_start,
             last_ip=pool_end)
-        session.add(ip_pool)
-        return ip_pool
+        ip_pool_obj.create()
+        return ip_pool_obj
 
-    def delete_allocation_pools(self, session):
+    def delete_allocation_pools(self, context):
         """Remove all allocation pools for the current subnet.
 
-        :param session: database session
+        :param context: neutron api request context
         """
-        session.query(db_models.IpamAllocationPool).filter_by(
-            ipam_subnet_id=self._ipam_subnet_id).delete()
+        ipam_objs.IpamAllocationPool.delete_objects(
+            context, ipam_subnet_id=self._ipam_subnet_id)
 
-    def list_pools(self, session):
+    def list_pools(self, context):
         """Return pools for the current subnet."""
-        return session.query(
-            db_models.IpamAllocationPool).filter_by(
-            ipam_subnet_id=self._ipam_subnet_id)
+        return ipam_objs.IpamAllocationPool.get_objects(
+            context, ipam_subnet_id=self._ipam_subnet_id)
 
-    def check_unique_allocation(self, session, ip_address):
+    def check_unique_allocation(self, context, ip_address):
         """Validate that the IP address on the subnet is not in use."""
-        iprequest = session.query(db_models.IpamAllocation).filter_by(
-            ipam_subnet_id=self._ipam_subnet_id, status='ALLOCATED',
-            ip_address=ip_address).first()
-        if iprequest:
-            return False
-        return True
+        return not ipam_objs.IpamAllocation.objects_exist(
+                     context, ipam_subnet_id=self._ipam_subnet_id,
+                     status=const.IPAM_ALLOCATION_STATUS_ALLOCATED,
+                     ip_address=ip_address)
 
-    def list_allocations(self, session, status='ALLOCATED'):
+    def list_allocations(self, context,
+                         status=const.IPAM_ALLOCATION_STATUS_ALLOCATED):
         """Return current allocations for the subnet.
 
-        :param session: database session
+        :param context: neutron api request context
         :param status: IP allocation status
-        :returns: a list of IP allocation as instance of
-            neutron.ipam.drivers.neutrondb_ipam.db_models.IpamAllocation
+        :returns: a list of IpamAllocation OVO objects
         """
-        return session.query(
-            db_models.IpamAllocation).filter_by(
-            ipam_subnet_id=self._ipam_subnet_id,
-            status=status)
+        return ipam_objs.IpamAllocation.get_objects(
+            context, ipam_subnet_id=self._ipam_subnet_id, status=status)
 
-    def create_allocation(self, session, ip_address,
-                          status='ALLOCATED'):
+    def create_allocation(self, context, ip_address,
+                          status=const.IPAM_ALLOCATION_STATUS_ALLOCATED):
         """Create an IP allocation entry.
 
-        :param session: database session
+        :param context: neutron api request context
         :param ip_address: the IP address to allocate
         :param status: IP allocation status
         """
-        ip_request = db_models.IpamAllocation(
-            ip_address=ip_address,
-            status=status,
-            ipam_subnet_id=self._ipam_subnet_id)
-        session.add(ip_request)
+        ipam_objs.IpamAllocation(
+            context, ip_address=ip_address, status=status,
+            ipam_subnet_id=self._ipam_subnet_id).create()
 
-    def delete_allocation(self, session, ip_address):
+    def delete_allocation(self, context, ip_address):
         """Remove an IP allocation for this subnet.
 
-        :param session: database session
+        :param context: neutron api request context
         :param ip_address: IP address for which the allocation entry should
             be removed.
+        :returns: number of deleted allocation entries.
         """
-        return session.query(db_models.IpamAllocation).filter_by(
-            ip_address=ip_address,
-            ipam_subnet_id=self._ipam_subnet_id).delete(
-                synchronize_session=False)
+        return ipam_objs.IpamAllocation.delete_objects(
+            context,
+            ipam_subnet_id=self._ipam_subnet_id,
+            ip_address=ip_address)
