@@ -14,11 +14,14 @@
 
 import netaddr
 from neutron_lib import constants
+from neutron_lib.db import api as db_api
 from neutron_lib.objects import common_types
 from neutron_lib.utils import net as net_utils
 from oslo_log import log as logging
 from oslo_utils import versionutils
 from oslo_versionedobjects import fields as obj_fields
+import sqlalchemy
+from sqlalchemy import and_
 
 from neutron.common import _constants
 from neutron.db.models import dns as dns_models
@@ -83,6 +86,30 @@ class PortBinding(PortBindingBase):
 
     primary_keys = ['port_id', 'host']
 
+    @classmethod
+    def get_port_id_and_host(cls, context, vif_type, vnic_type, status):
+        """Returns only the port_id and the host of matching registers
+
+        This method returns only the primary keys of a "PortBinding" register,
+        reducing the query complexity and increasing the retrieval speed.
+        This query does not check the "PortBinding" owner or RBACs.
+        """
+        with cls.db_context_reader(context):
+            query = context.session.query(cls.db_model.port_id,
+                                          cls.db_model.host)
+            query = query.filter(and_(
+                cls.db_model.vif_type == vif_type,
+                cls.db_model.vnic_type == vnic_type,
+                cls.db_model.status == status))
+            return query.all()
+
+    @classmethod
+    @db_api.CONTEXT_READER
+    def get_duplicated_port_bindings(cls, context):
+        return context.session.query(
+            cls.db_model).group_by(
+            cls.db_model.port_id).having(sqlalchemy.func.count() > 1).all()
+
 
 @base.NeutronObjectRegistry.register
 class DistributedPortBinding(PortBindingBase):
@@ -204,12 +231,12 @@ class IPAllocation(base.NeutronDbObject):
             alloc_db = (context.session.query(models_v2.IPAllocation).
                         filter_by(subnet_id=subnet_id).join(models_v2.Port).
                         filter(~models_v2.Port.device_owner.
-                        in_(device_owner)).first())
+                               in_(device_owner)).first())
         else:
             alloc_db = (context.session.query(models_v2.IPAllocation).
                         filter_by(subnet_id=subnet_id).join(models_v2.Port).
                         filter(models_v2.Port.device_owner.
-                        in_(device_owner)).first())
+                               in_(device_owner)).first())
         if exclude and alloc_db:
             return super(IPAllocation, cls)._load_object(context, alloc_db)
         if alloc_db:
@@ -223,6 +250,22 @@ class IPAllocation(base.NeutronDbObject):
             alloc_obj = super(IPAllocation, cls)._load_object(context, alloc)
             alloc_obj.delete()
 
+    @classmethod
+    @db_api.CONTEXT_READER
+    def get_alloc_routerports(cls, context, subnet_id, gateway_ip=None,
+                              first=False):
+        alloc_qry = context.session.query(cls.db_model.port_id)
+        alloc_qry = alloc_qry.join(
+            l3.RouterPort, l3.RouterPort.port_id == cls.db_model.port_id)
+        alloc_qry = alloc_qry.filter(cls.db_model.subnet_id == subnet_id)
+        if gateway_ip:
+            alloc_qry = alloc_qry.filter(cls.db_model.ip_address == gateway_ip)
+
+        if first:
+            return alloc_qry.first()
+        else:
+            return alloc_qry.all()
+
 
 @base.NeutronObjectRegistry.register
 class PortDNS(base.NeutronDbObject):
@@ -273,7 +316,9 @@ class Port(base.NeutronDbObject):
     # Version 1.3: distributed_binding -> distributed_bindings
     # Version 1.4: Attribute binding becomes ListOfObjectsField
     # Version 1.5: Added qos_network_policy_id field
-    VERSION = '1.5'
+    # Version 1.6: Added numa_affinity_policy field
+    # Version 1.7: Added port_device field
+    VERSION = '1.7'
 
     db_model = models_v2.Port
 
@@ -323,6 +368,8 @@ class Port(base.NeutronDbObject):
         'binding_levels': obj_fields.ListOfObjectsField(
             'PortBindingLevel', nullable=True
         ),
+        'numa_affinity_policy': obj_fields.StringField(nullable=True),
+        'device_profile': obj_fields.StringField(nullable=True),
 
         # TODO(ihrachys): consider adding a 'dns_assignment' fully synthetic
         # field in later object iterations
@@ -337,10 +384,12 @@ class Port(base.NeutronDbObject):
         'bindings',
         'binding_levels',
         'data_plane_status',
+        'device_profile',
         'dhcp_options',
         'distributed_bindings',
         'dns',
         'fixed_ips',
+        'numa_affinity_policy',
         'qos_policy_id',
         'qos_network_policy_id',
         'security',
@@ -426,6 +475,7 @@ class Port(base.NeutronDbObject):
         return port_array
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_auto_deletable_port_ids_and_proper_port_count_by_segment(
             cls, context, segment_id):
 
@@ -501,6 +551,15 @@ class Port(base.NeutronDbObject):
                 db_obj.qos_network_policy_binding.policy_id)
             fields_to_change.append('qos_network_policy_binding')
 
+        if db_obj.get('numa_affinity_policy'):
+            self.numa_affinity_policy = (
+                db_obj.numa_affinity_policy.numa_affinity_policy)
+            fields_to_change.append('numa_affinity_policy')
+
+        if db_obj.get('device_profile'):
+            self.device_profile = db_obj.device_profile.device_profile
+            fields_to_change.append('device_profile')
+
         self.obj_reset_changes(fields_to_change)
 
     def obj_make_compatible(self, primitive, target_version):
@@ -529,8 +588,13 @@ class Port(base.NeutronDbObject):
                         break
         if _target_version < (1, 5):
             primitive.pop('qos_network_policy_id', None)
+        if _target_version < (1, 6):
+            primitive.pop('numa_affinity_policy', None)
+        if _target_version < (1, 7):
+            primitive.pop('device_profile', None)
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_by_router_and_network(cls, context, router_id, owner,
                                         network_id):
         """Returns port objects filtering by router ID, owner and network ID"""
@@ -540,6 +604,7 @@ class Port(base.NeutronDbObject):
                                         rports_filter, router_filter)
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_by_router_and_port(cls, context, router_id, owner, port_id):
         """Returns port objects filtering by router ID, owner and port ID"""
         rports_filter = (l3.RouterPort.port_id == port_id, )
@@ -592,6 +657,7 @@ class Port(base.NeutronDbObject):
         return ports_rports
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_ids_by_security_groups(cls, context, security_group_ids,
                                          excluded_device_owners=None):
         query = context.session.query(sg_models.SecurityGroupPortBinding)
@@ -605,6 +671,7 @@ class Port(base.NeutronDbObject):
         return [port_binding['port_id'] for port_binding in query.all()]
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_by_host(cls, context, host):
         query = context.session.query(models_v2.Port.id).join(
             ml2_models.PortBinding)
@@ -613,6 +680,7 @@ class Port(base.NeutronDbObject):
         return [port_id[0] for port_id in query.all()]
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_by_binding_type_and_host(cls, context,
                                            binding_type, host):
         query = context.session.query(models_v2.Port).join(
@@ -623,16 +691,17 @@ class Port(base.NeutronDbObject):
         return [cls._load_object(context, db_obj) for db_obj in query.all()]
 
     @classmethod
-    def get_ports_by_vnic_type_and_host(
-            cls, context, vnic_type, host):
+    @db_api.CONTEXT_READER
+    def get_ports_by_vnic_type_and_host(cls, context, vnic_type, host=None):
         query = context.session.query(models_v2.Port).join(
             ml2_models.PortBinding)
-        query = query.filter(
-            ml2_models.PortBinding.vnic_type == vnic_type,
-            ml2_models.PortBinding.host == host)
+        query = query.filter(ml2_models.PortBinding.vnic_type == vnic_type)
+        if host:
+            query = query.filter(ml2_models.PortBinding.host == host)
         return [cls._load_object(context, db_obj) for db_obj in query.all()]
 
     @classmethod
+    @db_api.CONTEXT_READER
     def check_network_ports_by_binding_types(
             cls, context, network_id, binding_types, negative_search=False):
         """This method is to check whether networks have ports with given
@@ -657,8 +726,33 @@ class Port(base.NeutronDbObject):
         return bool(query.count())
 
     @classmethod
+    @db_api.CONTEXT_READER
     def get_ports_allocated_by_subnet_id(cls, context, subnet_id):
         """Return ports with fixed IPs in a subnet"""
         return context.session.query(models_v2.Port).filter(
             models_v2.IPAllocation.port_id == models_v2.Port.id).filter(
-            models_v2.IPAllocation.subnet_id == subnet_id).all()
+                models_v2.IPAllocation.subnet_id == subnet_id).all()
+
+    @classmethod
+    def get_port_from_mac_and_pci_slot(cls, context, device_mac,
+                                       pci_slot=None):
+        with db_api.CONTEXT_READER.using(context):
+            ports = cls.get_objects(context, mac_address=device_mac)
+
+        if not ports:
+            return
+        elif not pci_slot:
+            return ports.pop()
+        else:
+            for port in ports:
+                for _binding in port.bindings:
+                    if _binding.get('profile', {}).get('pci_slot') == pci_slot:
+                        return port
+
+    @classmethod
+    @db_api.CONTEXT_READER
+    def get_gateway_port_ids_by_network(cls, context, network_id):
+        gw_ports = context.session.query(models_v2.Port.id).filter_by(
+            device_owner=constants.DEVICE_OWNER_ROUTER_GW,
+            network_id=network_id)
+        return [gw_port[0] for gw_port in gw_ports]
