@@ -14,18 +14,15 @@
 #    under the License.
 
 from neutron_lib import constants as const
+from neutron_lib import context as n_context
 from neutron_lib import exceptions
-from oslo_config import cfg
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from neutron_lib.plugins.ml2 import api
 from oslo_log import log as logging
 
-from neutron._i18n import _, _LW
-from neutron import context as n_context
-from neutron.db import api as db_api
+from neutron._i18n import _
 from neutron.db import l3_hamode_db
-from neutron import manager
-from neutron.plugins.common import constants as service_constants
-from neutron.plugins.ml2 import driver_api as api
-from neutron.plugins.ml2.drivers.l2pop import config  # noqa
 from neutron.plugins.ml2.drivers.l2pop import db as l2pop_db
 from neutron.plugins.ml2.drivers.l2pop import rpc as l2pop_rpc
 
@@ -51,16 +48,24 @@ class L2populationMechanismDriver(api.MechanismDriver):
                                    ip_address=ip['ip_address'])
                 for ip in port['fixed_ips']]
 
+    def _remove_flooding(self, fdb_entries):
+        for network_fdb in fdb_entries.values():
+            for agent_fdb in network_fdb.get('ports', {}).values():
+                try:
+                    agent_fdb.remove(const.FLOODING_ENTRY)
+                except ValueError:
+                    pass
+
     def check_vlan_transparency(self, context):
         """L2population driver vlan transparency support."""
         return True
 
     def _get_ha_port_agents_fdb(
-            self, session, network_id, router_id):
+            self, context, network_id, router_id):
         other_fdb_ports = {}
-        for agent in l2pop_db.get_ha_agents_by_router_id(session, router_id):
+        for agent in l2pop_db.get_ha_agents_by_router_id(context, router_id):
             agent_active_ports = l2pop_db.get_agent_network_active_port_count(
-                session, agent.host, network_id)
+                context, agent.host, network_id)
             if agent_active_ports == 0:
                 ip = l2pop_db.get_agent_ip(agent)
                 other_fdb_ports[ip] = [const.FLOODING_ENTRY]
@@ -70,17 +75,18 @@ class L2populationMechanismDriver(api.MechanismDriver):
     def delete_port_postcommit(self, context):
         port = context.current
         agent_host = context.host
-        fdb_entries = self._get_agent_fdb(context.bottom_bound_segment,
-                                          port, agent_host)
-        if port['device_owner'] in l2pop_db.HA_ROUTER_PORTS and fdb_entries:
-            session = db_api.get_session()
+        plugin_context = context._plugin_context
+        fdb_entries = self._get_agent_fdb(
+            plugin_context, context.bottom_bound_segment, port, agent_host)
+        if fdb_entries and l3_hamode_db.is_ha_router_port(
+                plugin_context, port['device_owner'], port['device_id']):
             network_id = port['network_id']
             other_fdb_ports = self._get_ha_port_agents_fdb(
-                session, network_id, port['device_id'])
+                plugin_context, network_id, port['device_id'])
             fdb_entries[network_id]['ports'] = other_fdb_ports
 
         self.L2populationAgentNotify.remove_fdb_entries(self.rpc_ctx,
-            fdb_entries)
+                                                        fdb_entries)
 
     def filter_hosts_with_segment_access(
             self, context, segments, candidate_hosts, agent_getter):
@@ -110,7 +116,14 @@ class L2populationMechanismDriver(api.MechanismDriver):
         if not agent_host:
             return
 
-        agent_ip = l2pop_db.get_agent_ip_by_host(db_api.get_session(),
+        # We should not add arp responder for non tunnel network type
+        port_context = context._plugin_context
+        agent = l2pop_db.get_agent_by_host(port_context, agent_host)
+        segment = context.bottom_bound_segment
+        if not self._validate_segment(segment, port['id'], agent):
+            return
+
+        agent_ip = l2pop_db.get_agent_ip_by_host(context._plugin_context,
                                                  agent_host)
 
         orig_mac_ip = [l2pop_rpc.PortInfo(mac_address=port['mac_address'],
@@ -139,7 +152,7 @@ class L2populationMechanismDriver(api.MechanismDriver):
         orig = context.original
 
         if (orig['mac_address'] != port['mac_address'] and
-            context.status == const.PORT_STATUS_ACTIVE):
+                context.status == const.PORT_STATUS_ACTIVE):
             msg = _("unable to modify mac_address of ACTIVE port "
                     "%s") % port['id']
             raise exceptions.InvalidInput(error_message=msg)
@@ -147,7 +160,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
     def update_port_postcommit(self, context):
         port = context.current
         orig = context.original
-        if l3_hamode_db.is_ha_router_port(port['device_owner'],
+        plugin_context = context._plugin_context
+        if l3_hamode_db.is_ha_router_port(plugin_context, port['device_owner'],
                                           port['device_id']):
             return
         diff_ips = self._get_diff_ips(orig, port)
@@ -159,16 +173,17 @@ class L2populationMechanismDriver(api.MechanismDriver):
             if context.status == const.PORT_STATUS_DOWN:
                 agent_host = context.host
                 fdb_entries = self._get_agent_fdb(
-                        context.bottom_bound_segment, port, agent_host)
+                    plugin_context, context.bottom_bound_segment, port,
+                    agent_host)
                 self.L2populationAgentNotify.remove_fdb_entries(
                     self.rpc_ctx, fdb_entries)
-        elif (context.host != context.original_host
-              and context.original_status == const.PORT_STATUS_ACTIVE
-              and context.status == const.PORT_STATUS_DOWN):
+        elif (context.host != context.original_host and
+              context.original_status == const.PORT_STATUS_ACTIVE and
+              context.status == const.PORT_STATUS_DOWN):
             # The port has been migrated. Send notification about port
             # removal from old host.
             fdb_entries = self._get_agent_fdb(
-                context.original_bottom_bound_segment,
+                plugin_context, context.original_bottom_bound_segment,
                 orig, context.original_host)
             self.L2populationAgentNotify.remove_fdb_entries(
                 self.rpc_ctx, fdb_entries)
@@ -177,7 +192,8 @@ class L2populationMechanismDriver(api.MechanismDriver):
                 self.update_port_up(context)
             elif context.status == const.PORT_STATUS_DOWN:
                 fdb_entries = self._get_agent_fdb(
-                    context.bottom_bound_segment, port, context.host)
+                    plugin_context, context.bottom_bound_segment, port,
+                    context.host)
                 self.L2populationAgentNotify.remove_fdb_entries(
                     self.rpc_ctx, fdb_entries)
 
@@ -195,15 +211,15 @@ class L2populationMechanismDriver(api.MechanismDriver):
 
         return True
 
-    def _create_agent_fdb(self, session, agent, segment, network_id):
+    def _create_agent_fdb(self, context, agent, segment, network_id):
         agent_fdb_entries = {network_id:
                              {'segment_id': segment['segmentation_id'],
                               'network_type': segment['network_type'],
                               'ports': {}}}
         tunnel_network_ports = (
-            l2pop_db.get_distributed_active_network_ports(session, network_id))
+            l2pop_db.get_distributed_active_network_ports(context, network_id))
         fdb_network_ports = (
-            l2pop_db.get_nondistributed_active_network_ports(session,
+            l2pop_db.get_nondistributed_active_network_ports(context,
                                                              network_id))
         ports = agent_fdb_entries[network_id]['ports']
         ports.update(self._get_tunnels(
@@ -236,48 +252,57 @@ class L2populationMechanismDriver(api.MechanismDriver):
     def update_port_down(self, context):
         port = context.current
         agent_host = context.host
-        l3plugin = manager.NeutronManager.get_service_plugins().get(
-            service_constants.L3_ROUTER_NAT)
+        l3plugin = directory.get_plugin(plugin_constants.L3)
         # when agent transitions to backup, don't remove flood flows
-        if agent_host and l3plugin and getattr(
-            l3plugin, "list_router_ids_on_host", None):
+        if agent_host and l3plugin and getattr(l3plugin,
+                                               "list_router_ids_on_host",
+                                               None):
             admin_context = n_context.get_admin_context()
-            if l3plugin.list_router_ids_on_host(
-                admin_context, agent_host, [port['device_id']]):
-                return
+            port_context = context._plugin_context
             fdb_entries = self._get_agent_fdb(
-                context.bottom_bound_segment, port, agent_host)
+                port_context, context.bottom_bound_segment, port, agent_host,
+                include_ha_router_ports=True)
+            if (fdb_entries and
+                l3plugin.list_router_ids_on_host(
+                    admin_context, agent_host, [port['device_id']])):
+                # NOTE(slaweq): in case this is HA router, remove unicast
+                # entries to this port but don't remove flood entry
+                self._remove_flooding(fdb_entries)
             self.L2populationAgentNotify.remove_fdb_entries(
                 self.rpc_ctx, fdb_entries)
 
-    def update_port_up(self, context):
+    def update_port_up(self, context, refresh_tunnels=False):
         port = context.current
         agent_host = context.host
-        session = db_api.get_session()
-        agent = l2pop_db.get_agent_by_host(session, agent_host)
+        port_context = context._plugin_context
+        agent = l2pop_db.get_agent_by_host(port_context, agent_host)
         if not agent:
-            LOG.warning(_LW("Unable to retrieve active L2 agent on host %s"),
+            LOG.warning("Unable to retrieve active L2 agent on host %s",
                         agent_host)
             return
 
+        segment = context.bottom_bound_segment
+        if not self._validate_segment(segment, port['id'], agent):
+            return
+
         network_id = port['network_id']
 
         agent_active_ports = l2pop_db.get_agent_network_active_port_count(
-            session, agent_host, network_id)
-
+            port_context, agent_host, network_id)
+        LOG.debug("host: %s, agent_active_ports: %s, refresh_tunnels: %s",
+                  agent_host, agent_active_ports, refresh_tunnels)
         agent_ip = l2pop_db.get_agent_ip(agent)
-        segment = context.bottom_bound_segment
-        if not self._validate_segment(segment, port['id'], agent):
-            return
         other_fdb_entries = self._get_fdb_entries_template(
             segment, agent_ip, network_id)
         other_fdb_ports = other_fdb_entries[network_id]['ports']
 
-        if agent_active_ports == 1 or (l2pop_db.get_agent_uptime(agent) <
-                                       cfg.CONF.l2pop.agent_boot_time):
-            # First port activated on current agent in this network,
+        # with high concurrency more than 1 port may be activated on an agent
+        # at the same time (like VM port + a DVR port) so checking for 1 or 2
+        is_first_port = agent_active_ports in (1, 2)
+        if is_first_port or refresh_tunnels:
+            # First port(s) activated on current agent in this network,
             # we have to provide it with the whole list of fdb entries
-            agent_fdb_entries = self._create_agent_fdb(session,
+            agent_fdb_entries = self._create_agent_fdb(port_context,
                                                        agent,
                                                        segment,
                                                        network_id)
@@ -291,26 +316,31 @@ class L2populationMechanismDriver(api.MechanismDriver):
 
         # Notify other agents to add fdb rule for current port
         if (port['device_owner'] != const.DEVICE_OWNER_DVR_INTERFACE and
-            not l3_hamode_db.is_ha_router_port(port['device_owner'],
-                                               port['device_id'])):
+            not l3_hamode_db.is_ha_router_port(
+                port_context, port['device_owner'], port['device_id'])):
             other_fdb_ports[agent_ip] += self._get_port_fdb_entries(port)
 
         self.L2populationAgentNotify.add_fdb_entries(self.rpc_ctx,
                                                      other_fdb_entries)
 
-    def _get_agent_fdb(self, segment, port, agent_host):
+    def _get_agent_fdb(self, context, segment, port, agent_host,
+                       include_ha_router_ports=False):
         if not agent_host:
             return
 
+        agent = l2pop_db.get_agent_by_host(context,
+                                           agent_host)
+        if not agent:
+            LOG.warning("Unable to retrieve active L2 agent on host %s",
+                        agent_host)
+            return
+        if not self._validate_segment(segment, port['id'], agent):
+            return
+
         network_id = port['network_id']
 
-        session = db_api.get_session()
         agent_active_ports = l2pop_db.get_agent_network_active_port_count(
-            session, agent_host, network_id)
-
-        agent = l2pop_db.get_agent_by_host(db_api.get_session(), agent_host)
-        if not self._validate_segment(segment, port['id'], agent):
-            return
+            context, agent_host, network_id)
 
         agent_ip = l2pop_db.get_agent_ip(agent)
         other_fdb_entries = self._get_fdb_entries_template(
@@ -322,8 +352,10 @@ class L2populationMechanismDriver(api.MechanismDriver):
                 const.FLOODING_ENTRY)
         # Notify other agents to remove fdb rules for current port
         if (port['device_owner'] != const.DEVICE_OWNER_DVR_INTERFACE and
-            not l3_hamode_db.is_ha_router_port(port['device_owner'],
-                                               port['device_id'])):
+           (include_ha_router_ports or
+            not l3_hamode_db.is_ha_router_port(context,
+                                               port['device_owner'],
+                                               port['device_id']))):
             fdb_entries = self._get_port_fdb_entries(port)
             other_fdb_entries[network_id]['ports'][agent_ip] += fdb_entries
 
