@@ -293,13 +293,14 @@ class API(api.API, metaclass=abc.ABCMeta):
 
     @abc.abstractmethod
     def get_unhosted_gateways(self, port_physnet_dict, chassis_physnets,
-                              gw_chassis):
+                              gw_chassis, chassis_with_azs):
         """Return a list of gateways not hosted on chassis
 
         :param port_physnet_dict: Dictionary of gateway ports and their physnet
         :param chassis_physnets:  Dictionary of chassis and physnets
         :param gw_chassis:        List of gateway chassis provided by admin
                                   through ovn-cms-options
+        :param chassis_with_azs:  Dictionary of chassis and available zones
         :returns:                 List of gateways not hosted on a valid
                                   chassis
         """
@@ -626,7 +627,8 @@ class SbAPI(api.API, metaclass=abc.ABCMeta):
         value. And hostname and physnets are related to the same host.
         """
 
-    def get_gateway_chassis_from_cms_options(self):
+    @abc.abstractmethod
+    def get_gateway_chassis_from_cms_options(self, name_only=True):
         """Get chassis eligible for external connectivity from CMS options.
 
         When admin wants to enable router gateway on few chassis,
@@ -635,7 +637,10 @@ class SbAPI(api.API, metaclass=abc.ABCMeta):
         ovs-vsctl set open .
            external_ids:ovn-cms-options="enable-chassis-as-gw"
         In this function, we parse ovn-cms-options and return these chassis
-        :returns:              List with chassis names.
+
+        :param name_only: Return only the chassis names instead of
+                          objects. Defaults to True.
+        :returns: List with chassis.
         """
 
     @abc.abstractmethod
@@ -653,16 +658,3 @@ class SbAPI(api.API, metaclass=abc.ABCMeta):
         :param chassis_type:    The type of chassis
         :type chassis_type:     string
         """
-
-    @abc.abstractmethod
-    def get_chassis_data_for_ml2_bind_port(self, hostname):
-        """Return chassis data for ML2 port binding.
-
-        @param hostname:       The hostname of the chassis
-        @type hostname:        string
-        :returns:              Tuple containing the chassis datapath type,
-                               iface types and physical networks for the
-                               OVN bridge mappings.
-        :raises:               RuntimeError exception if an OVN chassis
-                               does not exist.
-        """
