@@ -34,6 +34,7 @@ from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import exceptions as ovn_exc
 from neutron.common.ovn import utils
+from neutron.common import utils as n_utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf as cfg
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import commands as cmd
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
@@ -48,10 +49,10 @@ LOG = log.getLogger(__name__)
 def wait_for_change(idl_, timeout, seqno=None):
     if seqno is None:
         seqno = idl_.change_seqno
-        while idl_.change_seqno == seqno and not idl_.run():
-            poller = idlutils.poller.Poller()
-            idl_.wait(poller)
-            poller.block()
+    while idl_.change_seqno == seqno and not idl_.run():
+        poller = idlutils.poller.Poller()
+        idl_.wait(poller)
+        poller.block()
 
 
 idlutils.wait_for_change = wait_for_change
@@ -139,6 +140,29 @@ class Backend(ovs_idl.Backend):
 
     _tables = tables
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        raise NotImplementedError()
+
+    @n_utils.classproperty
+    def schema_helper(cls):
+        # SchemaHelper.get_idl_schema() sets schema_json to None which is
+        # called in Idl.__init__(), so if we've done that return new helper
+        try:
+            if cls._schema_helper.schema_json:
+                return cls._schema_helper
+        except AttributeError:
+            pass
+
+        ovsdb_monitor._check_and_set_ssl_files(cls.schema)
+        cls._schema_helper = idlutils.get_schema_helper(cls.connection_string,
+                                                        cls.schema)
+        return cls._schema_helper
+
+    @classmethod
+    def schema_has_table(cls, table_name):
+        return table_name in cls.schema_helper.schema_json['tables']
+
     def is_table_present(self, table_name):
         return table_name in self._tables
 
@@ -146,6 +170,14 @@ class Backend(ovs_idl.Backend):
         return self.is_table_present(table_name) and (
             col_name in self._tables[table_name].columns)
 
+    def is_col_supports_value(self, table_name, col_name, value):
+        if not self.is_col_present(table_name, col_name):
+            return False
+        enum = self._tables[table_name].columns[col_name].type.key.enum
+        if not enum:
+            return False
+        return value in {k.value for k in enum.values}
+
     def create_transaction(self, check_error=False, log_errors=True):
         return idl_trans.Transaction(
             self, self.ovsdb_connection, self.ovsdb_connection.timeout,
@@ -199,9 +231,13 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def __init__(self, connection):
         super(OvsdbNbOvnIdl, self).__init__(connection)
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        return cfg.get_ovn_nb_connection()
+
     @classmethod
     def from_worker(cls, worker_class, driver=None):
-        args = (cfg.get_ovn_nb_connection(), 'OVN_Northbound')
+        args = (cls.connection_string, cls.schema_helper)
         if worker_class == worker.MaintenanceWorker:
             idl_ = ovsdb_monitor.BaseOvnIdl.from_server(*args)
         else:
@@ -227,11 +263,14 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         This method is just a wrapper around the ovsdbapp transaction
         to handle revision conflicts correctly.
         """
+        revision_mismatch_raise = kwargs.pop('revision_mismatch_raise', False)
         try:
             with super(OvsdbNbOvnIdl, self).transaction(*args, **kwargs) as t:
                 yield t
         except ovn_exc.RevisionConflict as e:
             LOG.info('Transaction aborted. Reason: %s', e)
+            if revision_mismatch_raise:
+                raise e
 
     def create_lswitch_port(self, lport_name, lswitch_name, may_exist=True,
                             **columns):
@@ -242,6 +281,10 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return cmd.SetLSwitchPortCommand(self, lport_name,
                                          if_exists, **columns)
 
+    def update_lswitch_qos_options(self, port, if_exists=True, **qos):
+        return cmd.UpdateLSwitchPortQosOptionsCommand(self, port, if_exists,
+                                                      **qos)
+
     def delete_lswitch_port(self, lport_name=None, lswitch_name=None,
                             ext_id=None, if_exists=True):
         if lport_name is not None:
@@ -251,6 +294,14 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             raise RuntimeError(_("Currently only supports "
                                  "delete by lport-name"))
 
+    def get_all_stateless_fip_nats(self):
+        cmd = self.db_find('NAT',
+            ('external_ids', '!=', {ovn_const.OVN_FIP_EXT_ID_KEY: ''}),
+            ('options', '=', {'stateless': 'true'}),
+            ('type', '=', 'dnat_and_snat')
+        )
+        return cmd.execute(check_error=True)
+
     def get_all_logical_switches_with_ports(self):
         result = []
         for lswitch in self._tables['Logical_Switch'].rows.values():
@@ -354,7 +405,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             # Iterate over each acl in a lswitch and store the acl in
             # a key:value representation for e.g. acl_string. This
             # key:value representation can invoke the code -
-            # self._ovn.add_acl(**acl_string)
+            # self.add_acl(**acl_string)
             for acl in acls:
                 ext_ids = getattr(acl, 'external_ids', {})
                 port_id = ext_ids.get('neutron:lport')
@@ -463,24 +514,42 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         except idlutils.RowNotFound:
             return []
 
+    def get_gateway_chassis_az_hints(self, gateway_name):
+        lrp = self.lookup('Logical_Router_Port', gateway_name,
+                          default=None)
+        if not lrp:
+            return []
+        router_id = lrp.external_ids.get(
+            ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY, "")
+        lrouter = self.lookup('Logical_Router', utils.ovn_name(router_id),
+                              default=None)
+        if not lrouter:
+            return []
+        az_string = lrouter.external_ids.get(
+            ovn_const.OVN_AZ_HINTS_EXT_ID_KEY, "")
+        if not az_string:
+            return []
+        return az_string.split(",")
+
     def get_chassis_gateways(self, chassis_name):
         gw_chassis = self.db_find_rows(
             'Gateway_Chassis', ('chassis_name', '=', chassis_name))
         return gw_chassis.execute(check_error=True)
 
     def get_unhosted_gateways(self, port_physnet_dict, chassis_with_physnets,
-                              all_gw_chassis):
+                              all_gw_chassis, chassis_with_azs):
         unhosted_gateways = set()
         for port, physnet in port_physnet_dict.items():
             lrp_name = '%s%s' % (ovn_const.LRP_PREFIX, port)
             original_state = self.get_gateway_chassis_binding(lrp_name)
-
+            az_hints = self.get_gateway_chassis_az_hints(lrp_name)
             # Filter out chassis that lost physnet, the cms option,
             # or has been deleted.
             actual_gw_chassis = [
                 chassis for chassis in original_state
                 if not utils.is_gateway_chassis_invalid(
-                    chassis, all_gw_chassis, physnet, chassis_with_physnets)]
+                    chassis, all_gw_chassis, physnet, chassis_with_physnets,
+                    az_hints, chassis_with_azs)]
 
             # Check if gw ports are fully scheduled.
             if len(actual_gw_chassis) >= ovn_const.MAX_GW_CHASSIS:
@@ -492,7 +561,8 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
             available_chassis = {
                 c for c in all_gw_chassis or chassis_with_physnets.keys()
                 if not utils.is_gateway_chassis_invalid(
-                    c, all_gw_chassis, physnet, chassis_with_physnets)}
+                    c, all_gw_chassis, physnet, chassis_with_physnets,
+                    az_hints, chassis_with_azs)}
 
             if available_chassis == set(original_state):
                 # The same situation as was before. Nothing
@@ -519,7 +589,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
                 'external_ids': ext_ids, 'uuid': row.uuid}
 
     def get_subnet_dhcp_options(self, subnet_id, with_ports=False):
-        subnet = None
+        subnet = {}
         ports = []
         for row in self._tables['DHCP_Options'].rows.values():
             external_ids = getattr(row, 'external_ids', {})
@@ -566,10 +636,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_address_sets(self):
         address_sets = {}
         for row in self._tables['Address_Set'].rows.values():
-            # TODO(lucasagomes): Remove OVN_SG_NAME_EXT_ID_KEY in the
-            # Rocky release
-            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-               ovn_const.OVN_SG_NAME_EXT_ID_KEY in row.external_ids):
+            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids):
                 continue
             name = getattr(row, 'name')
             data = {}
@@ -609,12 +676,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
 
         nat_rules = []
         for nat_rule in getattr(lrouter, 'nat', []):
-            ext_ids = {}
-            # TODO(dalvarez): remove this check once the minimum OVS required
-            # version contains the column (when OVS 2.8.2 is released).
-            if self.is_col_present('NAT', 'external_ids'):
-                ext_ids = dict(getattr(nat_rule, 'external_ids', {}))
-
+            ext_ids = dict(getattr(nat_rule, 'external_ids', {}))
             nat_rules.append({'external_ip': nat_rule.external_ip,
                               'logical_ip': nat_rule.logical_ip,
                               'type': nat_rule.type,
@@ -685,11 +747,6 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         return result[0] if result else None
 
     def get_floatingip(self, fip_id):
-        # TODO(dalvarez): remove this check once the minimum OVS required
-        # version contains the column (when OVS 2.8.2 is released).
-        if not self.is_col_present('NAT', 'external_ids'):
-            return
-
         fip = self.db_find('NAT', ('external_ids', '=',
                                    {ovn_const.OVN_FIP_EXT_ID_KEY: fip_id}))
         result = fip.execute(check_error=True)
@@ -713,12 +770,11 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_lrouter(self, lrouter_name):
         if uuidutils.is_uuid_like(lrouter_name):
             lrouter_name = utils.ovn_name(lrouter_name)
-
-        # TODO(lucasagomes): Use lr_get() once we start refactoring this
-        # API to use methods from ovsdbapp.
-        lr = self.db_find_rows('Logical_Router', ('name', '=', lrouter_name))
-        result = lr.execute(check_error=True)
-        return result[0] if result else None
+        try:
+            return self.lr_get(lrouter_name).execute(log_errors=False,
+                                                     check_error=True)
+        except idlutils.RowNotFound:
+            return None
 
     def get_lrouter_port(self, lrp_name):
         # TODO(mangelajo): Implement lrp_get() ovsdbapp and use from here
@@ -734,16 +790,7 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
     def get_port_group(self, pg_name):
         if uuidutils.is_uuid_like(pg_name):
             pg_name = utils.ovn_port_group_name(pg_name)
-        try:
-            for pg in self._tables['Port_Group'].rows.values():
-                if pg.name == pg_name:
-                    return pg
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            return
+        return self.lookup('Port_Group', pg_name, default=None)
 
     def get_sg_port_groups(self):
         """Returns OVN port groups used as Neutron Security Groups.
@@ -753,22 +800,15 @@ class OvsdbNbOvnIdl(nb_impl_idl.OvnNbApiIdlImpl, Backend):
         SGs, it will also not be returned.
         """
         port_groups = {}
-        try:
-            for row in self._tables['Port_Group'].rows.values():
-                name = getattr(row, 'name')
-                if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
-                   name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
-                    continue
-                data = {}
-                for row_key in getattr(row, "_data", {}):
-                    data[row_key] = getattr(row, row_key)
-                port_groups[name] = data
-        except KeyError:
-            # TODO(dalvarez): This except block is added for backwards compat
-            # with old OVN schemas (<=2.9) where Port Groups are not present.
-            # This (and other conditional code around this feature) shall be
-            # removed at some point.
-            pass
+        for row in self._tables['Port_Group'].rows.values():
+            name = getattr(row, 'name')
+            if not (ovn_const.OVN_SG_EXT_ID_KEY in row.external_ids or
+               name == ovn_const.OVN_DROP_PORT_GROUP_NAME):
+                continue
+            data = {}
+            for row_key in getattr(row, "_data", {}):
+                data[row_key] = getattr(row, row_key)
+            port_groups[name] = data
         return port_groups
 
     def check_liveness(self):
@@ -792,9 +832,13 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
     def __init__(self, connection):
         super(OvsdbSbOvnIdl, self).__init__(connection)
 
+    @n_utils.classproperty
+    def connection_string(cls):
+        return cfg.get_ovn_sb_connection()
+
     @classmethod
     def from_worker(cls, worker_class, driver=None):
-        args = (cfg.get_ovn_sb_connection(), 'OVN_Southbound')
+        args = (cls.connection_string, cls.schema_helper)
         if worker_class == worker.MaintenanceWorker:
             idl_ = ovsdb_monitor.BaseOvnSbIdl.from_server(*args)
         else:
@@ -804,8 +848,7 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
 
     def _get_chassis_physnets(self, chassis):
         bridge_mappings = chassis.external_ids.get('ovn-bridge-mappings', '')
-        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','),
-                                              unique_values=False)
+        mapping_dict = helpers.parse_mappings(bridge_mappings.split(','))
         return list(mapping_dict.keys())
 
     def chassis_exists(self, hostname):
@@ -818,13 +861,11 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             chassis_info_dict[ch.hostname] = self._get_chassis_physnets(ch)
         return chassis_info_dict
 
-    def get_gateway_chassis_from_cms_options(self):
-        gw_chassis = []
-        for ch in self.chassis_list().execute(check_error=True):
-            cms_options = ch.external_ids.get('ovn-cms-options', '')
-            if 'enable-chassis-as-gw' in cms_options.split(','):
-                gw_chassis.append(ch.name)
-        return gw_chassis
+    def get_gateway_chassis_from_cms_options(self, name_only=True):
+        return [ch.name if name_only else ch
+                for ch in self.chassis_list().execute(check_error=True)
+                if ovn_const.CMS_OPT_CHASSIS_AS_GW in
+                ch.external_ids.get(ovn_const.OVN_CMS_OPTIONS, '').split(',')]
 
     def get_chassis_and_physnets(self):
         chassis_info_dict = {}
@@ -832,21 +873,31 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             chassis_info_dict[ch.name] = self._get_chassis_physnets(ch)
         return chassis_info_dict
 
+    def get_chassis_and_azs(self):
+        chassis_azs = {}
+        for ch in self.chassis_list().execute(check_error=True):
+            chassis_azs[ch.name] = utils.get_chassis_availability_zones(ch)
+        return chassis_azs
+
     def get_all_chassis(self, chassis_type=None):
         # TODO(azbiswas): Use chassis_type as input once the compute type
         # preference patch (as part of external ids) merges.
         return [c.name for c in self.chassis_list().execute(check_error=True)]
 
-    def get_chassis_data_for_ml2_bind_port(self, hostname):
-        try:
-            cmd = self.db_find_rows('Chassis', ('hostname', '=', hostname))
-            chassis = next(c for c in cmd.execute(check_error=True))
-        except StopIteration:
-            msg = _('Chassis with hostname %s does not exist') % hostname
-            raise RuntimeError(msg)
-        return (chassis.external_ids.get('datapath-type', ''),
-                chassis.external_ids.get('iface-types', ''),
-                self._get_chassis_physnets(chassis))
+    def get_chassis_by_card_serial_from_cms_options(self,
+                                                    card_serial_number):
+        for ch in self.chassis_list().execute(check_error=True):
+            if ('{}={}'
+                .format(ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
+                        card_serial_number)
+                    in ch.external_ids.get(
+                        ovn_const.OVN_CMS_OPTIONS, '').split(',')):
+                return ch
+        msg = _('Chassis with %s %s %s does not exist'
+                ) % (ovn_const.OVN_CMS_OPTIONS,
+                     ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
+                     card_serial_number)
+        raise RuntimeError(msg)
 
     def get_metadata_port_network(self, network):
         # TODO(twilson) This function should really just take a Row/RowView
@@ -855,23 +906,9 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         except idlutils.RowNotFound:
             return None
         cmd = self.db_find_rows('Port_Binding', ('datapath', '=', dp),
-                                ('type', '=', 'localport'))
+                                ('type', '=', ovn_const.LSP_TYPE_LOCALPORT))
         return next(iter(cmd.execute(check_error=True)), None)
 
-    def get_chassis_metadata_networks(self, chassis_name):
-        """Return a list with the metadata networks the chassis is hosting."""
-        chassis = self.lookup('Chassis', chassis_name)
-        proxy_networks = chassis.external_ids.get(
-            'neutron-metadata-proxy-networks', None)
-        return proxy_networks.split(',') if proxy_networks else []
-
-    def set_chassis_metadata_networks(self, chassis, networks):
-        nets = ','.join(networks) if networks else ''
-        # TODO(twilson) This could just use DbSetCommand
-        return cmd.UpdateChassisExtIdsCommand(
-            self, chassis, {'neutron-metadata-proxy-networks': nets},
-            if_exists=True)
-
     def set_chassis_neutron_description(self, chassis, description,
                                         agent_type):
         desc_key = (ovn_const.OVN_AGENT_METADATA_DESC_KEY
@@ -890,16 +927,11 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
             if not port.chassis:
                 return False
 
-            # TODO(dalvarez): Remove the comparison to port.datapath.uuid in Y
-            # cycle when we are sure that all namespaces will be created with
-            # the Neutron network UUID and not anymore with the OVN datapath
-            # UUID.
-            is_in_network = lambda port: (
-                str(port.datapath.uuid) == network or
-                utils.get_network_name_from_datapath(port.datapath) == network)
-
-            return port.mac and is_in_network(port) and (
-                    ip_address in port.mac[0].split(' '))
+            is_in_network = utils.get_network_name_from_datapath(
+                port.datapath) == network
+            return (port.mac and
+                    is_in_network and
+                    (ip_address in port.mac[0].split(' ')))
 
         return [r for r in rows if check_net_and_ip(r)]
 
@@ -913,10 +945,3 @@ class OvsdbSbOvnIdl(sb_impl_idl.OvnSbApiIdlImpl, Backend):
         # and just start using chassis objects so db_find_rows could be used
         rows = self.db_list_rows('Port_Binding').execute(check_error=True)
         return [r for r in rows if r.chassis and r.chassis[0].name == chassis]
-
-    def get_logical_port_chassis_and_datapath(self, name):
-        for port in self._tables['Port_Binding'].rows.values():
-            if port.logical_port == name:
-                datapath = str(port.datapath.uuid)
-                chassis = port.chassis[0].name if port.chassis else None
-                return chassis, datapath
