@@ -14,12 +14,14 @@
 #    under the License.
 
 import abc
+import copy
 import inspect
+import re
 import threading
 
 from futurist import periodics
-from neutron_lib.api.definitions import external_net
-from neutron_lib.api.definitions import segment as segment_def
+from neutron_lib.api.definitions import portbindings
+from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
 from neutron_lib import exceptions as n_exc
@@ -33,14 +35,13 @@ from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db as hash_ring_db
 from neutron.db import ovn_revision_numbers_db as revision_numbers_db
-from neutron.db import segments_db
+from neutron.objects import ports as ports_obj
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_db_sync
 
 
 CONF = cfg.CONF
 LOG = log.getLogger(__name__)
 
-DB_CONSISTENCY_CHECK_INTERVAL = 300  # 5 minutes
 INCONSISTENCY_TYPE_CREATE_UPDATE = 'create/update'
 INCONSISTENCY_TYPE_DELETE = 'delete'
 
@@ -274,6 +275,26 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         else:
             self._ovn_client.update_subnet(context, sn_db_obj, n_db_obj)
 
+    # The migration will run just once per neutron-server instance. If the lock
+    # is held by some other neutron-server instance in the cloud, we'll attempt
+    # to perform the migration every 10 seconds until completed.
+    # TODO(ihrachys): Remove the migration to stateful fips in Z+1.
+    @periodics.periodic(spacing=10, run_immediately=True)
+    @rerun_on_schema_updates
+    def migrate_to_stateful_fips(self):
+        """Perform the migration from stateless to stateful Floating IPs. """
+        # Only the worker holding a valid lock within OVSDB will perform the
+        # migration.
+        if not self.has_lock:
+            return
+
+        admin_context = n_context.get_admin_context()
+        nb_sync = ovn_db_sync.OvnNbSynchronizer(
+            self._ovn_client._plugin, self._nb_idl, self._ovn_client._sb_idl,
+            None, None)
+        nb_sync.migrate_to_stateful_fips(admin_context)
+        raise periodics.NeverAgain()
+
     # The migration will run just once per neutron-server instance. If the lock
     # is held by some other neutron-server instance in the cloud, we'll attempt
     # to perform the migration every 10 seconds until completed.
@@ -327,7 +348,7 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         _log(create_update_inconsistencies, INCONSISTENCY_TYPE_CREATE_UPDATE)
         _log(delete_inconsistencies, INCONSISTENCY_TYPE_DELETE)
 
-    @periodics.periodic(spacing=DB_CONSISTENCY_CHECK_INTERVAL,
+    @periodics.periodic(spacing=ovn_const.DB_CONSISTENCY_CHECK_INTERVAL,
                         run_immediately=True)
     def check_for_inconsistencies(self):
         # Only the worker holding a valid lock within OVSDB will run
@@ -383,6 +404,9 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
                 if row.resource_type == ovn_const.TYPE_SUBNETS:
                     self._ovn_client.delete_subnet(admin_context,
                                                    row.resource_uuid)
+                elif row.resource_type == ovn_const.TYPE_PORTS:
+                    self._ovn_client.delete_port(admin_context,
+                                                 row.resource_uuid)
                 else:
                     self._fix_delete(admin_context, row)
             except Exception:
@@ -480,75 +504,6 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
         raise periodics.NeverAgain()
 
-    # A static spacing value is used here, but this method will only run
-    # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=1800, run_immediately=True)
-    def check_metadata_ports(self):
-        # If OVN metadata is disabled do not run this task again
-        if not ovn_conf.is_ovn_metadata_enabled():
-            raise periodics.NeverAgain()
-
-        # Make sure that only one worker is executing this
-        if not self.has_lock:
-            return
-
-        admin_context = n_context.get_admin_context()
-        for n in self._ovn_client._plugin.get_networks(admin_context):
-            self._ovn_client.create_metadata_port(admin_context, n)
-
-        raise periodics.NeverAgain()
-
-    # TODO(lucasagomes): Remove this in the U cycle
-    # A static spacing value is used here, but this method will only run
-    # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
-    def check_for_port_security_unknown_address(self):
-
-        if not self.has_lock:
-            return
-
-        for port in self._nb_idl.lsp_list().execute(check_error=True):
-
-            if port.type == ovn_const.LSP_TYPE_LOCALNET:
-                continue
-
-            addresses = port.addresses
-            type_ = port.type.strip()
-            if not port.port_security:
-                if not type_ and ovn_const.UNKNOWN_ADDR not in addresses:
-                    addresses.append(ovn_const.UNKNOWN_ADDR)
-                elif type_ and ovn_const.UNKNOWN_ADDR in addresses:
-                    addresses.remove(ovn_const.UNKNOWN_ADDR)
-            else:
-                if type_ and ovn_const.UNKNOWN_ADDR in addresses:
-                    addresses.remove(ovn_const.UNKNOWN_ADDR)
-                elif not type_ and ovn_const.UNKNOWN_ADDR in addresses:
-                    addresses.remove(ovn_const.UNKNOWN_ADDR)
-
-            if addresses:
-                self._nb_idl.lsp_set_addresses(
-                    port.name, addresses=addresses).execute(check_error=True)
-            else:
-                self._nb_idl.db_clear(
-                    'Logical_Switch_Port', port.name,
-                    'addresses').execute(check_error=True)
-
-        raise periodics.NeverAgain()
-
-    # A static spacing value is used here, but this method will only run
-    # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
-    def check_for_fragmentation_support(self):
-        if not self.has_lock:
-            return
-
-        context = n_context.get_admin_context()
-        for net in self._ovn_client._plugin.get_networks(
-                context, {external_net.EXTERNAL: [True]}):
-            self._ovn_client.set_gateway_mtu(context, net)
-
-        raise periodics.NeverAgain()
-
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
     @periodics.periodic(spacing=600, run_immediately=True)
@@ -560,7 +515,8 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
             value = ('true' if ovn_conf.is_igmp_snooping_enabled()
                      else 'false')
             for ls in self._nb_idl.ls_list().execute(check_error=True):
-                if ls.other_config.get(ovn_const.MCAST_SNOOP, None) == value:
+                if (ls.other_config.get(ovn_const.MCAST_SNOOP,
+                                        None) == value or not ls.name):
                     continue
                 txn.add(self._nb_idl.db_set(
                     'Logical_Switch', ls.name,
@@ -570,10 +526,18 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
         raise periodics.NeverAgain()
 
+    def _delete_default_ha_chassis_group(self, txn):
+        # TODO(lucasgomes): Remove the deletion of the
+        # HA_CHASSIS_GROUP_DEFAULT_NAME in the Y cycle. We no longer
+        # have a default HA Chassis Group.
+        cmd = [self._nb_idl.ha_chassis_group_del(
+            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, if_exists=True)]
+        self._ovn_client._transaction(cmd, txn=txn)
+
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
     @periodics.periodic(spacing=600, run_immediately=True)
-    def check_for_ha_chassis_group_address(self):
+    def check_for_ha_chassis_group(self):
         # If external ports is not supported stop running
         # this periodic task
         if not self._ovn_client.is_external_ports_supported():
@@ -582,86 +546,31 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         if not self.has_lock:
             return
 
-        default_ch_grp = self._nb_idl.ha_chassis_group_add(
-            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, may_exist=True).execute(
-            check_error=True)
-
-        # NOTE(lucasagomes): Find the existing chassis with the highest
-        # priority and keep it as being the highest to avoid moving
-        # things around
-        high_prio_ch = max(default_ch_grp.ha_chassis, key=lambda x: x.priority,
-                           default=None)
-
-        all_ch = self._sb_idl.get_all_chassis()
-        gw_ch = self._sb_idl.get_gateway_chassis_from_cms_options()
-        ch_to_del = set(all_ch) - set(gw_ch)
+        external_ports = self._nb_idl.db_find_rows(
+            'Logical_Switch_Port', ('type', '=', ovn_const.LSP_TYPE_EXTERNAL)
+        ).execute(check_error=True)
 
+        context = n_context.get_admin_context()
         with self._nb_idl.transaction(check_error=True) as txn:
-            for ch in ch_to_del:
-                txn.add(self._nb_idl.ha_chassis_group_del_chassis(
-                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, ch,
-                        if_exists=True))
+            for port in external_ports:
+                network_id = port.external_ids[
+                    ovn_const.OVN_NETWORK_NAME_EXT_ID_KEY].replace(
+                        ovn_const.OVN_NAME_PREFIX, '')
+                ha_ch_grp = self._ovn_client.sync_ha_chassis_group(
+                    context, network_id, txn)
+                try:
+                    port_ha_ch_uuid = port.ha_chassis_group[0].uuid
+                except IndexError:
+                    port_ha_ch_uuid = None
+                if port_ha_ch_uuid != ha_ch_grp:
+                    txn.add(self._nb_idl.set_lswitch_port(
+                        port.name, ha_chassis_group=ha_ch_grp))
 
-            # NOTE(lucasagomes): If the high priority chassis is in
-            # the list of chassis to be added/updated. Add it first with
-            # the highest priority number possible and then add the rest
-            # (the priority of the rest of the chassis does not matter
-            # since only the highest one is active)
-            priority = ovn_const.HA_CHASSIS_GROUP_HIGHEST_PRIORITY
-            if high_prio_ch and high_prio_ch.chassis_name in gw_ch:
-                txn.add(self._nb_idl.ha_chassis_group_add_chassis(
-                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME,
-                        high_prio_ch.chassis_name, priority=priority))
-                gw_ch.remove(high_prio_ch.chassis_name)
-                priority -= 1
-
-            for ch in gw_ch:
-                txn.add(self._nb_idl.ha_chassis_group_add_chassis(
-                        ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME,
-                        ch, priority=priority))
-                priority -= 1
+            self._delete_default_ha_chassis_group(txn)
 
         raise periodics.NeverAgain()
 
-    # A static spacing value is used here, but this method will only run
-    # once per lock due to the use of periodics.NeverAgain().
-    @periodics.periodic(spacing=600, run_immediately=True)
-    def check_for_localnet_legacy_port_name(self):
-        if not self.has_lock:
-            return
-
-        admin_context = n_context.get_admin_context()
-        cmds = []
-        for ls in self._nb_idl.ls_list().execute(check_error=True):
-            network_id = ls.name.replace('neutron-', '')
-            legacy_name = utils.ovn_provnet_port_name(network_id)
-            legacy_port = None
-            segment_id = None
-            for lsp in ls.ports:
-                if legacy_name == lsp.name:
-                    legacy_port = lsp
-                    break
-            else:
-                continue
-            for segment in segments_db.get_network_segments(
-                    admin_context, network_id):
-                if (segment.get(segment_def.PHYSICAL_NETWORK) ==
-                        legacy_port.options['network_name']):
-                    segment_id = segment['id']
-                    break
-            if not segment_id:
-                continue
-            new_p_name = utils.ovn_provnet_port_name(segment_id)
-            cmds.append(self._nb_idl.db_set('Logical_Switch_Port',
-                                            legacy_port.uuid,
-                                            ('name', new_p_name)))
-        if cmds:
-            with self._nb_idl.transaction(check_error=True) as txn:
-                for cmd in cmds:
-                    txn.add(cmd)
-        raise periodics.NeverAgain()
-
-    # TODO(lucasagomes): Remove this in the Y cycle
+    # TODO(lucasagomes): Remove this in the Z cycle
     # A static spacing value is used here, but this method will only run
     # once per lock due to the use of periodics.NeverAgain().
     @periodics.periodic(spacing=600, run_immediately=True)
@@ -672,17 +581,20 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
         cmds = []
         for port in self._nb_idl.lsp_list().execute(check_error=True):
             port_type = port.type.strip()
-            if port_type in ("vtep", "localport", "router"):
+            if port_type in ("vtep", ovn_const.LSP_TYPE_LOCALPORT, "router"):
                 continue
 
             options = port.options
-            if ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS in options:
+            if port_type == ovn_const.LSP_TYPE_LOCALNET:
+                mcast_flood_value = options.get(
+                    ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS)
+                if mcast_flood_value == 'false':
+                    continue
+                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'false'})
+            elif ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS in options:
                 continue
 
             options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD_REPORTS: 'true'})
-            if port_type == ovn_const.LSP_TYPE_LOCALNET:
-                options.update({ovn_const.LSP_OPTIONS_MCAST_FLOOD: 'false'})
-
             cmds.append(self._nb_idl.lsp_set_options(port.name, **options))
 
         if cmds:
@@ -692,6 +604,220 @@ class DBInconsistenciesPeriodics(SchemaAwarePeriodicsBase):
 
         raise periodics.NeverAgain()
 
+    # TODO(lucasagomes): Remove this in the Z cycle
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_router_mac_binding_options(self):
+        if not self.has_lock:
+            return
+
+        cmds = []
+        for router in self._nb_idl.lr_list().execute(check_error=True):
+            if (router.options.get('always_learn_from_arp_request') and
+                    router.options.get('dynamic_neigh_routers')):
+                continue
+
+            opts = copy.deepcopy(router.options)
+            opts.update({'always_learn_from_arp_request': 'false',
+                         'dynamic_neigh_routers': 'true'})
+            cmds.append(self._nb_idl.update_lrouter(router.name, options=opts))
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
+    # TODO(ralonsoh): Remove this in the Z+2 cycle
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def update_port_qos_with_external_ids_reference(self):
+        """Update all OVN QoS registers with the port ID
+
+        This method will only update the OVN QoS registers related to port QoS,
+        not FIP QoS. FIP QoS have the corresponding "external_ids" reference.
+        """
+        if not self.has_lock:
+            return
+
+        regex = re.compile(
+            r'(inport|outport) == \"(?P<port_id>[a-z0-9\-]{36})\"')
+        cmds = []
+        for ls in self._nb_idl.ls_list().execute(check_error=True):
+            for qos in self._nb_idl.qos_list(ls.name).execute(
+                    check_error=True):
+                if qos.external_ids:
+                    continue
+                match = re.match(regex, qos.match)
+                if not match:
+                    continue
+                port_id = match.group('port_id')
+                external_ids = {ovn_const.OVN_PORT_EXT_ID_KEY: port_id}
+                cmds.append(self._nb_idl.db_set(
+                    'QoS', qos.uuid, ('external_ids', external_ids)))
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_vlan_distributed_ports(self):
+        """Check VLAN distributed ports
+        Check for the option "reside-on-redirect-chassis" value for
+        distributed VLAN ports.
+        """
+        if not self.has_lock:
+            return
+        context = n_context.get_admin_context()
+        cmds = []
+        # Get router ports belonging to VLAN networks
+        vlan_nets = self._ovn_client._plugin.get_networks(
+            context, {pnet.NETWORK_TYPE: [n_const.TYPE_VLAN]})
+        vlan_net_ids = [vn['id'] for vn in vlan_nets]
+        router_ports = self._ovn_client._plugin.get_ports(
+            context, {'network_id': vlan_net_ids,
+                      'device_owner': n_const.ROUTER_PORT_OWNERS})
+        expected_value = ('false' if ovn_conf.is_ovn_distributed_floating_ip()
+                          else 'true')
+        for rp in router_ports:
+            lrp_name = utils.ovn_lrouter_port_name(rp['id'])
+            lrp = self._nb_idl.get_lrouter_port(lrp_name)
+            if lrp.options.get(
+                    ovn_const.LRP_OPTIONS_RESIDE_REDIR_CH) != expected_value:
+                opt = {ovn_const.LRP_OPTIONS_RESIDE_REDIR_CH: expected_value}
+                cmds.append(self._nb_idl.db_set(
+                    'Logical_Router_Port', lrp_name, ('options', opt)))
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
+    # TODO(ralonsoh): Remove this in the Z+3 cycle. This method adds the
+    # "external_ids:OVN_GW_NETWORK_EXT_ID_KEY" to each router that has
+    # a gateway (that means, that has "external_ids:OVN_GW_PORT_EXT_ID_KEY").
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def update_logical_router_with_gateway_network_id(self):
+        """Update all OVN logical router registers with the GW network ID"""
+        if not self.has_lock:
+            return
+
+        cmds = []
+        context = n_context.get_admin_context()
+        for lr in self._nb_idl.lr_list().execute(check_error=True):
+            gw_port = lr.external_ids.get(ovn_const.OVN_GW_PORT_EXT_ID_KEY)
+            gw_net = lr.external_ids.get(ovn_const.OVN_GW_NETWORK_EXT_ID_KEY)
+            if not gw_port or (gw_port and gw_net):
+                # This router does not have a gateway network assigned yet or
+                # it has a gateway port and its corresponding network.
+                continue
+
+            port = self._ovn_client._plugin.get_port(context, gw_port)
+            external_ids = {
+                ovn_const.OVN_GW_NETWORK_EXT_ID_KEY: port['network_id']}
+            cmds.append(self._nb_idl.db_set(
+                'Logical_Router', lr.uuid, ('external_ids', external_ids)))
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
+    # A static spacing value is used here, but this method will only run
+    # once per lock due to the use of periodics.NeverAgain().
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def check_baremetal_ports_dhcp_options(self):
+        """Update baremetal ports DHCP options
+
+        Update baremetal ports DHCP options based on the
+        "disable_ovn_dhcp_for_baremetal_ports" configuration option.
+        """
+        # If external ports is not supported stop running
+        # this periodic task
+        if not self._ovn_client.is_external_ports_supported():
+            raise periodics.NeverAgain()
+
+        if not self.has_lock:
+            return
+
+        context = n_context.get_admin_context()
+        ports = ports_obj.Port.get_ports_by_vnic_type_and_host(
+            context, portbindings.VNIC_BAREMETAL)
+        ports = self._ovn_client._plugin.get_ports(
+            context, filters={'id': [p.id for p in ports]})
+        if not ports:
+            raise periodics.NeverAgain()
+
+        with self._nb_idl.transaction(check_error=True) as txn:
+            for port in ports:
+                lsp = self._nb_idl.lsp_get(port['id']).execute(
+                    check_error=True)
+                if not lsp:
+                    continue
+
+                update_dhcp = False
+                if ovn_conf.is_ovn_dhcp_disabled_for_baremetal():
+                    if lsp.dhcpv4_options or lsp.dhcpv6_options:
+                        update_dhcp = True
+                else:
+                    if not lsp.dhcpv4_options and not lsp.dhcpv6_options:
+                        update_dhcp = True
+
+                if update_dhcp:
+                    port_info = self._ovn_client._get_port_options(port)
+                    dhcpv4_options, dhcpv6_options = (
+                        self._ovn_client.update_port_dhcp_options(
+                            port_info, txn))
+                    txn.add(self._nb_idl.set_lswitch_port(
+                                lport_name=port['id'],
+                                dhcpv4_options=dhcpv4_options,
+                                dhcpv6_options=dhcpv6_options,
+                                if_exists=False))
+
+        raise periodics.NeverAgain()
+
+    # TODO(ralonsoh): Remove this in the Z+4 cycle
+    @periodics.periodic(spacing=600, run_immediately=True)
+    def update_port_virtual_type(self):
+        """Set type=virtual to those ports with parents
+        Before LP#1973276, any virtual port with "device_owner" defined, lost
+        its type=virtual. This task restores the type for those ports updated
+        before the fix https://review.opendev.org/c/openstack/neutron/+/841711.
+        """
+        if not self.has_lock:
+            return
+
+        context = n_context.get_admin_context()
+        cmds = []
+        for lsp in self._nb_idl.lsp_list().execute(check_error=True):
+            if lsp.type != '':
+                continue
+
+            port = self._ovn_client._plugin.get_port(context, lsp.name)
+            for ip in port.get('fixed_ips', []):
+                if utils.get_virtual_port_parents(
+                        self._nb_idl, ip['ip_address'], port['network_id'],
+                        port['id']):
+                    cmds.append(self._nb_idl.db_set(
+                        'Logical_Switch_Port', lsp.uuid,
+                        ('type', ovn_const.LSP_TYPE_VIRTUAL)))
+                    break
+
+        if cmds:
+            with self._nb_idl.transaction(check_error=True) as txn:
+                for cmd in cmds:
+                    txn.add(cmd)
+        raise periodics.NeverAgain()
+
 
 class HashRingHealthCheckPeriodics(object):
 
