@@ -19,6 +19,7 @@ from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import segment as segment_def
 from neutron_lib import constants
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
@@ -30,6 +31,9 @@ from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron import manager
+from neutron.objects.port_forwarding import PortForwarding
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.extensions import qos \
+    as ovn_qos
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
 from neutron.services.segments import db as segments_db
 
@@ -85,7 +89,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
     def stop(self):
         if utils.is_ovn_l3(self.l3_plugin):
-            self.l3_plugin._ovn.ovsdb_connection.stop()
+            self.l3_plugin._nb_ovn.ovsdb_connection.stop()
             self.l3_plugin._sb_ovn.ovsdb_connection.stop()
         super(OvnNbSynchronizer, self).stop()
 
@@ -102,6 +106,9 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         self.sync_port_dns_records(ctx)
         self.sync_acls(ctx)
         self.sync_routers_and_rports(ctx)
+        self.migrate_to_stateful_fips(ctx)
+        self.sync_port_qos_policies(ctx)
+        self.sync_fip_qos_policies(ctx)
 
     def _create_port_in_ovn(self, ctx, port):
         # Remove any old ACLs for the port to avoid creating duplicate ACLs.
@@ -164,7 +171,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         neutron_sgs = {}
         neutron_pgs = set()
-        with ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_READER.using(ctx):
             for sg in self.core_plugin.get_security_groups(ctx):
                 pg_name = utils.ovn_port_group_name(sg['id'])
                 neutron_pgs.add(pg_name)
@@ -223,16 +230,14 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
     def _get_acls_from_port_groups(self):
         ovn_acls = []
-        port_groups = self.ovn_api.db_list_rows('Port_Group').execute()
-        for pg in port_groups:
+        acl_columns = (self.ovn_api._tables['ACL'].columns.keys() &
+                       set(ovn_const.ACL_EXPECTED_COLUMNS_NBDB))
+        acl_columns.discard('external_ids')
+        for pg in self.ovn_api.db_list_rows('Port_Group').execute():
             acls = getattr(pg, 'acls', [])
             for acl in acls:
-                acl_string = {}
+                acl_string = {k: getattr(acl, k) for k in acl_columns}
                 acl_string['port_group'] = pg.name
-                for acl_key in getattr(acl, "_data", {}):
-                    acl_string[acl_key] = getattr(acl, acl_key)
-                acl_string.pop('meter')
-                acl_string.pop('external_ids')
                 ovn_acls.append(acl_string)
         return ovn_acls
 
@@ -246,10 +251,24 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         LOG.debug('ACL-SYNC: started @ %s', str(datetime.now()))
 
         neutron_acls = []
-        for sgr in self.core_plugin.get_security_group_rules(ctx):
-            pg_name = utils.ovn_port_group_name(sgr['security_group_id'])
-            neutron_acls.append(acl_utils._add_sg_rule_acl_for_port_group(
-                pg_name, sgr))
+        # if allow-stateless supported, we have to fetch groups to determine if
+        # stateful is set
+        if self._ovn_client.is_allow_stateless_supported():
+            for sg in self.core_plugin.get_security_groups(ctx):
+                stateful = sg.get("stateful", True)
+                pg_name = utils.ovn_port_group_name(sg['id'])
+                for sgr in self.core_plugin.get_security_group_rules(
+                        ctx, {'security_group_id': sg['id']}):
+                    neutron_acls.append(
+                        acl_utils._add_sg_rule_acl_for_port_group(
+                            pg_name, stateful, sgr)
+                    )
+        else:
+            # TODO(ihrachys) remove when min OVN version >= 21.06
+            for sgr in self.core_plugin.get_security_group_rules(ctx):
+                pg_name = utils.ovn_port_group_name(sgr['security_group_id'])
+                neutron_acls.append(acl_utils._add_sg_rule_acl_for_port_group(
+                    pg_name, True, sgr))
         neutron_acls += acl_utils.add_acls_for_drop_port_group(
             ovn_const.OVN_DROP_PORT_GROUP_NAME)
 
@@ -284,7 +303,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 for acla in neutron_acls:
                     LOG.warning('ACL found in Neutron but not in '
                                 'OVN DB for port group %s', acla['port_group'])
-                    txn.add(self.ovn_api.pg_acl_add(**acla))
+                    txn.add(self.ovn_api.pg_acl_add(**acla, may_exist=True))
 
             with self.ovn_api.transaction(check_error=True) as txn:
                 for aclr in ovn_acls:
@@ -305,6 +324,27 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         LOG.debug('ACL-SYNC: finished @ %s', str(datetime.now()))
 
+    def _calculate_routes_differences(self, ovn_routes, db_routes):
+        to_add = []
+        to_remove = []
+        for db_route in db_routes:
+            for ovn_route in ovn_routes:
+                if (ovn_route['destination'] == db_route['destination'] and
+                   ovn_route['nexthop'] == db_route['nexthop']):
+                    break
+            else:
+                to_add.append(db_route)
+
+        for ovn_route in ovn_routes:
+            for db_route in db_routes:
+                if (ovn_route['destination'] == db_route['destination'] and
+                   ovn_route['nexthop'] == db_route['nexthop']):
+                    break
+            else:
+                to_remove.append(ovn_route)
+
+        return to_add, to_remove
+
     def _calculate_fips_differences(self, ovn_fips, ovn_rtr_lb_pfs, db_fips):
         to_add = []
         to_remove = []
@@ -330,6 +370,11 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         return to_add, to_remove
 
+    def _unroll_port_forwarding(self, db_pf):
+        pf = PortForwarding(**db_pf)
+        pfs = pf.unroll_port_ranges()
+        return [p.to_dict() for p in pfs]
+
     def _calculate_fip_pfs_differences(self, ovn_rtr_lb_pfs, db_pfs):
         to_add_or_update = set()
         to_remove = []
@@ -339,20 +384,21 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         # a set for each protocol and then comparing it with ovn_pfs
         db_mapped_pfs = {}
         for db_pf in db_pfs:
-            fip_id = db_pf.get('floatingip_id')
-            protocol = self.l3_plugin.port_forwarding.ovn_lb_protocol(
-                db_pf.get('protocol'))
-            db_vip = "{}:{} {}:{}".format(
-                db_pf.get('floating_ip_address'), db_pf.get('external_port'),
-                db_pf.get('internal_ip_address'), db_pf.get('internal_port'))
+            for pf in self._unroll_port_forwarding(db_pf):
+                fip_id = pf.get('floatingip_id')
+                protocol = self.l3_plugin.port_forwarding.ovn_lb_protocol(
+                    pf.get('protocol'))
+                db_vip = "{}:{} {}:{}".format(
+                    pf.get('floating_ip_address'), pf.get('external_port'),
+                    pf.get('internal_ip_address'), pf.get('internal_port'))
 
-            fip_dict = db_mapped_pfs.get(fip_id, {})
-            fip_dict_proto = fip_dict.get(protocol, set())
-            fip_dict_proto.add(db_vip)
-            if protocol not in fip_dict:
-                fip_dict[protocol] = fip_dict_proto
-            if fip_id not in db_mapped_pfs:
-                db_mapped_pfs[fip_id] = fip_dict
+                fip_dict = db_mapped_pfs.get(fip_id, {})
+                fip_dict_proto = fip_dict.get(protocol, set())
+                fip_dict_proto.add(db_vip)
+                if protocol not in fip_dict:
+                    fip_dict[protocol] = fip_dict_proto
+                if fip_id not in db_mapped_pfs:
+                    db_mapped_pfs[fip_id] = fip_dict
         for fip_id in db_mapped_pfs:
             ovn_pfs_fip_id = ovn_pfs.get(fip_id, {})
             # check for cases when ovn has lbs for protocols that are not in
@@ -430,7 +476,11 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 if gw_info.gateway_ip:
                     db_extends[router['id']]['routes'].append(
                         {'destination': prefix,
-                         'nexthop': gw_info.gateway_ip})
+                         'nexthop': gw_info.gateway_ip,
+                         'external_ids': {
+                             ovn_const.OVN_ROUTER_IS_EXT_GW: 'true',
+                             ovn_const.OVN_SUBNET_EXT_ID_KEY:
+                             gw_info.subnet_id}})
                 if gw_info.ip_version == constants.IP_VERSION_6:
                     continue
                 if gw_info.router_ip and utils.is_snat_enabled(router):
@@ -494,7 +544,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     db_routes.extend(db_extends[lrouter['name']]['routes'])
 
                 ovn_routes = lrouter['static_routes']
-                add_routes, del_routes = helpers.diff_list_of_dict(
+                add_routes, del_routes = self._calculate_routes_differences(
                     ovn_routes, db_routes)
                 update_sroutes_list.append({'id': lrouter['name'],
                                             'add': add_routes,
@@ -622,10 +672,15 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                         LOG.warning("Add static routes %s to OVN NB DB",
                                     sroute['add'])
                         for route in sroute['add']:
+                            columns = {}
+                            if 'external_ids' in route:
+                                columns['external_ids'] = route['external_ids']
                             txn.add(self.ovn_api.add_static_route(
                                 utils.ovn_name(sroute['id']),
                                 ip_prefix=route['destination'],
-                                nexthop=route['nexthop']))
+                                nexthop=route['nexthop'],
+                                **columns))
+
                 if sroute['del']:
                     LOG.warning("Router %(id)s static routes %(route)s "
                                 "found in OVN but not in Neutron",
@@ -856,27 +911,19 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
         if not ovn_conf.is_ovn_metadata_enabled():
             return
         LOG.debug('OVN sync metadata ports started')
-        # TODO(mjozefcz): Remove constants.DEVICE_OWNER_DHCP
-        # from get_ports in W-release.
         for net in self.core_plugin.get_networks(ctx):
-            # Get only DHCP ports that don't belong to agent, it should return
-            # only OVN metadata ports
-            dhcp_ports = [
-                p for p in self.core_plugin.get_ports(
+            metadata_ports = self.core_plugin.get_ports(
                     ctx, filters=dict(
                         network_id=[net['id']],
-                        device_owner=[
-                            constants.DEVICE_OWNER_DISTRIBUTED,
-                            constants.DEVICE_OWNER_DHCP]))
-                if not utils.is_neutron_dhcp_agent_port(p)]
+                        device_owner=[constants.DEVICE_OWNER_DISTRIBUTED]))
 
-            if not dhcp_ports:
+            if not metadata_ports:
                 LOG.warning('Missing metadata port found in Neutron for '
                             'network %s', net['id'])
                 if self.mode == SYNC_MODE_REPAIR:
                     try:
                         # Create the missing port in both Neutron and OVN.
-                        LOG.warning('Creating missing metadadata port in '
+                        LOG.warning('Creating missing metadata port in '
                                     'Neutron and OVN for network %s',
                                     net['id'])
                         self._ovn_client.create_metadata_port(ctx, net)
@@ -887,7 +934,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
             else:
                 # Delete all but one DHCP ports. Only one is needed for
                 # metadata.
-                for port in dhcp_ports[1:]:
+                for port in metadata_ports[1:]:
                     LOG.warning('Unnecessary DHCP port %s for network %s '
                                 'found in Neutron', port['id'], net['id'])
                     if self.mode == SYNC_MODE_REPAIR:
@@ -895,7 +942,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                                     'network %s', port['id'], net['id'])
                         self.core_plugin.delete_port(ctx, port['id'])
                     db_ports.pop(port['id'], None)
-                port = dhcp_ports[0]
+                port = metadata_ports[0]
                 if port['id'] in db_ports.keys():
                     LOG.warning('Metadata port %s for network %s found in '
                                 'Neutron but not in OVN',
@@ -908,8 +955,13 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                     db_ports.pop(port['id'])
 
             if self.mode == SYNC_MODE_REPAIR:
-                # Make sure that this port has an IP address in all the subnets
-                self._ovn_client.update_metadata_port(ctx, net['id'])
+                try:
+                    # Make sure that this port has an IP address in all the
+                    # subnets
+                    self._ovn_client.update_metadata_port(ctx, net['id'])
+                except n_exc.IpAddressGenerationFailure:
+                    LOG.error('Could not allocate IP addresses for '
+                              'metadata port in network %s', net['id'])
         LOG.debug('OVN sync metadata ports finished')
 
     def sync_networks_ports_and_dhcp_opts(self, ctx):
@@ -956,7 +1008,7 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                             {'network': db_network,
                              'segment': db_segment,
                              'lswitch': lswitch['name']})
-                # Delete orhpaned provnet ports
+                # Delete orphaned provnet ports
                 for provnet_port in lswitch['provnet_ports']:
                     if provnet_port in segments_provnet_port_names:
                         continue
@@ -1135,12 +1187,22 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
                 ext_ids = {ovn_const.OVN_SG_EXT_ID_KEY: sg['id']}
                 txn.add(self.ovn_api.pg_add(
                     name=pg_name, acls=[], external_ids=ext_ids))
-                acl_utils.add_acls_for_sg_port_group(self.ovn_api, sg, txn)
+                acl_utils.add_acls_for_sg_port_group(
+                    self.ovn_api, sg, txn,
+                    self._ovn_client.is_allow_stateless_supported())
             for port in db_ports:
                 for sg in port['security_groups']:
                     txn.add(self.ovn_api.pg_add_ports(
                         utils.ovn_port_group_name(sg), port['id']))
 
+    def migrate_to_stateful_fips(self, ctx):
+        # This routine will clear options:stateless=true for all dnat_and_snats
+        # that belong to neutron fips. Since we don't set any other options,
+        # just clear the whole column.
+        with self.ovn_api.transaction(check_error=True) as txn:
+            for nat in self.ovn_api.get_all_stateless_fip_nats():
+                txn.add(self.ovn_api.db_clear('NAT', nat['_uuid'], 'options'))
+
     def migrate_to_port_groups(self, ctx):
         # This routine is responsible for migrating the current Security
         # Groups and SG Rules to the new Port Groups implementation.
@@ -1170,6 +1232,36 @@ class OvnNbSynchronizer(OvnDbSynchronizer):
 
         LOG.debug('Port Groups Migration task finished')
 
+    def sync_port_qos_policies(self, ctx):
+        """Sync port QoS policies.
+
+        This method reads the port QoS policy assigned or the one inherited
+        from the network. Does not apply to "network" owned ports.
+        """
+        LOG.debug('Port QoS policies migration task started')
+        ovn_qos_ext = ovn_qos.OVNClientQosExtension(nb_idl=self.ovn_api)
+        with db_api.CONTEXT_READER.using(ctx), \
+                self.ovn_api.transaction(check_error=True) as txn:
+            for port in self.core_plugin.get_ports(ctx):
+                if not ovn_qos_ext.port_effective_qos_policy_id(port)[0]:
+                    continue
+                ovn_qos_ext.create_port(txn, port, None)
+
+        LOG.debug('Port QoS policies migration task finished')
+
+    def sync_fip_qos_policies(self, ctx):
+        """Sync floating IP QoS policies."""
+        LOG.debug('Floating IP QoS policies migration task started')
+        ovn_qos_ext = ovn_qos.OVNClientQosExtension(nb_idl=self.ovn_api)
+        with db_api.CONTEXT_READER.using(ctx), \
+                self.ovn_api.transaction(check_error=True) as txn:
+            for fip in self.l3_plugin.get_floatingips(ctx):
+                if not fip.get('qos_policy_id'):
+                    continue
+                ovn_qos_ext.create_floatingip(txn, fip)
+
+        LOG.debug('Floating IP QoS policies migration task finished')
+
 
 class OvnSbSynchronizer(OvnDbSynchronizer):
     """Synchronizer class for SB."""
@@ -1192,6 +1284,9 @@ class OvnSbSynchronizer(OvnDbSynchronizer):
         self.sync_hostname_and_physical_networks(ctx)
         if utils.is_ovn_l3(self.l3_plugin):
             self.l3_plugin.schedule_unhosted_gateways()
+        # NOTE(ralonsoh): this could be called using a resource event.
+        self.ovn_driver._ovn_client.placement_extension.\
+            read_initial_chassis_config()
 
     def sync_hostname_and_physical_networks(self, ctx):
         LOG.debug('OVN-SB Sync hostname and physical networks started')
