@@ -162,9 +162,9 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         net_segments = segments_db.get_networks_segments(context, ids)
         for network in networks:
             segments = net_segments[network['id']]
-            self._extend_network_dict_provider(network, segments)
+            self.extend_network_with_provider_segments(network, segments)
 
-    def _extend_network_dict_provider(self, network, segments):
+    def extend_network_with_provider_segments(self, network, segments):
         if not segments:
             LOG.debug("Network %s has no segments", network['id'])
             for attr in provider.ATTRIBUTES:
@@ -183,6 +183,22 @@ class TypeManager(stevedore.named.NamedExtensionManager):
             network[provider.SEGMENTATION_ID] = segment[
                 api.SEGMENTATION_ID]
 
+    @staticmethod
+    def pop_segments_from_network(network):
+        multiple_segments = network.pop(mpnet_apidef.SEGMENTS, [])
+        if multiple_segments:
+            network_segments = multiple_segments
+        else:
+            network_segments = [
+                {provider_key: network.pop(provider_key)
+                for provider_key in provider.ATTRIBUTES}]
+
+        return (
+            [{api.NETWORK_TYPE: network_segment[provider.NETWORK_TYPE],
+             api.PHYSICAL_NETWORK: network_segment[provider.PHYSICAL_NETWORK],
+             api.SEGMENTATION_ID: network_segment[provider.SEGMENTATION_ID]}
+             for network_segment in network_segments])
+
     def initialize(self):
         for network_type, driver in self.drivers.items():
             LOG.info("Initializing driver for type '%s'", network_type)
@@ -269,8 +285,7 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         self.validate_provider_segment(segment)
 
         # Reserve segment in type driver
-        with db_api.CONTEXT_WRITER.using(context):
-            return self.reserve_provider_segment(context, segment)
+        return self.reserve_provider_segment(context, segment)
 
     def is_partial_segment(self, segment):
         network_type = segment[api.NETWORK_TYPE]
@@ -340,6 +355,7 @@ class TypeManager(stevedore.named.NamedExtensionManager):
             LOG.error("Failed to release segment '%s' because "
                       "network type is not supported.", segment)
 
+    @db_api.retry_if_session_inactive()
     def allocate_dynamic_segment(self, context, network_id, segment):
         """Allocate a dynamic segment using a partial or full segment dict."""
         dynamic_segment = segments_db.get_dynamic_segment(
@@ -349,31 +365,35 @@ class TypeManager(stevedore.named.NamedExtensionManager):
         if dynamic_segment:
             return dynamic_segment
 
-        driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
-        if isinstance(driver.obj, api.TypeDriver):
-            dynamic_segment = driver.obj.reserve_provider_segment(
-                context.session, segment)
-        else:
-            dynamic_segment = driver.obj.reserve_provider_segment(
-                context, segment)
-        segments_db.add_network_segment(context, network_id, dynamic_segment,
-                                        is_dynamic=True)
+        with db_api.CONTEXT_WRITER.using(context):
+            driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
+            if isinstance(driver.obj, api.TypeDriver):
+                dynamic_segment = driver.obj.reserve_provider_segment(
+                    context.session, segment)
+            else:
+                dynamic_segment = driver.obj.reserve_provider_segment(
+                    context, segment)
+            segments_db.add_network_segment(context, network_id,
+                                            dynamic_segment,
+                                            is_dynamic=True)
         return dynamic_segment
 
+    @db_api.retry_if_session_inactive()
     def release_dynamic_segment(self, context, segment_id):
         """Delete a dynamic segment."""
         segment = segments_db.get_segment_by_id(context, segment_id)
         if segment:
-            driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
-            if driver:
-                if isinstance(driver.obj, api.TypeDriver):
-                    driver.obj.release_segment(context.session, segment)
+            with db_api.CONTEXT_WRITER.using(context):
+                driver = self.drivers.get(segment.get(api.NETWORK_TYPE))
+                if driver:
+                    if isinstance(driver.obj, api.TypeDriver):
+                        driver.obj.release_segment(context.session, segment)
+                    else:
+                        driver.obj.release_segment(context, segment)
+                    segments_db.delete_network_segment(context, segment_id)
                 else:
-                    driver.obj.release_segment(context, segment)
-                segments_db.delete_network_segment(context, segment_id)
-            else:
-                LOG.error("Failed to release segment '%s' because "
-                          "network type is not supported.", segment)
+                    LOG.error("Failed to release segment '%s' because "
+                              "network type is not supported.", segment)
         else:
             LOG.debug("No segment found with id %(segment_id)s", segment_id)
 
@@ -907,15 +927,14 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
                         # traits of the allocated resource provider on
                         # the top binding_level (==0).
                         if (context.current['binding:profile'] is not None and
-                                'allocation' in context.current[
-                                    'binding:profile'] and
+                                context.current[
+                                    'binding:profile'].get('allocation') and
                                 not redoing_bottom):
                             LOG.debug(
                                 "Undo bottom bound level and redo it "
-                                "according to binding_profile.allocation, "
-                                "resource provider uuid: %s",
-                                context.current[
-                                    'binding:profile']['allocation'])
+                                "according to binding_profile.allocation: %s",
+                                context.current['binding:profile'][
+                                    'allocation'])
                             context._pop_binding_level()
                             context._unset_binding()
                             return self._bind_port_level(
@@ -961,21 +980,22 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
             if driver.obj.responsible_for_ports_allocation(context):
                 drivers.append(driver)
 
+        allocation = context.current['binding:profile']['allocation']
+
         if len(drivers) == 0:
             LOG.error("Failed to bind port %(port)s on host "
-                      "%(host)s allocated on resource provider "
-                      "%(rsc_provider)s, because no mechanism driver "
+                      "%(host)s allocated on resource providers: "
+                      "%(rsc_providers)s, because no mechanism driver "
                       "reports being responsible",
                       {'port': context.current['id'],
                        'host': context.host,
-                       'rsc_provider': context.current[
-                           'binding:profile']['allocation']})
+                       'rsc_providers': ','.join(allocation.values())})
             raise place_exc.UnknownResourceProvider(
-                rsc_provider=context.current['binding:profile']['allocation'])
+                rsc_provider=','.join(allocation.values()))
 
         if len(drivers) >= 2:
             raise place_exc.AmbiguousResponsibilityForResourceProvider(
-                rsc_provider=context.current['binding:profile']['allocation'],
+                rsc_provider=','.join(allocation.values()),
                 drivers=','.join([driver.name for driver in drivers]))
 
         # NOTE(bence romsics): The error conditions for raising either
@@ -1050,9 +1070,7 @@ class MechanismManager(stevedore.named.NamedExtensionManager):
             return drivers
 
         return [d for d in drivers if
-                getattr(d.obj, 'vif_details', {}).get(
-                    portbindings.VIF_DETAILS_CONNECTIVITY) ==
-                portbindings.CONNECTIVITY_L2]
+                d.obj.connectivity == portbindings.CONNECTIVITY_L2]
 
     def get_workers(self):
         workers = []
