@@ -13,7 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import collections
+from collections import abc
 import itertools
 import re
 import sys
@@ -28,6 +28,7 @@ from neutron_lib.services import constants as service_const
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
+from oslo_policy import opts
 from oslo_policy import policy
 from oslo_utils import excutils
 import stevedore
@@ -51,6 +52,13 @@ _RESOURCE_FOREIGN_KEYS = {
 }
 
 
+# TODO(gmann): Remove setting the default value of config policy_file
+# once oslo_policy change the default value to 'policy.yaml'.
+# https://github.com/openstack/oslo.policy/blob/a626ad12fe5a3abd49d70e3e5b95589d279ab578/oslo_policy/opts.py#L49
+DEFAULT_POLICY_FILE = 'policy.yaml'
+opts.set_defaults(cfg.CONF, DEFAULT_POLICY_FILE)
+
+
 def reset():
     global _ENFORCER
     if _ENFORCER:
@@ -68,12 +76,21 @@ def register_rules(enforcer):
     enforcer.register_defaults(itertools.chain(*policies))
 
 
-def init(conf=cfg.CONF, policy_file=None):
+def init(conf=cfg.CONF, policy_file=None, suppress_deprecation_warnings=False):
     """Init an instance of the Enforcer class."""
 
     global _ENFORCER
     if not _ENFORCER:
         _ENFORCER = policy.Enforcer(conf, policy_file=policy_file)
+        # TODO(slaweq) Explictly disable the warnings for policies
+        # changing their default check_str. During policy-defaults-refresh
+        # work, all the policy defaults have been changed and warning for
+        # each policy started filling the logs limit for various tool.
+        # Once we move to new defaults only world then we can enable these
+        # warning again.
+        _ENFORCER.suppress_default_change_warnings = True
+        if suppress_deprecation_warnings:
+            _ENFORCER.suppress_deprecation_warnings = True
         register_rules(_ENFORCER)
         _ENFORCER.load_rules(True)
 
@@ -125,7 +142,7 @@ def _is_attribute_explicitly_set(attribute_name, resource, target, action):
 def _should_validate_sub_attributes(attribute, sub_attr):
     """Verify that sub-attributes are iterable and should be validated."""
     validate = attribute.get('validate')
-    return (validate and isinstance(sub_attr, collections.Iterable) and
+    return (validate and isinstance(sub_attr, abc.Iterable) and
             any([k.startswith('type:dict') and
                  v for (k, v) in validate.items()]))
 
@@ -189,6 +206,10 @@ def _build_match_rule(action, target, pluralized):
        (e.g.: create_router:external_gateway_info:network_id)
     """
     match_rule = policy.RuleCheck('rule', action)
+    registered_rule = _ENFORCER.registered_rules.get(action)
+    if registered_rule and registered_rule.scope_types:
+        match_rule.scope_types = registered_rule.scope_types
+
     resource, enforce_attr_based_check = get_resource_and_action(
         action, pluralized)
     if enforce_attr_based_check:
@@ -226,7 +247,7 @@ def _build_match_rule(action, target, pluralized):
 # This check is registered as 'tenant_id' so that it can override
 # GenericCheck which was used for validating parent resource ownership.
 # This will prevent us from having to handling backward compatibility
-# for policy.json
+# for policy.yaml
 # TODO(salv-orlando): Reinstate GenericCheck for simple tenant_id checks
 @policy.register('tenant_id')
 class OwnerCheck(policy.Check):
@@ -239,6 +260,9 @@ class OwnerCheck(policy.Check):
     resource and perform the check.
     """
     def __init__(self, kind, match):
+        self._orig_kind = kind
+        self._orig_match = match
+
         # Process the match
         try:
             self.target_field = re.findall(r'^\%\((.*)\)s$',
@@ -254,6 +278,14 @@ class OwnerCheck(policy.Check):
         self._cache = cache._get_memory_cache_region(expiration_time=5)
         super(OwnerCheck, self).__init__(kind, match)
 
+    # NOTE(slaweq): It seems we need to have it like that, otherwise we hit
+    # TypeError: cannot pickle '_thread.RLock' object
+    # during initialization of the policy rules when Neutron is run with
+    # mod_uwsgi, see bug https://bugs.launchpad.net/neutron/+bug/1915494 for
+    # details
+    def __deepcopy__(self, memo):
+        return OwnerCheck(self._orig_kind, self._orig_match)
+
     @cache.cache_method_results
     def _extract(self, resource_type, resource_id, field):
         # NOTE(salv-orlando): This check currently assumes the parent
@@ -340,6 +372,9 @@ class OwnerCheck(policy.Check):
 @policy.register('field')
 class FieldCheck(policy.Check):
     def __init__(self, kind, match):
+        self._orig_kind = kind
+        self._orig_match = match
+
         # Process the match
         resource, field_value = match.split(':', 1)
         field, value = field_value.split('=', 1)
@@ -359,6 +394,11 @@ class FieldCheck(policy.Check):
         self.value = conv_func(value)
         self.regex = re.compile(value[1:]) if value.startswith('~') else None
 
+    # TODO(stephenfin): Remove this when we drop support for Python 3.6, since
+    # that supports copying regex objects natively
+    def __deepcopy__(self, memo):
+        return FieldCheck(self._orig_kind, self._orig_match)
+
     def __call__(self, target_dict, cred_dict, enforcer):
         target_value = self._get_target_value(target_dict)
         # target_value might be a boolean, explicitly compare with None
@@ -396,13 +436,12 @@ class FieldCheck(policy.Check):
 
 
 def _prepare_check(context, action, target, pluralized):
-    """Prepare rule, target, and credentials for the policy engine."""
+    """Prepare rule, target, and context for the policy engine."""
     # Compare with None to distinguish case in which target is {}
     if target is None:
         target = {}
     match_rule = _build_match_rule(action, target, pluralized)
-    credentials = context.to_policy_values()
-    return match_rule, target, credentials
+    return match_rule, target, context
 
 
 def log_rule_list(match_rule):
@@ -433,7 +472,10 @@ def check(context, action, target, plugin=None, might_not_exist=False,
     """
     # If we already know the context has admin rights do not perform an
     # additional check and authorize the operation
-    if context.is_admin:
+    # TODO(slaweq): Remove that is_admin check and always perform rules checks
+    # when old, deprecated rules will be removed and only rules with new
+    # personas will be supported
+    if not cfg.CONF.oslo_policy.enforce_new_defaults and context.is_admin:
         return True
     if might_not_exist and not (_ENFORCER.rules and action in _ENFORCER.rules):
         return True
@@ -441,11 +483,10 @@ def check(context, action, target, plugin=None, might_not_exist=False,
                                                      action,
                                                      target,
                                                      pluralized)
-    result = _ENFORCER.enforce(match_rule,
-                               target,
-                               credentials,
-                               pluralized=pluralized)
-    return result
+    return _ENFORCER.enforce(match_rule,
+                             target,
+                             credentials,
+                             pluralized=pluralized)
 
 
 def enforce(context, action, target, plugin=None, pluralized=None):
@@ -467,19 +508,19 @@ def enforce(context, action, target, plugin=None, pluralized=None):
     """
     # If we already know the context has admin rights do not perform an
     # additional check and authorize the operation
-    if context.is_admin:
+    # TODO(slaweq): Remove that is_admin check and always perform rules checks
+    # when old, deprecated rules will be removed and only rules with new
+    # personas will be supported
+    if not cfg.CONF.oslo_policy.enforce_new_defaults and context.is_admin:
         return True
-    rule, target, credentials = _prepare_check(context,
-                                               action,
-                                               target,
-                                               pluralized)
+    rule, target, context = _prepare_check(context, action, target, pluralized)
     try:
-        result = _ENFORCER.enforce(rule, target, credentials, action=action,
+        result = _ENFORCER.enforce(rule, target, context, action=action,
                                    do_raise=True)
-    except policy.PolicyNotAuthorized:
+    except (policy.PolicyNotAuthorized, policy.InvalidScope):
         with excutils.save_and_reraise_exception():
             log_rule_list(rule)
-            LOG.debug("Failed policy check for '%s'", action)
+            LOG.debug("Failed policy enforce for '%s'", action)
     return result
 
 
