@@ -17,13 +17,14 @@ import socket
 from neutron_lib import constants
 from oslo_log import log as logging
 import pyroute2
-from pyroute2 import netlink
-from pyroute2.netlink import exceptions as netlink_exceptions
-from pyroute2.netlink import rtnl
-from pyroute2.netlink.rtnl import ifinfmsg
-from pyroute2.netlink.rtnl import ndmsg
-from pyroute2 import NetlinkError
-from pyroute2 import netns
+from pyroute2 import netlink  # pylint: disable=no-name-in-module
+from pyroute2.netlink import exceptions \
+    as netlink_exceptions  # pylint: disable=no-name-in-module
+from pyroute2.netlink import rtnl  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import ifinfmsg  # pylint: disable=no-name-in-module
+from pyroute2.netlink.rtnl import ndmsg  # pylint: disable=no-name-in-module
+from pyroute2 import netns  # pylint: disable=no-name-in-module
+import tenacity
 
 from neutron._i18n import _
 from neutron import privileged
@@ -36,8 +37,10 @@ _IP_VERSION_FAMILY_MAP = {4: socket.AF_INET, 6: socket.AF_INET6}
 
 NETNS_RUN_DIR = '/var/run/netns'
 
+NUD_STATES = {state[1]: state[0] for state in ndmsg.states.items()}
 
-def _get_scope_name(scope):
+
+def get_scope_name(scope):
     """Return the name of the scope (given as a number), or the scope number
     if the name is unknown.
 
@@ -136,54 +139,6 @@ def _make_route_dict(destination, nexthop, device, scope):
             'scope': scope}
 
 
-@privileged.default.entrypoint
-def get_routing_table(ip_version, namespace=None):
-    """Return a list of dictionaries, each representing a route.
-
-    :param ip_version: IP version of routes to return, for example 4
-    :param namespace: The name of the namespace from which to get the routes
-    :return: a list of dictionaries, each representing a route.
-    The dictionary format is: {'destination': cidr,
-                               'nexthop': ip,
-                               'device': device_name,
-                               'scope': scope}
-    """
-    family = _IP_VERSION_FAMILY_MAP[ip_version]
-    try:
-        netns = pyroute2.NetNS(namespace, flags=0) if namespace else None
-    except OSError as e:
-        if e.errno == errno.ENOENT:
-            raise NetworkNamespaceNotFound(netns_name=namespace)
-        raise
-    routes = []
-    with pyroute2.IPDB(nl=netns) as ipdb:
-        ipdb_routes = ipdb.routes
-        ipdb_interfaces = ipdb.interfaces
-        for route in ipdb_routes:
-            if route['family'] != family:
-                continue
-            dst = route['dst']
-            nexthop = route.get('gateway')
-            oif = route.get('oif')
-            scope = _get_scope_name(route['scope'])
-
-            # If there is not a valid outgoing interface id, check if
-            # this is a multipath route (i.e. same destination with
-            # multiple outgoing interfaces)
-            if oif:
-                device = ipdb_interfaces[oif]['ifname']
-                rt = _make_route_dict(dst, nexthop, device, scope)
-                routes.append(rt)
-            elif route.get('multipath'):
-                for mpr in route['multipath']:
-                    oif = mpr['oif']
-                    device = ipdb_interfaces[oif]['ifname']
-                    rt = _make_route_dict(dst, nexthop, device, scope)
-                    routes.append(rt)
-
-    return routes
-
-
 def get_iproute(namespace):
     # From iproute.py:
     # `IPRoute` -- RTNL API to the current network namespace
@@ -241,12 +196,7 @@ def _translate_ip_device_exception(e, device=None, namespace=None):
 
 def get_link_id(device, namespace, raise_exception=True):
     with get_iproute(namespace) as ip:
-        # TODO(ralonsoh): remove try block when the mininimum pyroute2 version
-        # is >= 0.5.13
-        try:
-            link_id = ip.link_lookup(ifname=device)
-        except NetlinkError:
-            link_id = None
+        link_id = ip.link_lookup(ifname=device)
     if not link_id or len(link_id) < 1:
         if raise_exception:
             raise NetworkInterfaceNotFound(device=device, namespace=namespace)
@@ -261,7 +211,7 @@ def _run_iproute_link(command, device, namespace=None, **kwargs):
         with get_iproute(namespace) as ip:
             idx = get_link_id(device, namespace)
             return ip.link(command, index=idx, **kwargs)
-    except NetlinkError as e:
+    except netlink.NetlinkError as e:
         _translate_ip_device_exception(e, device, namespace)
         raise
     except OSError as e:
@@ -275,7 +225,7 @@ def _run_iproute_neigh(command, device, namespace, **kwargs):
         with get_iproute(namespace) as ip:
             idx = get_link_id(device, namespace)
             return ip.neigh(command, ifindex=idx, **kwargs)
-    except NetlinkError as e:
+    except netlink_exceptions.NetlinkError as e:
         _translate_ip_device_exception(e, device, namespace)
         raise
     except OSError as e:
@@ -289,7 +239,7 @@ def _run_iproute_addr(command, device, namespace, **kwargs):
         with get_iproute(namespace) as ip:
             idx = get_link_id(device, namespace)
             return ip.addr(command, index=idx, **kwargs)
-    except NetlinkError as e:
+    except netlink_exceptions.NetlinkError as e:
         _translate_ip_device_exception(e, device, namespace)
         raise
     except OSError as e:
@@ -298,6 +248,11 @@ def _run_iproute_addr(command, device, namespace, **kwargs):
         raise
 
 
+@privileged.default.entrypoint
+def privileged_get_link_id(device, namespace, raise_exception=True):
+    return get_link_id(device, namespace, raise_exception=raise_exception)
+
+
 @privileged.default.entrypoint
 def add_ip_address(ip_version, ip, prefixlen, device, namespace, scope,
                    broadcast=None):
@@ -310,8 +265,8 @@ def add_ip_address(ip_version, ip, prefixlen, device, namespace, scope,
                           mask=prefixlen,
                           family=family,
                           broadcast=broadcast,
-                          scope=_get_scope_name(scope))
-    except NetlinkError as e:
+                          scope=get_scope_name(scope))
+    except netlink_exceptions.NetlinkError as e:
         if e.code == errno.EEXIST:
             raise IpAddressAlreadyExists(ip=ip, device=device)
         raise
@@ -327,7 +282,7 @@ def delete_ip_address(ip_version, ip, prefixlen, device, namespace):
                           address=ip,
                           mask=prefixlen,
                           family=family)
-    except NetlinkError as e:
+    except netlink_exceptions.NetlinkError as e:
         # when trying to delete a non-existent IP address, pyroute2 raises
         # NetlinkError with code EADDRNOTAVAIL (99, 'Cannot assign requested
         # address')
@@ -359,8 +314,8 @@ def create_interface(ifname, namespace, kind, **kwargs):
             if physical_interface:
                 link_key = "vxlan_link" if kind == "vxlan" else "link"
                 kwargs[link_key] = get_link_id(physical_interface, namespace)
-            return ip.link("add", ifname=ifname, kind=kind, **kwargs)
-    except NetlinkError as e:
+            ip.link("add", ifname=ifname, kind=kind, **kwargs)
+    except netlink_exceptions.NetlinkError as e:
         if e.code == errno.EEXIST:
             raise InterfaceAlreadyExists(device=ifname)
         raise
@@ -386,43 +341,48 @@ def interface_exists(ifname, namespace):
         raise
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_flags(device, namespace, flags):
     link = _run_iproute_link("get", device, namespace)[0]
     new_flags = flags | link['flags']
-    return _run_iproute_link("set", device, namespace, flags=new_flags)
+    _run_iproute_link("set", device, namespace, flags=new_flags)
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_attribute(device, namespace, **attributes):
-    return _run_iproute_link("set", device, namespace, **attributes)
+    _run_iproute_link("set", device, namespace, **attributes)
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_vf_feature(device, namespace, vf_config):
-    return _run_iproute_link("set", device, namespace=namespace, vf=vf_config)
+    _run_iproute_link("set", device, namespace=namespace, vf=vf_config)
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_bridge_forward_delay(device, forward_delay, namespace=None):
-    return _run_iproute_link('set', device, namespace=namespace,
-                             kind='bridge', br_forward_delay=forward_delay)
+    _run_iproute_link('set', device, namespace=namespace, kind='bridge',
+                      br_forward_delay=forward_delay)
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_bridge_stp(device, stp, namespace=None):
-    return _run_iproute_link('set', device, namespace=namespace,
-                             kind='bridge', br_stp_state=stp)
+    _run_iproute_link('set', device, namespace=namespace, kind='bridge',
+                      br_stp_state=stp)
 
 
-@privileged.default.entrypoint
+@privileged.link_cmd.entrypoint
 def set_link_bridge_master(device, bridge, namespace=None):
     bridge_idx = get_link_id(bridge, namespace) if bridge else 0
-    return _run_iproute_link('set', device, namespace=namespace,
-                             master=bridge_idx)
+    _run_iproute_link('set', device, namespace=namespace, master=bridge_idx)
 
 
-@privileged.default.entrypoint
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
+@privileged.link_cmd.entrypoint
 def get_link_attributes(device, namespace):
     link = _run_iproute_link("get", device, namespace)[0]
     return {
@@ -438,7 +398,13 @@ def get_link_attributes(device, namespace):
     }
 
 
-@privileged.default.entrypoint
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
+@privileged.link_cmd.entrypoint
 def get_link_vfs(device, namespace):
     link = _run_iproute_link('get', device, namespace=namespace, ext_mask=1)[0]
     num_vfs = link.get_attr('IFLA_NUM_VF')
@@ -450,21 +416,27 @@ def get_link_vfs(device, namespace):
     for vinfo in vfinfo_list.get_attrs('IFLA_VF_INFO'):
         mac = vinfo.get_attr('IFLA_VF_MAC')
         link_state = vinfo.get_attr('IFLA_VF_LINK_STATE')
+        rate = vinfo.get_attr('IFLA_VF_RATE', default={})
         vfs[mac['vf']] = {'mac': mac['mac'],
-                          'link_state': link_state['link_state']}
+                          'link_state': link_state['link_state'],
+                          'max_tx_rate': rate.get('max_tx_rate'),
+                          'min_tx_rate': rate.get('min_tx_rate'),
+                          }
 
     return vfs
 
 
 @privileged.default.entrypoint
 def add_neigh_entry(ip_version, ip_address, mac_address, device, namespace,
-                    **kwargs):
+                    nud_state, **kwargs):
     """Add a neighbour entry.
 
     :param ip_address: IP address of entry to add
     :param mac_address: MAC address of entry to add
     :param device: Device name to use in adding entry
     :param namespace: The name of the namespace in which to add the entry
+    :param nud_state: The NUD (Neighbour Unreachability Detection) state of
+                      the entry
     """
     family = _IP_VERSION_FAMILY_MAP[ip_version]
     _run_iproute_neigh('replace',
@@ -473,7 +445,7 @@ def add_neigh_entry(ip_version, ip_address, mac_address, device, namespace,
                        dst=ip_address,
                        lladdr=mac_address,
                        family=family,
-                       state=ndmsg.states['permanent'],
+                       state=ndmsg.states[nud_state],
                        **kwargs)
 
 
@@ -496,13 +468,19 @@ def delete_neigh_entry(ip_version, ip_address, mac_address, device, namespace,
                            lladdr=mac_address,
                            family=family,
                            **kwargs)
-    except NetlinkError as e:
+    except netlink_exceptions.NetlinkError as e:
         # trying to delete a non-existent entry shouldn't raise an error
         if e.code == errno.ENOENT:
             return
         raise
 
 
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
 @privileged.default.entrypoint
 def dump_neigh_entries(ip_version, device, namespace, **kwargs):
     """Dump all neighbour entries.
@@ -526,13 +504,14 @@ def dump_neigh_entries(ip_version, device, namespace, **kwargs):
 
     for entry in dump:
         attrs = dict(entry['attrs'])
-        entries += [{'dst': attrs['NDA_DST'],
-                     'lladdr': attrs.get('NDA_LLADDR'),
-                     'device': device}]
+        entries.append({'dst': attrs['NDA_DST'],
+                        'lladdr': attrs.get('NDA_LLADDR'),
+                        'device': device,
+                        'state': NUD_STATES[entry['state']]})
     return entries
 
 
-@privileged.default.entrypoint
+@privileged.namespace_cmd.entrypoint
 def create_netns(name, **kwargs):
     """Create a network namespace.
 
@@ -553,7 +532,7 @@ def create_netns(name, **kwargs):
             raise RuntimeError(_('Error creating namespace %s' % name))
 
 
-@privileged.default.entrypoint
+@privileged.namespace_cmd.entrypoint
 def remove_netns(name, **kwargs):
     """Remove a network namespace.
 
@@ -564,13 +543,14 @@ def remove_netns(name, **kwargs):
     except OSError as e:
         if e.errno != errno.ENOENT:
             raise
+    LOG.debug("Namespace %s deleted.", name)
 
 
-@privileged.default.entrypoint
+@privileged.namespace_cmd.entrypoint
 def list_netns(**kwargs):
     """List network namespaces.
 
-    Caller requires raised priveleges to list namespaces
+    Caller requires raised privileges to list namespaces
     """
     return netns.listnetns(**kwargs)
 
@@ -598,6 +578,12 @@ def make_serializable(value):
     return _ensure_string(value)
 
 
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
 @privileged.default.entrypoint
 def get_link_devices(namespace, **kwargs):
     """List interfaces in a namespace
@@ -628,6 +614,12 @@ def get_device_names(namespace, **kwargs):
     return device_names
 
 
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
 @privileged.default.entrypoint
 def get_ip_addresses(namespace, **kwargs):
     """List of IP addresses in a namespace
@@ -643,6 +635,12 @@ def get_ip_addresses(namespace, **kwargs):
         raise
 
 
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
 @privileged.default.entrypoint
 def list_ip_rules(namespace, ip_version, match=None, **kwargs):
     """List all IP rules"""
@@ -698,7 +696,8 @@ def _make_pyroute2_route_args(namespace, ip_version, cidr, device, via, table,
     :param ip_version: (int) [4, 6]
     :param cidr: (string) source IP or CIDR address (IPv4, IPv6)
     :param device: (string) input interface name
-    :param via: (string) gateway IP address
+    :param via: (string) gateway IP address or (list of dicts) for multipath
+                definition.
     :param table: (string, int) table number or name
     :param metric: (int) route metric
     :param scope: (int) route scope
@@ -708,31 +707,47 @@ def _make_pyroute2_route_args(namespace, ip_version, cidr, device, via, table,
     args = {'family': _IP_VERSION_FAMILY_MAP[ip_version]}
     if not scope:
         scope = 'global' if via else 'link'
-    scope = _get_scope_name(scope)
+    scope = get_scope_name(scope)
     if scope:
         args['scope'] = scope
     if cidr:
         args['dst'] = cidr
-    if device:
-        args['oif'] = get_link_id(device, namespace)
-    if via:
-        args['gateway'] = via
     if table:
         args['table'] = int(table)
     if metric:
         args['priority'] = int(metric)
     if protocol:
+        if isinstance(protocol, str) and protocol in rtnl.rt_proto:
+            protocol = rtnl.rt_proto[protocol]
         args['proto'] = protocol
+    if isinstance(via, (list, tuple)):
+        args['multipath'] = []
+        for mp in via:
+            multipath = {}
+            if mp.get('device'):
+                multipath['oif'] = get_link_id(mp['device'], namespace)
+            if mp.get('via'):
+                multipath['gateway'] = mp['via']
+            if mp.get('weight'):
+                multipath['hops'] = mp['weight'] - 1
+            args['multipath'].append(multipath)
+    else:
+        if via:
+            args['gateway'] = via
+        if device:
+            args['oif'] = get_link_id(device, namespace)
+
     return args
 
 
 @privileged.default.entrypoint
 def add_ip_route(namespace, cidr, ip_version, device=None, via=None,
-                 table=None, metric=None, scope=None, **kwargs):
+                 table=None, metric=None, scope=None,
+                 proto=rtnl.rt_proto['static'], **kwargs):
     """Add an IP route"""
     kwargs.update(_make_pyroute2_route_args(
         namespace, ip_version, cidr, device, via, table, metric, scope,
-        'static'))
+        proto))
     try:
         with get_iproute(namespace) as ip:
             ip.route('replace', **kwargs)
@@ -742,11 +757,18 @@ def add_ip_route(namespace, cidr, ip_version, device=None, via=None,
         raise
 
 
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
 @privileged.default.entrypoint
 def list_ip_routes(namespace, ip_version, device=None, table=None, **kwargs):
     """List IP routes"""
     kwargs.update(_make_pyroute2_route_args(
-        namespace, ip_version, None, device, None, table, None, None, None))
+        namespace, ip_version, None, device, None, table, None, 'universe',
+        None))
     try:
         with get_iproute(namespace) as ip:
             return make_serializable(ip.route('show', **kwargs))
@@ -769,3 +791,65 @@ def delete_ip_route(namespace, cidr, ip_version, device=None, via=None,
         if e.errno == errno.ENOENT:
             raise NetworkNamespaceNotFound(netns_name=namespace)
         raise
+
+
+@tenacity.retry(
+    retry=tenacity.retry_if_exception_type(
+        netlink_exceptions.NetlinkDumpInterrupted),
+    wait=tenacity.wait_exponential(multiplier=0.02, max=1),
+    stop=tenacity.stop_after_delay(8),
+    reraise=True)
+@privileged.default.entrypoint
+def list_bridge_fdb(namespace=None, **kwargs):
+    """List bridge fdb table"""
+    # NOTE(ralonsoh): fbd does not support ifindex filtering in pyroute2 0.5.14
+    try:
+        with get_iproute(namespace) as ip:
+            return make_serializable(ip.fdb('dump', **kwargs))
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            raise NetworkNamespaceNotFound(netns_name=namespace)
+        raise
+
+
+def _command_bridge_fdb(command, mac, device, dst_ip=None, namespace=None,
+                        **kwargs):
+    try:
+        kwargs['lladdr'] = mac
+        kwargs['ifindex'] = get_link_id(device, namespace)
+        if dst_ip:
+            kwargs['dst'] = dst_ip
+        with get_iproute(namespace) as ip:
+            return make_serializable(ip.fdb(command, **kwargs))
+    except OSError as e:
+        if e.errno == errno.ENOENT:
+            raise NetworkNamespaceNotFound(netns_name=namespace)
+        raise
+
+
+@privileged.default.entrypoint
+def add_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
+    """Add a FDB entry"""
+    _command_bridge_fdb('add', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
+
+
+@privileged.default.entrypoint
+def append_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
+    """Add a FDB entry"""
+    _command_bridge_fdb('append', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
+
+
+@privileged.default.entrypoint
+def replace_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
+    """Add a FDB entry"""
+    _command_bridge_fdb('replace', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
+
+
+@privileged.default.entrypoint
+def delete_bridge_fdb(mac, device, dst_ip=None, namespace=None, **kwargs):
+    """Add a FDB entry"""
+    _command_bridge_fdb('del', mac, device, dst_ip=dst_ip,
+                        namespace=namespace, **kwargs)
