@@ -16,6 +16,7 @@
 import collections
 
 from neutron_lib import constants as lib_const
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
 from neutron_lib.services.logapi import constants as log_const
 from os_ken.base import app_manager
 from os_ken.lib.packet import packet
@@ -27,8 +28,6 @@ from oslo_log import log as logging
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import firewall as ovsfw
 from neutron.agent.linux.openvswitch_firewall import rules
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
-        as ovs_consts
 from neutron.services.logapi.agent import log_extension as log_ext
 from neutron.services.logapi.common import exceptions as log_exc
 from neutron.services.logapi.drivers.openvswitch import log_oskenapp
@@ -145,7 +144,7 @@ class OVSFirewallLoggingDriver(log_ext.LoggingDriver):
         self.log_ports = collections.defaultdict(dict)
         self.cookies_table = set()
         self.cookie_ids_to_delete = set()
-        self.conj_id_map = ovsfw.ConjIdMap()
+        self.conj_id_map = ovsfw.ConjIdMap(self.int_br.br)
 
     def initialize(self, resource_rpc, **kwargs):
         self.resource_rpc = resource_rpc
@@ -385,7 +384,8 @@ class OVSFirewallLoggingDriver(log_ext.LoggingDriver):
     def _create_conj_flows_log(self, remote_rule, port):
         ethertype = remote_rule['ethertype']
         direction = remote_rule['direction']
-        remote_sg_id = remote_rule['remote_group_id']
+        remote_sg_id = remote_rule.get('remote_group_id')
+        remote_ag_id = remote_rule.get('remote_address_group_id')
         secgroup_id = remote_rule['security_group_id']
         # we only want to log first accept packet, that means a packet with
         # ct_state=+new-est, reg_remote_group=conj_id + 1 will be logged
@@ -394,7 +394,8 @@ class OVSFirewallLoggingDriver(log_ext.LoggingDriver):
             'dl_type': ovsfw_consts.ethertype_to_dl_type_map[ethertype],
             'reg_port': port.ofport,
             'reg_remote_group': self.conj_id_map.get_conj_id(
-                secgroup_id, remote_sg_id, direction, ethertype) + 1,
+                secgroup_id, remote_sg_id or remote_ag_id,
+                direction, ethertype) + 1,
         }
         if direction == lib_const.INGRESS_DIRECTION:
             flow_template['table'] = ovs_consts.RULES_INGRESS_TABLE
@@ -406,7 +407,7 @@ class OVSFirewallLoggingDriver(log_ext.LoggingDriver):
         cookie = self.generate_cookie(port.id, log_const.ACCEPT_EVENT,
                                       log_id, project_id)
         for rule in self.create_rules_generator_for_port(port):
-            if 'remote_group_id' in rule:
+            if 'remote_group_id' in rule or 'remote_address_group_id' in rule:
                 flows = self._create_conj_flows_log(rule, port)
             else:
                 flows = rules.create_flows_from_rule_and_port(rule, port)
@@ -465,7 +466,7 @@ class OVSFirewallLoggingDriver(log_ext.LoggingDriver):
         if not cookie:
             return
         for rule in del_rules:
-            if 'remote_group_id' in rule:
+            if 'remote_group_id' in rule or 'remote_address_group_id' in rule:
                 flows = self._create_conj_flows_log(rule, port)
             else:
                 flows = rules.create_flows_from_rule_and_port(rule, port)
