@@ -344,7 +344,8 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
         labels = router.get(constants.METERING_LABEL_KEY, [])
         for label in labels:
             label_id = label['id']
-            del rm.metering_labels[label_id]
+            if rm.metering_labels.get(label_id):
+                del rm.metering_labels[label_id]
 
     @log_helpers.log_method_call
     def add_metering_label(self, context, routers):
@@ -505,9 +506,12 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
 
             chain_acc = rm.iptables_manager.get_traffic_counters(
                 chain, wrap=False, zero=True)
-        except RuntimeError:
-            LOG.exception('Failed to get traffic counters, '
-                          'router: %s', router)
+        except RuntimeError as e:
+            LOG.warning('Failed to get traffic counters for router [%s] due '
+                        'to [%s]. This error message can happen when routers '
+                        'are migrated; therefore, most of the times they can '
+                        'be ignored.', router, e)
+
             routers_to_reconfigure.add(router['id'])
             return {}
         return chain_acc
@@ -537,9 +541,9 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
         `traffic_counters` dictionary.
                 * labels -- label-<label_id>
                 * routers -- router-<router_id>
-                * project -- project-<tenant_id>
+                * project -- project-<project_id>
                 * router-label -- router-<router_id>-label-<label_id>
-                * project-label -- project-<tenant_id>-label-<label_id>
+                * project-label -- project-<project_id>-label-<label_id>
 
         And last, but not least, if we are not able to retrieve the traffic
         counters from `iptables` for a given router, we will add it to
@@ -556,7 +560,7 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
 
         default_traffic_counters = {'pkts': 0, 'bytes': 0}
         project_traffic_counter_key = self.get_project_traffic_counter_key(
-            router['tenant_id'])
+            router['project_id'])
         router_traffic_counter_key = self.get_router_traffic_counter_key(
             router_id)
 
@@ -635,7 +639,7 @@ class IptablesMeteringDriver(abstract_driver.MeteringAbstractDriver):
 
             # NOTE(bno1): Sometimes a router is added before its namespaces are
             # created. The metering agent has to periodically check if the
-            # namespaces for the missing iptables managers have appearead and
+            # namespaces for the missing iptables managers have appeared and
             # create the managers for them. When a new manager is created, the
             # metering rules have to be added to it.
             if rm.create_iptables_managers():
