@@ -12,16 +12,17 @@
 #    under the License.
 #
 
-from neutron_lib.api.definitions import dns as dns_apidef
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
-from neutron_lib.api import extensions as api_extensions
+from neutron_lib.api.definitions import qos_fip as qos_fip_apidef
+from neutron_lib.api.definitions import qos_gateway_ip as qos_gateway_ip_apidef
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
 from neutron_lib.exceptions import availability_zone as az_exc
 from neutron_lib.plugins import constants as plugin_constants
@@ -30,13 +31,18 @@ from neutron_lib.services import base as service_base
 from oslo_log import log
 from oslo_utils import excutils
 
+from neutron._i18n import _
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import extensions
 from neutron.common.ovn import utils
+from neutron.common import utils as common_utils
 from neutron.db.availability_zone import router as router_az_db
 from neutron.db import dns_db
 from neutron.db import extraroute_db
+from neutron.db import l3_fip_pools_db
 from neutron.db import l3_fip_port_details
+from neutron.db import l3_fip_qos
+from neutron.db import l3_gateway_ip_qos
 from neutron.db import l3_gwmode_db
 from neutron.db.models import l3 as l3_models
 from neutron.db import ovn_revision_numbers_db as db_rev
@@ -57,7 +63,11 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                         l3_gwmode_db.L3_NAT_db_mixin,
                         dns_db.DNSDbMixin,
                         l3_fip_port_details.Fip_port_details_db_mixin,
-                        router_az_db.RouterAvailabilityZoneMixin):
+                        router_az_db.RouterAvailabilityZoneMixin,
+                        l3_fip_qos.FloatingQoSDbMixin,
+                        l3_gateway_ip_qos.L3_gw_ip_qos_db_mixin,
+                        l3_fip_pools_db.FloatingIPPoolsMixin,
+                        ):
     """Implementation of the OVN L3 Router Service Plugin.
 
     This class implements a L3 service plugin that provides
@@ -69,6 +79,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
     # once available.
     _supported_extension_aliases = (
         extensions.ML2_SUPPORTED_API_EXTENSIONS_OVN_L3)
+    __filter_validation_support = True
 
     @resource_registry.tracked_resources(router=l3_models.Router,
                                          floatingip=l3_models.FloatingIP)
@@ -91,28 +102,31 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             events.PRECOMMIT_CREATE)
 
     @staticmethod
-    def disable_dns_extension_by_extension_drivers(aliases):
-        core_plugin = directory.get_plugin()
-        if not api_extensions.is_extension_supported(
-                core_plugin, dns_apidef.ALIAS):
-            aliases.remove(dns_apidef.ALIAS)
+    def _disable_qos_extensions_by_extension_drivers(aliases):
+        qos_service_plugin = directory.get_plugin(plugin_constants.QOS)
+        qos_fip_in_aliases = qos_fip_apidef.ALIAS in aliases
+        qos_gwip_in_aliases = qos_gateway_ip_apidef.ALIAS in aliases
+        if not qos_service_plugin and qos_fip_in_aliases:
+            aliases.remove(qos_fip_apidef.ALIAS)
+        if not qos_service_plugin and qos_gwip_in_aliases:
+            aliases.remove(qos_gateway_ip_apidef.ALIAS)
 
     @property
     def supported_extension_aliases(self):
         if not hasattr(self, '_aliases'):
             self._aliases = self._supported_extension_aliases[:]
-            self.disable_dns_extension_by_extension_drivers(self._aliases)
+            self._disable_qos_extensions_by_extension_drivers(self._aliases)
         return self._aliases
 
     @property
     def _ovn_client(self):
         if self._ovn_client_inst is None:
-            self._ovn_client_inst = ovn_client.OVNClient(self._ovn,
+            self._ovn_client_inst = ovn_client.OVNClient(self._nb_ovn,
                                                          self._sb_ovn)
         return self._ovn_client_inst
 
     @property
-    def _ovn(self):
+    def _nb_ovn(self):
         return self._plugin_driver.nb_ovn
 
     @property
@@ -149,10 +163,14 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return ("L3 Router Service Plugin for basic L3 forwarding"
                 " using OVN")
 
-    def create_router_precommit(self, resource, event, trigger, context,
-                                router, router_id, router_db):
+    def create_router_precommit(self, resource, event, trigger, payload):
+        context = payload.context
+        router_id = payload.resource_id
+        router_db = payload.metadata['router_db']
+
         db_rev.create_initial_revision(
-            context, router_id, ovn_const.TYPE_ROUTERS)
+            context, router_id, ovn_const.TYPE_ROUTERS,
+            std_attr_id=router_db.standard_attr.id)
 
     def create_router(self, context, router):
         router = super(OVNL3RouterPlugin, self).create_router(context, router)
@@ -244,10 +262,14 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                     context, router_id, interface_info)
         return router_interface_info
 
-    def create_floatingip_precommit(self, resource, event, trigger, context,
-                                    floatingip, floatingip_id, floatingip_db):
+    def create_floatingip_precommit(self, resource, event, trigger, payload):
+        context = payload.context
+        floatingip_id = payload.resource_id
+        floatingip_db = payload.desired_state
+
         db_rev.create_initial_revision(
-            context, floatingip_id, ovn_const.TYPE_FLOATINGIPS)
+            context, floatingip_id, ovn_const.TYPE_FLOATINGIPS,
+            std_attr_id=floatingip_db.standard_attr.id)
 
     def create_floatingip(self, context, floatingip,
                           initial_status=n_const.FLOATINGIP_STATUS_DOWN):
@@ -257,33 +279,27 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         return fip
 
     def delete_floatingip(self, context, id):
-        # TODO(lucasagomes): Passing ``original_fip`` object as a
-        # parameter to the OVNClient's delete_floatingip() method is done
-        # for backward-compatible reasons. Remove it in the Rocky release
-        # of OpenStack.
-        original_fip = self.get_floatingip(context, id)
         super(OVNL3RouterPlugin, self).delete_floatingip(context, id)
-        self._ovn_client.delete_floatingip(context, id,
-                                           fip_object=original_fip)
+        self._ovn_client.delete_floatingip(context, id)
 
     def update_floatingip(self, context, id, floatingip):
-        # TODO(lucasagomes): Passing ``original_fip`` object as a
-        # parameter to the OVNClient's update_floatingip() method is done
-        # for backward-compatible reasons. Remove it in the Rocky release
-        # of OpenStack.
-        original_fip = self.get_floatingip(context, id)
         fip = super(OVNL3RouterPlugin, self).update_floatingip(context, id,
                                                                floatingip)
-        self._ovn_client.update_floatingip(context, fip,
-                                           fip_object=original_fip)
+        self._ovn_client.update_floatingip(context, fip)
         return fip
 
     def update_floatingip_status(self, context, floatingip_id, status):
-        fip = super(OVNL3RouterPlugin, self).update_floatingip_status(
+        fip = self.update_floatingip_status_retry(
             context, floatingip_id, status)
         self._ovn_client.update_floatingip_status(context, fip)
         return fip
 
+    @db_api.retry_if_session_inactive()
+    def update_floatingip_status_retry(self, context, floatingip_id, status):
+        with db_api.CONTEXT_WRITER.using(context):
+            return super(OVNL3RouterPlugin, self).update_floatingip_status(
+                context, floatingip_id, status)
+
     def disassociate_floatingips(self, context, port_id, do_notify=True):
         fips = self.get_floatingips(context.elevated(),
                                     filters={'port_id': [port_id]})
@@ -293,8 +309,11 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             router_id = fip.get('router_id')
             fixed_ip_address = fip.get('fixed_ip_address')
             if router_id and fixed_ip_address:
-                update_fip = {'logical_ip': fixed_ip_address,
-                              'external_ip': fip['floating_ip_address']}
+                update_fip = {
+                    'id': fip['id'],
+                    'logical_ip': fixed_ip_address,
+                    'external_ip': fip['floating_ip_address'],
+                    'floating_network_id': fip['floating_network_id']}
                 try:
                     self._ovn_client.disassociate_floatingip(update_fip,
                                                              router_id)
@@ -339,7 +358,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
             # device owner is router_gateway. We need to look into it and
             # fix the problem in Neutron before updating it here.
             if host:
-                self._plugin.update_port(
+                port = self._plugin.update_port(
                     context, port['id'],
                     {'port': {portbindings.HOST_ID: host}})
 
@@ -356,10 +375,10 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         if not self._plugin_driver.list_availability_zones(context):
             return []
 
-        lrp = self._ovn.get_lrouter_port(lrp_name)
+        lrp = self._nb_ovn.get_lrouter_port(lrp_name)
         router = self.get_router(
             context, lrp.external_ids[ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY])
-        az_hints = utils.get_az_hints(router)
+        az_hints = common_utils.get_az_hints(router)
         return az_hints
 
     def schedule_unhosted_gateways(self, event_from_chassis=None):
@@ -367,7 +386,7 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         port_physnet_dict = self._get_gateway_port_physnet_mapping()
         # Filter out unwanted ports in case of event.
         if event_from_chassis:
-            gw_chassis = self._ovn.get_chassis_gateways(
+            gw_chassis = self._nb_ovn.get_chassis_gateways(
                 chassis_name=event_from_chassis)
             if not gw_chassis:
                 return
@@ -388,26 +407,29 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
         chassis_with_physnets = self._sb_ovn.get_chassis_and_physnets()
         # All chassis with enable_as_gw_chassis set
         all_gw_chassis = self._sb_ovn.get_gateway_chassis_from_cms_options()
-        unhosted_gateways = self._ovn.get_unhosted_gateways(
+        chassis_with_azs = self._sb_ovn.get_chassis_and_azs()
+        unhosted_gateways = self._nb_ovn.get_unhosted_gateways(
             port_physnet_dict, chassis_with_physnets,
-            all_gw_chassis)
+            all_gw_chassis, chassis_with_azs)
         for g_name in unhosted_gateways:
             physnet = port_physnet_dict.get(g_name[len(ovn_const.LRP_PREFIX):])
             # Remove any invalid gateway chassis from the list, otherwise
             # we can have a situation where all existing_chassis are invalid
-            existing_chassis = self._ovn.get_gateway_chassis_binding(g_name)
+            existing_chassis = self._nb_ovn.get_gateway_chassis_binding(g_name)
             primary = existing_chassis[0] if existing_chassis else None
+            az_hints = self._nb_ovn.get_gateway_chassis_az_hints(g_name)
             existing_chassis = self.scheduler.filter_existing_chassis(
-                nb_idl=self._ovn, gw_chassis=all_gw_chassis,
+                nb_idl=self._nb_ovn, gw_chassis=all_gw_chassis,
                 physnet=physnet, chassis_physnets=chassis_with_physnets,
-                existing_chassis=existing_chassis)
-            az_hints = self._get_availability_zones_from_router_port(g_name)
+                existing_chassis=existing_chassis, az_hints=az_hints,
+                chassis_with_azs=chassis_with_azs)
+
             candidates = self._ovn_client.get_candidates_for_scheduling(
                 physnet, cms=all_gw_chassis,
                 chassis_physnets=chassis_with_physnets,
                 availability_zone_hints=az_hints)
             chassis = self.scheduler.select(
-                self._ovn, self._sb_ovn, g_name, candidates=candidates,
+                self._nb_ovn, self._sb_ovn, g_name, candidates=candidates,
                 existing_chassis=existing_chassis)
             if primary and primary != chassis[0]:
                 if primary not in chassis:
@@ -427,19 +449,19 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                     chassis[0], chassis[index] = chassis[index], chassis[0]
             # NOTE(dalvarez): Let's commit the changes in separate transactions
             # as we will rely on those for scheduling subsequent gateways.
-            with self._ovn.transaction(check_error=True) as txn:
-                txn.add(self._ovn.update_lrouter_port(
+            with self._nb_ovn.transaction(check_error=True) as txn:
+                txn.add(self._nb_ovn.update_lrouter_port(
                     g_name, gateway_chassis=chassis))
 
     @staticmethod
     @registry.receives(resources.SUBNET, [events.AFTER_UPDATE])
-    def _subnet_update(resource, event, trigger, **kwargs):
+    def _subnet_update(resource, event, trigger, payload):
         l3plugin = directory.get_plugin(plugin_constants.L3)
         if not l3plugin:
             return
-        context = kwargs['context']
-        orig = kwargs['original_subnet']
-        current = kwargs['subnet']
+        context = payload.context
+        orig = payload.states[0]
+        current = payload.latest_state
         orig_gw_ip = orig['gateway_ip']
         current_gw_ip = current['gateway_ip']
         if orig_gw_ip == current_gw_ip:
@@ -454,35 +476,54 @@ class OVNL3RouterPlugin(service_base.ServicePluginBase,
                   ] if orig_gw_ip else []
         add = [{'destination': '0.0.0.0/0', 'nexthop': current_gw_ip}
                ] if current_gw_ip else []
-        with l3plugin._ovn.transaction(check_error=True) as txn:
+        with l3plugin._nb_ovn.transaction(check_error=True) as txn:
             for router_id in router_ids:
                 l3plugin._ovn_client.update_router_routes(
                     context, router_id, add, remove, txn=txn)
 
     @staticmethod
-    @registry.receives(resources.PORT, [events.AFTER_UPDATE])
-    def _port_update(resource, event, trigger, **kwargs):
+    @registry.receives(
+        resources.PORT,
+        [events.BEFORE_UPDATE, events.AFTER_UPDATE]
+    )
+    def _port_update(resource, event, trigger, payload):
         l3plugin = directory.get_plugin(plugin_constants.L3)
         if not l3plugin:
             return
 
-        current = kwargs['port']
+        context = payload.context
+        current = payload.latest_state
+        original = payload.states[0]
 
-        if utils.is_lsp_router_port(current):
+        # The OVN NB DB has a constraint where network has to be
+        # greater than 0. Updating it with an empty network would
+        # cause a constraint violation error. This problem happens
+        # when the last IP of a LRP is deleted, in order to avoid it
+        # an exception needs to be thrown before any write is performed
+        # to the DB, since if not it would leave the Neutron DB and the
+        # OVN DB unsync.
+        # https://bugs.launchpad.net/neutron/+bug/1948457
+        if (event == events.BEFORE_UPDATE and
+                'fixed_ips' in current and not current['fixed_ips'] and
+                utils.is_lsp_router_port(original)):
+            reason = _("Router port must have at least one IP.")
+            raise n_exc.ServicePortInUse(port_id=original['id'], reason=reason)
+
+        if event == events.AFTER_UPDATE and utils.is_lsp_router_port(current):
             # We call the update_router port with if_exists, because neutron,
             # internally creates the port, and then calls update, which will
             # trigger this callback even before we had the chance to create
             # the OVN NB DB side
-            l3plugin._ovn_client.update_router_port(kwargs['context'],
+            l3plugin._ovn_client.update_router_port(context,
                                                     current, if_exists=True)
 
     def get_router_availability_zones(self, router):
-        lr = self._ovn.get_lrouter(router['id'])
+        lr = self._nb_ovn.get_lrouter(router['id'])
         if not lr:
             return []
 
         return [az.strip() for az in lr.external_ids.get(
-                ovn_const.OVN_ROUTER_AZ_HINTS_EXT_ID_KEY, '').split(',')
+                ovn_const.OVN_AZ_HINTS_EXT_ID_KEY, '').split(',')
                 if az.strip()]
 
     def validate_availability_zones(self, context, resource_type,
