@@ -26,6 +26,7 @@ import os.path
 import queue
 import threading
 from unittest import mock
+import warnings
 
 import eventlet.timeout
 import fixtures
@@ -61,7 +62,6 @@ from neutron.tests import post_mortem_debug
 
 
 CONF = cfg.CONF
-CONF.import_opt('state_path', 'neutron.conf.common')
 
 ROOTDIR = os.path.dirname(__file__)
 ETCDIR = os.path.join(ROOTDIR, 'etc')
@@ -226,6 +226,17 @@ class DietTestCase(base.BaseTestCase, metaclass=_CatchTimeoutMetaclass):
     def setUp(self):
         super(DietTestCase, self).setUp()
 
+        # NOTE(slaweq): Make deprecation warnings only happen once.
+        warnings.simplefilter("once", DeprecationWarning)
+
+        # NOTE(slaweq): Let's not display such warnings in tests as we know
+        # that we have many places where policy enforcement depends on values
+        # like is_admin or project_id and there can be a lot of such warnings
+        # in the logs
+        warnings.filterwarnings(
+            'ignore', message=(
+                'Policy enforcement is depending on the value of '))
+
         # Suppress some log messages during test runs, otherwise it may cause
         # issues with subunit parser when running on Python 3. It happened for
         # example for neutron-functional tests.
@@ -371,6 +382,7 @@ class BaseTestCase(DietTestCase):
         self.useFixture(lockutils.ExternalLockFixture())
         self.useFixture(fixture.APIDefinitionFixture())
 
+        config.register_common_config_options()
         cfg.CONF.set_override('state_path', self.get_default_temp_dir().path)
 
         self.addCleanup(CONF.reset)
@@ -394,7 +406,7 @@ class BaseTestCase(DietTestCase):
         # Give a private copy of the directory to each test.
         self.useFixture(fixture.PluginDirectoryFixture())
 
-        policy.init()
+        policy.init(suppress_deprecation_warnings=True)
         self.addCleanup(policy.reset)
         self.addCleanup(resource_registry.unregister_all_resources)
         self.addCleanup(db_api.sqla_remove_all)
