@@ -86,6 +86,29 @@ def wait_until_dscp_marking_rule_applied_ovs(bridge, port_vif, rule):
     common_utils.wait_until_true(_dscp_marking_rule_applied)
 
 
+def wait_until_pkt_meter_rule_applied_ovs(bridge, port_vif, port_id,
+                                          direction, mac=None):
+    def _pkt_rate_limit_rule_applied():
+        port_num = bridge.get_port_ofport(port_vif)
+        port_vlan = bridge.get_port_tag_by_name(port_vif)
+        key = "%s_%s" % (port_id, direction)
+        meter_id = bridge.get_value_from_other_config(
+            port_vif, key, value_type=int)
+
+        if direction == "egress":
+            flows = bridge.dump_flows_for(table='59', in_port=str(port_num),
+                                          dl_src=str(mac))
+        else:
+            flows = bridge.dump_flows_for(table='59', dl_vlan=str(port_vlan),
+                                          dl_dst=str(mac))
+        if mac:
+            return bool(flows) and meter_id
+        else:
+            return not bool(flows) and not meter_id
+
+    common_utils.wait_until_true(_pkt_rate_limit_rule_applied)
+
+
 def wait_until_dscp_marking_rule_applied_linuxbridge(namespace, port_vif,
                                                      expected_rule):
 
@@ -135,3 +158,22 @@ def wait_for_dscp_marked_packet(sender_vm, receiver_vm, dscp_mark):
         "to %(dst)s" % {'dscp_mark': dscp_mark,
                         'src': sender_vm.ip,
                         'dst': receiver_vm.ip})
+
+
+def extract_vlan_id(flows):
+    if flows:
+        flow_list = flows.splitlines()
+        for flow in flow_list:
+            if 'mod_vlan_vid' in flow:
+                actions = flow.partition('actions=')[2]
+                after_mod = actions.partition('mod_vlan_vid:')[2]
+                return int(after_mod.partition(',')[0])
+
+
+def wait_for_mod_vlan_id_applied(bridge, expected_vlan_id):
+    def _vlan_id_rule_applied():
+        flows = bridge.dump_flows_for(table='0')
+        vlan_id = extract_vlan_id(flows)
+        return vlan_id == expected_vlan_id
+
+    common_utils.wait_until_true(_vlan_id_rule_applied)
