@@ -45,6 +45,7 @@ from neutron.db import db_base_plugin_common as db_base
 from neutron.plugins.ml2.drivers.linuxbridge.agent import \
     linuxbridge_neutron_agent as linuxbridge_agent
 from neutron.tests.common import base as common_base
+from neutron.tests.common import helpers
 from neutron.tests import tools
 
 LOG = logging.getLogger(__name__)
@@ -110,8 +111,9 @@ def assert_ping(src_namespace, dst_ip, timeout=1, count=3):
     ipversion = netaddr.IPAddress(dst_ip).version
     ping_command = 'ping' if ipversion == 4 else 'ping6'
     ns_ip_wrapper = ip_lib.IPWrapper(src_namespace)
-    ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', count,
-                                 dst_ip])
+    ns_ip_wrapper.netns.execute(
+        [ping_command, '-W', timeout, '-c', count, dst_ip],
+        privsep_exec=True)
 
 
 def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
@@ -123,8 +125,9 @@ def assert_async_ping(src_namespace, dst_ip, timeout=1, count=1, interval=1):
     # cannot be used and it needs to be done using the following workaround.
     for _index in range(count):
         start_time = time.time()
-        ns_ip_wrapper.netns.execute([ping_command, '-W', timeout, '-c', '1',
-                                     dst_ip])
+        ns_ip_wrapper.netns.execute(
+            [ping_command, '-W', timeout, '-c', '1', dst_ip],
+            privsep_exec=True)
         end_time = time.time()
         diff = end_time - start_time
         if 0 < diff < interval:
@@ -166,7 +169,7 @@ def assert_arping(src_namespace, dst_ip, source=None, timeout=1, count=1):
     if source:
         arping_cmd.extend(['-s', source])
     arping_cmd.append(dst_ip)
-    ns_ip_wrapper.netns.execute(arping_cmd)
+    ns_ip_wrapper.netns.execute(arping_cmd, privsep_exec=True)
 
 
 def assert_no_arping(src_namespace, dst_ip, source=None, timeout=1, count=1):
@@ -192,7 +195,8 @@ def _get_source_ports_from_ss_output(output):
 def get_unused_port(used, start=1024, end=None):
     if end is None:
         port_range = utils.execute(
-            ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+            ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True,
+            privsep_exec=True)
         end = int(port_range.split()[0]) - 1
 
     candidates = set(range(start, end + 1))
@@ -222,7 +226,8 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
         raise ValueError("Unsupported protocol %s" % protocol)
 
     ip_wrapper = ip_lib.IPWrapper(namespace=namespace)
-    output = ip_wrapper.netns.execute(['ss', param], run_as_root=True)
+    output = ip_wrapper.netns.execute(['ss', param], run_as_root=True,
+                                      privsep_exec=True)
     used_ports = _get_source_ports_from_ss_output(output)
 
     return get_unused_port(used_ports, start, end)
@@ -231,11 +236,12 @@ def get_free_namespace_port(protocol, namespace=None, start=1024, end=None):
 def set_local_port_range(start, end):
     utils.execute(
         ['sysctl', '-w', 'net.ipv4.ip_local_port_range=%d %d' % (start, end)],
-        run_as_root=True)
-    utils.execute(['sysctl', '-p'], run_as_root=True)
+        run_as_root=True, privsep_exec=True)
+    utils.execute(['sysctl', '-p'], run_as_root=True, privsep_exec=True)
     # verify
     port_range = utils.execute(
-        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True)
+        ['sysctl', '-n', 'net.ipv4.ip_local_port_range'], run_as_root=True,
+        privsep_exec=True)
     assert int(port_range.split()[0]) == start
     assert int(port_range.split()[1]) == end
 
@@ -299,9 +305,27 @@ class RootHelperProcess(subprocess.Popen):
         super(RootHelperProcess, self).__init__(cmd, *args, **kwargs)
         self._wait_for_child_process()
 
-    def kill(self, sig=signal.SIGKILL):
+    def kill(self, sig=signal.SIGKILL, skip_errors=None):
+        skip_errors = (
+            ["No such process"] if skip_errors is None else skip_errors)
         pid = self.child_pid or str(self.pid)
-        utils.execute(['kill', '-%d' % sig, pid], run_as_root=True)
+        try:
+            utils.execute(['kill', '-%d' % sig, pid], run_as_root=True)
+        except n_exc.ProcessExecutionError as e:
+            # NOTE(slaweq): kill command returns 1 for many
+            # different issues, e.g. when there is no such process to kill
+            # (which we want to handle here) but also in cases like e.g. when
+            # user don't have privileges to kill process (which we don't want
+            # to silently hide). Sometimes we don't want really to fail if e.g.
+            # we are trying to kill process which already don't exists, so we
+            # can check if that was the error and not fail than.
+            for skip_error in skip_errors:
+                if skip_error in str(e):
+                    LOG.debug('Kill process %(pid)s failed due to error: '
+                              '%(err)s. This error can be ignored.',
+                              {'pid': pid, 'err': e})
+                    return
+            raise e
 
     def read_stdout(self, timeout=None):
         return self._read_stream(self.stdout, timeout)
@@ -339,7 +363,7 @@ class RootHelperProcess(subprocess.Popen):
                 pass
             msg = ("Process %(cmd)s hasn't been spawned in %(seconds)d "
                    "seconds. Return code: %(ret_code)s, stdout: %(stdout)s, "
-                   "sdterr: %(stderr)s" %
+                   "stderr: %(stderr)s" %
                    {'cmd': self.cmd, 'seconds': timeout,
                     'ret_code': self.returncode, 'stdout': stdout,
                     'stderr': stderr})
@@ -544,7 +568,7 @@ class NetcatTester(object):
             self.server_process.writeline(message)
             message = self.client_process.read_stdout(READ_TIMEOUT).strip()
         except ConnectionError as e:
-            LOG.debug("Error: %s occured during connectivity test.", e)
+            LOG.debug("Error: %s occurred during connectivity test.", e)
             message = ""
 
         return message == testing_string
@@ -574,14 +598,12 @@ class NetcatTester(object):
         return proc
 
     def stop_processes(self, skip_errors=None):
-        skip_errors = (['No such process'] if skip_errors is None
-                       else skip_errors)
         for proc_attr in ('_client_process', '_server_process'):
             proc = getattr(self, proc_attr)
             if proc:
                 try:
                     if proc.poll() is None:
-                        proc.kill()
+                        proc.kill(skip_errors=skip_errors)
                         proc.wait()
                 except n_exc.ProcessExecutionError as exc:
                     for skip_error in skip_errors:
@@ -612,10 +634,18 @@ class NamespaceFixture(fixtures.Fixture):
         self.addCleanup(self.destroy)
 
     def destroy(self):
-        if self.ip_wrapper.netns.exists(self.name):
-            for pid in ip_lib.list_namespace_pids(self.name):
-                utils.kill_process(pid, signal.SIGKILL, run_as_root=True)
-            self.ip_wrapper.netns.delete(self.name)
+        # TODO(ralonsoh): once the issue in LP#1838793 is properly fixed, we
+        # can remove this workaround (TestTimer context).
+        with helpers.TestTimer(5):
+            try:
+                if self.ip_wrapper.netns.exists(self.name):
+                    for pid in ip_lib.list_namespace_pids(self.name):
+                        utils.kill_process(pid, signal.SIGKILL,
+                                           run_as_root=True)
+                    self.ip_wrapper.netns.delete(self.name)
+            except helpers.TestTimerTimeout:
+                LOG.warning('Namespace %s was not deleted due to a timeout.',
+                            self.name)
 
 
 class VethFixture(fixtures.Fixture):
@@ -637,8 +667,8 @@ class VethFixture(fixtures.Fixture):
     def destroy(self):
         for port in self.ports:
             ip_wrapper = ip_lib.IPWrapper(port.namespace)
-            if (ip_wrapper.netns.exists(port.namespace) or
-                    port.namespace is None):
+            if (port.namespace is None or
+                    ip_wrapper.netns.exists(port.namespace)):
                 try:
                     ip_wrapper.del_veth(port.name)
                     break
@@ -828,6 +858,9 @@ class OVSPortFixture(PortFixture):
         # on ports that we intend to use as fake vm interfaces, they
         # need to be flat. This is related to lp#1767422
         self.bridge.clear_db_attribute("Port", port_name, "tag")
+        # Clear vlan_mode that is added for each new port. lp#1930414
+        self.bridge.clear_db_attribute("Port", port_name, "vlan_mode")
+        self.bridge.clear_db_attribute("Port", port_name, "trunks")
         self.addCleanup(self.bridge.delete_port, port_name)
         self.port = ip_lib.IPDevice(port_name, self.namespace)
 
