@@ -30,6 +30,7 @@ from neutron.tests.common import helpers
 from neutron.tests.common import machine_fixtures
 from neutron.tests.common import net_helpers
 from neutron.tests.fullstack.resources import client as client_resource
+from neutron.tests.fullstack.resources import machine
 from neutron.tests.unit import testlib_api
 
 
@@ -159,6 +160,22 @@ class BaseFullStackTestCase(testlib_api.MySQLTestCaseMixin,
             "tag", network.get("provider:segmentation_id"))
         return vm
 
+    def _prepare_vms_in_net(self, tenant_uuid, network, use_dhcp=False):
+        vms = machine.FakeFullstackMachinesList(
+            self.useFixture(
+                machine.FakeFullstackMachine(
+                    host,
+                    network['id'],
+                    tenant_uuid,
+                    self.safe_client,
+                    use_dhcp=use_dhcp))
+            for host in self.environment.hosts)
+
+        vms.block_until_all_boot()
+        if use_dhcp:
+            vms.block_until_all_dhcp_config_done()
+        return vms
+
     def assert_namespace_exists(self, ns_name):
         common_utils.wait_until_true(
             lambda: ip_lib.network_namespace_exists(ns_name,
