@@ -20,7 +20,6 @@ from neutron.agent.linux import iptables_firewall
 from neutron.agent.linux import iptables_manager
 from neutron.cmd.sanity import checks
 from neutron.common import utils as common_utils
-from neutron.tests import base as tests_base
 from neutron.tests.common import net_helpers
 from neutron.tests.fullstack import base
 from neutron.tests.fullstack.resources import environment
@@ -53,7 +52,7 @@ class BaseSecurityGroupsSameNetworkTest(base.BaseFullStackTestCase):
             environment.HostDescription(
                 l2_agent_type=self.l2_agent_type,
                 firewall_driver=self.firewall_driver,
-                dhcp_agent=True) for _ in range(self.num_hosts)]
+                dhcp_agent=False) for _ in range(self.num_hosts)]
         env = environment.Environment(
             environment.EnvironmentDescription(
                 network_type=self.network_type,
@@ -109,7 +108,6 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
     # NOTE(toshii): As a firewall_driver can interfere with others,
     # the recommended way to add test is to expand this method, not
     # adding another.
-    @tests_base.unstable_test("bug 1742401")
     def test_securitygroup(self):
         """Tests if a security group rules are working, by confirming
         that 0. traffic is allowed when port security is disabled,
@@ -248,7 +246,7 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                     tenant_uuid,
                     self.safe_client,
                     neutron_port=ports[-1],
-                    use_dhcp=True)))
+                    use_dhcp=False)))
         self.assertEqual(5, len(vms))
 
         vms[4].block_until_boot()
@@ -567,10 +565,9 @@ class TestSecurityGroupsSameNetwork(BaseSecurityGroupsSameNetworkTest):
                     tenant_uuid,
                     self.safe_client,
                     neutron_port=ports[port],
-                    use_dhcp=True))
+                    use_dhcp=False))
             for port, host in enumerate(index_to_host)]
         map(lambda vm: vm.block_until_boot(), vms)
-        map(lambda vm: vm.block_until_dhcp_config_done(), vms)
 
         return vms, ports, sgs, network, index_to_host
 
@@ -691,3 +688,18 @@ class SecurityGroupRulesTest(base.BaseFullStackTestCase):
 
         self.assertRaises(nc_exc.OverQuotaClient,
                           self.safe_client.create_security_group, project_id)
+
+    def test_normalized_cidr_in_rule(self):
+        project_id = uuidutils.generate_uuid()
+        sg = self.safe_client.create_security_group(project_id)
+
+        rule = self.safe_client.create_security_group_rule(
+            project_id, sg['id'], direction='ingress',
+            remote_ip_prefix='10.0.0.34/24')
+        self.assertEqual('10.0.0.0/24', rule['normalized_cidr'])
+        self.assertEqual('10.0.0.34/24', rule['remote_ip_prefix'])
+
+        rule = self.safe_client.create_security_group_rule(
+            project_id, sg['id'], direction='ingress')
+        self.assertIsNone(rule['normalized_cidr'])
+        self.assertIsNone(rule['remote_ip_prefix'])
