@@ -20,6 +20,7 @@ from unittest import mock
 import eventlet
 import fixtures
 from neutron_lib import constants as n_const
+from neutron_lib.plugins.ml2 import ovs_constants
 from neutron_lib.utils import net
 from oslo_config import cfg
 from oslo_utils import uuidutils
@@ -34,7 +35,6 @@ from neutron.conf.agent import ovs_conf as ovs_agent_config
 from neutron.conf import common as common_config
 from neutron.conf.plugins.ml2.drivers import agent
 from neutron.conf.plugins.ml2.drivers import ovs_conf
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.plugins.ml2.drivers.openvswitch.agent.extension_drivers \
     import qos_driver as ovs_qos_driver
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
@@ -176,7 +176,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
         self._bridge_classes()['br_phys'](self.br_phys).create()
         ext_mgr = ext_manager.L2AgentExtensionsManager(self.config)
         with mock.patch.object(ovs_qos_driver.QosOVSAgentDriver,
-                               '_minimum_bandwidth_initialize'):
+                               '_qos_bandwidth_initialize'):
             agent = ovs_agent.OVSNeutronAgent(self._bridge_classes(),
                                               ext_mgr, self.config)
         self.addCleanup(self.ovs.delete_bridge, self.br_int)
@@ -228,7 +228,7 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
         utils.wait_until_true(
             polling_manager._monitor.is_active)
         agent.check_ovs_status = mock.Mock(
-            return_value=constants.OVS_NORMAL)
+            return_value=ovs_constants.OVS_NORMAL)
         self.agent_thread = eventlet.spawn(agent.rpc_loop,
                                            polling_manager)
 
@@ -465,13 +465,16 @@ class OVSAgentTestFramework(base.BaseOVSLinuxTestCase, OVSOFControllerHelper):
         self._plug_ports(network, ports, self.agent, bridge=phys_br,
                          namespace=namespace)
 
-        if network_type == 'flat':
-            # NOTE(slaweq): for OVS implementations remove the DEAD VLAN tag
-            # on ports that belongs to flat network. DEAD VLAN tag is added
-            # to each newly created port. This is related to lp#1767422
-            for port in ports:
+        for port in ports:
+            if network_type == 'flat':
+                # NOTE(slaweq): for OVS implementations remove the DEAD VLAN
+                # tag on ports that belongs to flat network. DEAD VLAN tag is
+                # added to each newly created port.
+                # This is related to lp#1767422
                 phys_br.clear_db_attribute("Port", port['vif_name'], "tag")
-        elif phys_segmentation_id and network_type == 'vlan':
-            for port in ports:
+            elif phys_segmentation_id and network_type == 'vlan':
                 phys_br.set_db_attribute(
                     "Port", port['vif_name'], "tag", phys_segmentation_id)
+            # Clear vlan_mode that is added for each new port. lp#1930414
+            phys_br.clear_db_attribute("Port", port['vif_name'], "vlan_mode")
+            phys_br.clear_db_attribute("Port", port['vif_name'], "trunks")
