@@ -27,11 +27,11 @@ import testtools
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.l3 import agent as neutron_l3_agent
+from neutron.agent.l3 import dvr_local_router
 from neutron.agent.l3 import namespaces
 from neutron.agent.l3 import router_info as l3_router_info
 from neutron.agent import l3_agent as l3_agent_main
 from neutron.agent.linux import external_process
-from neutron.agent.linux import interface
 from neutron.agent.linux import ip_lib
 from neutron.agent.linux import keepalived
 from neutron.agent.metadata import driver as metadata_driver
@@ -39,11 +39,14 @@ from neutron.common import utils as common_utils
 from neutron.conf.agent import common as agent_config
 from neutron.conf.agent.l3 import config as l3_config
 from neutron.conf import common as common_config
+from neutron.tests import base as test_base
 from neutron.tests.common import l3_test_common
 from neutron.tests.common import net_helpers
 from neutron.tests.functional import base
 
 
+LOG = logging.getLogger(__name__)
+
 _uuid = uuidutils.generate_uuid
 
 OVS_INTERFACE_DRIVER = 'neutron.agent.linux.interface.OVSInterfaceDriver'
@@ -95,6 +98,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         self.mock_plugin_api = mock.patch(
             'neutron.agent.l3.agent.L3PluginApi').start().return_value
         mock.patch('neutron.agent.rpc.PluginReportStateAPI').start()
+        mock.patch('neutron.agent.common.ovs_lib.'
+                   'OVSBridge._set_port_dead').start()
         l3_config.register_l3_agent_config_opts(l3_config.OPTS, cfg.CONF)
         self.conf = self._configure_agent('agent1')
         self.agent = neutron_l3_agent.L3NATAgentWithStateReport('agent1',
@@ -168,6 +173,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                                                   qos_policy_id=qos_policy_id)
 
     def change_router_state(self, router_id, state):
+        LOG.debug("Router %s state changed to '%s'", router_id, state)
         ri = self.agent.router_info.get(router_id)
         if not ri:
             self.fail('Router %s is not present in the L3 agent' % router_id)
@@ -199,13 +205,14 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         self.addCleanup(netcat.stop_processes)
 
         def assert_num_of_conntrack_rules(n):
-            out = router_ns.netns.execute(["conntrack", "-L",
-                                           "--orig-src", client_address])
+            out = router_ns.netns.execute(
+                ["conntrack", "-L", "--orig-src", client_address],
+                privsep_exec=True)
             self.assertEqual(
                 n, len([line for line in out.strip().split('\n') if line]))
 
         if ha:
-            common_utils.wait_until_true(lambda: router.ha_state == 'primary')
+            self.wait_until_ha_router_has_state(router, 'primary')
 
         with self.assert_max_execution_time(100):
             assert_num_of_conntrack_rules(0)
@@ -269,13 +276,27 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
     def _assert_external_device(self, router):
         self.assertTrue(self._check_external_device(router))
 
+    def _wait_until_addr_gen_mode_has_state(
+            self, ns_name, state):
+        ip_wrapper = ip_lib.IPWrapper(namespace=ns_name)
+
+        def _addr_gen_mode_state():
+            addr_gen_mode_state = ip_wrapper.netns.execute(
+                ['sysctl', '-b', 'net.ipv6.conf.all.addr_gen_mode'],
+                privsep_exec=True)
+            return (
+                state == int(addr_gen_mode_state))
+
+        common_utils.wait_until_true(_addr_gen_mode_state)
+
     def _wait_until_ipv6_accept_ra_has_state(
             self, ns_name, device_name, enabled):
         ip_wrapper = ip_lib.IPWrapper(namespace=ns_name)
 
         def _ipv6_accept_ra_state():
-            ra_state = ip_wrapper.netns.execute(['sysctl', '-b',
-                'net.ipv6.conf.%s.accept_ra' % device_name])
+            ra_state = ip_wrapper.netns.execute(
+                ['sysctl', '-b', 'net.ipv6.conf.%s.accept_ra' % device_name],
+                privsep_exec=True)
             return (
                 enabled == (int(ra_state) != constants.ACCEPT_RA_DISABLED))
 
@@ -325,7 +346,7 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
         router.process()
 
         if enable_ha:
-            common_utils.wait_until_true(lambda: router.ha_state == 'primary')
+            self.wait_until_ha_router_has_state(router, 'primary')
 
             # Keepalived notifies of a state transition when it starts,
             # not when it ends. Thus, we have to wait until keepalived finishes
@@ -387,26 +408,8 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
 
     def manage_router(self, agent, router):
         self.addCleanup(agent._safe_router_removed, router['id'])
-
-        # NOTE(mangelajo): Neutron functional for l3 don't rely on openvswitch
-        #                  agent tagging ports, and all ports remain untagged
-        #                  during test execution.
-        #                  Workaround related to lp#1767422 plugs new ports as
-        #                  dead vlan (4095) to avoid issues, we need to remove
-        #                  such tag during functional l3 testing.
-        original_plug_new = interface.OVSInterfaceDriver.plug_new
-
-        def new_ovs_plug(self, *args, **kwargs):
-            original_plug_new(self, *args, **kwargs)
-            bridge = (kwargs.get('bridge') or args[4] or
-                      self.conf.OVS.integration_bridge)
-            device_name = kwargs.get('device_name') or args[2]
-            ovsbr = ovs_lib.OVSBridge(bridge)
-            ovsbr.clear_db_attribute('Port', device_name, 'tag')
-
-        with mock.patch(OVS_INTERFACE_DRIVER + '.plug_new', autospec=True) as (
-                ovs_plug):
-            ovs_plug.side_effect = new_ovs_plug
+        with mock.patch.object(dvr_local_router.DvrLocalRouter,
+                               '_load_used_fip_information'):
             agent._process_added_router(router)
 
         return agent.router_info[router['id']]
@@ -519,19 +522,22 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                 msg = 'PID file %s is not present.' % pid_file
             self.fail(msg)
 
-    def _assert_snat_chains(self, router):
-        self.assertFalse(router.iptables_manager.is_chain_empty(
-            'nat', 'snat'))
-        self.assertFalse(router.iptables_manager.is_chain_empty(
-            'nat', 'POSTROUTING'))
+    def _assert_snat_chains(self, router, enable_gw=True):
+        check = self.assertFalse if enable_gw else self.assertTrue
+        check(router.iptables_manager.is_chain_empty('nat', 'snat'))
+        check(router.iptables_manager.is_chain_empty('nat', 'POSTROUTING'))
 
-    def _assert_floating_ip_chains(self, router, snat_bound_fip=False):
+    def _assert_floating_ip_chains(self, router, snat_bound_fip=False,
+                                   enable_gw=True):
         if snat_bound_fip:
-            self.assertFalse(router.snat_iptables_manager.is_chain_empty(
-                'nat', 'float-snat'))
+            if enable_gw:
+                self.assertFalse(router.snat_iptables_manager.is_chain_empty(
+                    'nat', 'float-snat'))
+            else:
+                self.assertIsNone(router.snat_iptables_manager)
 
-        self.assertFalse(router.iptables_manager.is_chain_empty(
-            'nat', 'float-snat'))
+        check = self.assertFalse if enable_gw else self.assertTrue
+        check(router.iptables_manager.is_chain_empty('nat', 'float-snat'))
 
     def _assert_iptables_rules_converged(self, router):
         # if your code is failing on this line, it means you are not generating
@@ -558,26 +564,32 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             self.assertTrue(self.device_exists_with_ips_and_mac(
                 device, router.get_internal_device_name, router.ns_name))
 
-    def _assert_extra_routes(self, router, namespace=None):
+    def _assert_extra_routes(self, router, namespace=None, enable_gw=True):
         if namespace is None:
             namespace = router.ns_name
-        routes = ip_lib.get_routing_table(4, namespace=namespace)
-        routes = [{'nexthop': route['nexthop'],
-                   'destination': route['destination']} for route in routes]
+        routes = ip_lib.list_ip_routes(namespace, constants.IP_VERSION_4)
+        routes = [{'nexthop': route['via'],
+                   'destination': route['cidr']} for route in routes]
 
         for extra_route in router.router['routes']:
-            self.assertIn(extra_route, routes)
+            check = self.assertIn if enable_gw else self.assertNotIn
+            check(extra_route, routes)
 
     def _assert_onlink_subnet_routes(
-            self, router, ip_versions, namespace=None):
+            self, router, ip_versions, namespace=None, enable_gw=True):
         ns_name = namespace or router.ns_name
         routes = []
         for ip_version in ip_versions:
-            _routes = ip_lib.get_routing_table(ip_version,
-                                               namespace=ns_name)
+            _routes = ip_lib.list_ip_routes(ns_name, ip_version)
             routes.extend(_routes)
-        routes = set(route['destination'] for route in routes)
-        extra_subnets = router.get_ex_gw_port()['extra_subnets']
+        routes = set(route['cidr'] for route in routes)
+        ex_gw_port = router.get_ex_gw_port()
+        if not ex_gw_port:
+            if not enable_gw:
+                return
+            self.fail('GW port is enabled but not present in the router')
+
+        extra_subnets = ex_gw_port['extra_subnets']
         for extra_subnet in (route['cidr'] for route in extra_subnets):
             self.assertIn(extra_subnet, routes)
 
@@ -649,36 +661,49 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
                                         check_external_device=True):
 
         try:
-            common_utils.wait_until_true(
-                lambda: router1.ha_state == 'primary')
+            self.wait_until_ha_router_has_state(router1, 'primary')
             if check_external_device:
                 common_utils.wait_until_true(
                     lambda: self._check_external_device(router1))
             primary_router = router1
             backup_router = router2
         except common_utils.WaitTimeout:
-            common_utils.wait_until_true(
-                lambda: router2.ha_state == 'primary')
+            self.wait_until_ha_router_has_state(router2, 'primary')
             if check_external_device:
                 common_utils.wait_until_true(
                     lambda: self._check_external_device(router2))
             primary_router = router2
             backup_router = router1
 
-        common_utils.wait_until_true(
-                lambda: primary_router.ha_state == 'primary')
+        self.wait_until_ha_router_has_state(primary_router, 'primary')
         if check_external_device:
             common_utils.wait_until_true(
                 lambda: self._check_external_device(primary_router))
-        common_utils.wait_until_true(
-            lambda: backup_router.ha_state == 'backup')
+        self.wait_until_ha_router_has_state(backup_router, 'backup')
+
+        LOG.debug("Found primary router %s and backup router %s",
+                  primary_router.router_id, backup_router.router_id)
         return primary_router, backup_router
 
     def fail_ha_router(self, router):
         device_name = router.get_ha_device_name()
+        LOG.debug("Failing HA router %s by setting device %s to DOWN",
+                  router.router_id, device_name)
         ha_device = ip_lib.IPDevice(device_name, router.ha_namespace)
         ha_device.link.set_down()
 
+    @test_base.unstable_test("bug 1956958")
+    def wait_until_ha_router_has_state(self, router, expected_state):
+
+        def router_has_expected_state():
+            state = router.ha_state
+            LOG.debug("Router %s; current state is '%s', "
+                      "expected state is '%s'",
+                      router.router_id, state, expected_state)
+            return state == expected_state
+
+        common_utils.wait_until_true(router_has_expected_state)
+
     @staticmethod
     def fail_gw_router_port(router):
         r_br = ip_lib.IPDevice(router.driver.conf.OVS.integration_bridge)
@@ -724,3 +749,26 @@ class L3AgentTestFramework(base.BaseSudoTestCase):
             expected_address, ping_results[1],
             ("Expect to see %s in the reply of ping, but failed" %
              expected_address))
+
+    def _assert_route_in_routes(self, router, expected_route):
+        updated_route = ip_lib.list_ip_routes(
+            router.ns_name,
+            ip_version=constants.IP_VERSION_4, )
+
+        actual_routes = [{key: route[key] for key in expected_route.keys()}
+                         for route in updated_route]
+        self.assertIn(expected_route, actual_routes)
+
+    def _assert_ecmp_route_in_routes(self, router, expected_route):
+        updated_route = ip_lib.list_ip_routes(
+            router.ns_name,
+            ip_version=constants.IP_VERSION_4)
+        routes_actual = [{key: route[key] for key in expected_route.keys()}
+                         for route in updated_route]
+        for entry in routes_actual:
+            if entry['via']:
+                if isinstance(entry['via'], (list, tuple)):
+                    via_list = [{'via': hop['via']}
+                                for hop in entry['via']]
+                    entry['via'] = sorted(via_list, key=lambda i: i['via'])
+        self.assertIn(expected_route, routes_actual)
