@@ -47,6 +47,13 @@ WRONG_IP = '0.0.0.0'
 TEST_IP = '240.0.0.1'
 TEST_IP_NEIGH = '240.0.0.2'
 TEST_IP_SECONDARY = '240.0.0.3'
+TEST_IP6_NEIGH = 'fd00::2'
+TEST_IP6_SECONDARY = 'fd00::3'
+TEST_IP6_VXLAN_GROUP = 'ff00::1'
+TEST_IP_NUD_STATES = ((TEST_IP_NEIGH, 'permanent'),
+                      (TEST_IP_SECONDARY, 'reachable'),
+                      (TEST_IP6_NEIGH, 'permanent'),
+                      (TEST_IP6_SECONDARY, 'reachable'))
 
 
 class IpLibTestFramework(functional_base.BaseSudoTestCase):
@@ -190,6 +197,20 @@ class IpLibTestCase(IpLibTestFramework):
             for expected_rule in expected_rules[ip_version]:
                 self.assertNotIn(expected_rule, rules)
 
+    def test_add_ip_rule_default_table(self):
+        attr = self.generate_device_details()
+        device = self.manage_device(attr)
+        test_cases = {
+            constants.IP_VERSION_4: {'ip': '1.1.1.1', 'to': '8.8.8.0/24'},
+            constants.IP_VERSION_6: {'ip': 'abcd::1', 'to': '1234::/64'}
+        }
+        for ip_version, rule in test_cases.items():
+            ip_lib.add_ip_rule(namespace=device.namespace, **rule)
+            rules = ip_lib.list_ip_rules(device.namespace, ip_version)
+            for _rule in rules:
+                if _rule['from'] == rule['ip'] and _rule['to'] == rule['to']:
+                    self.assertEqual('default', _rule['table'])
+
     def test_device_exists(self):
         attr = self.generate_device_details()
 
@@ -232,9 +253,26 @@ class IpLibTestCase(IpLibTestFramework):
         attr = self.generate_device_details()
         ip = ip_lib.IPWrapper(namespace=attr.namespace)
         ip.netns.add(attr.namespace)
+        ip.add_dummy('dummy_device')
         self.addCleanup(ip.netns.delete, attr.namespace)
         self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
-        device = ip.add_vxlan(attr.name, 9999)
+        device = ip.add_vxlan(attr.name, 9999, 'dummy_device')
+        self.addCleanup(self._safe_delete_device, device)
+        self.assertTrue(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
+        device.link.delete()
+        self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
+
+    def test_ipv6_vxlan_exists(self):
+        attr = self.generate_device_details(
+            name='test_device', ip_cidrs=["%s/24" % TEST_IP, 'fd00::1/64']
+        )
+        self.manage_device(attr)
+        ip = ip_lib.IPWrapper(namespace=attr.namespace)
+        ip.netns.add(attr.namespace)
+        self.addCleanup(ip.netns.delete, attr.namespace)
+        self.assertFalse(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
+        device = ip.add_vxlan('test_vxlan_device', 9999, local='fd00::1',
+            group=TEST_IP6_VXLAN_GROUP, dev='test_device')
         self.addCleanup(self._safe_delete_device, device)
         self.assertTrue(ip_lib.vxlan_in_use(9999, namespace=attr.namespace))
         device.link.delete()
@@ -364,49 +402,6 @@ class IpLibTestCase(IpLibTestFramework):
             self.assertIsNone(
                 device.route.get_gateway(ip_version=ip_version))
 
-    def test_get_routing_table(self):
-        attr = self.generate_device_details(
-            ip_cidrs=["%s/24" % TEST_IP, "fd00::1/64"]
-        )
-        device = self.manage_device(attr)
-        device_ip = attr.ip_cidrs[0].split('/')[0]
-        destination = '8.8.8.0/24'
-        device.route.add_route(destination, device_ip)
-
-        destination6 = 'fd01::/64'
-        device.route.add_route(destination6, "fd00::2")
-
-        expected_routes = [{'nexthop': device_ip,
-                            'device': attr.name,
-                            'destination': destination,
-                            'scope': 'universe'},
-                           {'nexthop': None,
-                            'device': attr.name,
-                            'destination': str(
-                                netaddr.IPNetwork(attr.ip_cidrs[0]).cidr),
-                            'scope': 'link'}]
-
-        routes = ip_lib.get_routing_table(4, namespace=attr.namespace)
-        self.assertItemsEqual(expected_routes, routes)
-        self.assertIsInstance(routes, list)
-
-        expected_routes6 = [{'nexthop': "fd00::2",
-                             'device': attr.name,
-                             'destination': destination6,
-                             'scope': 'universe'},
-                            {'nexthop': None,
-                             'device': attr.name,
-                             'destination': str(
-                                 netaddr.IPNetwork(attr.ip_cidrs[1]).cidr),
-                             'scope': 'universe'}]
-        routes6 = ip_lib.get_routing_table(6, namespace=attr.namespace)
-        self.assertItemsEqual(expected_routes6, routes6)
-        self.assertIsInstance(routes6, list)
-
-    def test_get_routing_table_no_namespace(self):
-        with testtools.ExpectedException(ip_lib.NetworkNamespaceNotFound):
-            ip_lib.get_routing_table(4, namespace="nonexistent-netns")
-
     def test_get_neigh_entries(self):
         attr = self.generate_device_details(
             ip_cidrs=["%s/24" % TEST_IP, "fd00::1/64"]
@@ -417,10 +412,11 @@ class IpLibTestCase(IpLibTestFramework):
 
         expected_neighs = [{'dst': TEST_IP_NEIGH,
                             'lladdr': mac_address,
-                            'device': attr.name}]
+                            'device': attr.name,
+                            'state': 'permanent'}]
 
         neighs = device.neigh.dump(4)
-        self.assertItemsEqual(expected_neighs, neighs)
+        self.assertCountEqual(expected_neighs, neighs)
         self.assertIsInstance(neighs, list)
 
         device.neigh.delete(TEST_IP_NEIGH, mac_address)
@@ -450,6 +446,41 @@ class IpLibTestCase(IpLibTestFramework):
         # trying to delete a non-existent entry shouldn't raise an error
         device.neigh.delete(TEST_IP_NEIGH, mac_address)
 
+    def test_flush_neigh_ipv4(self):
+        # Entry with state "reachable" deleted.
+        self._flush_neigh(constants.IP_VERSION_4, TEST_IP_SECONDARY,
+                          {TEST_IP_NEIGH})
+        # Entries belong to "ip_to_flush" passed CIDR, but "permanent" entry
+        # is not deleted.
+        self._flush_neigh(constants.IP_VERSION_4, '240.0.0.0/28',
+                          {TEST_IP_NEIGH})
+        # "all" passed, but "permanent" entry is not deleted.
+        self._flush_neigh(constants.IP_VERSION_4, 'all', {TEST_IP_NEIGH})
+
+    def test_flush_neigh_ipv6(self):
+        # Entry with state "reachable" deleted.
+        self._flush_neigh(constants.IP_VERSION_6, TEST_IP6_SECONDARY,
+                          {TEST_IP6_NEIGH})
+        # Entries belong to "ip_to_flush" passed CIDR, but "permanent" entry
+        # is not deleted.
+        self._flush_neigh(constants.IP_VERSION_6, 'fd00::0/64',
+                          {TEST_IP6_NEIGH})
+        # "all" passed, but "permanent" entry is not deleted.
+        self._flush_neigh(constants.IP_VERSION_6, 'all', {TEST_IP6_NEIGH})
+
+    def _flush_neigh(self, version, ip_to_flush, ips_expected):
+        attr = self.generate_device_details(
+            ip_cidrs=['%s/24' % TEST_IP, 'fd00::1/64'],
+            namespace=utils.get_rand_name(20, 'ns-'))
+        device = self.manage_device(attr)
+        for test_ip, nud_state in TEST_IP_NUD_STATES:
+            mac_address = net.get_random_mac('fa:16:3e:00:00:00'.split(':'))
+            device.neigh.add(test_ip, mac_address, nud_state)
+
+        device.neigh.flush(version, ip_to_flush)
+        ips = {e['dst'] for e in device.neigh.dump(version)}
+        self.assertEqual(ips_expected, ips)
+
     def _check_for_device_name(self, ip, name, should_exist):
         exist = any(d for d in ip.get_devices() if d.name == name)
         self.assertEqual(should_exist, exist)
@@ -574,7 +605,7 @@ class IpLibTestCase(IpLibTestFramework):
              ip_info['scope'],
              ip_info['broadcast']) for
             ip_info in device.addr.list()]
-        self.assertItemsEqual(ip_addresses, device_ips_info)
+        self.assertCountEqual(ip_addresses, device_ips_info)
 
     def _flush_ips(self, device, ip_version):
         device.addr.flush(ip_version)
@@ -660,10 +691,16 @@ class NamespaceTestCase(functional_base.BaseSudoTestCase):
         ip_lib.delete_network_namespace(self.namespace)
 
     def test_network_namespace_exists_ns_exists(self):
-        self.assertTrue(ip_lib.network_namespace_exists(self.namespace))
+        for use_helper_for_ns_read in (True, False):
+            cfg.CONF.set_override('use_helper_for_ns_read',
+                                  use_helper_for_ns_read, 'AGENT')
+            self.assertTrue(ip_lib.network_namespace_exists(self.namespace))
 
     def test_network_namespace_exists_ns_doesnt_exists(self):
-        self.assertFalse(ip_lib.network_namespace_exists('another_ns'))
+        for use_helper_for_ns_read in (True, False):
+            cfg.CONF.set_override('use_helper_for_ns_read',
+                                  use_helper_for_ns_read, 'AGENT')
+            self.assertFalse(ip_lib.network_namespace_exists('another_ns'))
 
     def test_network_namespace_exists_ns_exists_try_is_ready(self):
         self.assertTrue(ip_lib.network_namespace_exists(self.namespace,
@@ -734,7 +771,7 @@ class IpMonitorTestCase(testscenarios.WithScenarios,
                 if ip_address not in registers:
                     return False
             return True
-        except (OSError, IOError):
+        except (OSError, IOError, ValueError):
             return False
 
     def _check_read_file(self, ip_addresses):
@@ -789,7 +826,7 @@ class IpMonitorTestCase(testscenarios.WithScenarios,
              'name': self.devices[4]}]
         self._check_read_file(ip_addresses)
 
-    def test_interface_added_after_initilization(self):
+    def test_interface_added_after_initialization(self):
         for device in self.devices[:len(self.devices) - 1]:
             self.ip_wrapper.add_dummy(device)
         utils.wait_until_true(lambda: self._read_file({}), timeout=30)
@@ -877,7 +914,8 @@ class IpRouteCommandTestCase(functional_base.BaseSudoTestCase):
                'scope': scope,
                'device': 'test_device',
                'via': via,
-               'metric': metric}
+               'metric': metric,
+               'proto': 'static'}
         try:
             utils.wait_until_true(fn, timeout=5)
         except utils.WaitTimeout:
@@ -1094,3 +1132,63 @@ class GetDevicesWithIpTestCase(functional_base.BaseSudoTestCase):
             ip_addresses = self._remove_loopback_interface(ip_addresses)
             ip_addresses = self._remove_ipv6_scope_link(ip_addresses)
             self.assertEqual(0, len(ip_addresses))
+
+
+class ListIpRoutesTestCase(functional_base.BaseSudoTestCase):
+
+    def setUp(self):
+        super().setUp()
+        self.namespace = self.useFixture(net_helpers.NamespaceFixture()).name
+        self.device_names = ['test_device1', 'test_device2']
+        self.device_ips = ['10.0.0.1/24', '10.0.1.1/24']
+        self.device_cidrs = [netaddr.IPNetwork(ip_address).cidr for ip_address
+                             in self.device_ips]
+        for idx, dev in enumerate(self.device_names):
+            ip_lib.IPWrapper(self.namespace).add_dummy(dev)
+            device = ip_lib.IPDevice(dev, namespace=self.namespace)
+            device.link.set_up()
+            device.addr.add(self.device_ips[idx])
+
+    def test_list_ip_routes_multipath(self):
+        multipath = [
+            {'device': self.device_names[0],
+             'via': str(self.device_cidrs[0].ip + 100), 'weight': 10},
+            {'device': self.device_names[1],
+             'via': str(self.device_cidrs[1].ip + 100), 'weight': 20},
+            {'via': str(self.device_cidrs[1].ip + 101), 'weight': 30},
+            {'via': str(self.device_cidrs[1].ip + 102)}]
+        ip_lib.add_ip_route(self.namespace, '1.2.3.0/24',
+                            constants.IP_VERSION_4, via=multipath)
+
+        routes = ip_lib.list_ip_routes(self.namespace, constants.IP_VERSION_4)
+        multipath[2]['device'] = self.device_names[1]
+        multipath[3]['device'] = self.device_names[1]
+        multipath[3]['weight'] = 1
+        for route in (route for route in routes if
+                      route['cidr'] == '1.2.3.0/24'):
+            if not isinstance(route['via'], list):
+                continue
+
+            self.assertEqual(len(multipath), len(route['via']))
+            for nexthop in multipath:
+                for mp in route['via']:
+                    if nexthop != mp:
+                        continue
+                    break
+                else:
+                    self.fail('Not matching route, routes: %s' % routes)
+
+            return
+
+        self.fail('Not matching route, routes: %s' % routes)
+
+
+class IpLinkCommandTestCase(IpLibTestFramework):
+
+    def test_set_netns(self):
+        device_name = ('int_' + uuidutils.generate_uuid())[
+                      :constants.DEVICE_NAME_MAX_LEN]
+        device = ip_lib.IPDevice(device_name, kind='dummy')
+        device.link.create()
+        namespace = self.useFixture(net_helpers.NamespaceFixture())
+        device.link.set_netns(namespace.name)
