@@ -22,6 +22,7 @@ import fixtures
 import netaddr
 from neutron_lib import constants as lib_const
 from oslo_config import fixture as fixture_config
+from oslo_log import log as logging
 from oslo_utils import uuidutils
 
 from neutron.agent.common import ovs_lib
@@ -39,6 +40,8 @@ from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import helpers
 from neutron.tests.functional import base
 
+LOG = logging.getLogger(__name__)
+
 
 class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
@@ -85,6 +88,9 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
         self.conf.set_override('check_child_processes_interval', 1, 'AGENT')
 
+        mock.patch('neutron.agent.common.ovs_lib.'
+                   'OVSBridge._set_port_dead').start()
+
     def network_dict_for_dhcp(self, dhcp_enabled=True,
                               ip_version=lib_const.IP_VERSION_4,
                               prefix_override=None):
@@ -105,6 +111,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
                            ip_version=lib_const.IP_VERSION_4,
                            prefix_override=None):
         cidr = self._IP_ADDRS[ip_version]['cidr']
+        spool_id = uuidutils.generate_uuid()
         if prefix_override is not None:
             cidr = '/'.join((cidr.split('/')[0], str(prefix_override)))
         sn_dict = dhcp.DictModel(
@@ -112,6 +119,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
             network_id=net_id,
             ip_version=ip_version,
             cidr=cidr,
+            subnetpool_id=spool_id,
             gateway_ip=self._IP_ADDRS[ip_version]['gateway'],
             enable_dhcp=dhcp_enabled,
             dns_nameservers=[],
@@ -147,7 +155,7 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
                                  non_local_subnets=non_local_subnets,
                                  ports=ports,
                                  admin_state_up=True,
-                                 tenant_id=uuidutils.generate_uuid())
+                                 project_id=uuidutils.generate_uuid())
         return net_dict
 
     def get_interface_name(self, network, port):
@@ -178,7 +186,8 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
     def assert_accept_ra_disabled(self, namespace):
         actual = ip_lib.IPWrapper(namespace=namespace).netns.execute(
-            ['sysctl', '-b', 'net.ipv6.conf.default.accept_ra'])
+            ['sysctl', '-b', 'net.ipv6.conf.default.accept_ra'],
+            privsep_exec=True)
         self.assertEqual('0', actual)
 
     def assert_dhcp_device(self, namespace, dhcp_iface_name, dhcp_enabled):
@@ -198,7 +207,9 @@ class DHCPAgentOVSTestFramework(base.BaseSudoTestCase):
 
     def _ip_list_for_vif(self, vif_name, namespace):
         ip_device = ip_lib.IPDevice(vif_name, namespace)
-        return ip_device.addr.list(ip_version=lib_const.IP_VERSION_4)
+        res = ip_device.addr.list(ip_version=lib_const.IP_VERSION_4)
+        LOG.debug("IP addr list of %s: %s", vif_name, res)
+        return res
 
     def _get_network_port_for_allocation_test(self):
         network = self.network_dict_for_dhcp()
