@@ -18,14 +18,13 @@ from unittest import mock
 import uuid
 
 from neutron_lib import constants as const
+from neutron_lib.plugins.ml2 import ovs_constants as agent_const
 from oslo_config import cfg
 from ovsdbapp.backend.ovs_idl import idlutils
 
 from neutron.agent.common import ovs_lib
 from neutron.agent.linux import ip_lib
 from neutron.common import utils
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import (
-    constants as agent_const)
 from neutron.tests.common import net_helpers
 from neutron.tests.functional.agent.linux import base
 
@@ -92,6 +91,10 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
                                                     'external_ids')['test'])
         self.assertEqual(agent_const.DEAD_VLAN_TAG,
                          self.br.db_get_val('Port', port_name, 'tag'))
+        self.assertEqual("trunk",
+                         self.br.db_get_val('Port', port_name, 'vlan_mode'))
+        self.assertEqual(4095,
+                         self.br.db_get_val('Port', port_name, 'trunks'))
 
     def test_attribute_lifecycle(self):
         (port_name, ofport) = self.create_ovs_port()
@@ -417,55 +420,6 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
                                                 controller,
                                                 'connection_mode'))
 
-    def test_egress_bw_limit(self):
-        port_name, _ = self.create_ovs_port()
-        self.br.create_egress_bw_limit_for_port(port_name, 700, 70)
-        max_rate, burst = self.br.get_egress_bw_limit_for_port(port_name)
-        self.assertEqual(700, max_rate)
-        self.assertEqual(70, burst)
-        self.br.delete_egress_bw_limit_for_port(port_name)
-        max_rate, burst = self.br.get_egress_bw_limit_for_port(port_name)
-        self.assertIsNone(max_rate)
-        self.assertIsNone(burst)
-
-    def test_ingress_bw_limit(self):
-        port_name, _ = self.create_ovs_port()
-        self.br.update_ingress_bw_limit_for_port(port_name, 700, 70)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_port(port_name)
-        self.assertEqual(700, max_rate)
-        self.assertEqual(70, burst)
-
-        self.br.update_ingress_bw_limit_for_port(port_name, 750, 100)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_port(port_name)
-        self.assertEqual(750, max_rate)
-        self.assertEqual(100, burst)
-
-        self.br.delete_ingress_bw_limit_for_port(port_name)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_port(port_name)
-        self.assertIsNone(max_rate)
-        self.assertIsNone(burst)
-
-    def test_ingress_bw_limit_dpdk_port(self):
-        port_name, _ = self.create_ovs_port(
-            ('type', agent_const.OVS_DPDK_VHOST_USER))
-        self.br.update_ingress_bw_limit_for_port(port_name, 700, 70)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_dpdk_port(
-            port_name)
-        self.assertEqual(700, max_rate)
-        self.assertEqual(70, burst)
-
-        self.br.update_ingress_bw_limit_for_port(port_name, 750, 100)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_dpdk_port(
-            port_name)
-        self.assertEqual(750, max_rate)
-        self.assertEqual(100, burst)
-
-        self.br.delete_ingress_bw_limit_for_port(port_name)
-        max_rate, burst = self.br.get_ingress_bw_limit_for_dpdk_port(
-            port_name)
-        self.assertIsNone(max_rate)
-        self.assertIsNone(burst)
-
     def test_db_create_references(self):
         with self.ovs.ovsdb.transaction(check_error=True) as txn:
             queue = txn.add(self.ovs.ovsdb.db_create("Queue",
@@ -489,7 +443,7 @@ class OVSBridgeTestCase(OVSBridgeTestBase):
         expected = self.br.initial_protocols.union(protocols)
         self.br.ovsdb.db_add("Bridge", self.br.br_name, "protocols",
                              *protocols).execute(check_error=True)
-        self.assertItemsEqual(expected,
+        self.assertCountEqual(expected,
                               self.br.db_get_val('Bridge',
                                                  self.br.br_name, "protocols"))
 
@@ -637,4 +591,4 @@ class OVSLibTestCase(base.BaseOVSLinuxTestCase):
         self.assertTrue(tags_present)
         tags_42 = [t for t in tags_present if t['tag'] == 42]
         self.assertEqual(tags_42, single_value.result)
-        self.assertItemsEqual(len_0_list.result, tags_present)
+        self.assertCountEqual(len_0_list.result, tags_present)
