@@ -29,10 +29,12 @@ from oslo_config import cfg
 from oslo_db import exception as os_db_exc
 from oslo_db.sqlalchemy import provision
 from oslo_log import log
+from oslo_utils import timeutils
 from oslo_utils import uuidutils
 
 from neutron.agent.linux import utils
 from neutron.api import extensions as exts
+from neutron.common import utils as n_utils
 from neutron.conf.agent import common as config
 from neutron.conf.agent import ovs_conf
 from neutron.conf.plugins.ml2 import config as ml2_config
@@ -41,6 +43,9 @@ from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 # the SqlFixture
 from neutron.db import models  # noqa
 from neutron import manager
+from neutron.plugins.ml2.drivers.ovn.agent import neutron_agent
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.extensions import \
+    placement as ovn_client_placement
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import worker
 from neutron.plugins.ml2.drivers import type_geneve  # noqa
 from neutron import service  # noqa
@@ -69,7 +74,7 @@ def config_decorator(method_to_decorate, config_tuples):
 
 class BaseLoggingTestCase(base.BaseTestCase):
     def setUp(self):
-        super(BaseLoggingTestCase, self).setUp()
+        super().setUp()
         # NOTE(slaweq): Because of issue with stestr and Python3, we need
         # to avoid too much output to be produced during tests, so we will
         # ignore python warnings here
@@ -162,7 +167,11 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
     _counter = 0
     l3_plugin = 'neutron.services.ovn_l3.plugin.OVNL3RouterPlugin'
 
-    def setUp(self, maintenance_worker=False):
+    def preSetUp(self):
+        self.temp_dir = self.useFixture(fixtures.TempDir()).path
+        self._start_ovsdb_server()
+
+    def setUp(self, maintenance_worker=False, service_plugins=None):
         ml2_config.cfg.CONF.set_override('extension_drivers',
                                      self._extension_drivers,
                                      group='ml2')
@@ -172,12 +181,18 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         ml2_config.cfg.CONF.set_override('vni_ranges',
                                      ['1:65536'],
                                      group='ml2_type_geneve')
+        # ensure viable minimum is set for OVN's Geneve
+        ml2_config.cfg.CONF.set_override('max_header_size', 38,
+                                         group='ml2_type_geneve')
+        ovn_conf.register_opts()
         ovn_conf.cfg.CONF.set_override('dns_servers',
                                        ['10.10.10.10'],
                                        group='ovn')
         ovn_conf.cfg.CONF.set_override('api_workers', 1)
 
         self.addCleanup(exts.PluginAwareExtensionManager.clear_instance)
+        self.ovsdb_server_mgr = None
+        self._service_plugins = service_plugins
         super(TestOVNFunctionalBase, self).setUp()
         self.test_log_dir = os.path.join(DEFAULT_LOG_DIR, self.id())
         base.setup_test_logging(
@@ -191,12 +206,33 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self.pf_plugin = manager.NeutronManager.load_class_for_provider(
             'neutron.service_plugins', 'port_forwarding')()
         self.pf_plugin._rpc_notifications_required = False
-        self.ovsdb_server_mgr = None
+        self.log_plugin = directory.get_plugin(constants.LOG_API)
+        if not self.log_plugin:
+            self.log_plugin = manager.NeutronManager.load_class_for_provider(
+                'neutron.service_plugins', 'log')()
+            directory.add_plugin(constants.LOG_API, self.log_plugin)
+            self.log_plugin.driver_manager.register_driver(
+                self.mech_driver.log_driver)
+        self.mech_driver.log_driver.plugin_driver = self.mech_driver
+        self.mech_driver.log_driver._log_plugin_property = None
         self.ovn_northd_mgr = None
         self.maintenance_worker = maintenance_worker
-        self.temp_dir = self.useFixture(fixtures.TempDir()).path
-        self._start_ovsdb_server_and_idls()
+        mock.patch(
+            'neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
+            'maintenance.MaintenanceThread').start()
+        mock.patch(
+            'neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb.'
+            'maintenance.HashRingHealthCheckPeriodics').start()
+        self._start_idls()
         self._start_ovn_northd()
+        self.addCleanup(self._reset_agent_cache_singleton)
+        self.addCleanup(self._reset_ovn_client_placement_extension)
+
+    def _reset_agent_cache_singleton(self):
+        neutron_agent.AgentCache._instance = None
+
+    def _reset_ovn_client_placement_extension(self):
+        ovn_client_placement.OVNClientPlacementExtension._instance = None
 
     def _get_install_share_path(self):
         lookup_paths = set()
@@ -227,8 +263,10 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
 
     def get_additional_service_plugins(self):
         p = super(TestOVNFunctionalBase, self).get_additional_service_plugins()
-        p.update({'revision_plugin_name': 'revisions'})
-        p.update({'segments': 'neutron.services.segments.plugin.Plugin'})
+        p.update({'revision_plugin_name': 'revisions',
+                  'segments': 'neutron.services.segments.plugin.Plugin'})
+        if self._service_plugins:
+            p.update(self._service_plugins)
         return p
 
     @property
@@ -241,14 +279,43 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
     def _start_ovn_northd(self):
         if not self.ovsdb_server_mgr:
             return
+
+        def wait_for_northd():
+            try:
+                self.nb_api.nb_global
+            except StopIteration:
+                LOG.debug("NB_Global is not ready yet")
+                return False
+
+            try:
+                next(iter(self.sb_api.db_list_rows('SB_Global').execute(
+                    check_error=True)))
+            except StopIteration:
+                LOG.debug("SB_Global is not ready yet")
+                return False
+            except KeyError:
+                # Maintenance worker doesn't register SB_Global therefore
+                # we don't need to wait for it
+                LOG.debug("SB_Global is not registered in this IDL")
+
+            return True
+
+        timeout = 20
         ovn_nb_db = self.ovsdb_server_mgr.get_ovsdb_connection_path('nb')
         ovn_sb_db = self.ovsdb_server_mgr.get_ovsdb_connection_path('sb')
+        LOG.debug("Starting OVN northd")
         self.ovn_northd_mgr = self.useFixture(
             process.OvnNorthd(self.temp_dir,
                               ovn_nb_db, ovn_sb_db,
                               protocol=self._ovsdb_protocol))
+        LOG.debug("OVN northd started: %r", self.ovn_northd_mgr)
+        n_utils.wait_until_true(
+            wait_for_northd, timeout, sleep=1,
+            exception=Exception(
+                "ovn-northd didn't initialize OVN DBs in %d"
+                "seconds" % timeout))
 
-    def _start_ovsdb_server_and_idls(self):
+    def _start_ovsdb_server(self):
         # Start 2 ovsdb-servers one each for OVN NB DB and OVN SB DB
         # ovsdb-server with OVN SB DB can be used to test the chassis up/down
         # events.
@@ -257,6 +324,8 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             process.OvsdbServer(self.temp_dir, install_share_path,
                                 ovn_nb_db=True, ovn_sb_db=True,
                                 protocol=self._ovsdb_protocol))
+        LOG.debug("OVSDB server manager instantiated: %r",
+                  self.ovsdb_server_mgr)
         set_cfg = cfg.CONF.set_override
         set_cfg('ovn_nb_connection',
                 self.ovsdb_server_mgr.get_ovsdb_connection_path(), 'ovn')
@@ -272,11 +341,13 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
 
         # NOTE(mjozefcz): We can find occasional functional test
         # failures because of low timeout value - set it to 30
-        # seconds, should be enought. More info: 1868110
+        # seconds, should be enough. More info: 1868110
         cfg.CONF.set_override(
             'ovsdb_connection_timeout', 30,
             'ovn')
+        self.addCleanup(self._collect_processes_logs)
 
+    def _start_idls(self):
         class TriggerCls(mock.MagicMock):
             def trigger(self):
                 pass
@@ -288,8 +359,10 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         else:
             trigger_cls.trigger.__self__.__class__ = neutron.wsgi.WorkerService
 
-        self.addCleanup(self._collect_processes_logs)
         self.addCleanup(self.stop)
+        # NOTE(ralonsoh): do not access to the DB at exit when the SQL
+        # connection is already closed, to avoid useless exception messages.
+        mock.patch.object(self.mech_driver, '_clean_hash_ring').start()
         self.mech_driver.pre_fork_initialize(
             mock.ANY, mock.ANY, trigger_cls.trigger)
 
@@ -297,10 +370,11 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
         self.mech_driver.post_fork_initialize(
             mock.ANY, mock.ANY, trigger_cls.trigger)
 
-        self.nb_api = self.mech_driver._nb_ovn
-        self.sb_api = self.mech_driver._sb_ovn
+        self.nb_api = self.mech_driver.nb_ovn
+        self.sb_api = self.mech_driver.sb_ovn
 
     def _collect_processes_logs(self):
+        timestamp = datetime.now().strftime('%y-%m-%d_%H-%M-%S')
         for database in ("nb", "sb"):
             for file_suffix in ("log", "db"):
                 src_filename = "ovn_%(db)s.%(suffix)s" % {
@@ -310,19 +384,30 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
                 dst_filename = "ovn_%(db)s-%(timestamp)s.%(suffix)s" % {
                     'db': database,
                     'suffix': file_suffix,
-                    'timestamp': datetime.now().strftime('%y-%m-%d_%H-%M-%S'),
+                    'timestamp': timestamp,
                 }
+                self._copy_log_file(src_filename, dst_filename)
 
-                filepath = os.path.join(self.temp_dir, src_filename)
-                shutil.copyfile(
-                    filepath, os.path.join(self.test_log_dir, dst_filename))
+        # Copy northd logs
+        northd_log = "ovn_northd"
+        dst_northd = "%(northd)s-%(timestamp)s.log" % {
+            "northd": northd_log,
+            "timestamp": timestamp,
+        }
+        self._copy_log_file("%s.log" % northd_log, dst_northd)
+
+    def _copy_log_file(self, src_filename, dst_filename):
+        """Copy log file from temporary dict to the test directory."""
+        filepath = os.path.join(self.temp_dir, src_filename)
+        shutil.copyfile(
+            filepath, os.path.join(self.test_log_dir, dst_filename))
 
     def stop(self):
         if self.maintenance_worker:
             self.mech_driver.nb_synchronizer.stop()
             self.mech_driver.sb_synchronizer.stop()
-        self.mech_driver._nb_ovn.ovsdb_connection.stop()
-        self.mech_driver._sb_ovn.ovsdb_connection.stop()
+        self.mech_driver.nb_ovn.ovsdb_connection.stop()
+        self.mech_driver.sb_ovn.ovsdb_connection.stop()
 
     def restart(self):
         self.stop()
@@ -333,13 +418,27 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             self.ovn_northd_mgr.stop()
 
         self.ovsdb_server_mgr.delete_dbs()
-        self._start_ovsdb_server_and_idls()
+        self._start_ovsdb_server()
+        self._start_idls()
         self._start_ovn_northd()
 
     def add_fake_chassis(self, host, physical_nets=None, external_ids=None,
-                         name=None):
+                         name=None, azs=None, enable_chassis_as_gw=False):
+        def append_cms_options(ext_ids, value):
+            if 'ovn-cms-options' not in ext_ids:
+                ext_ids['ovn-cms-options'] = value
+            else:
+                ext_ids['ovn-cms-options'] += ',' + value
+
         physical_nets = physical_nets or []
         external_ids = external_ids or {}
+        if azs is None:
+            azs = ['ovn']
+        if azs:
+            append_cms_options(external_ids, 'availability-zones=')
+            external_ids['ovn-cms-options'] += ':'.join(azs)
+        if enable_chassis_as_gw:
+            append_cms_options(external_ids, 'enable-chassis-as-gw')
 
         bridge_mapping = ",".join(["%s:br-provider%s" % (phys_net, i)
                                   for i, phys_net in enumerate(physical_nets)])
@@ -358,11 +457,16 @@ class TestOVNFunctionalBase(test_plugin.Ml2PluginV2TestCase,
             name, ['geneve'], '172.24.4.%d' % self._counter,
             external_ids=external_ids, hostname=host).execute(check_error=True)
         if self.sb_api.is_table_present('Chassis_Private'):
+            nb_cfg_timestamp = timeutils.utcnow_ts() * 1000
             self.sb_api.db_create(
                 'Chassis_Private', name=name, external_ids=external_ids,
-                chassis=chassis.uuid).execute(check_error=True)
+                chassis=chassis.uuid, nb_cfg_timestamp=nb_cfg_timestamp
+            ).execute(check_error=True)
         return name
 
     def del_fake_chassis(self, chassis, if_exists=True):
         self.sb_api.chassis_del(
             chassis, if_exists=if_exists).execute(check_error=True)
+        if self.sb_api.is_table_present('Chassis_Private'):
+            self.sb_api.db_destroy(
+                'Chassis_Private', chassis).execute(check_error=True)
