@@ -27,6 +27,7 @@ import sqlalchemy
 from sqlalchemy import event  # noqa
 from sqlalchemy.sql import ddl as sqla_ddl
 
+from neutron.common import config
 from neutron.db import migration as migration_root
 from neutron.db.migration.alembic_migrations import external
 from neutron.db.migration import cli as migration
@@ -35,7 +36,6 @@ from neutron.tests import base as test_base
 from neutron.tests.functional import base as functional_base
 from neutron.tests.unit import testlib_api
 
-cfg.CONF.import_opt('core_plugin', 'neutron.conf.common')
 
 CREATION_OPERATIONS = {
     'sqla': (sqla_ddl.CreateIndex,
@@ -143,6 +143,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
     TIMEOUT_SCALING_FACTOR = 4
 
     def setUp(self):
+        config.register_common_config_options()
         super(_TestModelsMigrations, self).setUp()
         self.cfg = self.useFixture(config_fixture.Config())
         self.cfg.config(core_plugin='ml2')
@@ -175,8 +176,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
     # Remove some difference that are not mistakes just specific of
     # dialects, etc
     def remove_unrelated_errors(self, element):
-        insp = sqlalchemy.engine.reflection.Inspector.from_engine(
-            self.get_engine())
+        insp = sqlalchemy.inspect(self.get_engine())
         dialect = self.get_engine().dialect.name
         if isinstance(element, tuple):
             if dialect == 'mysql' and element[0] == 'remove_index':
@@ -272,7 +272,7 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
             """Identify excepted operations that are allowed for the branch."""
             # For alembic the clause is AddColumn or DropColumn
             column = clauseelement.column.name
-            table = clauseelement.column.table.name
+            table = clauseelement.table_name
             element_name = '.'.join([table, column])
             for alembic_type, excepted_names in exceptions.items():
                 if alembic_type == sqlalchemy.Column:
@@ -287,11 +287,13 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
                 return is_excepted_alembic(clauseelement, exceptions)
             return True
 
-        def check_expand_branch(conn, clauseelement, multiparams, params):
+        def check_expand_branch(conn, clauseelement, multiparams, params,
+                                execution_options):
             if not is_allowed(clauseelement, drop_exceptions, DROP_OPERATIONS):
                 self.fail("Migration in expand branch contains drop command")
 
-        def check_contract_branch(conn, clauseelement, multiparams, params):
+        def check_contract_branch(conn, clauseelement, multiparams, params,
+                                  execution_options):
             if not is_allowed(clauseelement, creation_exceptions,
                               CREATION_OPERATIONS):
                 self.fail("Migration in contract branch contains create "
@@ -339,17 +341,11 @@ class _TestModelsMigrations(test_migrations.ModelsMigrationsSync):
             msg='Offline contract migration scripts are forbidden for Ocata+')
 
 
-class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
+class TestModelsMigrationsMySQL(testlib_api.MySQLTestCaseMixin,
                                 _TestModelsMigrations,
                                 testlib_api.SqlTestCaseLight,
                                 functional_base.BaseLoggingTestCase):
 
-    @test_base.skip_if_timeout("bug 1687027")
-    def test_forbid_offline_migrations_starting_newton(self):
-        super(TestModelsMigrationsMysql,
-              self).test_forbid_offline_migrations_starting_newton()
-
-    @test_base.skip_if_timeout("bug 1687027")
     def test_check_mysql_engine(self):
         engine = self.get_engine()
         cfg.CONF.set_override('connection', engine.url, group='database')
@@ -357,7 +353,7 @@ class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
             self.alembic_config.attributes['connection'] = connection
             migration.do_alembic_command(self.alembic_config, 'upgrade',
                                          'heads')
-            insp = sqlalchemy.engine.reflection.Inspector.from_engine(engine)
+            insp = sqlalchemy.inspect(engine)
             # Test that table creation on MySQL only builds InnoDB tables
             tables = insp.get_table_names()
             self.assertGreater(len(tables), 0,
@@ -368,26 +364,13 @@ class TestModelsMigrationsMysql(testlib_api.MySQLTestCaseMixin,
                    table != 'alembic_version']
             self.assertEqual(0, len(res), "%s non InnoDB tables created" % res)
 
-    @test_base.skip_if_timeout("bug 1687027")
-    def test_upgrade_expand_branch(self):
-        super(TestModelsMigrationsMysql, self).test_upgrade_expand_branch()
-
-    @test_base.skip_if_timeout("bug 1687027")
-    def test_upgrade_contract_branch(self):
-        super(TestModelsMigrationsMysql, self).test_upgrade_contract_branch()
-
-    @test_base.skip_if_timeout("bug 1687027")
-    def test_branches(self):
-        super(TestModelsMigrationsMysql, self).test_branches()
-
-    @test_base.skip_if_timeout("bug 1687027")
     def test_models_sync(self):
-        super(TestModelsMigrationsMysql, self).test_models_sync()
+        super(TestModelsMigrationsMySQL, self).test_models_sync()
 
 
-class TestModelsMigrationsPsql(testlib_api.PostgreSQLTestCaseMixin,
-                               _TestModelsMigrations,
-                               testlib_api.SqlTestCaseLight):
+class TestModelsMigrationsPostgreSQL(testlib_api.PostgreSQLTestCaseMixin,
+                                     _TestModelsMigrations,
+                                     testlib_api.SqlTestCaseLight):
     pass
 
 
@@ -593,7 +576,7 @@ class _TestWalkMigrations(object):
             migration.do_alembic_command(config, 'upgrade', dest)
 
 
-class TestWalkMigrationsMysql(testlib_api.MySQLTestCaseMixin,
+class TestWalkMigrationsMySQL(testlib_api.MySQLTestCaseMixin,
                               _TestWalkMigrations,
                               testlib_api.SqlTestCaseLight):
 
@@ -606,12 +589,11 @@ class TestWalkMigrationsMysql(testlib_api.MySQLTestCaseMixin,
     # on slow nodes than 'psycopg2' and because of that this increased
     # timeout is required only when for testing with 'mysql' backend.
     @test_base.set_timeout(600)
-    @test_base.skip_if_timeout("bug 1687027")
     def test_walk_versions(self):
-        super(TestWalkMigrationsMysql, self).test_walk_versions()
+        super(TestWalkMigrationsMySQL, self).test_walk_versions()
 
 
-class TestWalkMigrationsPsql(testlib_api.PostgreSQLTestCaseMixin,
-                             _TestWalkMigrations,
-                             testlib_api.SqlTestCaseLight):
+class TestWalkMigrationsPostgreSQL(testlib_api.PostgreSQLTestCaseMixin,
+                                   _TestWalkMigrations,
+                                   testlib_api.SqlTestCaseLight):
     pass
