@@ -174,9 +174,8 @@ class _TestMaintenanceHelper(base.TestOVNFunctionalBase):
         return self.deserialize(self.fmt, res)['security_group']
 
     def _find_security_group_row_by_id(self, sg_id):
-        for row in self.nb_api._tables['Port_Group'].rows.values():
-            if row.name == utils.ovn_port_group_name(sg_id):
-                return row
+        return self.nb_api.lookup(
+            'Port_Group', utils.ovn_port_group_name(sg_id), default=None)
 
     def _create_security_group_rule(self, sg_id):
         data = {'security_group_rule': {'security_group_id': sg_id,
@@ -722,105 +721,6 @@ class TestMaintenance(_TestMaintenanceHelper):
             self.context,
             neutron_obj['port_id']))
 
-    def test_check_metadata_ports(self):
-        ovn_config.cfg.CONF.set_override('ovn_metadata_enabled', True,
-                                         group='ovn')
-        neutron_net = self._create_network('network1')
-        metadata_port = self._ovn_client._find_metadata_port(
-            self.context, neutron_net['id'])
-
-        # Assert the metadata port exists
-        self.assertIsNotNone(metadata_port)
-
-        # Delete the metadata port
-        self._delete('ports', metadata_port['id'])
-
-        metadata_port = self._ovn_client._find_metadata_port(
-            self.context, neutron_net['id'])
-
-        # Assert the metadata port is gone
-        self.assertIsNone(metadata_port)
-
-        # Call the maintenance thread to fix the problem, it will raise
-        # NeverAgain so that the job only runs once at startup
-        self.assertRaises(periodics.NeverAgain,
-                          self.maint.check_metadata_ports)
-
-        metadata_port = self._ovn_client._find_metadata_port(
-            self.context, neutron_net['id'])
-
-        # Assert the metadata port was re-created
-        self.assertIsNotNone(metadata_port)
-
-    def test_check_metadata_ports_not_enabled(self):
-        ovn_config.cfg.CONF.set_override('ovn_metadata_enabled', False,
-                                         group='ovn')
-        with mock.patch.object(self._ovn_client,
-                               'create_metadata_port') as mock_create_port:
-            self.assertRaises(periodics.NeverAgain,
-                              self.maint.check_metadata_ports)
-            # Assert create_metadata_port() wasn't called since metadata
-            # is not enabled
-            self.assertFalse(mock_create_port.called)
-
-    def test_check_for_port_security_unknown_address(self):
-        neutron_net = self._create_network('network1')
-        neutron_port = self._create_port('port1', neutron_net['id'])
-
-        # Let's force disabling port security for the LSP
-        self.nb_api.lsp_set_port_security(neutron_port['id'], []).execute(
-            check_error=True)
-
-        ovn_port = self.nb_api.db_find(
-            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
-            check_error=True)[0]
-
-        # Assert that port security is now disabled but the 'unknown'
-        # is not set in the addresses column
-        self.assertFalse(ovn_port['port_security'])
-        self.assertNotIn('unknown', ovn_port['addresses'])
-
-        # Call the maintenance task to fix the problem. Note that
-        # NeverAgain is raised so it only runs once at start up
-        self.assertRaises(periodics.NeverAgain,
-                          self.maint.check_for_port_security_unknown_address)
-
-        ovn_port = self.nb_api.db_find(
-            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
-            check_error=True)[0]
-
-        # Assert that 'unknown' was set in the addresses column for
-        # the port
-        self.assertFalse(ovn_port['port_security'])
-        self.assertIn('unknown', ovn_port['addresses'])
-
-        # Now the other way around, let's set port_security in the OVN
-        # table while the 'unknown' address is set in the addresses column
-        self.nb_api.lsp_set_port_security(
-            neutron_port['id'], ovn_port['addresses']).execute(
-            check_error=True)
-
-        ovn_port = self.nb_api.db_find(
-            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
-            check_error=True)[0]
-
-        self.assertTrue(ovn_port['port_security'])
-        self.assertIn('unknown', ovn_port['addresses'])
-
-        # Call the maintenance task to fix the problem. Note that
-        # NeverAgain is raised so it only runs once at start up
-        self.assertRaises(periodics.NeverAgain,
-                          self.maint.check_for_port_security_unknown_address)
-
-        ovn_port = self.nb_api.db_find(
-            'Logical_Switch_Port', ('name', '=', neutron_port['id'])).execute(
-            check_error=True)[0]
-
-        # Assert that 'unknown' was removed from the addresses column
-        # for the port
-        self.assertTrue(ovn_port['port_security'])
-        self.assertNotIn('unknown', ovn_port['addresses'])
-
     def test_check_for_igmp_snooping_enabled(self):
         cfg.CONF.set_override('igmp_snooping_enable', False, group='OVS')
         net = self._create_network('net')
@@ -990,7 +890,7 @@ class TestMaintenance(_TestMaintenanceHelper):
         p1 = self._create_port('testp1', net1['id'])
         p1_ip = p1['fixed_ips'][0]['ip_address']
 
-        with mock.patch('neutron_lib.callbacks.registry.notify') as m_notify:
+        with mock.patch('neutron_lib.callbacks.registry.publish') as m_publish:
             # > Create
             fip_pf_args = {
                 pf_def.EXTERNAL_PORT: 2222,
@@ -1000,7 +900,7 @@ class TestMaintenance(_TestMaintenanceHelper):
                 pf_def.INTERNAL_IP_ADDRESS: p1_ip}
             pf_obj = self.pf_plugin.create_floatingip_port_forwarding(
                 self.context, fip_id, **fip_attrs(fip_pf_args))
-            m_notify.assert_called_once()
+            m_publish.assert_called_once()
 
             # Assert load balancer for port forwarding was not created
             self.assertFalse(self._find_pf_lb(router_id, fip_id))
@@ -1015,10 +915,10 @@ class TestMaintenance(_TestMaintenanceHelper):
             fip_pf_args = {pf_def.EXTERNAL_PORT: 5353,
                            pf_def.INTERNAL_PORT: 53,
                            pf_def.PROTOCOL: 'udp'}
-            m_notify.reset_mock()
+            m_publish.reset_mock()
             self.pf_plugin.update_floatingip_port_forwarding(
                 self.context, pf_obj['id'], fip_id, **fip_attrs(fip_pf_args))
-            m_notify.assert_called_once()
+            m_publish.assert_called_once()
 
             # Assert load balancer for port forwarding is stale
             _verify_lb(self, 'tcp', 2222, 22)
@@ -1030,10 +930,10 @@ class TestMaintenance(_TestMaintenanceHelper):
             _verify_lb(self, 'udp', 5353, 53)
 
             # > Delete
-            m_notify.reset_mock()
+            m_publish.reset_mock()
             self.pf_plugin.delete_floatingip_port_forwarding(
                 self.context, pf_obj['id'], fip_id)
-            m_notify.assert_called_once()
+            m_publish.assert_called_once()
 
             # Assert load balancer for port forwarding is stale
             _verify_lb(self, 'udp', 5353, 53)
