@@ -346,12 +346,16 @@ class TestNBDbMonitor(base.TestOVNFunctionalBase):
         # This test checks that the VIP "Port_Binding" register gets the type
         # and the corresponding "virtual-parents".
         self._check_port_binding_type(vip['id'], ovn_const.LSP_TYPE_VIRTUAL)
+        self._check_port_virtual_parents(vip['id'], port['id'])
         mock_update_vip_host.assert_not_called()
 
         data = {'port': {'allowed_address_pairs': []}}
         req = self.new_update_request('ports', data, port['id'])
         req.get_response(self.api)
         self._check_port_binding_type(vip['id'], '')
+        self._check_port_virtual_parents(vip['id'], None)
+        n_utils.wait_until_true(lambda: mock_update_vip_host.called,
+                                timeout=10)
         mock_update_vip_host.assert_called_once_with(vip['id'], None)
 
 
@@ -456,19 +460,30 @@ class TestAgentMonitor(base.TestOVNFunctionalBase):
                 type(neutron_agent.AgentCache()[self.chassis_name]))
 
     def test_agent_updated_at_use_nb_cfg_timestamp(self):
+        def check_agent_ts():
+            agent = neutron_agent.AgentCache()[self.chassis_name]
+            chassis_ts = self.sb_api.db_get(
+                'Chassis_Private', self.chassis_name,
+                'nb_cfg_timestamp').execute(check_error=True)
+            updated_at = datetime.datetime.fromtimestamp(
+                int(chassis_ts / 1000), datetime.timezone.utc)
+            return agent.updated_at == updated_at
+
         if not self.sb_api.is_table_present('Chassis_Private'):
             self.skipTest('Ovn sb not support Chassis_Private')
         timestamp = timeutils.utcnow_ts()
         nb_cfg_timestamp = timestamp * 1000
-        updated_at = datetime.datetime.fromtimestamp(
-            timestamp, datetime.timezone.utc)
         self.sb_api.db_set('Chassis_Private', self.chassis_name, (
             'nb_cfg_timestamp', nb_cfg_timestamp)).execute(check_error=True)
-        n_utils.wait_until_true(lambda:
-                neutron_agent.AgentCache()[self.chassis_name].
-                chassis_private.nb_cfg_timestamp == nb_cfg_timestamp)
-        agent = neutron_agent.AgentCache()[self.chassis_name]
-        self.assertEqual(updated_at, agent.updated_at)
+        try:
+            n_utils.wait_until_true(check_agent_ts, timeout=5)
+        except n_utils.WaitTimeout:
+            agent = neutron_agent.AgentCache()[self.chassis_name]
+            chassis_ts = self.sb_api.db_get(
+                'Chassis_Private', self.chassis_name,
+                'nb_cfg_timestamp').execute(check_error=True)
+            self.fail('Chassis timestamp: %s, agent updated_at: %s' %
+                      (chassis_ts, str(agent.updated_at)))
 
 
 class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
@@ -501,3 +516,39 @@ class TestOvnIdlProbeInterval(base.TestOVNFunctionalBase):
         interval = ovn_conf.get_ovn_ovsdb_probe_interval()
         for idl in idls:
             self.assertEqual(interval, idl._session.reconnect.probe_interval)
+
+
+class TestOvnIdlConnections(base.TestOVNFunctionalBase):
+    def setUp(self):
+        temp_dir = self.useFixture(og_fixtures.TempDir()).path
+        install_share_path = self._get_install_share_path()
+        mgr = self.useFixture(
+            process.OvsdbServer(temp_dir, install_share_path,
+                                ovn_nb_db=True, ovn_sb_db=True,
+                                protocol='tcp'))
+        connection = mgr.get_ovsdb_connection_path
+
+        nb_conns = connection()
+        sb_conns = connection(db_type='sb')
+        # add fake address, idl support multiple addresses, as long as there
+        # is an available address, it will run successfully.
+        nb_conns += ',tcp:192.168.0.1:6641'
+        sb_conns += ',tcp:192.168.0.1:6642'
+        self.connections = {'OVN_Northbound': nb_conns,
+                            'OVN_Southbound': sb_conns}
+        super().setUp()
+
+    def test_ovsdb_connections(self):
+        klasses = {
+            ovsdb_monitor.OvnNbIdl: ('OVN_Northbound',
+                                     {'driver': self.mech_driver}),
+            ovsdb_monitor.OvnSbIdl: ('OVN_Southbound',
+                                     {'driver': self.mech_driver})}
+        for kls, (schema, kwargs) in klasses.items():
+            conns = self.connections[schema]
+            idl = kls.from_server(
+                conns,
+                idlutils.get_schema_helper(conns, schema),
+                **kwargs)
+            self.assertEqual(set(idlutils.parse_connection(conns)),
+                             set(idl._session.remotes))
