@@ -13,8 +13,10 @@
 #    under the License.
 
 import copy
+import datetime
 import functools
 import re
+import time
 from unittest import mock
 
 import netaddr
@@ -27,7 +29,6 @@ from oslo_utils import uuidutils
 from ovsdbapp.backend.ovs_idl import event
 from ovsdbapp.tests.functional import base as ovs_base
 
-from neutron.agent.linux import utils as linux_utils
 from neutron.common.ovn import constants as ovn_const
 from neutron.common.ovn import utils
 from neutron.common import utils as n_utils
@@ -36,6 +37,7 @@ from neutron.db import ovn_revision_numbers_db as db_rev
 from neutron.plugins.ml2.drivers.ovn.mech_driver import mech_driver
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovn_client
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.tests import base as tests_base
 from neutron.tests.functional import base
 
@@ -600,6 +602,74 @@ class TestExternalPorts(base.TestOVNFunctionalBase):
         self.assertEqual(utils.ovn_name(net_id),
                          str(ovn_port.ha_chassis_group[0].name))
 
+    def _create_router_port(self, vnic_type):
+        net_id = self.n1['network']['id']
+        port_data = {
+            'port': {'network_id': net_id,
+                     'tenant_id': self._tenant_id,
+                     portbindings.VNIC_TYPE: 'normal'}}
+
+        # Create port
+        port_req = self.new_create_request('ports', port_data, self.fmt)
+        port_res = port_req.get_response(self.api)
+        port = self.deserialize(self.fmt, port_res)['port']
+
+        # Update it as lsp port
+        port_upt_data = {
+            'port': {'device_owner': "network:router_gateway"}
+        }
+        port_req = self.new_update_request(
+            'ports', port_upt_data, port['id'], self.fmt)
+        port_res = port_req.get_response(self.api)
+
+    def test_add_external_port_avoid_flapping(self):
+        class LogicalSwitchPortUpdateUpEventTest(event.RowEvent):
+            def __init__(self):
+                self.count = 0
+                table = 'Logical_Switch_Port'
+                events = (self.ROW_UPDATE,)
+                super(LogicalSwitchPortUpdateUpEventTest, self).__init__(
+                    events, table, (('up', '=', True),),
+                    old_conditions=(('up', '=', False),))
+
+            def run(self, event, row, old):
+                self.count += 1
+
+            def get_count(self):
+                return self.count
+
+        class LogicalSwitchPortUpdateDownEventTest(event.RowEvent):
+            def __init__(self):
+                self.count = 0
+                table = 'Logical_Switch_Port'
+                events = (self.ROW_UPDATE,)
+                super(LogicalSwitchPortUpdateDownEventTest, self).__init__(
+                    events, table, (('up', '=', False),),
+                    old_conditions=(('up', '=', True),))
+
+            def run(self, event, row, old):
+                self.count += 1
+
+            def get_count(self):
+                return self.count
+
+        og_up_event = ovsdb_monitor.LogicalSwitchPortUpdateUpEvent(None)
+        og_down_event = ovsdb_monitor.LogicalSwitchPortUpdateDownEvent(None)
+        test_down_event = LogicalSwitchPortUpdateDownEventTest()
+        test_up_event = LogicalSwitchPortUpdateUpEventTest()
+        self.nb_api.idl.notify_handler.unwatch_events(
+            [og_up_event, og_down_event])
+        self.nb_api.idl.notify_handler.watch_events(
+            [test_down_event, test_up_event])
+        # Creating a port the same way as the osp cli cmd
+        # openstack router add port ROUTER PORT
+        # shouldn't trigger an status flapping (up -> down -> up)
+        # it should be created with status false and then change the
+        # status as up, triggering only a LogicalSwitchPortUpdateUpEvent.
+        self._create_router_port(portbindings.VNIC_DIRECT)
+        self.assertEqual(test_down_event.get_count(), 0)
+        self.assertEqual(test_up_event.get_count(), 1)
+
     def test_external_port_create_vnic_direct(self):
         self._test_external_port_create(portbindings.VNIC_DIRECT)
 
@@ -731,6 +801,7 @@ class TestCreateDefaultDropPortGroup(base.BaseLoggingTestCase,
 
     def setUp(self):
         super(TestCreateDefaultDropPortGroup, self).setUp()
+        ovn_conf.register_opts()
         self.api = impl_idl_ovn.OvsdbNbOvnIdl(
             self.connection['OVN_Northbound'])
         self.addCleanup(self.api.pg_del(self.PG_NAME, if_exists=True).execute,
@@ -782,6 +853,12 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
         super(TestSecurityGroup, self).setUp()
         self._ovn_client = self.mech_driver._ovn_client
         self.plugin = self.mech_driver._plugin
+        self.sg_data = {
+            'name': 'testsg',
+            'description': 'Test Security Group',
+            'tenant_id': self._tenant_id,
+            'is_default': True,
+        }
 
     def _find_acls_for_sg(self, sg_id):
         rows = self.nb_api.db_find_rows('ACL').execute(check_error=True)
@@ -798,6 +875,13 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
             return [r for r in rows if get_api_id(r) in rule_ids]
         return []
 
+    def _find_acl_remote_sg(self, remote_sg_id):
+        # NOTE: the ACL to be found has ethertype=IPv4 and protocol=ICMP.
+        sg_match = '$pg_' + remote_sg_id.replace('-', '_') + '_ip4 && icmp4'
+        for row in self.nb_api.db_find_rows('ACL').execute(check_error=True):
+            if sg_match in row.match:
+                return row
+
     def test_sg_stateful_toggle_updates_ovn_acls(self):
         def check_acl_actions(sg_id, expected):
             self.assertEqual(
@@ -805,21 +889,15 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
                 set(a.action for a in self._find_acls_for_sg(sg_id))
             )
 
-        sg_data = {
-            'name': 'testsg',
-            'description': 'Test Security Group',
-            'tenant_id': self._tenant_id,
-            'is_default': True,
-        }
         sg = self.plugin.create_security_group(
-            self.context, security_group={'security_group': sg_data})
+            self.context, security_group={'security_group': self.sg_data})
         check_acl_actions(sg['id'], 'allow-related')
 
         def update_sg(stateful):
-            sg_data['stateful'] = stateful
+            self.sg_data['stateful'] = stateful
             self.plugin.update_security_group(
                 self.context, sg['id'],
-                security_group={'security_group': sg_data})
+                security_group={'security_group': self.sg_data})
 
         update_sg(False)
         check_acl_actions(sg['id'], 'allow-stateless')
@@ -830,6 +908,35 @@ class TestSecurityGroup(base.TestOVNFunctionalBase):
         update_sg(False)
         check_acl_actions(sg['id'], 'allow-stateless')
 
+    def test_remove_sg_with_related_rule_remote_sg(self):
+        self.sg_data['is_default'] = False
+        sg1 = self.plugin.create_security_group(
+            self.context, security_group={'security_group': self.sg_data})
+        sg2 = self.plugin.create_security_group(
+            self.context, security_group={'security_group': self.sg_data})
+        rule_data = {'direction': constants.INGRESS_DIRECTION,
+                     'ethertype': constants.IPv4,
+                     'protocol': constants.PROTO_NAME_ICMP,
+                     'port_range_max': None,
+                     'port_range_min': None,
+                     'remote_ip_prefix': None,
+                     'tenant_id': sg1['project_id'],
+                     'remote_address_group_id': None,
+                     'security_group_id': sg1['id'],
+                     'remote_group_id': sg2['id']}
+        sg_rule = {'security_group_rule': rule_data}
+        rule = self.plugin.create_security_group_rule(self.context, sg_rule)
+        acl = self._find_acl_remote_sg(sg2['id'])
+        self.assertEqual(rule['id'],
+                         acl.external_ids[ovn_const.OVN_SG_RULE_EXT_ID_KEY])
+        acls = self._find_acls_for_sg(sg1['id'])
+        self.assertEqual(3, len(acls))
+
+        self.plugin.delete_security_group(self.context, sg2['id'])
+        self.assertIsNone(self._find_acl_remote_sg(sg2['id']))
+        acls = self._find_acls_for_sg(sg1['id'])
+        self.assertEqual(2, len(acls))
+
 
 class TestProvnetPorts(base.TestOVNFunctionalBase):
 
@@ -1054,6 +1161,25 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         for agent_id in self.agent_types.values():
             self.assertTrue(self.plugin.get_agent(self.context, agent_id))
 
+    def test_agent_show_real_heartbeat_timestamp(self):
+        agent_id = self.agent_types[ovn_const.OVN_CONTROLLER_AGENT]
+        agent = self.plugin.get_agent(self.context, agent_id)
+        heartbeat_timestamp = agent['heartbeat_timestamp']
+        if self.sb_api.is_table_present('Chassis_Private'):
+            chassis_ts = self.sb_api.db_get(
+                'Chassis_Private', self.chassis, 'nb_cfg_timestamp'
+            ).execute(check_error=True)
+            updated_at = datetime.datetime.fromtimestamp(
+                int(chassis_ts / 1000), datetime.timezone.utc)
+            # if table Chassis_Private present, agent.updated_at is
+            # Chassis_Private.nb_cfg_timestamp
+            self.assertEqual(updated_at, heartbeat_timestamp)
+        time.sleep(1)
+        # if chassis is not updated, agent's heartbeat_timestamp shouldn't
+        # be updated.
+        n_agent = self.plugin.get_agent(self.context, agent['id'])
+        self.assertEqual(heartbeat_timestamp, n_agent['heartbeat_timestamp'])
+
     def test_agent_list(self):
         agent_ids = [a['id'] for a in self.plugin.get_agents(
             self.context, filters={'host': self.host})]
@@ -1094,56 +1220,3 @@ class TestAgentApi(base.TestOVNFunctionalBase):
         self.plugin.delete_agent(self.context, metadata_id)
         self.assertRaises(agent_exc.AgentNotFound, self.plugin.get_agent,
                           self.context, metadata_id)
-
-
-class ConnectionInactivityProbeSetEvent(event.WaitEvent):
-    """Wait for a Connection (NB/SB) to have the inactivity probe set"""
-
-    ONETIME = False
-
-    def __init__(self, target, inactivity_probe):
-        table = 'Connection'
-        events = (self.ROW_UPDATE,)
-        super().__init__(events, table, None)
-        self.event_name = "ConnectionEvent"
-        self.target = target
-        self.inactivity_probe = inactivity_probe
-
-    def match_fn(self, event, row, old):
-        return row.target in self.target
-
-    def run(self, event, row, old):
-        if (row.inactivity_probe and
-                row.inactivity_probe[0] == self.inactivity_probe):
-            self.event.set()
-
-
-class TestSetInactivityProbe(base.TestOVNFunctionalBase):
-
-    def setUp(self):
-        super().setUp()
-        self.dbs = [(ovn_conf.get_ovn_nb_connection(), 'ptcp:1000:1.2.3.4'),
-                    (ovn_conf.get_ovn_sb_connection(), 'ptcp:1001:1.2.3.4')]
-        linux_utils.execute(
-            ['ovn-nbctl', '--db=%s' % self.dbs[0][0],
-             'set-connection', self.dbs[0][1]], run_as_root=True,
-            privsep_exec=True)
-        linux_utils.execute(
-            ['ovn-sbctl', '--db=%s' % self.dbs[1][0],
-             'set-connection', self.dbs[1][1]], run_as_root=True,
-            privsep_exec=True)
-
-    def test_1(self):
-        mock.patch.object(ovn_conf, 'get_ovn_ovsdb_probe_interval',
-                          return_value='2500').start()
-        nb_connection = ConnectionInactivityProbeSetEvent(self.dbs[0][1], 2500)
-        sb_connection = ConnectionInactivityProbeSetEvent(self.dbs[1][1], 2500)
-        self.nb_api.idl.notify_handler.watch_event(nb_connection)
-        self.sb_api.idl.notify_handler.watch_event(sb_connection)
-        with mock.patch.object(utils, 'connection_config_to_target_string') \
-                as mock_target:
-            mock_target.side_effect = [self.dbs[0][1], self.dbs[1][1]]
-            self.mech_driver._set_inactivity_probe()
-
-        self.assertTrue(nb_connection.wait())
-        self.assertTrue(sb_connection.wait())
