@@ -22,6 +22,7 @@ from neutron_lib import constants as n_cons
 from oslo_utils import uuidutils
 from pyroute2.ipdb import routes as ipdb_routes
 from pyroute2.iproute import linux as iproute_linux
+from pyroute2.netlink import rtnl
 import testtools
 
 from neutron.agent.linux import ip_lib
@@ -320,13 +321,15 @@ class RuleTestCase(functional_base.BaseSudoTestCase):
             ip_lenght = common_utils.get_network_length(ip_version)
             ip_family = common_utils.get_socket_address_family(ip_version)
             priv_ip_lib.add_ip_rule(self.namespace, src=ip_address,
-                                    src_len=ip_lenght, family=ip_family)
+                                    src_len=ip_lenght, family=ip_family,
+                                    table=ip_lib.IP_RULE_TABLES['default'])
             rules = ip_lib.list_ip_rules(self.namespace, ip_version)
             self._check_rules(rules, ['from'], [ip_address],
                               '"from" IP address %s' % ip_address)
 
             priv_ip_lib.delete_ip_rule(self.namespace, family=ip_family,
-                                       src=ip_address, src_len=ip_lenght)
+                                       src=ip_address, src_len=ip_lenght,
+                                       table=ip_lib.IP_RULE_TABLES['default'])
             rules = ip_lib.list_ip_rules(self.namespace, ip_version)
             self.assertFalse(
                 self._check_rules(rules, ['from'], [ip_address],
@@ -377,7 +380,8 @@ class RuleTestCase(functional_base.BaseSudoTestCase):
             ip_family = common_utils.get_socket_address_family(ip_version)
             priv_ip_lib.add_ip_rule(self.namespace, priority=priority,
                                     src=ip_address, src_len=ip_lenght,
-                                    family=ip_family)
+                                    family=ip_family,
+                                    table=ip_lib.IP_RULE_TABLES['default'])
             rules = ip_lib.list_ip_rules(self.namespace, ip_version)
             self._check_rules(
                 rules, ['priority', 'from'], [str(priority), ip_address],
@@ -386,7 +390,8 @@ class RuleTestCase(functional_base.BaseSudoTestCase):
 
             priv_ip_lib.delete_ip_rule(self.namespace, priority=priority,
                                        src=ip_address, src_len=ip_lenght,
-                                       family=ip_family)
+                                       family=ip_family,
+                                       table=ip_lib.IP_RULE_TABLES['default'])
             rules = ip_lib.list_ip_rules(self.namespace, ip_version)
             self.assertFalse(
                 self._check_rules(rules, ['priority', 'from'],
@@ -419,14 +424,16 @@ class RuleTestCase(functional_base.BaseSudoTestCase):
     def test_add_rule_exists(self):
         iif = 'iif_device_1'
         priv_ip_lib.create_interface(iif, self.namespace, 'dummy')
-        priv_ip_lib.add_ip_rule(self.namespace, iifname=iif)
+        priv_ip_lib.add_ip_rule(self.namespace, iifname=iif,
+                                table=ip_lib.IP_RULE_TABLES['default'])
         rules = ip_lib.list_ip_rules(self.namespace, 4)
         self._check_rules(rules, ['iif'], [iif], 'iif name %s' % iif)
         self.assertEqual(4, len(rules))
 
         # pyroute2.netlink.exceptions.NetlinkError(17, 'File exists')
         # exception is catch.
-        priv_ip_lib.add_ip_rule(self.namespace, iifname=iif)
+        priv_ip_lib.add_ip_rule(self.namespace, iifname=iif,
+                                table=ip_lib.IP_RULE_TABLES['default'])
         rules = ip_lib.list_ip_rules(self.namespace, 4)
         self._check_rules(rules, ['iif'], [iif], 'iif name %s' % iif)
         self.assertEqual(4, len(rules))
@@ -475,12 +482,40 @@ class RouteTestCase(functional_base.BaseSudoTestCase):
         self.device = ip_lib.IPDevice(self.device_name, self.namespace)
         self.device.link.set_up()
 
+    def _check_gateway_or_multipath(self, route, gateway):
+        if gateway is None or isinstance(gateway, str):
+            self.assertEqual(gateway, ip_lib.get_attr(route, 'RTA_GATEWAY'))
+            return
+
+        rta_multipath = ip_lib.get_attr(route, 'RTA_MULTIPATH')
+        self.assertEqual(len(gateway), len(rta_multipath))
+        for nexthop in gateway:
+            to_check = {'hops': 0}
+            if nexthop.get('device'):
+                to_check['oif'] = priv_ip_lib.privileged_get_link_id(
+                    nexthop['device'], self.namespace)
+            if nexthop.get('weight'):
+                to_check['hops'] = nexthop['weight'] - 1
+            if nexthop.get('via'):
+                to_check['gateway'] = nexthop['via']
+
+            for mp in rta_multipath:
+                mp['gateway'] = ip_lib.get_attr(mp, 'RTA_GATEWAY')
+                for key in to_check:
+                    if to_check[key] != mp[key]:
+                        break
+                else:
+                    break
+            else:
+                self.fail('Route nexthops %s do not match with defined ones '
+                          '%s' % (rta_multipath, gateway))
+
     def _check_routes(self, cidrs, table=None, gateway=None, metric=None,
-                      scope=None):
+                      scope=None, proto='static'):
         table = table or iproute_linux.DEFAULT_TABLE
         if not scope:
             scope = 'universe' if gateway else 'link'
-        scope = priv_ip_lib._get_scope_name(scope)
+        scope = priv_ip_lib.get_scope_name(scope)
         for cidr in cidrs:
             ip_version = common_utils.get_ip_version(cidr)
             if ip_version == n_cons.IP_VERSION_6 and not metric:
@@ -498,11 +533,11 @@ class RouteTestCase(functional_base.BaseSudoTestCase):
                 self.assertEqual(
                     priv_ip_lib._IP_VERSION_FAMILY_MAP[ip_version],
                     route['family'])
-                self.assertEqual(gateway,
-                                 ip_lib.get_attr(route, 'RTA_GATEWAY'))
+                self._check_gateway_or_multipath(route, gateway)
                 self.assertEqual(metric,
                                  ip_lib.get_attr(route, 'RTA_PRIORITY'))
                 self.assertEqual(scope, route['scope'])
+                self.assertEqual(rtnl.rt_proto[proto], route['proto'])
                 break
             else:
                 self.fail('CIDR %s not found in the list of routes' % cidr)
@@ -533,16 +568,17 @@ class RouteTestCase(functional_base.BaseSudoTestCase):
                       % gateway)
 
     def _add_route_device_and_check(self, table=None, metric=None,
-                                    scope='link'):
+                                    scope='link', proto='static'):
         cidrs = ['192.168.0.0/24', '172.90.0.0/16', '10.0.0.0/8',
                  '2001:db8::/64']
         for cidr in cidrs:
             ip_version = common_utils.get_ip_version(cidr)
             priv_ip_lib.add_ip_route(self.namespace, cidr, ip_version,
                                      device=self.device_name, table=table,
-                                     metric=metric, scope=scope)
+                                     metric=metric, scope=scope, proto=proto)
 
-        self._check_routes(cidrs, table=table, metric=metric, scope=scope)
+        self._check_routes(cidrs, table=table, metric=metric, scope=scope,
+                           proto=proto)
 
     def test_add_route_device(self):
         self._add_route_device_and_check(table=None)
@@ -565,6 +601,18 @@ class RouteTestCase(functional_base.BaseSudoTestCase):
     def test_add_route_device_scope_host(self):
         self._add_route_device_and_check(scope='host')
 
+    def test_add_route_device_proto_static(self):
+        self._add_route_device_and_check(proto='static')  # default
+
+    def test_add_route_device_proto_redirect(self):
+        self._add_route_device_and_check(proto='redirect')
+
+    def test_add_route_device_proto_kernel(self):
+        self._add_route_device_and_check(proto='kernel')
+
+    def test_add_route_device_proto_boot(self):
+        self._add_route_device_and_check(proto='boot')
+
     def test_add_route_via_ipv4(self):
         cidrs = ['192.168.0.0/24', '172.90.0.0/16', '10.0.0.0/8']
         int_cidr = '192.168.20.1/24'
@@ -601,6 +649,22 @@ class RouteTestCase(functional_base.BaseSudoTestCase):
                                      device=self.device_name, via=_ip)
             self._check_gateway(_ip)
 
+    def test_add_multipath_route(self):
+        self.device_name2 = 'test_device2'
+        ip_lib.IPWrapper(self.namespace).add_dummy(self.device_name2)
+        self.device2 = ip_lib.IPDevice(self.device_name2, self.namespace)
+        self.device2.link.set_up()
+        self.device.addr.add('10.1.0.1/24')
+        self.device2.addr.add('10.2.0.1/24')
+        multipath = [
+            {'device': self.device_name, 'via': '10.1.0.100', 'weight': 10},
+            {'device': self.device_name2, 'via': '10.2.0.100', 'weight': 20},
+            {'via': '10.2.0.101', 'weight': 30},
+            {'via': '10.2.0.102'}]
+        priv_ip_lib.add_ip_route(self.namespace, '192.168.0.0/24',
+                                 n_cons.IP_VERSION_4, via=multipath)
+        self._check_routes(['192.168.0.0/24'], gateway=multipath)
+
 
 class GetLinkAttributesTestCase(functional_base.BaseSudoTestCase):
 
@@ -638,6 +702,9 @@ class ListNamespacePids(functional_base.BaseSudoTestCase):
     @staticmethod
     def _run_sleep(namespace, timeout):
         ip_wrapper = ip_lib.IPWrapper(namespace=namespace)
+        # NOTE(ralonsoh): this is a "long" (more than one second) lived
+        # process. It should not be executed in a privsep context to avoid
+        # a possible privsep thread starvation.
         ip_wrapper.netns.execute(['sleep', timeout], check_exit_code=False)
 
     def _check_pids(self, num_pids, namespace=None):
