@@ -14,20 +14,22 @@
 
 from unittest import mock
 
-from neutron.common.ovn import constants as ovn_const
-from neutron.common.ovn import utils as ovn_utils
-from neutron.common import utils as n_utils
-from neutron.scheduler import l3_ovn_scheduler as l3_sched
-from neutron.tests.functional import base
-from neutron.tests.functional.resources.ovsdb import events
 from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.api.definitions import provider_net as pnet
 from neutron_lib import constants as n_consts
 from neutron_lib.plugins import directory
+from oslo_db import exception as db_exc
 from ovsdbapp.backend.ovs_idl import idlutils
 
+from neutron.common.ovn import constants as ovn_const
+from neutron.common.ovn import utils as ovn_utils
+from neutron.common import utils as n_utils
+from neutron.scheduler import l3_ovn_scheduler as l3_sched
+from neutron.tests.functional import base
+from neutron.tests.functional.resources.ovsdb import events
+
 
 class TestRouter(base.TestOVNFunctionalBase):
     def setUp(self):
@@ -39,11 +41,13 @@ class TestRouter(base.TestOVNFunctionalBase):
         self.cr_lrp_pb_event = events.WaitForCrLrpPortBindingEvent()
         self.sb_api.idl.notify_handler.watch_event(self.cr_lrp_pb_event)
 
-    def _create_router(self, name, gw_info=None):
+    def _create_router(self, name, gw_info=None, az_hints=None):
         router = {'router':
                   {'name': name,
                    'admin_state_up': True,
                    'tenant_id': self._tenant_id}}
+        if az_hints:
+            router['router']['availability_zone_hints'] = az_hints
         if gw_info:
             router['router']['external_gateway_info'] = gw_info
         return self.l3_plugin.create_router(self.context, router)
@@ -85,12 +89,13 @@ class TestRouter(base.TestOVNFunctionalBase):
                 'Logical_Router_Port'].rows.values():
             if self._l3_ha_supported():
                 chassis = [gwc.chassis_name for gwc in row.gateway_chassis]
-                self.assertItemsEqual(expected, chassis)
+                self.assertCountEqual(expected, chassis)
             else:
                 rc = row.options.get(ovn_const.OVN_GATEWAY_CHASSIS_KEY)
                 self.assertIn(rc, expected)
 
-    def _check_gateway_chassis_candidates(self, candidates):
+    def _check_gateway_chassis_candidates(self, candidates,
+                                          router_az_hints=None):
         # In this test, fake_select() is called once from _create_router()
         # and later from schedule_unhosted_gateways()
         ovn_client = self.l3_plugin._ovn_client
@@ -100,9 +105,9 @@ class TestRouter(base.TestOVNFunctionalBase):
         # candidates.
 
         def fake_select(*args, **kwargs):
-            self.assertItemsEqual(candidates, kwargs['candidates'])
+            self.assertCountEqual(candidates, kwargs['candidates'])
             # We are not interested in further processing, let us return
-            # INVALID_CHASSIS to avoid erros
+            # INVALID_CHASSIS to avoid errors
             return [ovn_const.OVN_GATEWAY_INVALID_CHASSIS]
 
         with mock.patch.object(ovn_client._ovn_scheduler, 'select',
@@ -110,7 +115,8 @@ class TestRouter(base.TestOVNFunctionalBase):
             mock.patch.object(self.l3_plugin.scheduler, 'select',
                               side_effect=fake_select) as plugin_select:
             gw_info = {'network_id': ext1['network']['id']}
-            self._create_router('router1', gw_info=gw_info)
+            self._create_router('router1', gw_info=gw_info,
+                                az_hints=router_az_hints)
             self.assertFalse(plugin_select.called)
             self.assertTrue(client_select.called)
             client_select.reset_mock()
@@ -158,6 +164,53 @@ class TestRouter(base.TestOVNFunctionalBase):
             # and do not updated the lrp port.
             ulrp.assert_not_called()
 
+    def test_gateway_chassis_with_cms_and_azs(self):
+        # Both chassis3 and chassis4 are having azs,
+        # but only chassis3's azs is ['ovn'].
+        # Test if chassis3 is selected as candidate or not.
+        self.chassis3 = self.add_fake_chassis(
+            'ovs-host3', physical_nets=['physnet1'],
+            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
+            azs=['ovn'])
+        self.chassis4 = self.add_fake_chassis(
+            'ovs-host4', physical_nets=['physnet1'],
+            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
+            azs=['ovn2'])
+        self._check_gateway_chassis_candidates([self.chassis3],
+                                               router_az_hints=['ovn'])
+
+    def test_gateway_chassis_with_cms_and_not_match_azs(self):
+        # add chassis3 is having azs [ovn], match router az_hints,
+        # if not add chassis3, create router will fail with
+        # AvailabilityZoneNotFound. after create will delete if.
+        # add chassis4 is having azs [ovn2], not match routers az_hints [ovn]
+        self.chassis3 = self.add_fake_chassis(
+            'ovs-host3', physical_nets=['physnet1'],
+            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'})
+        self.chassis4 = self.add_fake_chassis(
+            'ovs-host4', physical_nets=['physnet1'],
+            external_ids={'ovn-cms-options': 'enable-chassis-as-gw'},
+            azs=['ovn2'])
+        ovn_client = self.l3_plugin._ovn_client
+        ext1 = self._create_ext_network(
+            'ext1', 'vlan', 'physnet1', 1, "10.0.0.1", "10.0.0.0/24")
+        # As we have 'gateways' in the system, but without required
+        # chassis we should not schedule gw in that case at all.
+        self._set_redirect_chassis_to_invalid_chassis(ovn_client)
+        with mock.patch.object(ovn_client._ovn_scheduler, 'select',
+                              return_value=[self.chassis1]), \
+            mock.patch.object(self.l3_plugin.scheduler, 'select',
+                              side_effect=[self.chassis1]):
+            gw_info = {'network_id': ext1['network']['id']}
+            self._create_router('router1', gw_info=gw_info, az_hints=['ovn'])
+        self.del_fake_chassis(self.chassis3)
+        with mock.patch.object(
+                ovn_client._nb_idl, 'update_lrouter_port') as ulrp:
+            self.l3_plugin.schedule_unhosted_gateways()
+            # Make sure that we don't schedule on chassis3
+            # and do not updated the lrp port.
+            ulrp.assert_not_called()
+
     def test_gateway_chassis_with_bridge_mappings_and_no_cms(self):
         # chassis1 is configured with proper bridge mappings,
         # but none of the chassis having enable-chassis-as-gw.
@@ -289,7 +342,7 @@ class TestRouter(base.TestOVNFunctionalBase):
             self.assertIsNotNone(gw_port)
 
             expected_networks = ['%s/24' % subnet4_ip, '%s/64' % subnet6_ip]
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 expected_networks, gw_port.networks,
                 'networks in ovn port must match fixed_ips in neutron')
 
@@ -341,9 +394,9 @@ class TestRouter(base.TestOVNFunctionalBase):
         self.candidates = []
 
         def fake_select(*args, **kwargs):
-            self.assertItemsEqual(self.candidates, kwargs['candidates'])
+            self.assertCountEqual(self.candidates, kwargs['candidates'])
             # We are not interested in further processing, let us return
-            # INVALID_CHASSIS to avoid erros
+            # INVALID_CHASSIS to avoid errors
             return [ovn_const.OVN_GATEWAY_INVALID_CHASSIS]
 
         with mock.patch.object(ovn_client._ovn_scheduler, 'select',
@@ -402,7 +455,8 @@ class TestRouter(base.TestOVNFunctionalBase):
 
         # Set chassis on chassisredirect port in Port_Binding table
         logical_port = 'cr-lrp-%s' % gw_port_id
-        self.assertTrue(self.cr_lrp_pb_event.wait(logical_port))
+        self.assertTrue(self.cr_lrp_pb_event.wait(logical_port),
+                        msg='lrp %s failed to bind' % logical_port)
         self.sb_api.lsp_bind(logical_port, self.chassis1,
                              may_exist=True).execute(check_error=True)
 
@@ -447,7 +501,7 @@ class TestRouter(base.TestOVNFunctionalBase):
                 'address_mode': ovn_utils.get_ovn_ipv6_address_mode(
                     address_mode),
                 'send_periodic': 'true',
-                'mtu': '1450'}
+                'mtu': '1442'}
         else:
             expected_ra_configs = {}
         self._validate_router_ipv6_ra_configs(lrp_name, expected_ra_configs)
@@ -487,17 +541,28 @@ class TestRouter(base.TestOVNFunctionalBase):
         ext1 = self._create_ext_network(
             'ext1', 'flat', 'physnet4', None, "30.0.0.1", "30.0.0.0/24")
         gw_info = {'network_id': ext1['network']['id']}
-        # Create 20 routers with a gateway. Since we're using physnet4, the
-        # chassis candidates will be chassis4 initially.
-        for i in range(20):
-            router = self._create_router('router%d' % i, gw_info=gw_info)
+        # Tries to create 20 routers with a gateway. Since we're using
+        # physnet4, the chassis candidates will be chassis4 initially.
+        num_routers = 20
+        for i in range(num_routers):
+            try:
+                router = self._create_router('router%d' % i, gw_info=gw_info)
+            except db_exc.DBReferenceError:
+                # NOTE(ralonsoh): this is a workaround for LP#1956344. There
+                # seems to be a bug in SQLite3. The "port" DB object is not
+                # persistently stored in the DB and raises a "DBReferenceError"
+                # exception occasionally.
+                num_routers -= 1
+                continue
+
             gw_port_id = router.get('gw_port_id')
             logical_port = 'cr-lrp-%s' % gw_port_id
-            self.assertTrue(self.cr_lrp_pb_event.wait(logical_port))
+            self.assertTrue(self.cr_lrp_pb_event.wait(logical_port),
+                            msg='lrp %s failed to bind' % logical_port)
             self.sb_api.lsp_bind(logical_port, chassis4,
                                  may_exist=True).execute(check_error=True)
         self.l3_plugin.schedule_unhosted_gateways()
-        expected = {chassis4: {1: 20}}
+        expected = {chassis4: {1: num_routers}}
         self.assertEqual(expected, _get_result_dict())
 
         # Add another chassis as a gateway chassis
@@ -512,9 +577,9 @@ class TestRouter(base.TestOVNFunctionalBase):
 
         # Chassis4 should have all ports at Priority 2
         self.l3_plugin.schedule_unhosted_gateways()
-        self.assertEqual({2: 20}, _get_result_dict()[chassis4])
+        self.assertEqual({2: num_routers}, _get_result_dict()[chassis4])
         # Chassis5 should have all ports at Priority 1
-        self.assertEqual({1: 20}, _get_result_dict()[chassis5])
+        self.assertEqual({1: num_routers}, _get_result_dict()[chassis5])
 
         # delete chassis that hosts all the gateways
         self.del_fake_chassis(chassis4)
@@ -523,7 +588,7 @@ class TestRouter(base.TestOVNFunctionalBase):
         # As Chassis4 has been removed so all gateways that were
         # hosted there are now primaries on chassis5 and have
         # priority 1.
-        self.assertEqual({1: 20}, _get_result_dict()[chassis5])
+        self.assertEqual({1: num_routers}, _get_result_dict()[chassis5])
 
     def test_gateway_chassis_rebalance_max_chassis(self):
         chassis_list = []
@@ -541,7 +606,8 @@ class TestRouter(base.TestOVNFunctionalBase):
         router = self._create_router('router', gw_info=gw_info)
         gw_port_id = router.get('gw_port_id')
         logical_port = 'cr-lrp-%s' % gw_port_id
-        self.assertTrue(self.cr_lrp_pb_event.wait(logical_port))
+        self.assertTrue(self.cr_lrp_pb_event.wait(logical_port),
+                        msg='lrp %s failed to bind' % logical_port)
         self.sb_api.lsp_bind(logical_port, chassis_list[0],
                              may_exist=True).execute(check_error=True)
 
