@@ -18,7 +18,9 @@ from neutron_lib.callbacks import events as callbacks_events
 from neutron_lib.callbacks import registry as callbacks_registry
 from neutron_lib.callbacks import resources as callbacks_resources
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
 from neutron_lib.utils import helpers
+from os_ken.ofproto import ofproto_v1_3_parser
 from oslo_config import cfg
 import testtools
 
@@ -29,14 +31,42 @@ from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import exceptions
 from neutron.agent.linux.openvswitch_firewall import firewall as ovsfw
 from neutron.conf.agent import securitygroups_rpc
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
-        as ovs_consts
 from neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge
 from neutron.tests import base
 
 TESTING_VLAN_TAG = 1
 TESTING_SEGMENT = 1000
+MATCH_1 = ofproto_v1_3_parser.OFPMatch(
+    _ordered_fields=[('conj_id', 100), ('eth_type', 2048), ('reg5', 12),
+                     ('ct_state', (2, 14))])
+MATCH_2 = ofproto_v1_3_parser.OFPMatch(
+    _ordered_fields=[('conj_id', 200), ('eth_type', 2048), ('reg5', 12),
+                     ('ct_state', (2, 14))])
+MATCH_3 = ofproto_v1_3_parser.OFPMatch(
+    _ordered_fields=[('reg5', 13), ('ct_state', (10, 14)), ('ct_zone', 1),
+                     ('ct_mark', 0)])
+MATCH_4 = ofproto_v1_3_parser.OFPMatch(
+    _ordered_fields=[('eth_type', 34525), ('ip_proto', 58),
+                     ('icmpv6_type', 136), ('reg5', 11)])
+INIT_OF_RULES = [
+    mock.Mock(match=MATCH_1),
+    mock.Mock(match=MATCH_2),
+    mock.Mock(match=MATCH_3),
+    mock.Mock(match=MATCH_4),
+]
+INIT_OF_RULES_VSCTL = [
+    'priority=40,ct_state=+est,ip,reg5=0xd actions=ct(commit,zone=NXM_NX_REG6['
+    '0..15],exec(load:0x1->NXM_NX_CT_MARK[]))',
+    'priority=70,conj_id=100,ct_state=+est-rel-rpl,ip,reg5=0xc actions=load:0x'
+    'f0->NXM_NX_REG7[],output:12',
+    'priority=70,conj_id=200,ct_state=+est-rel-rpl,ipv6,reg5=0xc actions=load:'
+    '0xf8->NXM_NX_REG7[],output:12',
+    'priority=73,ct_state=+est-rel-rpl,ip,reg6=0x1,nw_src=10.10.0.42 actions=c'
+    'onjunction(118,1/2)',
+    'priority=70,ct_state=+est-rel-rpl,ip,reg6=0x1,nw_src=10.10.0.61 actions=c'
+    'onjunction(120,1/2)'
+]
 
 
 def create_ofport(port_dict, network_type=None,
@@ -81,9 +111,12 @@ class TestSecurityGroup(base.BaseTestCase):
     def test_update_rules_split(self):
         rules = [
             {'foo': 'bar', 'rule': 'all'}, {'bar': 'foo'},
-            {'remote_group_id': '123456', 'foo': 'bar'}]
+            {'remote_group_id': '123456', 'foo': 'bar'},
+            {'remote_address_group_id': 'fake_ag_id', 'bar': 'foo'}]
         expected_raw_rules = [{'foo': 'bar', 'rule': 'all'}, {'bar': 'foo'}]
-        expected_remote_rules = [{'remote_group_id': '123456', 'foo': 'bar'}]
+        expected_remote_rules = [{'remote_group_id': '123456', 'foo': 'bar'},
+                                 {'remote_address_group_id': 'fake_ag_id',
+                                  'bar': 'foo'}]
         self.sg.update_rules(rules)
 
         self.assertEqual(expected_raw_rules, self.sg.raw_rules)
@@ -269,7 +302,11 @@ class TestSGPortMap(base.BaseTestCase):
 class TestConjIdMap(base.BaseTestCase):
     def setUp(self):
         super(TestConjIdMap, self).setUp()
-        self.conj_id_map = ovsfw.ConjIdMap()
+        self.mock_int_br = mock.Mock()
+        self.dump_flows_ret = [[]] * len(ovs_consts.OVS_FIREWALL_TABLES)
+        self.dump_flows_ret[0] = INIT_OF_RULES
+        self.mock_int_br.dump_flows.side_effect = self.dump_flows_ret
+        self.conj_id_map = ovsfw.ConjIdMap(self.mock_int_br)
 
     def test_get_conj_id(self):
         allocated = []
@@ -290,6 +327,7 @@ class TestConjIdMap(base.BaseTestCase):
                           constants.IPv6)
 
     def test_delete_sg(self):
+        self.conj_id_map._max_id = 0
         test_data = [
             # conj_id: 8
             ('sg1', 'sg1', constants.INGRESS_DIRECTION, constants.IPv6, 0),
@@ -335,11 +373,43 @@ class TestConjIdMap(base.BaseTestCase):
             reallocated.add(self.conj_id_map.get_conj_id(*conj_id_tuple))
         self.assertEqual(reallocated, conj_id_segment)
 
+    def test__init_max_id_os_ken(self):
+        self.mock_int_br.dump_flows.side_effect = self.dump_flows_ret
+        self.assertEqual(208, self.conj_id_map._init_max_id(self.mock_int_br))
+
+        match = ofproto_v1_3_parser.OFPMatch(
+            _ordered_fields=[('conj_id', 237), ('eth_type', 2048),
+                             ('reg5', 12), ('ct_state', (2, 14))])
+        new_rule = mock.Mock(match=match)
+        self.dump_flows_ret[0] = INIT_OF_RULES + [new_rule]
+        self.mock_int_br.dump_flows.side_effect = self.dump_flows_ret
+        self.assertEqual(240, self.conj_id_map._init_max_id(self.mock_int_br))
+
+    def test__init_max_id_vsctl(self):
+        self.mock_int_br.dump_flows.side_effect = AttributeError()
+        dump_flows_ret = [[]] * len(ovs_consts.OVS_FIREWALL_TABLES)
+        dump_flows_ret[0] = INIT_OF_RULES_VSCTL
+        self.mock_int_br.dump_flows_for_table.side_effect = dump_flows_ret
+
+        self.assertEqual(208, self.conj_id_map._init_max_id(self.mock_int_br))
+
+        new_rule = ('priority=70,conj_id=237,ct_state=+est-rel-rpl,ipv6,reg5=0'
+                    'xc actions=load:0xf8->NXM_NX_REG7[],output:12')
+        dump_flows_ret[0] = INIT_OF_RULES_VSCTL + [new_rule]
+        self.mock_int_br.dump_flows_for_table.side_effect = dump_flows_ret
+        self.assertEqual(240, self.conj_id_map._init_max_id(self.mock_int_br))
+
+    def test__next_max_id(self):
+        self.assertEqual(8, self.conj_id_map._next_max_id(0))
+        self.assertEqual(0, self.conj_id_map._next_max_id(
+            self.conj_id_map.MAX_CONJ_ID - 1))
+
 
 class TestConjIPFlowManager(base.BaseTestCase):
     def setUp(self):
         super(TestConjIPFlowManager, self).setUp()
         self.driver = mock.Mock()
+        self.driver.int_br.br.dump_flows.return_value = INIT_OF_RULES
         self.manager = ovsfw.ConjIPFlowManager(self.driver)
         self.vlan_tag = 100
         self.conj_id = 16
@@ -352,7 +422,7 @@ class TestConjIPFlowManager(base.BaseTestCase):
             get_conj_id_mock.return_value = self.conj_id
             self.manager.add(self.vlan_tag, 'sg', 'remote_id',
                              constants.INGRESS_DIRECTION, constants.IPv4, 0)
-            self.manager.update_flows_for_vlan(self.vlan_tag)
+            self.manager.update_flows_for_vlan(self.vlan_tag, mock.ANY)
         self.assertFalse(remote_group.get_ethertype_filtered_addresses.called)
         self.assertFalse(self.driver._add_flow.called)
 
@@ -368,11 +438,11 @@ class TestConjIPFlowManager(base.BaseTestCase):
             get_conj_id_mock.return_value = self.conj_id
             self.manager.add(self.vlan_tag, 'sg', 'remote_id',
                              constants.INGRESS_DIRECTION, constants.IPv4, 0)
-            self.manager.update_flows_for_vlan(self.vlan_tag)
+            self.manager.update_flows_for_vlan(self.vlan_tag, mock.ANY)
         self.assertTrue(remote_group.get_ethertype_filtered_addresses.called)
         self.assertTrue(self.driver._add_flow.called)
 
-    def test_update_flows_for_vlan(self):
+    def test_update_flows_for_vlan_remote_group(self):
         remote_group = self.driver.sg_port_map.get_sg.return_value
         remote_group.get_ethertype_filtered_addresses.return_value = [
             ('10.22.3.4', 'fa:16:3e:aa:bb:cc'), ]
@@ -383,20 +453,25 @@ class TestConjIPFlowManager(base.BaseTestCase):
                              constants.INGRESS_DIRECTION, constants.IPv4, 0)
             self.manager.add(self.vlan_tag, 'sg', 'remote_id',
                              constants.INGRESS_DIRECTION, constants.IPv4, 3)
-            self.manager.update_flows_for_vlan(self.vlan_tag)
-        self.assertEqual(self.driver._add_flow.call_args_list,
-            [mock.call(actions='conjunction(16,1/2)', ct_state='+est-rel-rpl',
-                       dl_type=2048, nw_src='10.22.3.4/32', priority=70,
-                       reg_net=self.vlan_tag, table=82),
-             mock.call(actions='conjunction(17,1/2)', ct_state='+new-est',
-                       dl_type=2048, nw_src='10.22.3.4/32', priority=70,
-                       reg_net=self.vlan_tag, table=82),
-             mock.call(actions='conjunction(22,1/2)', ct_state='+est-rel-rpl',
-                       dl_type=2048, nw_src='10.22.3.4/32', priority=73,
-                       reg_net=self.vlan_tag, table=82),
-             mock.call(actions='conjunction(23,1/2)', ct_state='+new-est',
-                       dl_type=2048, nw_src='10.22.3.4/32', priority=73,
-                       reg_net=self.vlan_tag, table=82)])
+            self.manager.update_flows_for_vlan(self.vlan_tag, 'ofport1')
+        calls = [
+            mock.call(actions='conjunction(16,1/2)', ct_state='+est-rel-rpl',
+                      dl_type=2048, nw_src='10.22.3.4/32', priority=70,
+                      reg_net=self.vlan_tag, table=82,
+                      flow_group_id='ofport1'),
+            mock.call(actions='conjunction(17,1/2)', ct_state='+new-est',
+                      dl_type=2048, nw_src='10.22.3.4/32', priority=70,
+                      reg_net=self.vlan_tag, table=82,
+                      flow_group_id='ofport1'),
+            mock.call(actions='conjunction(22,1/2)', ct_state='+est-rel-rpl',
+                      dl_type=2048, nw_src='10.22.3.4/32', priority=73,
+                      reg_net=self.vlan_tag, table=82,
+                      flow_group_id='ofport1'),
+            mock.call(actions='conjunction(23,1/2)', ct_state='+new-est',
+                      dl_type=2048, nw_src='10.22.3.4/32', priority=73,
+                      reg_net=self.vlan_tag, table=82,
+                      flow_group_id='ofport1')]
+        self.assertEqual(self.driver._add_flow.call_args_list, calls)
 
     def _sg_removed(self, sg_name):
         with mock.patch.object(self.manager.conj_id_map,
@@ -471,7 +546,12 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             {'ethertype': constants.IPv6,
              'protocol': constants.PROTO_NAME_TCP,
              'remote_group_id': 2,
-             'direction': constants.EGRESS_DIRECTION}]
+             'direction': constants.EGRESS_DIRECTION},
+            {'ethertype': constants.IPv4,
+             'protocol': constants.PROTO_NAME_TCP,
+             'remote_address_group_id': 3,
+             'direction': constants.EGRESS_DIRECTION}
+        ]
         self.firewall.update_security_group_rules(2, security_group_rules)
 
     def _assert_invalid_conntrack_entries_deleted(self, port_dict):
@@ -775,16 +855,12 @@ class TestOVSFirewallDriver(base.BaseTestCase):
 
         self.firewall.delete_all_port_flows(port)
 
-        call_args1 = {"strict": True,
-                      "priority": 90,
-                      "table": ovs_consts.TRANSIENT_TABLE,
+        call_args1 = {"table": ovs_consts.TRANSIENT_TABLE,
                       "dl_dst": port.mac,
                       "dl_vlan": port.vlan_tag}
         flow1 = mock.call(**call_args1)
 
-        call_args2 = {"strict": True,
-                      "priority": 90,
-                      "table": ovs_consts.TRANSIENT_TABLE,
+        call_args2 = {"table": ovs_consts.TRANSIENT_TABLE,
                       "dl_dst": port.mac,
                       "dl_vlan": port.segment_id}
         flow2 = mock.call(**call_args2)
@@ -795,8 +871,6 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         flow3 = mock.call(**call_args3)
 
         call_args4 = {"in_port": port.ofport,
-                      "strict": True,
-                      "priority": 100,
                       "table": ovs_consts.TRANSIENT_TABLE}
         flow4 = mock.call(**call_args4)
 
@@ -836,8 +910,10 @@ class TestOVSFirewallDriver(base.BaseTestCase):
 
         self.firewall.update_port_filter(port_dict)
         self.assertTrue(self.mock_bridge.br.delete_flows.called)
-        conj_id = self.firewall.conj_ip_manager.conj_id_map.get_conj_id(
+        rsg_conj_id = self.firewall.conj_ip_manager.conj_id_map.get_conj_id(
             2, 2, constants.EGRESS_DIRECTION, constants.IPv6)
+        rag_conj_id = self.firewall.conj_ip_manager.conj_id_map.get_conj_id(
+            2, 3, constants.EGRESS_DIRECTION, constants.IPv4)
         filter_rules = [mock.call(
             actions='resubmit(,{:d})'.format(
                 ovs_consts.ACCEPT_OR_INGRESS_TABLE),
@@ -848,12 +924,20 @@ class TestOVSFirewallDriver(base.BaseTestCase):
             reg5=self.port_ofport,
             table=ovs_consts.RULES_EGRESS_TABLE),
                         mock.call(
-            actions='conjunction({:d},2/2)'.format(conj_id + 6),
+            actions='conjunction({:d},2/2)'.format(rsg_conj_id + 6),
             ct_state=ovsfw_consts.OF_STATE_ESTABLISHED_NOT_REPLY,
             dl_type=mock.ANY,
             nw_proto=6,
             priority=73, reg5=self.port_ofport,
-            table=ovs_consts.RULES_EGRESS_TABLE)]
+            table=ovs_consts.RULES_EGRESS_TABLE),
+                        mock.call(
+            actions='conjunction({:d},2/2)'.format(rag_conj_id + 6),
+            ct_state=ovsfw_consts.OF_STATE_ESTABLISHED_NOT_REPLY,
+            dl_type=mock.ANY,
+            nw_proto=6,
+            priority=73, reg5=self.port_ofport,
+            table=ovs_consts.RULES_EGRESS_TABLE)
+        ]
         self.mock_bridge.br.add_flow.assert_has_calls(
             filter_rules, any_order=True)
         self._assert_invalid_conntrack_entries_deleted(port_dict)
@@ -897,7 +981,7 @@ class TestOVSFirewallDriver(base.BaseTestCase):
         self.firewall.prepare_port_filter(port_dict)
         with self.firewall.defer_apply():
             self.firewall.update_port_filter(port_dict)
-        self.assertEqual(2, self.mock_bridge.apply_flows.call_count)
+        self.mock_bridge.apply_flows.assert_called_once()
 
     def test_update_port_filter_clean_when_port_not_found(self):
         """Check flows are cleaned if port is not found in the bridge."""
@@ -1146,6 +1230,8 @@ class TestCookieContext(base.BaseTestCase):
             ovsfw.OVSFirewallDriver, 'initialize_bridge',
             return_value=bridge.deferred(
                 full_ordered=True, use_bundle=True)).start()
+        mock.patch.object(ovsfw.ConjIdMap, '_init_max_id',
+                          return_value=0).start()
 
         securitygroups_rpc.register_securitygroups_opts()
         self.firewall = ovsfw.OVSFirewallDriver(bridge)
@@ -1163,7 +1249,7 @@ class TestCookieContext(base.BaseTestCase):
                 mock.ANY,
                 process_input='hard_timeout=0,idle_timeout=0,priority=1,'
                               'cookie=%d,actions=drop' % cookie,
-                run_as_root=mock.ANY,
+                run_as_root=True, privsep_exec=True
             ) for cookie in (update_cookie, default_cookie)
         ]
 
