@@ -15,12 +15,11 @@
 from unittest import mock
 
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_consts
 
 from neutron.agent.linux.openvswitch_firewall import constants as ovsfw_consts
 from neutron.agent.linux.openvswitch_firewall import firewall as ovsfw
 from neutron.agent.linux.openvswitch_firewall import rules
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants \
-        as ovs_consts
 from neutron.tests import base
 
 TESTING_VLAN_TAG = 1
@@ -451,7 +450,7 @@ class TestMergeRules(base.BaseTestCase):
                           ({'direction': 'ingress', 'ethertype': 'IPv4',
                             'protocol': 1}, 24)]
         result = rules.merge_common_rules(rule_conj_list)
-        self.assertItemsEqual(
+        self.assertCountEqual(
             [({'direction': 'ingress', 'ethertype': 'IPv4',
                'protocol': 1}, [8, 24]),
              ({'direction': 'ingress', 'ethertype': 'IPv4',
