@@ -16,6 +16,7 @@
 from unittest import mock
 
 from neutron_lib import constants
+from neutron_lib.plugins.ml2 import ovs_constants as ovs_const
 from oslo_utils import excutils
 from pyroute2.netlink import exceptions as pyroute2_exc
 
@@ -25,8 +26,6 @@ from neutron.agent.linux import ip_lib
 from neutron.common import utils
 from neutron.conf.agent import common as config
 from neutron.conf.plugins.ml2.drivers import ovs_conf
-from neutron.plugins.ml2.drivers.openvswitch.agent.common \
-    import constants as ovs_const
 from neutron.privileged.agent.linux import ethtool
 from neutron.tests import base
 
@@ -61,17 +60,6 @@ class FakePort(object):
     network_id = network.id
 
 
-class FakeLegacyInterfaceDriver(interface.LinuxInterfaceDriver):
-
-    def plug_new(self, network_id, port_id, device_name, mac_address,
-                 bridge=None, namespace=None, prefix=None, mtu=None):
-        """This is legacy method which don't accepts link_up argument."""
-        pass
-
-    def unplug(self, device_name, bridge=None, namespace=None, prefix=None):
-        pass
-
-
 class TestBase(base.BaseTestCase):
     def setUp(self):
         super(TestBase, self).setUp()
@@ -450,6 +438,11 @@ class TestOVSInterfaceDriver(TestBase):
                 self.device_exists.side_effect = device_exists
                 link = self.ip.return_value.device.return_value.link
                 link.set_address.side_effect = (RuntimeError, None)
+                namespace_obj = (
+                    self.ip.return_value.ensure_namespace.return_value)
+                self.ip.ensure_namespace.return_value = namespace_obj
+                namespace_obj.add_device_to_namespace.side_effect = (
+                    ip_lib.NetworkInterfaceNotFound, utils.WaitTimeout, None)
                 ovs.plug('01234567-1234-1234-99',
                          'port-1234',
                          'tap0',
@@ -473,6 +466,10 @@ class TestOVSInterfaceDriver(TestBase):
             if namespace:
                 expected.extend(
                     [mock.call().ensure_namespace(namespace),
+                     mock.call().ensure_namespace().add_device_to_namespace(
+                         mock.ANY),
+                     mock.call().ensure_namespace().add_device_to_namespace(
+                         mock.ANY),
                      mock.call().ensure_namespace().add_device_to_namespace(
                          mock.ANY)])
             expected.extend([
@@ -497,7 +494,8 @@ class TestOVSInterfaceDriver(TestBase):
                     reraise.start()
                     ip_wrapper = mock.Mock()
                     for exception in (OSError(),
-                                      pyroute2_exc.NetlinkError(22)):
+                                      pyroute2_exc.NetlinkError(22),
+                                      RuntimeError()):
                         ip_wrapper.ensure_namespace.side_effect = exception
                         self.ip.return_value = ip_wrapper
                         delete_port.reset_mock()
@@ -519,6 +517,20 @@ class TestOVSInterfaceDriver(TestBase):
             ovs_br.assert_has_calls([mock.call('br-int'),
                                      mock.call().delete_port('tap0')])
 
+    def test__add_device_to_namespace_retries(self):
+        ovs = interface.OVSInterfaceDriver(self.conf)
+        namespace_obj = self.ip.return_value.ensure_namespace.return_value
+        self.ip.ensure_namespace.return_value = namespace_obj
+        namespace_obj.add_device_to_namespace.side_effect = (
+            ip_lib.NetworkInterfaceNotFound)
+        device = mock.MagicMock()
+        self.assertRaises(
+            ip_lib.NetworkInterfaceNotFound,
+            ovs._add_device_to_namespace,
+            self.ip, device, "test-ns")
+        self.assertEqual(10, namespace_obj.add_device_to_namespace.call_count)
+        self.assertIsNone(device.namespace)
+
 
 class TestOVSInterfaceDriverWithVeth(TestOVSInterfaceDriver):
 
@@ -685,20 +697,3 @@ class TestBridgeInterfaceDriver(TestBase):
 
         self.ip_dev.assert_has_calls([mock.call('tap0', namespace=None),
                                       mock.call().link.delete()])
-
-
-class TestLegacyDriver(TestBase):
-
-    def test_plug(self):
-        self.device_exists.return_value = False
-        with mock.patch('neutron.agent.linux.interface.LOG.warning') as log:
-            driver = FakeLegacyInterfaceDriver(self.conf)
-            try:
-                driver.plug(
-                    '01234567-1234-1234-99', 'port-1234', 'tap0',
-                    'aa:bb:cc:dd:ee:ff')
-            except TypeError:
-                self.fail("LinuxInterfaceDriver class can not call properly "
-                          "plug_new method from the legacy drivers that "
-                          "do not accept 'link_up' parameter.")
-            log.assert_called_once()
