@@ -75,6 +75,11 @@ class AgentRPCPluginApi(base.BaseTestCase):
 
 
 class AgentPluginReportState(base.BaseTestCase):
+    def test_plugin_report_state_timeout_report_interval(self):
+        cfg.CONF.set_override('report_interval', 15, 'AGENT')
+        reportStateAPI = rpc.PluginReportStateAPI('test')
+        self.assertEqual(reportStateAPI.timeout, 15)
+
     def test_plugin_report_state_use_call(self):
         topic = 'test'
         reportStateAPI = rpc.PluginReportStateAPI(topic)
@@ -215,12 +220,15 @@ class TestCacheBackedPluginApi(base.BaseTestCase):
             binding_levels=[ports.PortBindingLevel(port_id=self._port_id,
                                                    host='host1',
                                                    level=0,
-                                                   segment=self._segment)])
+                                                   segment=self._segment)],
+            status='ACTIVE')
 
     def test__legacy_notifier_resource_delete(self):
         self._api._legacy_notifier(resources.PORT, events.AFTER_DELETE, self,
-                                   mock.ANY, resource_id=self._port_id,
-                                   existing=self._port)
+                                   payload=events.DBEventPayload(
+                                       mock.ANY,
+                                       resource_id=self._port_id,
+                                       states=(self._port,)))
         self._api._legacy_interface.port_update.assert_not_called()
         self._api._legacy_interface.port_delete.assert_called_once_with(
             mock.ANY, port={'id': self._port_id}, port_id=self._port_id)
@@ -229,9 +237,14 @@ class TestCacheBackedPluginApi(base.BaseTestCase):
     def test__legacy_notifier_resource_update(self):
         updated_port = ports.Port(id=self._port_id, name='updated_port')
         self._api._legacy_notifier(resources.PORT, events.AFTER_UPDATE, self,
-                                   mock.ANY, changed_fields=set(['name']),
-                                   resource_id=self._port_id,
-                                   existing=self._port, updated=updated_port)
+                                   payload=events.DBEventPayload(
+                                       mock.ANY,
+                                       metadata={
+                                           'changed_fields': set(['name'])
+                                       },
+                                       resource_id=self._port_id,
+                                       states=(self._port, updated_port)))
+
         self._api._legacy_interface.port_delete.assert_not_called()
         self._api._legacy_interface.port_update.assert_called_once_with(
             mock.ANY, port={'id': self._port_id}, port_id=self._port_id)
@@ -246,11 +259,16 @@ class TestCacheBackedPluginApi(base.BaseTestCase):
                       ports.PortBinding(port_id=self._port_id,
                                         host='host1',
                                         status=constants.INACTIVE)])
-        self._api._legacy_notifier(resources.PORT, events.AFTER_UPDATE, self,
-                                   mock.ANY,
-                                   changed_fields=set(['name', 'bindings']),
-                                   resource_id=self._port_id,
-                                   existing=self._port, updated=updated_port)
+        self._api._legacy_notifier(
+            resources.PORT, events.AFTER_UPDATE, self,
+            payload=events.DBEventPayload(
+                mock.ANY,
+                metadata={
+                    'changed_fields': set(['name', 'bindings'])
+                },
+                resource_id=self._port_id,
+                states=(self._port, updated_port)))
+
         self._api._legacy_interface.port_update.assert_not_called()
         self._api._legacy_interface.port_delete.assert_not_called()
 
@@ -267,27 +285,40 @@ class TestCacheBackedPluginApi(base.BaseTestCase):
             bindings=[ports.PortBinding(port_id=self._port_id,
                                         host='host2',
                                         status=constants.ACTIVE)])
-        self._api._legacy_notifier(resources.PORT, events.AFTER_UPDATE, self,
-                                   mock.ANY,
-                                   changed_fields=set(['name', 'bindings']),
-                                   resource_id=self._port_id,
-                                   existing=self._port, updated=updated_port)
+        self._api._legacy_notifier(
+            resources.PORT, events.AFTER_UPDATE, self,
+            payload=events.DBEventPayload(
+                mock.ANY,
+                metadata={
+                    'changed_fields': set(['name', 'bindings'])
+                },
+                resource_id=self._port_id,
+                states=(self._port, updated_port)))
+
         self._api._legacy_interface.port_update.assert_called_once_with(
             mock.ANY, port={'id': self._port_id}, port_id=self._port_id)
         self._api._legacy_interface.port_delete.assert_not_called()
         self._api._legacy_interface.binding_deactivate.assert_not_called()
 
     def test__legacy_notifier_existing_or_updated_is_none(self):
-        self._api._legacy_notifier(resources.PORT, events.AFTER_UPDATE,
-                                   self, mock.ANY,
-                                   changed_fields=set(['name', 'bindings']),
-                                   resource_id=self._port_id,
-                                   existing=None, updated=None)
-        self._api._legacy_notifier(resources.PORT, events.AFTER_UPDATE, self,
-                                   mock.ANY,
-                                   changed_fields=set(['name', 'bindings']),
-                                   resource_id=self._port_id,
-                                   existing=self._port, updated=None)
+        self._api._legacy_notifier(
+            resources.PORT, events.AFTER_UPDATE, self,
+            payload=events.DBEventPayload(
+                mock.ANY,
+                metadata={
+                    'changed_fields': set(['name', 'bindings'])
+                },
+                resource_id=self._port_id,
+                states=(None, None)))
+        self._api._legacy_notifier(
+            resources.PORT, events.AFTER_UPDATE, self,
+            payload=events.DBEventPayload(
+                mock.ANY,
+                metadata={
+                    'changed_fields': set(['name', 'bindings'])
+                },
+                resource_id=self._port_id,
+                states=(self._port, None)))
         call = mock.call(mock.ANY, port={'id': self._port_id},
                          port_id=self._port_id)
         self._api._legacy_interface.port_update.assert_has_calls([call, call])
@@ -321,21 +352,13 @@ class TestCacheBackedPluginApi(base.BaseTestCase):
         self.assertIn(constants.NO_ACTIVE_BINDING, entry)
 
     def test_get_device_details_migrating_to_host(self):
-        for live_migration_events, migrating_to in ((True, 'host2'),
-                                                    (False, 'irrelevant')):
-            cfg.CONF.set_override('live_migration_events',
-                                  live_migration_events, group='nova')
-            profile = jsonutils.dumps({'migrating_to': migrating_to})
-            self._port.bindings[0].profile = profile
-            self._api.remote_resource_cache.get_resource_by_id.side_effect = [
-                self._port, self._network]
-            entry = self._api.get_device_details(mock.ANY, self._port_id,
-                                                 mock.ANY, 'host2')
-            if live_migration_events:
-                self.assertEqual('host2', entry['migrating_to'])
-            else:
-                self.assertTrue(entry[constants.NO_ACTIVE_BINDING])
-                self.assertNotIn('migrating_to', entry)
+        profile = jsonutils.dumps({'migrating_to': 'host2'})
+        self._port.bindings[0].profile = profile
+        self._api.remote_resource_cache.get_resource_by_id.side_effect = [
+            self._port, self._network]
+        entry = self._api.get_device_details(mock.ANY, self._port_id,
+                                             mock.ANY, 'host2')
+        self.assertEqual('host2', entry['migrating_to'])
 
     @mock.patch('neutron.agent.resource_cache.RemoteResourceCache')
     def test_initialization_with_default_resources(self, rcache_class):
