@@ -102,8 +102,8 @@ class FakePluginWithExtension(service_base.ServicePluginBase):
 class ExtensionPathTest(base.BaseTestCase):
 
     def setUp(self):
-        self.base_path = extensions.get_extensions_path()
         super(ExtensionPathTest, self).setUp()
+        self.base_path = extensions.get_extensions_path()
 
     def test_get_extensions_path_with_plugins(self):
         cfg.CONF.set_override('api_extensions_path',
@@ -1032,7 +1032,8 @@ class ExtensionExtendedAttributeTestCase(base.BaseTestCase):
         self.agentscheduler_dbMinxin = directory.get_plugin()
 
         quota.QUOTAS._driver = None
-        cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
+        cfg.CONF.set_override('quota_driver',
+                              'neutron.db.quota.driver_null.DbQuotaDriverNull',
                               group='QUOTAS')
 
     def _do_request(self, method, path, data=None, params=None, action=None):
