@@ -40,9 +40,9 @@ from neutron.api import api_common
 from neutron.api import extensions
 from neutron.api.v2 import base as v2_base
 from neutron.api.v2 import router
+from neutron.conf import quota as quota_conf
 from neutron import policy
 from neutron import quota
-from neutron.quota import resource_registry
 from neutron.tests import base
 from neutron.tests import tools
 from neutron.tests.unit import dummy_plugin
@@ -50,6 +50,7 @@ from neutron.tests.unit import testlib_api
 
 
 EXTDIR = os.path.join(base.ROOTDIR, 'unit/extensions')
+NULL_QUOTA_DRIVER = 'neutron.db.quota.driver_null.DbQuotaDriverNull'
 
 _uuid = uuidutils.generate_uuid
 
@@ -98,7 +99,7 @@ class APIv2TestBase(base.BaseTestCase):
         self.api = webtest.TestApp(api)
 
         quota.QUOTAS._driver = None
-        cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
+        cfg.CONF.set_override('quota_driver', quota_conf.QUOTA_DB_DRIVER,
                               group='QUOTAS')
 
         # APIRouter initialization resets policy module, re-initializing it
@@ -1302,6 +1303,9 @@ class NotificationTest(APIv2TestBase):
     def setUp(self):
         super(NotificationTest, self).setUp()
         fake_notifier.reset()
+        quota.QUOTAS._driver = None
+        cfg.CONF.set_override('quota_driver', NULL_QUOTA_DRIVER,
+                              group='QUOTAS')
 
     def _resource_op_notifier(self, opname, resource, expected_errors=False):
         initial_input = {resource: {'name': 'myname'}}
@@ -1341,29 +1345,30 @@ class NotificationTest(APIv2TestBase):
 
         self.assertEqual(expected_code, res.status_int)
 
-    def test_network_create_notifer(self):
+    def test_network_create_notifier(self):
         self._resource_op_notifier('create', 'network')
 
-    def test_network_delete_notifer(self):
+    def test_network_delete_notifier(self):
         self._resource_op_notifier('delete', 'network')
 
-    def test_network_update_notifer(self):
+    def test_network_update_notifier(self):
         self._resource_op_notifier('update', 'network')
 
 
 class RegistryNotificationTest(APIv2TestBase):
 
     def setUp(self):
-        # This test does not have database support so tracking cannot be used
-        cfg.CONF.set_override('track_quota_usage', False, group='QUOTAS')
         super(RegistryNotificationTest, self).setUp()
+        quota.QUOTAS._driver = None
+        cfg.CONF.set_override('quota_driver', NULL_QUOTA_DRIVER,
+                              group='QUOTAS')
 
-    def _test_registry_notify(self, opname, resource, initial_input=None):
+    def _test_registry_publish(self, opname, resource, initial_input=None):
         instance = self.plugin.return_value
         instance.get_networks.return_value = initial_input
         instance.get_networks_count.return_value = 0
         expected_code = exc.HTTPCreated.code
-        with mock.patch.object(registry, 'publish') as notify:
+        with mock.patch.object(registry, 'publish') as publish:
             if opname == 'create':
                 res = self.api.post_json(
                     _get_path('networks'),
@@ -1376,93 +1381,52 @@ class RegistryNotificationTest(APIv2TestBase):
             if opname == 'delete':
                 res = self.api.delete(_get_path('networks', id=_uuid()))
                 expected_code = exc.HTTPNoContent.code
-            self.assertTrue(notify.called)
+            self.assertTrue(publish.called)
         self.assertEqual(expected_code, res.status_int)
 
-    def test_network_create_registry_notify(self):
+    def test_network_create_registry_publish(self):
         input = {'network': {'name': 'net',
                              'tenant_id': _uuid()}}
-        self._test_registry_notify('create', 'network', input)
+        self._test_registry_publish('create', 'network', input)
 
-    def test_network_delete_registry_notify(self):
-        self._test_registry_notify('delete', 'network')
+    def test_network_delete_registry_publish(self):
+        self._test_registry_publish('delete', 'network')
 
-    def test_network_update_registry_notify(self):
+    def test_network_update_registry_publish(self):
         input = {'network': {'name': 'net'}}
-        self._test_registry_notify('update', 'network', input)
+        self._test_registry_publish('update', 'network', input)
 
-    def test_networks_create_bulk_registry_notify(self):
+    def test_networks_create_bulk_registry_publish(self):
         input = {'networks': [{'name': 'net1',
                                'tenant_id': _uuid()},
                               {'name': 'net2',
                                'tenant_id': _uuid()}]}
-        self._test_registry_notify('create', 'network', input)
+        self._test_registry_publish('create', 'network', input)
 
 
 class QuotaTest(APIv2TestBase):
+    """This class checks the quota enforcement API, regardless of the driver"""
 
-    def setUp(self):
-        # This test does not have database support so tracking cannot be used
-        cfg.CONF.set_override('track_quota_usage', False, group='QUOTAS')
-        super(QuotaTest, self).setUp()
-        # Use mock to let the API use a different QuotaEngine instance for
-        # unit test in this class. This will ensure resource are registered
-        # again and instantiated with neutron.quota.resource.CountableResource
-        replacement_registry = resource_registry.ResourceRegistry()
-        registry_patcher = mock.patch('neutron.quota.resource_registry.'
-                                      'ResourceRegistry.get_instance')
-        mock_registry = registry_patcher.start().return_value
-        mock_registry.get_resource = replacement_registry.get_resource
-        mock_registry.resources = replacement_registry.resources
-        # Register a resource
-        replacement_registry.register_resource_by_name('network')
-
-    def test_create_network_quota(self):
-        cfg.CONF.set_override('quota_network', 1, group='QUOTAS')
+    def test_create_network_quota_exceeded(self):
         initial_input = {'network': {'name': 'net1', 'tenant_id': _uuid()}}
-        full_input = {'network': {'admin_state_up': True, 'subnets': []}}
-        full_input['network'].update(initial_input['network'])
-
-        instance = self.plugin.return_value
-        instance.get_networks_count.return_value = 1
-        res = self.api.post_json(
-            _get_path('networks'), initial_input, expect_errors=True)
-        instance.get_networks_count.assert_called_with(mock.ANY,
-                                                       filters=mock.ANY)
-        self.assertIn("Quota exceeded for resources",
-                      res.json['NeutronError']['message'])
-
-    def test_create_network_quota_no_counts(self):
-        cfg.CONF.set_override('quota_network', 1, group='QUOTAS')
-        initial_input = {'network': {'name': 'net1', 'tenant_id': _uuid()}}
-        full_input = {'network': {'admin_state_up': True, 'subnets': []}}
-        full_input['network'].update(initial_input['network'])
-
-        instance = self.plugin.return_value
-        instance.get_networks_count.side_effect = (
-            NotImplementedError())
-        instance.get_networks.return_value = ["foo"]
-        res = self.api.post_json(
-            _get_path('networks'), initial_input, expect_errors=True)
-        instance.get_networks_count.assert_called_with(mock.ANY,
-                                                       filters=mock.ANY)
+        with mock.patch.object(quota.QUOTAS, 'make_reservation',
+                               side_effect=n_exc.OverQuota(overs='network')):
+            res = self.api.post_json(
+                _get_path('networks'), initial_input, expect_errors=True)
         self.assertIn("Quota exceeded for resources",
                       res.json['NeutronError']['message'])
 
     def test_create_network_quota_without_limit(self):
-        cfg.CONF.set_override('quota_network', -1, group='QUOTAS')
         initial_input = {'network': {'name': 'net1', 'tenant_id': _uuid()}}
-        instance = self.plugin.return_value
-        instance.get_networks_count.return_value = 3
-        res = self.api.post_json(
-            _get_path('networks'), initial_input)
+        with mock.patch.object(quota.QUOTAS, 'make_reservation'), \
+                mock.patch.object(quota.QUOTAS, 'commit_reservation'):
+            res = self.api.post_json(
+                _get_path('networks'), initial_input)
         self.assertEqual(exc.HTTPCreated.code, res.status_int)
 
 
 class ExtensionTestCase(base.BaseTestCase):
     def setUp(self):
-        # This test does not have database support so tracking cannot be used
-        cfg.CONF.set_override('track_quota_usage', False, group='QUOTAS')
         super(ExtensionTestCase, self).setUp()
         plugin = 'neutron.neutron_plugin_base_v2.NeutronPluginBaseV2'
         # Ensure existing ExtensionManager is not used
@@ -1487,7 +1451,7 @@ class ExtensionTestCase(base.BaseTestCase):
         self.api = webtest.TestApp(api)
 
         quota.QUOTAS._driver = None
-        cfg.CONF.set_override('quota_driver', 'neutron.quota.ConfDriver',
+        cfg.CONF.set_override('quota_driver', NULL_QUOTA_DRIVER,
                               group='QUOTAS')
 
     def test_extended_create(self):
