@@ -15,21 +15,30 @@
 
 from unittest import mock
 
+import ddt
 import netaddr
+from neutron_lib.api.definitions import external_net as extnet_apidef
+from neutron_lib.api.definitions import l3 as l3_apidef
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
 from neutron_lib.callbacks import resources
 from neutron_lib import constants as n_const
 from neutron_lib import context
+from neutron_lib.db import api as db_api
 from neutron_lib import exceptions as n_exc
+from neutron_lib.exceptions import extraroute as xroute_exc
 from neutron_lib.plugins import constants as plugin_constants
 from neutron_lib.plugins import directory
 from neutron_lib.plugins import utils as plugin_utils
 from oslo_utils import uuidutils
 import testtools
+import webob.exc
 
+from neutron.db import extraroute_db
 from neutron.db import l3_db
 from neutron.db.models import l3 as l3_models
+from neutron.db import models_v2
+from neutron.extensions import segment as segment_ext
 from neutron.objects import base as base_obj
 from neutron.objects import network as network_obj
 from neutron.objects import ports as port_obj
@@ -39,10 +48,18 @@ from neutron.tests import base
 from neutron.tests.unit.db import test_db_base_plugin_v2
 
 
-class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
-    def setUp(self):
-        super(TestL3_NAT_dbonly_mixin, self).setUp()
-        self.db = l3_db.L3_NAT_dbonly_mixin()
+@ddt.ddt
+class TestL3_NAT_dbonly_mixin(
+        test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
+
+    def setUp(self, *args, **kwargs):
+        super(TestL3_NAT_dbonly_mixin, self).setUp(*args, **kwargs)
+        # "extraroute_db.ExtraRoute_dbonly_mixin" inherits from
+        # "l3_db.L3_NAT_dbonly_mixin()", the class under test. This is used
+        # instead to test the validation of router routes and GW change because
+        # implements "_validate_routes".
+        self.db = extraroute_db.ExtraRoute_dbonly_mixin()
+        self.ctx = mock.Mock()
 
     def test__each_port_having_fixed_ips_none(self):
         """Be sure the method returns an empty list when None is passed"""
@@ -132,7 +149,8 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
 
         ports = [{'network_id': 'net_id',
                   'id': 'port_id',
-                  'fixed_ips': [{'subnet_id': mock.sentinel.subnet_id}]}]
+                  'fixed_ips': [{'subnet_id': mock.sentinel.subnet_id}],
+                  'device_owner': 'compute:nova'}]
         with mock.patch.object(directory, 'get_plugin') as get_p:
             get_p().get_networks.return_value = [{'id': 'net_id', 'mtu': 1446}]
             self.db._populate_mtu_and_subnets_for_ports(mock.sentinel.context,
@@ -148,7 +166,73 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
                                'mtu': 1446,
                                'network_id': 'net_id',
                                'subnets': [{k: subnet[k] for k in keys}],
-                               'address_scopes': address_scopes}], ports)
+                               'address_scopes': address_scopes,
+                               'device_owner': 'compute:nova'}], ports)
+
+    @ddt.unpack
+    @ddt.data({'plugin_loaded': False, 'seg1': None, 'seg2': None},
+              {'plugin_loaded': True, 'seg1': None, 'seg2': None},
+              {'plugin_loaded': True, 'seg1': 'seg1', 'seg2': 'seg2'})
+    @mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                       '_get_subnets_by_network_list')
+    def test__populate_ports_for_subnets_gw_port(self, get_subnets_by_network,
+                                                 plugin_loaded, seg1, seg2):
+        subnets = [
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.1.0.0/24',
+             'gateway_ip': mock.sentinel.gateway_ip,
+             'dns_nameservers': mock.sentinel.dns_nameservers,
+             'ipv6_ra_mode': mock.sentinel.ipv6_ra_mode,
+             'subnetpool_id': mock.sentinel.subnetpool_id,
+             'address_scope_id': mock.sentinel.address_scope_id,
+             'segment_id': seg1},
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.2.0.0/24',
+             'gateway_ip': mock.sentinel.gateway_ip,
+             'dns_nameservers': mock.sentinel.dns_nameservers,
+             'ipv6_ra_mode': mock.sentinel.ipv6_ra_mode,
+             'subnetpool_id': mock.sentinel.subnetpool_id,
+             'address_scope_id': mock.sentinel.address_scope_id,
+             'segment_id': seg1},
+            {'id': uuidutils.generate_uuid(),
+             'cidr': '10.3.0.0/24',
+             'gateway_ip': mock.sentinel.gateway_ip,
+             'dns_nameservers': mock.sentinel.dns_nameservers,
+             'ipv6_ra_mode': mock.sentinel.ipv6_ra_mode,
+             'subnetpool_id': mock.sentinel.subnetpool_id,
+             'address_scope_id': mock.sentinel.address_scope_id,
+             'segment_id': seg2}]
+        get_subnets_by_network.return_value = {'net_id': subnets}
+
+        ports = [{'network_id': 'net_id',
+                  'id': 'port_id',
+                  'fixed_ips': [{'subnet_id': subnets[0]['id']}],
+                  'device_owner': n_const.DEVICE_OWNER_ROUTER_GW}]
+        with mock.patch.object(directory, 'get_plugin') as get_p, \
+                mock.patch.object(segment_ext.SegmentPluginBase,
+                                  'is_loaded', return_value=plugin_loaded):
+            get_p().get_networks.return_value = [{'id': 'net_id', 'mtu': 1446}]
+            self.db._populate_mtu_and_subnets_for_ports(mock.sentinel.context,
+                                                        ports)
+            keys = ('id', 'cidr', 'gateway_ip', 'ipv6_ra_mode',
+                    'subnetpool_id', 'dns_nameservers')
+            address_scopes = {4: mock.sentinel.address_scope_id, 6: None}
+            reference = {'fixed_ips': [{'subnet_id': subnets[0]['id'],
+                                        'prefixlen': 24}],
+                         'id': 'port_id',
+                         'mtu': 1446,
+                         'network_id': 'net_id',
+                         'subnets': [{k: subnets[0][k] for k in keys}],
+                         'address_scopes': address_scopes,
+                         'device_owner': n_const.DEVICE_OWNER_ROUTER_GW,
+                         'extra_subnets': [{k: subnets[1][k] for k in keys}]}
+            # If RPN plugin is not enabled or the network subnets do not have
+            # associated segments (that means this is not a RPN), all subnets
+            # should be passed in "subnets" + "extra_subnets".
+            if not plugin_loaded or subnets[0]['segment_id'] is None:
+                reference['extra_subnets'].append(
+                    {k: subnets[2][k] for k in keys})
+            self.assertEqual([reference], ports)
 
     def test__get_sync_floating_ips_no_query(self):
         """Basic test that no query is performed if no router ids are passed"""
@@ -272,7 +356,7 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
                 mock.ANY, fip, floatingip_obj)
 
     def test__notify_attaching_interface(self):
-        with mock.patch.object(l3_db.registry, 'notify') as mock_notify:
+        with mock.patch.object(l3_db.registry, 'publish') as mock_notify:
             context = mock.MagicMock()
             router_id = 'router_id'
             net_id = 'net_id'
@@ -281,14 +365,22 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
             port = {'network_id': net_id}
             intf = {}
             self.db._notify_attaching_interface(context, router_db, port, intf)
-            kwargs = {'context': context, 'router_id': router_id,
-                      'network_id': net_id, 'interface_info': intf,
-                      'router_db': router_db, 'port': port}
+
             mock_notify.assert_called_once_with(
                 resources.ROUTER_INTERFACE, events.BEFORE_CREATE, self.db,
-                **kwargs)
+                payload=mock.ANY)
+            payload = mock_notify.mock_calls[0][2]['payload']
+            self.assertEqual(context, payload.context)
+            self.assertEqual(router_id, payload.resource_id)
+            self.assertEqual(net_id, payload.metadata.get('network_id'))
+            self.assertEqual(intf, payload.metadata.get('interface_info'))
+            self.assertEqual(router_db, payload.latest_state)
+            self.assertEqual(port, payload.metadata.get('port'))
 
     def test__create_gw_port(self):
+        # NOTE(slaweq): this test is probably wrong
+        # returning dict as gw_port breaks test later in L334 in
+        # neutron.db.l3_db file
         router_id = '2afb8434-7380-43a2-913f-ba3a5ad5f349'
         router = l3_models.Router(id=router_id)
         new_network_id = 'net-id'
@@ -298,37 +390,129 @@ class TestL3_NAT_dbonly_mixin(base.BaseTestCase):
                    'id': '8742d007-6f05-4b7e-abdb-11818f608959'}
         ctx = context.get_admin_context()
 
-        with mock.patch.object(directory, 'get_plugin') as get_p, \
-                mock.patch.object(get_p(), 'get_subnets_by_network',
-                                  return_value=mock.ANY), \
-                mock.patch.object(get_p(), '_get_port',
-                                  return_value=gw_port), \
-                mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
-                                  '_check_for_dup_router_subnets') as cfdrs,\
-                mock.patch.object(plugin_utils, 'create_port',
-                                  return_value=gw_port), \
-                mock.patch.object(ctx.session, 'add'), \
-                mock.patch.object(base_obj.NeutronDbObject, 'create'), \
-                mock.patch.object(l3_db.registry, 'publish') as mock_notify:
+        with db_api.CONTEXT_WRITER.using(ctx):
+            with mock.patch.object(directory, 'get_plugin') as get_p, \
+                    mock.patch.object(get_p(), 'get_subnets_by_network',
+                                      return_value=mock.ANY), \
+                    mock.patch.object(get_p(), '_get_port',
+                                      return_value=gw_port), \
+                    mock.patch.object(l3_db.L3_NAT_dbonly_mixin,
+                                      '_check_for_dup_router_subnets') as \
+                    cfdrs, \
+                    mock.patch.object(plugin_utils, 'create_port',
+                                      return_value=gw_port), \
+                    mock.patch.object(ctx.session, 'add'), \
+                    mock.patch.object(base_obj.NeutronDbObject, 'create'), \
+                    mock.patch.object(l3_db.registry, 'publish') as \
+                    mock_notify, \
+                    mock.patch.object(l3_db.L3_NAT_dbonly_mixin, '_get_router',
+                                      return_value=router):
 
-            self.db._create_gw_port(ctx, router_id=router_id,
-                                    router=router,
-                                    new_network_id=new_network_id,
-                                    ext_ips=ext_ips)
+                self.db._create_gw_port(ctx, router_id=router_id,
+                                        router=router,
+                                        new_network_id=new_network_id,
+                                        ext_ips=ext_ips)
 
-            expected_gw_ips = ['1.1.1.1']
+                expected_gw_ips = ['1.1.1.1']
 
-            self.assertTrue(cfdrs.called)
-            mock_notify.assert_called_with(
-                resources.ROUTER_GATEWAY, events.AFTER_CREATE,
-                self.db._create_gw_port, payload=mock.ANY)
-            cb_payload = mock_notify.mock_calls[1][2]['payload']
-            self.assertEqual(ctx, cb_payload.context)
-            self.assertEqual(expected_gw_ips,
-                             cb_payload.metadata.get('gateway_ips'))
-            self.assertEqual(new_network_id,
-                             cb_payload.metadata.get('network_id'))
-            self.assertEqual(router_id, cb_payload.resource_id)
+                self.assertTrue(cfdrs.called)
+                mock_notify.assert_called_with(
+                    resources.ROUTER_GATEWAY, events.AFTER_CREATE,
+                    self.db._create_gw_port, payload=mock.ANY)
+                cb_payload = mock_notify.mock_calls[1][2]['payload']
+                self.assertEqual(ctx, cb_payload.context)
+                self.assertEqual(expected_gw_ips,
+                                 cb_payload.metadata.get('gateway_ips'))
+                self.assertEqual(new_network_id,
+                                 cb_payload.metadata.get('network_id'))
+                self.assertEqual(router_id, cb_payload.resource_id)
+
+    def _create_router(self, gw_port=True, num_ports=2, create_routes=True):
+        # GW CIDR: 10.0.0.0/24
+        # Interface CIDRS: 10.0.1.0/24, 10.0.2.0/24, etc.
+        router_id = uuidutils.generate_uuid()
+        port_gw_cidr = netaddr.IPNetwork('10.0.0.0/24')
+        rports = []
+        if gw_port:
+            port_gw = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(port_gw_cidr.ip + 1))])
+            rports.append(l3_models.RouterPort(router_id=router_id,
+                                               port=port_gw))
+        else:
+            port_gw = None
+
+        port_cidrs = []
+        port_subnets = []
+        for idx in range(num_ports):
+            cidr = port_gw_cidr.cidr.next(idx + 1)
+            port = models_v2.Port(
+                id=uuidutils.generate_uuid(),
+                fixed_ips=[models_v2.IPAllocation(
+                    ip_address=str(cidr.ip + 1))])
+            port_cidrs.append(cidr)
+            rports.append(l3_models.RouterPort(router_id=router_id, port=port))
+            port_subnets.append({'cidr': str(cidr)})
+
+        routes = []
+        if create_routes:
+            for cidr in [*port_cidrs, port_gw_cidr]:
+                routes.append(l3_models.RouterRoute(
+                    destination=str(cidr.next(100)),
+                    nexthop=str(cidr.ip + 10)))
+        return (l3_models.Router(
+            id=router_id, attached_ports=rports, route_list=routes,
+            gw_port_id=port_gw.id if port_gw else None), port_subnets)
+
+    def test__validate_gw_info(self):
+        gw_network = mock.Mock(subnets=[mock.Mock(cidr='10.0.0.0/24')],
+                               external=True)
+        router, port_subnets = self._create_router(gw_port=False)
+        info = {'network_id': 'net_id'}
+        with mock.patch.object(self.db._core_plugin, '_get_network',
+                               return_value=gw_network), \
+                mock.patch.object(self.db._core_plugin, 'get_subnet',
+                                  side_effect=port_subnets):
+            self.assertEqual(
+                'net_id',
+                self.db._validate_gw_info(self.ctx, info, [], router))
+
+    def test__validate_gw_info_no_route_connectivity(self):
+        gw_network = mock.Mock(subnets=[mock.Mock(cidr='10.50.0.0/24')],
+                               external=True)
+        router, port_subnets = self._create_router(gw_port=False)
+        info = {'network_id': 'net_id'}
+        with mock.patch.object(self.db._core_plugin, '_get_network',
+                               return_value=gw_network), \
+                mock.patch.object(self.db._core_plugin, 'get_subnet',
+                                  side_effect=port_subnets):
+            self.assertRaises(
+                xroute_exc.InvalidRoutes, self.db._validate_gw_info, self.ctx,
+                info, [], router)
+
+    def test__validate_gw_info_delete_gateway(self):
+        router, port_subnets = self._create_router()
+        info = {'network_id': None}
+        with mock.patch.object(self.db._core_plugin, '_get_network',
+                               return_value=None), \
+                mock.patch.object(self.db._core_plugin, 'get_subnet',
+                                  side_effect=port_subnets):
+            self.assertRaises(
+                xroute_exc.InvalidRoutes, self.db._validate_gw_info, self.ctx,
+                info, [], router)
+
+    def test__validate_gw_info_delete_gateway_no_route(self):
+        gw_network = mock.Mock(subnets=[mock.Mock(cidr='10.50.0.0/24')],
+                               external=True)
+        router, port_subnets = self._create_router(create_routes=False)
+        info = {'network_id': None}
+        with mock.patch.object(self.db._core_plugin, '_get_network',
+                               return_value=gw_network), \
+                mock.patch.object(self.db._core_plugin, 'get_subnet',
+                                  side_effect=port_subnets):
+            self.assertIsNone(
+                self.db._validate_gw_info(mock.ANY, info, [], router))
 
 
 class L3_NAT_db_mixin(base.BaseTestCase):
@@ -425,23 +609,27 @@ class L3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
             self.ctx, network_id=self.network['network']['id'])
         network_obj.Network.get_object(
             self.ctx, id=self.network['network']['id']).delete()
+        router_ports = l3_obj.RouterPort.get_objects(
+            self.ctx, **{'router_id': self.router['id']})
+        for router_port in router_ports:
+            router_port.delete()
         l3_obj.Router.get_object(self.ctx, id=self.router['id']).delete()
 
     def create_router(self, router):
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self.mixin.create_router(self.ctx, router)
 
     def create_port(self, net_id, port_info):
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self._make_port(self.fmt, net_id, **port_info)
 
     def create_network(self, name=None, **kwargs):
         name = name or 'network1'
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self._make_network(self.fmt, name, True, **kwargs)
 
     def create_subnet(self, network, gateway, cidr, **kwargs):
-        with self.ctx.session.begin(subtransactions=True):
+        with db_api.CONTEXT_WRITER.using(self.ctx):
             return self._make_subnet(self.fmt, network, gateway, cidr,
                                      **kwargs)
 
@@ -527,3 +715,104 @@ class L3TestCase(test_db_base_plugin_v2.NeutronDbPluginV2TestCase):
         mock_log.warning.assert_called_once_with(self.GET_PORTS_BY_ROUTER_MSG,
                                                  msg_vars)
         self._check_routerports((False, True))
+
+    def test_create_router_notify(self):
+        with mock.patch.object(l3_db.registry, 'publish') as mock_publish:
+            router = {'router': {'name': 'foo_router',
+                                 'admin_state_up': True,
+                                 'tenant_id': 'foo_tenant'}}
+            self.create_router(router)
+            expected_calls = [
+                mock.call(resources.ROUTER, events.BEFORE_CREATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.PRECOMMIT_CREATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.AFTER_CREATE,
+                          self.mixin, payload=mock.ANY),
+            ]
+            mock_publish.assert_has_calls(expected_calls)
+
+    def test_update_router_notify(self):
+        with mock.patch.object(l3_db.registry, 'publish') as mock_publish:
+            self.mixin.update_router(self.ctx, self.router['id'],
+                                     {'router': {'name': 'test1'}})
+            expected_calls = [
+                mock.call(resources.ROUTER, events.PRECOMMIT_UPDATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.AFTER_UPDATE,
+                         self.mixin, payload=mock.ANY),
+            ]
+            mock_publish.assert_has_calls(expected_calls)
+
+    def _create_external_network(self, name=None, **kwargs):
+        name = name or 'network1'
+        kwargs[extnet_apidef.EXTERNAL] = True
+        with db_api.CONTEXT_WRITER.using(self.ctx):
+            res = self._create_network(
+                self.fmt, name, True,
+                arg_list=(extnet_apidef.EXTERNAL,), **kwargs)
+            if res.status_int >= webob.exc.HTTPClientError.code:
+                raise webob.exc.HTTPClientError(code=res.status_int)
+            return self.deserialize(self.fmt, res)
+
+    def test_update_router_gw_notify(self):
+        with mock.patch.object(l3_db.registry, 'publish') as mock_publish:
+            ext_net = self._create_external_network()
+            self.create_subnet(ext_net, '1.1.2.1', '1.1.2.0/24')
+            update_data = {
+                l3_apidef.EXTERNAL_GW_INFO: {
+                    'network_id': ext_net['network']['id']}}
+            self.mixin.update_router(
+                self.ctx, self.router['id'], {'router': update_data})
+            expected_calls = [
+                mock.call(resources.NETWORK, events.BEFORE_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SEGMENT, events.PRECOMMIT_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.NETWORK, events.PRECOMMIT_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.NETWORK, events.AFTER_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.NETWORK, events.BEFORE_RESPONSE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SUBNET, events.BEFORE_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SUBNET, events.AFTER_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SUBNET, events.BEFORE_RESPONSE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.ROUTER_GATEWAY, events.BEFORE_CREATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.PORT, events.BEFORE_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.PORT, events.PRECOMMIT_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.PORT, events.AFTER_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.ROUTER_GATEWAY, events.AFTER_CREATE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.PRECOMMIT_UPDATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.AFTER_UPDATE,
+                          self.mixin, payload=mock.ANY)]
+            mock_publish.assert_has_calls(expected_calls)
+            mock_publish.reset_mock()
+            update_data = {l3_apidef.EXTERNAL_GW_INFO: {}}
+            self.mixin.update_router(
+                self.ctx, self.router['id'], {'router': update_data})
+            expected_calls = [
+                mock.call(resources.ROUTER_GATEWAY, events.BEFORE_DELETE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.PORT, events.BEFORE_DELETE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.PORT, events.PRECOMMIT_DELETE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.PORT, events.AFTER_DELETE,
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.ROUTER_GATEWAY, events.AFTER_DELETE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.PRECOMMIT_UPDATE,
+                          self.mixin, payload=mock.ANY),
+                mock.call(resources.ROUTER, events.AFTER_UPDATE,
+                          self.mixin, payload=mock.ANY)]
+            mock_publish.assert_has_calls(expected_calls)
