@@ -50,6 +50,7 @@ FAKE_SECGROUP_RULE = {
         'remote_ip_prefix': '10.0.0.1',
         'ethertype': 'IPv4',
         'remote_group_id': None,
+        'remote_address_group_id': None,
         'security_group_id': 'None',
         'direction': 'ingress'
     }
@@ -73,10 +74,8 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         self.setup_coreplugin(core_plugin=DB_PLUGIN_KLASS)
         self.ctx = context.get_admin_context()
         self.mixin = SecurityGroupDbMixinImpl()
-        make_res = mock.patch.object(quota.QuotaEngine, 'make_reservation')
-        self.mock_quota_make_res = make_res.start()
-        commit_res = mock.patch.object(quota.QuotaEngine, 'commit_reservation')
-        self.mock_quota_commit_res = commit_res.start()
+        quota_check = mock.patch.object(quota.QuotaEngine, 'quota_limit_check')
+        self.mock_quota_check = quota_check.start()
         is_ext_supported = mock.patch(
             'neutron_lib.api.extensions.is_extension_supported')
         self.is_ext_supported = is_ext_supported.start()
@@ -94,8 +93,8 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         with mock.patch.object(self.mixin,
                                '_get_port_security_group_bindings'),\
                 mock.patch.object(self.mixin, '_get_security_group'),\
-                mock.patch.object(registry, "notify") as mock_notify:
-            mock_notify.side_effect = exceptions.CallbackFailure(Exception())
+                mock.patch.object(registry, "publish") as mock_publish:
+            mock_publish.side_effect = exceptions.CallbackFailure(Exception())
             with testtools.ExpectedException(securitygroup.SecurityGroupInUse):
                 self.mixin.delete_security_group(self.ctx, mock.ANY)
 
@@ -105,15 +104,15 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         FAKE_SECGROUP['security_group']['stateful'] = not sg_dict['stateful']
         with mock.patch.object(self.mixin,
                                '_get_port_security_group_bindings'), \
-                mock.patch.object(registry, "notify") as mock_notify:
-            mock_notify.side_effect = exceptions.CallbackFailure(Exception())
+                mock.patch.object(registry, "publish") as mock_publish:
+            mock_publish.side_effect = exceptions.CallbackFailure(Exception())
             with testtools.ExpectedException(securitygroup.SecurityGroupInUse):
                 self.mixin.update_security_group(self.ctx, sg_dict['id'],
                                                  FAKE_SECGROUP)
 
     def test_update_security_group_conflict(self):
-        with mock.patch.object(registry, "notify") as mock_notify:
-            mock_notify.side_effect = exceptions.CallbackFailure(Exception())
+        with mock.patch.object(registry, "publish") as mock_publish:
+            mock_publish.side_effect = exceptions.CallbackFailure(Exception())
             secgroup = {'security_group': FAKE_SECGROUP}
             with testtools.ExpectedException(
                     securitygroup.SecurityGroupConflict):
@@ -123,12 +122,12 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         with mock.patch.object(self.mixin, '_validate_security_group_rule'),\
                 mock.patch.object(self.mixin,
                                   '_check_for_duplicate_rules'),\
-                mock.patch.object(registry, "notify") as mock_notify:
-            mock_notify.side_effect = exceptions.CallbackFailure(Exception())
+                mock.patch.object(registry, "publish") as mock_publish:
+            mock_publish.side_effect = exceptions.CallbackFailure(Exception())
             with testtools.ExpectedException(
                     securitygroup.SecurityGroupConflict):
                 self.mixin.create_security_group_rule(
-                    self.ctx, mock.MagicMock())
+                    self.ctx, FAKE_SECGROUP_RULE)
 
     def test__check_for_duplicate_rules_does_not_drop_protocol(self):
         with mock.patch.object(self.mixin, 'get_security_group',
@@ -200,8 +199,8 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
                 context, 'fake', [rule_dict])
 
     def test_delete_security_group_rule_in_use(self):
-        with mock.patch.object(registry, "notify") as mock_notify:
-            mock_notify.side_effect = exceptions.CallbackFailure(Exception())
+        with mock.patch.object(registry, "publish") as mock_publish:
+            mock_publish.side_effect = exceptions.CallbackFailure(Exception())
             with testtools.ExpectedException(
                     securitygroup.SecurityGroupRuleInUse):
                 self.mixin.delete_security_group_rule(self.ctx, mock.ANY)
@@ -290,6 +289,8 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
             'name': 'default',
             'description': 'Default security group',
             'stateful': mock.ANY,
+            'standard_attr_id': mock.ANY,
+            'shared': False,
             'security_group_rules': [
                 # Four rules for egress/ingress and ipv4/ipv6
                 mock.ANY, mock.ANY, mock.ANY, mock.ANY,
@@ -299,33 +300,48 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
             DEFAULT_SECGROUP_DICT.update({
                 'revision_number': mock.ANY,
             })
-        with mock.patch.object(registry, 'publish') as publish, \
-                mock.patch.object(registry, "notify") as mock_notify:
+        with mock.patch.object(registry, 'publish') as publish:
             sg_dict = self.mixin.create_security_group(self.ctx, FAKE_SECGROUP)
-            mock_notify.assert_has_calls([
-                mock.call('security_group', 'precommit_create', mock.ANY,
-                          context=mock.ANY, is_default=True,
-                          security_group=DEFAULT_SECGROUP_DICT),
-                mock.call('security_group', 'after_create', mock.ANY,
-                          context=mock.ANY, is_default=True,
-                          security_group=DEFAULT_SECGROUP_DICT),
-                mock.call('security_group', 'precommit_create', mock.ANY,
-                          context=mock.ANY, is_default=False,
-                          security_group=sg_dict),
-                mock.call('security_group', 'after_create', mock.ANY,
-                          context=mock.ANY, is_default=False,
-                          security_group=sg_dict)])
 
             publish.assert_has_calls([
-                mock.call('security_group', 'before_create', mock.ANY,
+                mock.call(resources.SECURITY_GROUP, 'before_create', mock.ANY,
                           payload=mock.ANY),
-                mock.call('security_group', 'before_create', mock.ANY,
+                mock.call(resources.SECURITY_GROUP, 'before_create', mock.ANY,
+                          payload=mock.ANY),
+                mock.call(resources.SECURITY_GROUP, 'precommit_create',
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SECURITY_GROUP, 'after_create', mock.ANY,
+                          payload=mock.ANY),
+                mock.call(resources.SECURITY_GROUP, 'precommit_create',
+                          mock.ANY, payload=mock.ANY),
+                mock.call(resources.SECURITY_GROUP, 'after_create', mock.ANY,
                           payload=mock.ANY)])
+
             payload = publish.mock_calls[0][2]['payload']
             self.assertDictEqual(payload.desired_state,
                                  FAKE_SECGROUP['security_group'])
+
             payload = publish.mock_calls[1][2]['payload']
-            self.assertDictEqual(payload.desired_state, DEFAULT_SECGROUP)
+            self.assertDictEqual(payload.desired_state,
+                                 DEFAULT_SECGROUP)
+
+            payload = publish.mock_calls[2][2]['payload']
+            self.assertDictEqual(payload.latest_state,
+                                 DEFAULT_SECGROUP_DICT)
+            self.assertTrue(payload.metadata['is_default'])
+
+            payload = publish.mock_calls[3][2]['payload']
+            self.assertDictEqual(payload.latest_state,
+                                 DEFAULT_SECGROUP_DICT)
+            self.assertTrue(payload.metadata['is_default'])
+
+            payload = publish.mock_calls[4][2]['payload']
+            self.assertDictEqual(payload.latest_state, sg_dict)
+            self.assertFalse(payload.metadata['is_default'])
+
+            payload = publish.mock_calls[5][2]['payload']
+            self.assertDictEqual(payload.latest_state, sg_dict)
+            self.assertFalse(payload.metadata['is_default'])
 
             # Ensure that the result of create is same as get.
             # Especially we want to check the revision number here.
@@ -348,36 +364,45 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         sg_id = original_sg_dict['id']
         with mock.patch.object(self.mixin,
                                '_get_port_security_group_bindings'), \
-                mock.patch.object(registry, "publish") as mock_notify:
+                mock.patch.object(registry, "publish") as mock_publish:
             fake_secgroup = copy.deepcopy(FAKE_SECGROUP)
             fake_secgroup['security_group']['name'] = 'updated_fake'
             fake_secgroup['security_group']['stateful'] = mock.ANY
             sg_dict = self.mixin.update_security_group(
                     self.ctx, sg_id, fake_secgroup)
 
-            mock_notify.assert_has_calls(
-                [mock.call('security_group', 'precommit_update', mock.ANY,
-                           payload=mock.ANY)])
-            payload = mock_notify.call_args[1]['payload']
+            mock_publish.assert_has_calls(
+                [mock.call(resources.SECURITY_GROUP, events.PRECOMMIT_UPDATE,
+                           mock.ANY, payload=mock.ANY)])
+            payload = mock_publish.call_args[1]['payload']
             self.assertEqual(original_sg_dict, payload.states[0])
             self.assertEqual(sg_id, payload.resource_id)
-            self.assertEqual(sg_dict, payload.desired_state)
+            self.assertEqual(sg_dict, payload.latest_state)
 
     def test_security_group_precommit_and_after_delete_event(self):
         sg_dict = self.mixin.create_security_group(self.ctx, FAKE_SECGROUP)
-        with mock.patch.object(registry, "notify") as mock_notify:
+        with mock.patch.object(registry, "publish") as mock_publish:
             self.mixin.delete_security_group(self.ctx, sg_dict['id'])
             sg_dict['security_group_rules'] = mock.ANY
-            mock_notify.assert_has_calls(
+            mock_publish.assert_has_calls(
                 [mock.call('security_group', 'precommit_delete',
-                           mock.ANY, context=mock.ANY, security_group=sg_dict,
-                           security_group_id=sg_dict['id'],
-                           security_group_rule_ids=[mock.ANY, mock.ANY]),
+                           mock.ANY, payload=mock.ANY),
                  mock.call('security_group', 'after_delete',
-                           mock.ANY, context=mock.ANY,
-                           security_group_id=sg_dict['id'],
-                           security_group_rule_ids=[mock.ANY, mock.ANY],
-                           name=sg_dict['name'])])
+                           mock.ANY,
+                           payload=mock.ANY)])
+            payload = mock_publish.mock_calls[1][2]['payload']
+            self.assertEqual(mock.ANY, payload.context)
+            self.assertEqual(sg_dict, payload.latest_state)
+            self.assertEqual(sg_dict['id'], payload.resource_id)
+            self.assertEqual([mock.ANY, mock.ANY],
+                payload.metadata.get('security_group_rule_ids'))
+
+            payload = mock_publish.mock_calls[2][2]['payload']
+            self.assertEqual(mock.ANY, payload.context)
+            self.assertEqual(sg_dict, payload.latest_state)
+            self.assertEqual(sg_dict['id'], payload.resource_id)
+            self.assertEqual([mock.ANY, mock.ANY],
+                             payload.metadata.get('security_group_rule_ids'))
 
     def test_security_group_rule_precommit_create_event_fail(self):
         registry.subscribe(fake_callback, resources.SECURITY_GROUP_RULE,
@@ -413,34 +438,54 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
         sg_dict = self.mixin.create_security_group(self.ctx, FAKE_SECGROUP)
         fake_rule = FAKE_SECGROUP_RULE
         fake_rule['security_group_rule']['security_group_id'] = sg_dict['id']
-        with mock.patch.object(registry, "notify") as mock_notify, \
+        with mock.patch.object(registry, "publish") as mock_publish, \
                 mock.patch.object(self.mixin, '_get_security_group'):
-            mock_notify.assert_has_calls([mock.call('security_group_rule',
-                'precommit_create', mock.ANY, context=mock.ANY,
-                security_group_rule=self.mixin.create_security_group_rule(
-                    self.ctx, fake_rule))])
+            sg_rule = self.mixin.create_security_group_rule(self.ctx,
+                                                            fake_rule)
+            mock_publish.assert_has_calls([mock.call(
+                'security_group_rule',
+                'precommit_create', mock.ANY, payload=mock.ANY)])
+
+            payload = mock_publish.mock_calls[1][2]['payload']
+            self.assertEqual(sg_rule['id'], payload.resource_id)
+            self.assertEqual(sg_rule, payload.latest_state)
 
     def test_sg_rule_before_precommit_and_after_delete_event(self):
         sg_dict = self.mixin.create_security_group(self.ctx, FAKE_SECGROUP)
         fake_rule = FAKE_SECGROUP_RULE
         fake_rule['security_group_rule']['security_group_id'] = sg_dict['id']
-        with mock.patch.object(registry, "notify") as mock_notify, \
+        with mock.patch.object(registry, "publish") as mock_publish, \
                 mock.patch.object(self.mixin, '_get_security_group'):
             sg_rule_dict = self.mixin.create_security_group_rule(self.ctx,
-                   fake_rule)
+                                                                 fake_rule)
             self.mixin.delete_security_group_rule(self.ctx,
-                    sg_rule_dict['id'])
-            mock_notify.assert_has_calls([mock.call('security_group_rule',
-                'before_delete', mock.ANY, context=mock.ANY,
-                security_group_rule_id=sg_rule_dict['id'])])
-            mock_notify.assert_has_calls([mock.call('security_group_rule',
-                'precommit_delete', mock.ANY, context=mock.ANY,
-                security_group_id=sg_dict['id'],
-                security_group_rule_id=sg_rule_dict['id'])])
-            mock_notify.assert_has_calls([mock.call('security_group_rule',
-                'after_delete', mock.ANY, context=mock.ANY,
-                security_group_rule_id=sg_rule_dict['id'],
-                security_group_id=sg_dict['id'])])
+                                                  sg_rule_dict['id'])
+            mock_publish.assert_has_calls([mock.call('security_group_rule',
+                                                     'before_delete',
+                                                     mock.ANY,
+                                                     payload=mock.ANY)])
+            mock_publish.assert_has_calls([mock.call('security_group_rule',
+                                                    'precommit_delete',
+                                                     mock.ANY,
+                                                     payload=mock.ANY)])
+            mock_publish.assert_has_calls([mock.call('security_group_rule',
+                                                    'after_delete',
+                                                     mock.ANY,
+                                                     payload=mock.ANY)])
+
+            payload = mock_publish.mock_calls[1][2]['payload']
+            self.assertEqual(mock.ANY, payload.context)
+            self.assertEqual(sg_rule_dict, payload.latest_state)
+            self.assertEqual(sg_rule_dict['id'], payload.resource_id)
+
+            payload = mock_publish.mock_calls[2][2]['payload']
+            self.assertEqual(mock.ANY, payload.context)
+            self.assertEqual(sg_rule_dict, payload.latest_state)
+            self.assertEqual(sg_rule_dict['id'], payload.resource_id)
+
+            payload = mock_publish.mock_calls[3][2]['payload']
+            self.assertEqual(mock.ANY, payload.context)
+            self.assertEqual(sg_rule_dict['id'], payload.resource_id)
 
     def test_get_ip_proto_name_and_num(self):
         protocols = [constants.PROTO_NAME_UDP, str(constants.PROTO_NUM_TCP),
@@ -615,3 +660,15 @@ class SecurityGroupDbMixinTestCase(testlib_api.SqlTestCase):
             self.mixin._ensure_default_security_group(self.ctx, 'tenant_1')
             create_sg.assert_not_called()
             get_default_sg_id.assert_not_called()
+
+    def test__ensure_default_security_group_tenant_mismatch(self):
+        with mock.patch.object(
+                self.mixin, '_get_default_sg_id') as get_default_sg_id,\
+                mock.patch.object(
+                        self.mixin, 'create_security_group') as create_sg:
+            context = mock.Mock()
+            context.tenant_id = 'tenant_0'
+            context.is_admin = False
+            self.mixin._ensure_default_security_group(context, 'tenant_1')
+            create_sg.assert_not_called()
+            get_default_sg_id.assert_not_called()
