@@ -14,9 +14,12 @@
 
 import collections
 import copy
+import sys
 from unittest import mock
 
 from neutron_lib.api.definitions import l3
+from neutron_lib import constants as n_const
+from neutron_lib.utils import net
 from oslo_utils import uuidutils
 
 from neutron.common.ovn import constants as ovn_const
@@ -59,6 +62,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_acls_for_lswitches = mock.Mock()
         self.create_lrouter = mock.Mock()
         self.lrp_del = mock.Mock()
+        self.lrp_set_options = mock.Mock()
         self.update_lrouter = mock.Mock()
         self.delete_lrouter = mock.Mock()
         self.add_lrouter_port = mock.Mock()
@@ -74,6 +78,8 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_all_chassis_gateway_bindings = mock.Mock()
         self.get_chassis_gateways = mock.Mock()
         self.get_gateway_chassis_binding = mock.Mock()
+        self.get_gateway_chassis_az_hints = mock.Mock()
+        self.get_gateway_chassis_az_hints.return_value = []
         self.get_unhosted_gateways = mock.Mock()
         self.add_dhcp_options = mock.Mock()
         self.delete_dhcp_options = mock.Mock()
@@ -117,6 +123,8 @@ class FakeOvsdbNbOvnIdl(object):
         self.get_router_floatingip_lbs.return_value = []
         self.is_col_present = mock.Mock()
         self.is_col_present.return_value = False
+        self.is_col_supports_value = mock.Mock()
+        self.is_col_supports_value.return_value = False
         self.get_lrouter = mock.Mock()
         self.get_lrouter.return_value = None
         self.delete_lrouter_ext_gw = mock.Mock()
@@ -130,6 +138,7 @@ class FakeOvsdbNbOvnIdl(object):
         self.pg_del_ports = mock.Mock()
         self.lsp_get_up = mock.Mock()
         self.nb_global = mock.Mock()
+        self.nb_global.options.get.return_value = '100000'
         self.db_list_rows = mock.Mock()
         self.lsp_list = mock.MagicMock()
         self.db_find = mock.Mock()
@@ -146,6 +155,14 @@ class FakeOvsdbNbOvnIdl(object):
         self.ls_get = mock.Mock()
         self.check_liveness = mock.Mock()
         self.ha_chassis_group_get = mock.Mock()
+        self.qos_del = mock.Mock()
+        self.qos_del_ext_ids = mock.Mock()
+        self.meter_add = mock.Mock()
+        self.meter_del = mock.Mock()
+        self.ha_chassis_group_add = mock.Mock()
+        self.ha_chassis_group_del = mock.Mock()
+        self.ha_chassis_group_add_chassis = mock.Mock()
+        self.ha_chassis_group_del_chassis = mock.Mock()
 
 
 class FakeOvsdbSbOvnIdl(object):
@@ -155,13 +172,11 @@ class FakeOvsdbSbOvnIdl(object):
         self.chassis_exists.return_value = True
         self.get_chassis_hostname_and_physnets = mock.Mock()
         self.get_chassis_hostname_and_physnets.return_value = {}
+        self.get_chassis_and_azs = mock.Mock()
+        self.get_chassis_and_azs.return_value = {}
         self.get_all_chassis = mock.Mock()
-        self.get_chassis_data_for_ml2_bind_port = mock.Mock()
-        self.get_chassis_data_for_ml2_bind_port.return_value = \
-            ('fake', '', ['fake-physnet'])
-        self.get_logical_port_chassis_and_datapath = mock.Mock()
-        self.get_logical_port_chassis_and_datapath.return_value = \
-            ('fake', 'fake-dp')
+        self._get_chassis_physnets = mock.Mock()
+        self._get_chassis_physnets.return_value = ['fake-physnet']
         self.get_chassis_and_physnets = mock.Mock()
         self.get_gateway_chassis_from_cms_options = mock.Mock()
         self.is_col_present = mock.Mock()
@@ -171,6 +186,7 @@ class FakeOvsdbSbOvnIdl(object):
         self.chassis_list = mock.MagicMock()
         self.is_table_present = mock.Mock()
         self.is_table_present.return_value = False
+        self.get_chassis_by_card_serial_from_cms_options = mock.Mock()
 
 
 class FakeOvsdbTransaction(object):
@@ -185,6 +201,30 @@ class FakePlugin(object):
         self._get_port_security_group_bindings = mock.Mock()
 
 
+class FakeStandardAttribute(object):
+
+    def __init__(self, _id=1, resource_type=mock.ANY, description=mock.ANY,
+                 revision_number=1):
+        self.id = _id
+        self.resource_type = resource_type
+        self.description = description
+        self.revision_number = revision_number
+
+
+class FakeQosNetworkPolicyBinding(object):
+
+    def __init__(self, policy_id=mock.ANY, network_id=mock.ANY):
+        self.policy_id = policy_id
+        self.network_id = network_id
+
+
+class FakeQosFIPPolicyBinding(object):
+
+    def __init__(self, policy_id=mock.ANY, fip_id=mock.ANY):
+        self.policy_id = policy_id
+        self.fip_id = fip_id
+
+
 class FakeResource(dict):
 
     def __init__(self, manager=None, info=None, loaded=False, methods=None):
@@ -280,6 +320,7 @@ class FakeNetwork(object):
             'availability_zones': [],
             'availability_zone_hints': [],
             'is_default': False,
+            'standard_attr_id': 1,
         }
 
         # Overwrite default attributes.
@@ -395,11 +436,13 @@ class FakeOvsdbTable(FakeResource):
     """Fake one or more OVSDB tables."""
 
     @staticmethod
-    def create_one_ovsdb_table(attrs=None):
+    def create_one_ovsdb_table(attrs=None, max_rows=sys.maxsize):
         """Create a fake OVSDB table.
 
         :param Dictionary attrs:
             A dictionary with all attributes
+        :param Int max_rows:
+            A num of max rows
         :return:
             A FakeResource object faking the OVSDB table
         """
@@ -409,6 +452,8 @@ class FakeOvsdbTable(FakeResource):
         ovsdb_table_attrs = {
             'rows': collections.UserDict(),
             'columns': {},
+            'indexes': [],
+            'max_rows': max_rows,
         }
 
         # Overwrite default attributes.
@@ -544,6 +589,7 @@ class FakeSecurityGroupRule(object):
             'protocol': 'tcp',
             'remote_group_id': None,
             'remote_ip_prefix': '0.0.0.0/0',
+            'normalized_cidr': '0.0.0.0/0',
             'security_group_id': 'security-group-id-' + fake_uuid,
             'tenant_id': 'project-id-' + fake_uuid,
         }
@@ -551,6 +597,13 @@ class FakeSecurityGroupRule(object):
         # Overwrite default attributes.
         security_group_rule_attrs.update(attrs)
 
+        if ('remote_ip_prefix' in attrs and 'normalized_cidr' not in attrs):
+            if attrs['remote_ip_prefix'] is None:
+                security_group_rule_attrs['normalized_cidr'] = None
+            else:
+                security_group_rule_attrs['normalized_cidr'] = (
+                        net.AuthenticIPNetwork(attrs['remote_ip_prefix']))
+
         return FakeResource(info=copy.deepcopy(security_group_rule_attrs),
                             loaded=True)
 
@@ -657,6 +710,9 @@ class FakeFloatingIp(object):
             'dns_domain': '',
             'dns_name': '',
             'project_id': '',
+            'standard_attr': FakeStandardAttribute(),
+            'qos_policy_binding': FakeQosFIPPolicyBinding(),
+            'qos_network_policy_binding': FakeQosNetworkPolicyBinding(),
         }
 
         # Overwrite default attributes.
@@ -786,7 +842,10 @@ class FakeOVNRouter(object):
 class FakeChassis(object):
 
     @staticmethod
-    def create(attrs=None, az_list=None, chassis_as_gw=False):
+    def create(attrs=None, az_list=None, chassis_as_gw=False,
+               bridge_mappings=None, rp_bandwidths=None,
+               rp_inventory_defaults=None, rp_hypervisors=None,
+               card_serial_number=None):
         cms_opts = []
         if az_list:
             cms_opts.append("%s=%s" % (ovn_const.CMS_OPT_AVAILABILITY_ZONES,
@@ -794,11 +853,38 @@ class FakeChassis(object):
         if chassis_as_gw:
             cms_opts.append(ovn_const.CMS_OPT_CHASSIS_AS_GW)
 
+        if rp_bandwidths:
+            cms_opts.append('%s=%s' % (n_const.RP_BANDWIDTHS,
+                                       ';'.join(rp_bandwidths)))
+        elif rp_bandwidths == '':  # Test wrongly defined parameter
+            cms_opts.append('%s=' % n_const.RP_BANDWIDTHS)
+
+        if rp_inventory_defaults:
+            inv_defaults = ';'.join('%s:%s' % (key, value) for key, value in
+                                    rp_inventory_defaults.items())
+            cms_opts.append('%s=%s' % (n_const.RP_INVENTORY_DEFAULTS,
+                                       inv_defaults))
+        elif rp_inventory_defaults == '':  # Test wrongly defined parameter
+            cms_opts.append('%s=' % n_const.RP_INVENTORY_DEFAULTS)
+
+        if rp_hypervisors:
+            cms_opts.append('%s=%s' % (ovn_const.RP_HYPERVISORS,
+                                       ';'.join(rp_hypervisors)))
+        elif rp_hypervisors == '':  # Test wrongly defined parameter
+            cms_opts.append('%s=' % ovn_const.RP_HYPERVISORS)
+
+        if card_serial_number:
+            cms_opts.append('%s=%s' % (ovn_const.CMS_OPT_CARD_SERIAL_NUMBER,
+                                       card_serial_number))
+
         external_ids = {}
         if cms_opts:
             external_ids[ovn_const.OVN_CMS_OPTIONS] = ','.join(cms_opts)
 
-        attrs = {
+        if bridge_mappings:
+            external_ids['ovn-bridge-mappings'] = ','.join(bridge_mappings)
+
+        chassis_attrs = {
             'encaps': [],
             'external_ids': external_ids,
             'hostname': '',
@@ -809,5 +895,5 @@ class FakeChassis(object):
             'vtep_logical_switches': []}
 
         # Overwrite default attributes.
-        attrs.update(attrs)
-        return type('Chassis', (object, ), attrs)
+        chassis_attrs.update(attrs or {})
+        return type('Chassis', (object, ), chassis_attrs)
