@@ -13,8 +13,11 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import queue
 from unittest import mock
 
+import eventlet
+from keystoneauth1 import exceptions as ks_exc
 from neutron_lib import constants as n_const
 from neutron_lib import context as n_ctx
 from neutron_lib import exceptions as n_exc
@@ -139,7 +142,7 @@ class TestNovaNotify(base.BaseTestCase):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
         returned_obj = {'port':
                         {'device_owner': DEVICE_OWNER_COMPUTE,
-                         'id': u'bee50827-bcee-4cc8-91c1-a27b0ce54222',
+                         'id': 'bee50827-bcee-4cc8-91c1-a27b0ce54222',
                          'device_id': device_id}}
 
         expected_event = {'server_uuid': device_id,
@@ -152,7 +155,7 @@ class TestNovaNotify(base.BaseTestCase):
     def test_create_floatingip_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
         returned_obj = {'floatingip':
-                        {'port_id': u'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
+                        {'port_id': 'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
 
         expected_event = {'server_uuid': device_id,
                           'name': 'network-changed',
@@ -172,7 +175,7 @@ class TestNovaNotify(base.BaseTestCase):
     def test_delete_floatingip_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
         returned_obj = {'floatingip':
-                        {'port_id': u'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
+                        {'port_id': 'bee50827-bcee-4cc8-91c1-a27b0ce54222'}}
 
         expected_event = {'server_uuid': device_id,
                           'name': 'network-changed',
@@ -202,7 +205,7 @@ class TestNovaNotify(base.BaseTestCase):
     def test_associate_floatingip_notify(self):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
         returned_obj = {'floatingip':
-                        {'port_id': u'5a39def4-3d3f-473d-9ff4-8e90064b9cc1'}}
+                        {'port_id': '5a39def4-3d3f-473d-9ff4-8e90064b9cc1'}}
         original_obj = {'port_id': None}
 
         expected_event = {'server_uuid': device_id,
@@ -236,45 +239,62 @@ class TestNovaNotify(base.BaseTestCase):
 
     @mock.patch('novaclient.client.Client')
     def test_nova_send_events_returns_bad_list(self, mock_client):
-        mock_client.server_external_events.create.return_value = (
-            'i am a string!')
+        create = mock_client().server_external_events.create
+        create.return_value = 'i am a string!'
         self.nova_notifier.send_events([])
+        create.assert_called()
 
     @mock.patch('novaclient.client.Client')
     def test_nova_send_event_rasies_404(self, mock_client):
-        mock_client.server_external_events.create.return_value = (
-            nova_exceptions.NotFound)
+        create = mock_client().server_external_events.create
+        create.return_value = nova_exceptions.NotFound
         self.nova_notifier.send_events([])
+        create.assert_called()
+
+    @mock.patch('novaclient.client.Client')
+    def test_nova_send_events_raises_connect_exc(self, mock_client):
+        create = mock_client().server_external_events.create
+        create.side_effect = (
+            ks_exc.ConnectFailure, ks_exc.ConnectTimeout, [])
+        self.nova_notifier.send_events([])
+        self.assertEqual(3, create.call_count)
 
     @mock.patch('novaclient.client.Client')
     def test_nova_send_events_raises(self, mock_client):
-        mock_client.server_external_events.create.return_value = Exception
+        create = mock_client().server_external_events.create
+        create.return_value = Exception
         self.nova_notifier.send_events([])
+        create.assert_called()
 
     @mock.patch('novaclient.client.Client')
     def test_nova_send_events_returns_non_200(self, mock_client):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        mock_client.server_external_events.create.return_value = [
+        create = mock_client().server_external_events.create
+        create.return_value = [
             {'code': 404,
              'name': 'network-changed',
              'server_uuid': device_id}]
         self.nova_notifier.send_events([{'name': 'network-changed',
                                          'server_uuid': device_id}])
+        create.assert_called()
 
     @mock.patch('novaclient.client.Client')
     def test_nova_send_events_return_200(self, mock_client):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        mock_client.server_external_events.create.return_value = [
+        create = mock_client().server_external_events.create
+        create.return_value = [
             {'code': 200,
              'name': 'network-changed',
              'server_uuid': device_id}]
         self.nova_notifier.send_events([{'name': 'network-changed',
                                          'server_uuid': device_id}])
+        create.assert_called()
 
     @mock.patch('novaclient.client.Client')
-    def test_nova_send_events_multiple(self, mock_create):
+    def test_nova_send_events_multiple(self, mock_client):
         device_id = '32102d7b-1cf4-404d-b50a-97aae1f55f87'
-        mock_create.server_external_events.create.return_value = [
+        create = mock_client().server_external_events.create
+        create.return_value = [
             {'code': 200,
              'name': 'network-changed',
              'server_uuid': device_id},
@@ -284,6 +304,7 @@ class TestNovaNotify(base.BaseTestCase):
         self.nova_notifier.send_events([
             {'name': 'network-changed', 'server_uuid': device_id},
             {'name': 'network-changed', 'server_uuid': device_id}])
+        create.assert_called()
 
     def test_reassociate_floatingip_without_disassociate_event(self):
         returned_obj = {'floatingip':
@@ -371,3 +392,35 @@ class TestNovaNotify(base.BaseTestCase):
         self.assertEqual(
             expected_event,
             self.nova_notifier.batch_notifier._pending_events.get())
+
+    def test_notify_concurrent_enable_flag_update(self):
+        # This test assumes Neutron server uses eventlet.
+        # NOTE(ralonsoh): the exceptions raise inside a thread won't stop the
+        # test. The checks are stored in "_queue" and tested at the end of the
+        # test execution.
+        _queue = eventlet.queue.Queue()
+
+        def _local_executor(thread_idx):
+            # This thread has not yet initialized the local "enable" flag.
+            _queue.put(getattr(nova._notifier_store, 'enable', None) is None)
+            eventlet.sleep(0)  # Next thread execution.
+            new_enable = bool(thread_idx % 2)
+            with self.nova_notifier.context_enabled(new_enable):
+                # At this point, the Nova Notifier should have updated the
+                # "enable" flag.
+                _queue.put(new_enable == nova._notifier_store.enable)
+                eventlet.sleep(0)  # Next thread execution.
+                _queue.put(new_enable == nova._notifier_store.enable)
+            _queue.put(nova.NOTIFIER_ENABLE_DEFAULT ==
+                       nova._notifier_store.enable)
+
+        num_threads = 20
+        pool = eventlet.GreenPool(num_threads)
+        for idx in range(num_threads):
+            pool.spawn(_local_executor, idx)
+        pool.waitall()
+        try:
+            while True:
+                self.assertTrue(_queue.get(block=False))
+        except queue.Empty:
+            pass
