@@ -11,6 +11,7 @@
 #    under the License.
 
 import collections
+from collections import abc
 import copy
 import itertools
 import random
@@ -43,7 +44,10 @@ from neutron.objects import agent
 from neutron.objects import base
 from neutron.objects.db import api as obj_db_api
 from neutron.objects import flavor
+from neutron.objects import local_ip
 from neutron.objects import network as net_obj
+from neutron.objects.port.extensions import port_device_profile
+from neutron.objects.port.extensions import port_numa_affinity_policy
 from neutron.objects import ports
 from neutron.objects.qos import policy as qos_policy
 from neutron.objects import rbac_db
@@ -100,8 +104,6 @@ class FakeSmallNeutronObjectNewEngineFacade(base.NeutronDbObject):
 
     db_model = ObjectFieldsModel
 
-    new_facade = True
-
     primary_keys = ['field1']
 
     foreign_keys = {
@@ -386,6 +388,8 @@ class FakeNeutronObjectWithProjectId(base.NeutronDbObject):
         'field2': common_types.UUIDField(),
     }
 
+    fields_no_update = ['id', 'project_id']
+
 
 @base.NeutronObjectRegistry.register_if(False)
 class FakeNeutronObject(base.NeutronObject):
@@ -501,6 +505,11 @@ def get_set_of_random_uuids():
     }
 
 
+def get_random_port_ranges():
+    port = tools.LAST_RANDOM_PORT_RANGE_GENERATED
+    return "%d:%d" % (port, port)
+
+
 # NOTE: The keys in this dictionary have alphabetic order.
 FIELD_TYPE_VALUE_GENERATOR_MAP = {
     common_types.DictOfMiscValuesField: get_random_dict,
@@ -510,6 +519,8 @@ FIELD_TYPE_VALUE_GENERATOR_MAP = {
     common_types.EtherTypeEnumField: tools.get_random_ether_type,
     common_types.FloatingIPStatusEnumField: tools.get_random_floatingip_status,
     common_types.FlowDirectionEnumField: tools.get_random_flow_direction,
+    common_types.FlowDirectionAndAnyEnumField:
+        tools.get_random_flow_direction_or_any,
     common_types.HARouterEnumField: tools.get_random_ha_states,
     common_types.IpamAllocationStatusEnumField: tools.get_random_ipam_status,
     common_types.IPNetworkField: lib_test_tools.get_random_ip_network,
@@ -536,11 +547,18 @@ FIELD_TYPE_VALUE_GENERATOR_MAP = {
     obj_fields.IPAddressField: tools.get_random_ip_address,
     obj_fields.IPV4AddressField: lambda: tools.get_random_ip_address(
         version=constants.IP_VERSION_4),
+    obj_fields.IPV6AddressField: lambda: tools.get_random_ip_address(
+        version=constants.IP_VERSION_6),
     obj_fields.IntegerField: tools.get_random_integer,
     obj_fields.ListOfObjectsField: lambda: [],
     obj_fields.ListOfStringsField: tools.get_random_string_list,
     obj_fields.ObjectField: lambda: None,
     obj_fields.StringField: lambda: helpers.get_random_string(10),
+    port_numa_affinity_policy.NumaAffinityPoliciesEnumField:
+        tools.get_random_port_numa_affinity_policy,
+    port_device_profile.PortDeviceProfile:
+        lambda: helpers.get_random_string(255),
+    common_types.PortRangesField: get_random_port_ranges
 }
 
 
@@ -668,7 +686,7 @@ class _BaseObjectTestCase(object):
                     objs or self.objs):
                 val = v() if callable(v) else v
                 db_obj_key = obj.fields_need_translation.get(k, k)
-                if isinstance(val, collections.Mapping):
+                if isinstance(val, abc.Mapping):
                     self.update_obj_fields(
                         val, db_obj[db_obj_key], fields[k], obj[k])
                 else:
@@ -743,10 +761,10 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
             if getattr(self._test_class.rbac_db_cls, 'db_model', None):
                 mock.patch.object(
                     rbac_db.RbacNeutronDbObjectMixin,
-                    'is_shared_with_tenant', return_value=False).start()
+                    'is_shared_with_project', return_value=False).start()
                 mock.patch.object(
                     rbac_db.RbacNeutronDbObjectMixin,
-                    'get_shared_with_tenant', return_value=False).start()
+                    'get_shared_with_project', return_value=False).start()
 
     def fake_get_object(self, context, model, **kwargs):
         objs = self.model_map[model]
@@ -840,7 +858,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
                 obj_db_api, 'get_objects',
                 side_effect=self.fake_get_objects) as get_objects_mock:
             objs = self._test_class.get_objects(self.context)
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 [get_obj_persistent_fields(obj) for obj in self.objs],
                 [get_obj_persistent_fields(obj) for obj in objs])
         get_objects_mock.assert_any_call(
@@ -904,7 +922,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
             objs = self._test_class.get_objects(self.context,
                                                 validate_filters=False,
                                                 unknown_filter='value')
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 [get_obj_persistent_fields(obj) for obj in self.objs],
                 [get_obj_persistent_fields(obj) for obj in objs])
 
@@ -915,7 +933,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
                 obj_db_api, 'get_values',
                 side_effect=self.fake_get_values) as get_values_mock:
             values = self._test_class.get_values(self.context, field)
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 [getattr(obj, field) for obj in self.objs], values)
         get_values_mock.assert_any_call(
             self._test_class, self.context, db_field
@@ -936,7 +954,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
             values = self._test_class.get_values(self.context, field,
                                                  validate_filters=False,
                                                  unknown_filter='value')
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 [getattr(obj, field) for obj in self.objs], values)
 
     def test_get_values_mixed_field(self):
@@ -1080,7 +1098,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
                           fake_field='xxx')
 
     def _check_equal(self, expected, observed):
-        self.assertItemsEqual(get_obj_persistent_fields(expected),
+        self.assertCountEqual(get_obj_persistent_fields(expected),
                               get_obj_persistent_fields(observed))
 
     def test_count_validate_filters_false(self):
@@ -1182,6 +1200,7 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
         dict_ = obj.to_dict()
 
         self.assertNotIn('project_id', dict_)
+        # TODO(ralonsoh): remove once bp/keystone-v3 migration finishes.
         self.assertNotIn('tenant_id', dict_)
 
     def test_fields_no_update(self):
@@ -1189,13 +1208,13 @@ class BaseObjectIfaceTestCase(_BaseObjectTestCase, test_base.BaseTestCase):
         for field in self._test_class.fields_no_update:
             self.assertTrue(hasattr(obj, field))
 
-    def test_get_tenant_id(self):
+    def test_get_project_id(self):
         if not hasattr(self._test_class, 'project_id'):
             self.skipTest(
                 'Test class %r has no project_id field' % self._test_class)
         obj = self._test_class(self.context, **self.obj_fields[0])
         project_id = self.obj_fields[0]['project_id']
-        self.assertEqual(project_id, obj.tenant_id)
+        self.assertEqual(project_id, obj.project_id)
 
     # Adding delete_objects mock because some objects are using delete_objects
     # while calling update(), Port for example
@@ -1400,22 +1419,22 @@ class BaseObjectIfaceWithProjectIdTestCase(BaseObjectIfaceTestCase):
 
     _test_class = FakeNeutronObjectWithProjectId
 
-    def test_update_fields_using_tenant_id(self):
+    def test_update_fields_using_project_id(self):
         obj = self._test_class(self.context, **self.obj_fields[0])
         obj.obj_reset_changes()
 
-        tenant_id = obj['tenant_id']
+        project_id = obj['project_id']
         new_obj_fields = dict()
-        new_obj_fields['tenant_id'] = uuidutils.generate_uuid()
+        new_obj_fields['project_id'] = uuidutils.generate_uuid()
         new_obj_fields['field2'] = uuidutils.generate_uuid()
 
         obj.update_fields(new_obj_fields)
         self.assertEqual(set(['field2']), obj.obj_what_changed())
-        self.assertEqual(tenant_id, obj.project_id)
+        self.assertEqual(project_id, obj.project_id)
 
-    def test_tenant_id_filter_added_when_project_id_present(self):
+    def test_project_id_filter_added_when_project_id_present(self):
         self._test_class.get_objects(
-            self.context, tenant_id=self.obj_fields[0]['project_id'])
+            self.context, project_id=self.obj_fields[0]['project_id'])
 
 
 class BaseDbObjectMultipleForeignKeysTestCase(_BaseObjectTestCase,
@@ -1536,12 +1555,14 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         _network.create()
         return _network
 
-    def _create_test_network_id(self):
+    def _create_test_network_id(self, qos_policy_id=None):
         return self._create_test_network(
-            "test-network-%s" % helpers.get_random_string(4)).id
+            name="test-network-%s" % helpers.get_random_string(4),
+            qos_policy_id=qos_policy_id).id
 
-    def _create_external_network_id(self):
-        test_network_id = self._create_test_network_id()
+    def _create_external_network_id(self, qos_policy_id=None):
+        test_network_id = self._create_test_network_id(
+            qos_policy_id=qos_policy_id)
         ext_net = net_obj.ExternalNetwork(self.context,
             network_id=test_network_id)
         ext_net.create()
@@ -1620,9 +1641,9 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         segment.create()
         return segment.id
 
-    def _create_test_router_id(self, router_id=None):
+    def _create_test_router_id(self, router_id=None, name=None):
         attrs = {
-            'name': 'test_router',
+            'name': name or 'test_router',
         }
         if router_id:
             attrs['id'] = router_id
@@ -1652,6 +1673,23 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         _address_group.create()
         return _address_group.id
 
+    def _create_test_local_ip_id(self, **lip_attrs):
+        return self._create_test_local_ip(**lip_attrs)['id']
+
+    def _create_test_local_ip(self, **lip_attrs):
+        if 'network_id' not in lip_attrs:
+            lip_attrs['network_id'] = self._create_test_network_id()
+        if 'local_port_id' not in lip_attrs:
+            lip_attrs['local_port_id'] = self._create_test_port_id()
+        if 'local_ip_address' not in lip_attrs:
+            lip_attrs['local_ip_address'] = '10.10.10.10'
+        if 'ip_mode' not in lip_attrs:
+            lip_attrs['ip_mode'] = 'translate'
+
+        lip = local_ip.LocalIP(self.context, **lip_attrs)
+        lip.create()
+        return lip
+
     def _create_test_agent_id(self):
         attrs = self.get_random_object_fields(obj_cls=agent.Agent)
         _agent = agent.Agent(self.context, **attrs)
@@ -1793,7 +1831,7 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
 
     def test_get_objects_single_transaction(self):
         with mock.patch(self._get_ro_txn_exit_func_name()) as mock_exit:
-            with db_api.autonested_transaction(self.context.session):
+            with db_api.CONTEXT_READER.using(self.context):
                 self._test_class.get_objects(self.context)
         self.assertEqual(1, mock_exit.call_count)
 
@@ -1808,7 +1846,7 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
         obj.create()
 
         with mock.patch(self._get_ro_txn_exit_func_name()) as mock_exit:
-            with db_api.autonested_transaction(self.context.session):
+            with db_api.CONTEXT_READER.using(self.context):
                 obj = self._test_class.get_object(self.context,
                                                   **obj._get_composite_keys())
         self.assertEqual(1, mock_exit.call_count)
@@ -1852,7 +1890,7 @@ class BaseDbObjectTestCase(_BaseObjectTestCase,
             else:
                 filters = {field: obj[field]}
             new = self._test_class.get_objects(self.context, **filters)
-            self.assertItemsEqual(
+            self.assertCountEqual(
                 [obj._get_composite_keys()],
                 [obj_._get_composite_keys() for obj_ in new],
                 'Filtering by %s failed.' % field)
@@ -2174,7 +2212,7 @@ class GetObjectClassByModelTestCase(UniqueObjectBase):
             self.registered_object.db_model)
         self.assertIs(self.registered_object, found_obj)
 
-    def test_not_registed_object_raises_exception(self):
+    def test_not_registered_object_raises_exception(self):
         with testtools.ExpectedException(o_exc.NeutronDbObjectNotFoundByModel):
             base.get_object_class_by_model(self.not_registered_object.db_model)
 
