@@ -13,6 +13,7 @@
 from unittest import mock
 
 import netaddr
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants
 from neutron_lib.tests import tools
 from oslo_utils import uuidutils
@@ -53,6 +54,17 @@ class PortBindingDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
                                   BasePortBindingDbObjectTestCase):
     _test_class = ports.PortBinding
 
+    def test_get_duplicated_port_bindings(self):
+        port_id = self._create_test_port_id()
+        self.update_obj_fields({'port_id': port_id},
+                               objs=[self.objs[0], self.objs[1]])
+        for i in range(3):
+            _obj = self._make_object(self.obj_fields[i])
+            _obj.create()
+        dup_pb = ports.PortBinding.get_duplicated_port_bindings(self.context)
+        self.assertEqual(1, len(dup_pb))
+        self.assertEqual(port_id, dup_pb[0].port_id)
+
 
 class DistributedPortBindingIfaceObjTestCase(
         obj_test_base.BaseObjectIfaceTestCase):
@@ -482,6 +494,18 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
         self.assertNotIn('qos_network_policy_id',
                          port_v1_4['versioned_object.data'])
 
+    def test_v1_6_to_v1_5_drops_numa_affinity_policy(self):
+        port_new = self._create_test_port()
+        port_v1_5 = port_new.obj_to_primitive(target_version='1.5')
+        self.assertNotIn('numa_affinity_policy',
+                         port_v1_5['versioned_object.data'])
+
+    def test_v1_7_to_v1_6_drops_device_profile(self):
+        port_new = self._create_test_port()
+        port_v1_6 = port_new.obj_to_primitive(target_version='1.6')
+        self.assertNotIn('device_profile',
+                         port_v1_6['versioned_object.data'])
+
     def test_get_ports_ids_by_security_groups_except_router(self):
         sg_id = self._create_test_security_group_id()
         filter_owner = constants.ROUTER_INTERFACE_OWNERS_SNAT
@@ -597,3 +621,34 @@ class PortDbObjectTestCase(obj_test_base.BaseDbObjectTestCase,
             (0, 1),
             (len(dhcp_ports), count),
         )
+
+    def test_get_port_from_mac_and_pci_slot_no_ports(self):
+        self.assertIsNone(
+            ports.Port.get_port_from_mac_and_pci_slot(self.context,
+                                                      'ca:fe:ca:fe:ca:fe'))
+
+    def test_get_port_from_mac_and_pci_slot_no_pci_slot(self):
+        obj = self._make_object(self.obj_fields[0])
+        obj.create()
+        mac_address = obj.mac_address
+        port = ports.Port.get_port_from_mac_and_pci_slot(self.context,
+                                                         mac_address)
+        self.assertEqual(obj.id, port.id)
+
+    def test_get_port_from_mac_and_pci_slot(self):
+        obj = self._make_object(self.obj_fields[0])
+        obj.create()
+        mac_address = obj.mac_address
+        pci_slot = '0000:04:00.1'
+        port = ports.Port.get_port_from_mac_and_pci_slot(
+            self.context, mac_address, pci_slot=pci_slot)
+        self.assertIsNone(port)
+
+        port_binding = ports.PortBinding(
+            self.context, port_id=obj.id, host='any_host',
+            vif_type=portbindings.VIF_TYPE_OTHER,
+            vnic_type=portbindings.VNIC_DIRECT, profile={'pci_slot': pci_slot})
+        port_binding.create()
+        port = ports.Port.get_port_from_mac_and_pci_slot(
+            self.context, mac_address, pci_slot=pci_slot)
+        self.assertEqual(obj.id, port.id)
