@@ -17,8 +17,8 @@
 from unittest import mock
 
 from neutron_lib import constants as p_const
+from neutron_lib.plugins.ml2 import ovs_constants
 
-from neutron.plugins.ml2.drivers.openvswitch.agent.common import constants
 from neutron.tests.unit.plugins.ml2.drivers.openvswitch.agent.openflow.native \
     import ovs_bridge_test_base
 
@@ -33,7 +33,8 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
         self.stamp = self.br.default_cookie
 
     def test_setup_default_table(self):
-        self.br.setup_default_table()
+        self.br.setup_default_table(enable_openflow_dhcp=True,
+                                    enable_dhcpv6=True)
         (dp, ofp, ofpp) = self._get_dp()
         expected = [
             call._send_msg(ofpp.OFPFlowMod(dp,
@@ -46,12 +47,21 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(),
                 priority=0,
                 table_id=0),
                            active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[
+                    ofpp.OFPInstructionGotoTable(table_id=60),
+                ],
+                match=ofpp.OFPMatch(),
+                priority=0,
+                table_id=59),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
@@ -64,6 +74,42 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 priority=3,
                 table_id=60),
                            active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[ofpp.OFPInstructionGotoTable(table_id=77)],
+                match=ofpp.OFPMatch(eth_type=self.ether_types.ETH_TYPE_IP,
+                                    ip_proto=self.in_proto.IPPROTO_UDP,
+                                    ipv4_dst="255.255.255.255",
+                                    udp_dst=67,
+                                    udp_src=68),
+                priority=101,
+                table_id=60),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[],
+                match=ofpp.OFPMatch(),
+                priority=0,
+                table_id=77),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[ofpp.OFPInstructionGotoTable(table_id=78)],
+                match=ofpp.OFPMatch(eth_type=self.ether_types.ETH_TYPE_IPV6,
+                                    ip_proto=self.in_proto.IPPROTO_UDP,
+                                    ipv6_dst="ff02::1:2",
+                                    udp_dst=547,
+                                    udp_src=546),
+                priority=101,
+                table_id=60),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=[],
+                match=ofpp.OFPMatch(),
+                priority=0,
+                table_id=78),
+                           active_bundle=None),
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
@@ -74,7 +120,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[],
-                match=ofpp.OFPMatch(vlan_vid=4095),
+                match=ofpp.OFPMatch(vlan_vid=ofp.OFPVID_PRESENT | 4095),
                 priority=65535,
                 table_id=0),
                            active_bundle=None),
@@ -88,7 +134,25 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                 ],
                 match=ofpp.OFPMatch(),
                 priority=3,
-                table_id=constants.TRANSIENT_EGRESS_TABLE),
+                table_id=ovs_constants.TRANSIENT_EGRESS_TABLE),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(
+                dp, cookie=self.stamp,
+                instructions=[
+                    ofpp.OFPInstructionGotoTable(table_id=59),
+                ],
+                match=ofpp.OFPMatch(),
+                priority=0,
+                table_id=30),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(
+                dp, cookie=self.stamp,
+                instructions=[
+                    ofpp.OFPInstructionGotoTable(table_id=59),
+                ],
+                match=ofpp.OFPMatch(),
+                priority=0,
+                table_id=31),
                            active_bundle=None),
         ]
         self.assertEqual(expected, self.mock.mock_calls)
@@ -108,7 +172,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                         ofpp.OFPActionSetField(
                             vlan_vid=lvid | ofp.OFPVID_PRESENT),
                     ]),
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     in_port=port,
@@ -135,7 +199,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                         ofpp.OFPActionSetField(
                             vlan_vid=lvid | ofp.OFPVID_PRESENT),
                     ]),
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     in_port=port,
@@ -191,7 +255,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
                         ofpp.OFPActionSetField(eth_src=gateway_mac),
                     ]),
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -261,7 +325,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
                         ofpp.OFPActionSetField(eth_src=gateway_mac),
                     ]),
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -304,7 +368,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                     ofpp.OFPInstructionActions(ofp.OFPIT_APPLY_ACTIONS, [
                         ofpp.OFPActionSetField(eth_src=gateway_mac),
                     ]),
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     eth_dst=dst_mac,
@@ -447,7 +511,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     eth_type=self.ether_types.ETH_TYPE_IPV6,
@@ -462,7 +526,7 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
             call._send_msg(ofpp.OFPFlowMod(dp,
                 cookie=self.stamp,
                 instructions=[
-                    ofpp.OFPInstructionGotoTable(table_id=60),
+                    ofpp.OFPInstructionGotoTable(table_id=59),
                 ],
                 match=ofpp.OFPMatch(
                     eth_type=self.ether_types.ETH_TYPE_IPV6,
@@ -557,9 +621,9 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
 
     def _test_delete_dvr_dst_mac_for_arp(self, network_type):
         if network_type in (p_const.TYPE_VLAN, p_const.TYPE_FLAT):
-            table_id = constants.DVR_TO_SRC_MAC_PHYSICAL
+            table_id = ovs_constants.DVR_TO_SRC_MAC_PHYSICAL
         else:
-            table_id = constants.DVR_TO_SRC_MAC
+            table_id = ovs_constants.DVR_TO_SRC_MAC
 
         if network_type == p_const.TYPE_FLAT:
             vlan_tag = None
@@ -605,6 +669,123 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
     def test_delete_dvr_dst_mac_for_flat(self):
         self._test_delete_dvr_dst_mac_for_arp(network_type='flat')
 
+    def test_list_meter_features(self):
+        (dp, ofp, ofpp) = self._get_dp()
+        self.br.list_meter_features()
+        self.assertIn(
+            call._send_msg(ofpp.OFPMeterFeaturesStatsRequest(dp, 0),
+                           reply_cls=ofpp.OFPMeterFeaturesStatsReply),
+            self.mock.mock_calls)
+
+    def test_create_meter(self):
+        meter_id = 1
+        rate = 2
+        burst = 0
+        (dp, ofp, ofpp) = self._get_dp()
+        self.br.create_meter(meter_id, rate)
+
+        bands = [
+            ofpp.OFPMeterBandDrop(rate=rate, burst_size=burst)]
+        req = ofpp.OFPMeterMod(datapath=dp, command=ofp.OFPMC_ADD,
+                               flags=ofp.OFPMF_PKTPS, meter_id=meter_id,
+                               bands=bands)
+
+        expected = [call._send_msg(req)]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_delete_meter(self):
+        meter_id = 1
+        (dp, ofp, ofpp) = self._get_dp()
+        self.br.delete_meter(meter_id)
+
+        req = ofpp.OFPMeterMod(datapath=dp, command=ofp.OFPMC_DELETE,
+                               flags=ofp.OFPMF_PKTPS, meter_id=meter_id)
+        expected = [call._send_msg(req)]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_update_meter(self):
+        meter_id = 1
+        rate = 2
+        burst = 0
+        (dp, ofp, ofpp) = self._get_dp()
+        self.br.update_meter(meter_id, rate)
+
+        bands = [
+            ofpp.OFPMeterBandDrop(rate=rate, burst_size=burst)]
+        req = ofpp.OFPMeterMod(datapath=dp, command=ofp.OFPMC_MODIFY,
+                               flags=ofp.OFPMF_PKTPS, meter_id=meter_id,
+                               bands=bands)
+        expected = [call._send_msg(req)]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def _test_apply_meter_to_port(self, direction, mac,
+                            in_port=None, local_vlan=None):
+        meter_id = 1
+        (dp, ofp, ofpp) = self._get_dp()
+        self.br.apply_meter_to_port(meter_id, direction, mac,
+                                    in_port, local_vlan)
+
+        if direction == p_const.EGRESS_DIRECTION and in_port:
+            match = ofpp.OFPMatch(in_port=in_port, eth_src=mac)
+        elif direction == p_const.INGRESS_DIRECTION and local_vlan:
+            vlan_vid = local_vlan | ofp.OFPVID_PRESENT
+            match = ofpp.OFPMatch(vlan_vid=vlan_vid, eth_dst=mac)
+
+        instructions = [
+            ofpp.OFPInstructionMeter(meter_id, type_=ofp.OFPIT_METER),
+            ofpp.OFPInstructionGotoTable(
+                table_id=ovs_constants.TRANSIENT_TABLE)]
+
+        expected = [
+            call._send_msg(ofpp.OFPFlowMod(dp,
+                cookie=self.stamp,
+                instructions=instructions,
+                match=match,
+                priority=100,
+                table_id=ovs_constants.PACKET_RATE_LIMIT),
+                active_bundle=None)
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_apply_meter_to_port_egress(self):
+        self._test_apply_meter_to_port(p_const.EGRESS_DIRECTION,
+                                       mac="00:02:b3:13:fe:3e",
+                                       in_port=1)
+
+    def test_apply_meter_to_port_ingress(self):
+        self._test_apply_meter_to_port(p_const.INGRESS_DIRECTION,
+                                       mac="00:02:b3:13:fe:3e",
+                                       local_vlan=1)
+
+    def _test_remove_meter_from_port(self, direction, mac,
+                               in_port=None, local_vlan=None):
+        (_dp, ofp, ofpp) = self._get_dp()
+        self.br.remove_meter_from_port(direction,
+                                       mac, in_port, local_vlan)
+
+        if direction == p_const.EGRESS_DIRECTION and in_port:
+            match = ofpp.OFPMatch(in_port=in_port, eth_src=mac)
+        elif direction == p_const.INGRESS_DIRECTION and local_vlan:
+            vlan_vid = local_vlan | ofp.OFPVID_PRESENT
+            match = ofpp.OFPMatch(vlan_vid=vlan_vid, eth_dst=mac)
+
+        expected = [
+            call.uninstall_flows(
+                    table_id=ovs_constants.PACKET_RATE_LIMIT,
+                    match=match)
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_remove_meter_from_port_egress(self):
+        self._test_remove_meter_from_port(p_const.EGRESS_DIRECTION,
+                                          mac="00:02:b3:13:fe:3e",
+                                          in_port=1)
+
+    def test_remove_meter_from_port_ingress(self):
+        self._test_remove_meter_from_port(p_const.INGRESS_DIRECTION,
+                                          mac="00:02:b3:13:fe:3e",
+                                          local_vlan=1)
+
     def test_install_dscp_marking_rule(self):
         test_port = 8888
         test_mark = 38
@@ -634,3 +815,110 @@ class OVSIntegrationBridgeTest(ovs_bridge_test_base.OVSBridgeTestBase):
                            active_bundle=None)
         ]
         self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_setup_local_egress_flows(self):
+        in_port = 10
+        vlan = 3333
+        self.br.setup_local_egress_flows(in_port=in_port, vlan=vlan)
+
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call._send_msg(ofpp.OFPFlowMod(dp, cookie=self.stamp,
+                               instructions=[
+                                   ofpp.OFPInstructionGotoTable(table_id=30)],
+                               match=ofpp.OFPMatch(in_port=in_port),
+                               priority=8,
+                               table_id=0),
+                           active_bundle=None),
+            call._send_msg(ofpp.OFPFlowMod(dp, cookie=self.stamp,
+                               instructions=[ofpp.OFPInstructionActions(
+                                   ofp.OFPIT_APPLY_ACTIONS,
+                                   [ofpp.OFPActionSetField(reg6=vlan),
+                                    ofpp.NXActionResubmitTable(in_port=in_port,
+                                                               table_id=31)])],
+                               match=ofpp.OFPMatch(in_port=in_port),
+                               priority=10, table_id=30),
+                           active_bundle=None)
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_setup_local_egress_flows_ofport_invalid(self):
+        in_port = ovs_constants.OFPORT_INVALID
+        vlan = 3333
+        self.br.setup_local_egress_flows(in_port=in_port, vlan=vlan)
+
+        self.assertFalse(self.mock.called)
+
+    def test_install_garp_blocker(self):
+        vlan = 2222
+        ip = '192.0.0.10'
+        self.br.install_garp_blocker(vlan, ip)
+
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call._send_msg(ofpp.OFPFlowMod(dp, cookie=self.stamp,
+                               instructions=[],
+                               match=ofpp.OFPMatch(
+                                   vlan_vid=vlan | ofp.OFPVID_PRESENT,
+                                   eth_type=self.ether_types.ETH_TYPE_ARP,
+                                   arp_spa=ip),
+                               priority=10,
+                               table_id=0),
+                           active_bundle=None)]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_delete_garp_blocker(self):
+        vlan = 2222
+        ip = '192.0.0.10'
+        self.br.delete_garp_blocker(vlan, ip)
+
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call.uninstall_flows(
+                table_id=0,
+                priority=10,
+                match=ofpp.OFPMatch(
+                    vlan_vid=vlan | ofp.OFPVID_PRESENT,
+                    eth_type=self.ether_types.ETH_TYPE_ARP,
+                    arp_spa=ip))
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_install_garp_blocker_exception(self):
+        vlan = 2222
+        ip = '192.0.0.10'
+        except_ip = '192.0.0.20'
+        self.br.install_garp_blocker_exception(vlan, ip, except_ip)
+
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call._send_msg(ofpp.OFPFlowMod(dp, cookie=self.stamp,
+                               instructions=[
+                                   ofpp.OFPInstructionGotoTable(table_id=59)],
+                               match=ofpp.OFPMatch(
+                                   vlan_vid=vlan | ofp.OFPVID_PRESENT,
+                                   eth_type=self.ether_types.ETH_TYPE_ARP,
+                                   arp_spa=ip, arp_tpa=except_ip),
+                               priority=11,
+                               table_id=0),
+                           active_bundle=None)
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
+
+    def test_delete_garp_blocker_exception(self):
+        vlan = 2222
+        ip = '192.0.0.10'
+        except_ip = '192.0.0.20'
+        self.br.delete_garp_blocker_exception(vlan, ip, except_ip)
+
+        (dp, ofp, ofpp) = self._get_dp()
+        expected = [
+            call.uninstall_flows(
+                table_id=0,
+                priority=11,
+                match=ofpp.OFPMatch(
+                    vlan_vid=vlan | ofp.OFPVID_PRESENT,
+                    eth_type=self.ether_types.ETH_TYPE_ARP,
+                    arp_spa=ip, arp_tpa=except_ip))
+        ]
+        self.assertEqual(expected, self.mock.mock_calls)
