@@ -14,7 +14,6 @@
 import collections
 import copy
 from unittest import mock
-import uuid
 
 from ovsdbapp.backend import ovs_idl
 
@@ -148,10 +147,12 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         'lrouters': [
             {'name': utils.ovn_name('lr-id-a'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-a'}},
+                              'lr-name-a',
+                              ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: 'az-a'}},
             {'name': utils.ovn_name('lr-id-b'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
-                              'lr-name-b'}},
+                              'lr-name-b',
+                              ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: 'az-b'}},
             {'name': utils.ovn_name('lr-id-c'),
              'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
                               'lr-name-c'}},
@@ -163,7 +164,9 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                               'lr-name-e'}}],
         'lrouter_ports': [
             {'name': utils.ovn_lrouter_port_name('orp-id-a1'),
-             'external_ids': {}, 'networks': ['10.0.1.0/24'],
+             'external_ids': {ovn_const.OVN_ROUTER_NAME_EXT_ID_KEY:
+                              'lr-id-a'},
+             'networks': ['10.0.1.0/24'],
              'options': {ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-1'}},
             {'name': utils.ovn_lrouter_port_name('orp-id-a2'),
              'external_ids': {}, 'networks': ['10.0.2.0/24'],
@@ -354,6 +357,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         self._tables['DHCP_Options'] = self.dhcp_table
         self._tables['Address_Set'] = self.address_set_table
         self._tables['Load_Balancer'] = self.lb_table
+        self._tables['NAT'] = self.nat_table
 
         with mock.patch.object(impl_idl_ovn.OvsdbNbOvnIdl, 'from_worker',
                                return_value=mock.Mock()):
@@ -422,7 +426,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
     def test_get_all_logical_switches_with_ports(self):
         # Test empty
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
-        self.assertItemsEqual(mapping, {})
+        self.assertCountEqual(mapping, {})
         # Test loaded values
         self._load_nb_db()
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
@@ -440,12 +444,12 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                      'ports': ['lsp-id-51', 'lsp-id-52', 'lsp-rp-id-5',
                                'lsp-vpn-id-5'],
                      'provnet_ports': []}]
-        self.assertItemsEqual(mapping, expected)
+        self.assertCountEqual(mapping, expected)
 
     def test_get_all_logical_routers_with_rports(self):
         # Test empty
         mapping = self.nb_ovn_idl.get_all_logical_switches_with_ports()
-        self.assertItemsEqual(mapping, {})
+        self.assertCountEqual(mapping, {})
         # Test loaded values
         self._load_nb_db()
         mapping = self.nb_ovn_idl.get_all_logical_routers_with_rports()
@@ -481,7 +485,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                      'snats': [], 'dnat_and_snats': []},
                     {'name': 'lr-id-e', 'ports': {}, 'static_routes': [],
                      'snats': [], 'dnat_and_snats': []}]
-        self.assertItemsEqual(mapping, expected)
+        self.assertCountEqual(mapping, expected)
 
     def test_get_acls_for_lswitches(self):
         self._load_nb_db()
@@ -538,7 +542,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                  'direction': 'to-lport',
                  'match': 'outport == "lsp-id-52" && ip4.src == $as_ip4_id_5'}
             ]}
-        self.assertItemsEqual(acl_values, excepted_acl_values)
+        self.assertCountEqual(acl_values, excepted_acl_values)
         self.assertEqual(len(acl_objs), 8)
         self.assertEqual(len(lswitch_ovsdb_dict), len(lswitches))
 
@@ -546,7 +550,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         lswitches = ['ls-id-4']
         acl_values, acl_objs, lswitch_ovsdb_dict = \
             self.nb_ovn_idl.get_acls_for_lswitches(lswitches)
-        self.assertItemsEqual(acl_values, {})
+        self.assertCountEqual(acl_values, {})
         self.assertEqual(len(acl_objs), 0)
         self.assertEqual(len(lswitch_ovsdb_dict), 0)
 
@@ -558,15 +562,15 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                     'host-2': [utils.ovn_lrouter_port_name('orp-id-b2')],
                     ovn_const.OVN_GATEWAY_INVALID_CHASSIS: [
                         utils.ovn_name('orp-id-a3')]}
-        self.assertItemsEqual(bindings, expected)
+        self.assertCountEqual(bindings, expected)
 
         bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings([])
-        self.assertItemsEqual(bindings, expected)
+        self.assertCountEqual(bindings, expected)
 
         bindings = self.nb_ovn_idl.get_all_chassis_gateway_bindings(['host-1'])
         expected = {'host-1': [utils.ovn_lrouter_port_name('orp-id-a1'),
                                utils.ovn_lrouter_port_name('orp-id-a2')]}
-        self.assertItemsEqual(bindings, expected)
+        self.assertCountEqual(bindings, expected)
 
     def test_get_gateway_chassis_binding(self):
         self._load_nb_db()
@@ -596,17 +600,29 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         # Test only that orp-id-a3 is to be scheduled.
         # Rest ports don't have required chassis (physnet2)
         # or are already scheduled.
+        chassis_with_azs = {'host-1': ['az-a'], 'host-2': ['az-b']}
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             port_physnet_dict, {'host-1': 'physnet1', 'host-2': 'physnet3'},
-            ['host-1', 'host-2'])
+            ['host-1', 'host-2'], chassis_with_azs)
         expected = ['lrp-orp-id-a3']
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
         # Test both host-1, host-2 in valid list
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             port_physnet_dict, {'host-1': 'physnet1', 'host-2': 'physnet2'},
-            ['host-1', 'host-2'])
+            ['host-1', 'host-2'], chassis_with_azs)
         expected = ['lrp-orp-id-a3', 'lrp-orp-id-b6']
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
+        # Test lrp-orp-id-a1 az_hints not in host-1's azs
+        # lrp-orp-id-a2 not set az_hints, should schedule in host-1, host-3
+        # lrp-orp-id-a3 not scheduled
+        chassis_with_azs = {'host-1': ['az-b'], 'host-2': ['az-b'],
+                            'host-3': ['az-a']}
+        unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
+            port_physnet_dict, {'host-1': 'physnet1', 'host-2': 'physnet3',
+                                'host-3': 'physnet1'},
+            ['host-1', 'host-2', 'host-3'], chassis_with_azs)
+        expected = ['lrp-orp-id-a1', 'lrp-orp-id-a2', 'lrp-orp-id-a3']
+        self.assertCountEqual(unhosted_gateways, expected)
 
     def test_get_unhosted_gateways_deleted_physnet(self):
         self._load_nb_db()
@@ -616,18 +632,19 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         setattr(router_row, 'options', {
             ovn_const.OVN_GATEWAY_CHASSIS_KEY: 'host-2'})
         port_physnet_dict = {'orp-id-a1': 'physnet1'}
+        chassis_with_azs = {'host-1': ['az-a'], 'host-2': ['az-a']}
         # Lets spoof that physnet1 is deleted from host-2.
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             port_physnet_dict, {'host-1': 'physnet1', 'host-2': 'physnet3'},
-            ['host-1', 'host-2'])
+            ['host-1', 'host-2'], chassis_with_azs)
         # Make sure that lrp is rescheduled, because host-1 has physet1
         expected = ['lrp-orp-id-a1']
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
         # Spoof that there is no valid host with required physnet.
         unhosted_gateways = self.nb_ovn_idl.get_unhosted_gateways(
             port_physnet_dict, {'host-1': 'physnet4', 'host-2': 'physnet3'},
-            ['host-1', 'host-2'])
-        self.assertItemsEqual(unhosted_gateways, [])
+            ['host-1', 'host-2'], chassis_with_azs)
+        self.assertCountEqual(unhosted_gateways, [])
 
     def _test_get_unhosted_gateway_max_chassis(self, r):
         gw_chassis_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
@@ -647,10 +664,11 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             {'host-1': 'physnet1', 'host-2': 'physnet2',
              'host-3': 'physnet1', 'host-4': 'physnet2',
              'host-5': 'physnet1', 'host-6': 'physnet2'},
-            ['host-%s' % x for x in range(1, 7)])
+            ['host-%s' % x for x in range(1, 7)],
+            {'host-%s' % x: ['az-a'] for x in range(1, 7)})
         # We don't have required number of chassis
         expected = []
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
 
     def test_get_unhosted_gateway_max_chassis(self):
         # We have required number of chassis, and lrp
@@ -661,9 +679,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             {'host-1': 'physnet1', 'host-2': 'physnet1',
              'host-3': 'physnet1', 'host-4': 'physnet1',
              'host-5': 'physnet1', 'host-6': 'physnet1'},
-            ['host-%s' % x for x in range(1, 7)])
+            ['host-%s' % x for x in range(1, 7)],
+            {'host-%s' % x: ['az-a'] for x in range(1, 7)})
         expected = []
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
 
     def test_get_unhosed_gateway_schedule_to_max(self):
         # The LRP is not yet scheduled on all chassis
@@ -674,9 +693,10 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             {'host-1': 'physnet1', 'host-2': 'physnet1',
              'host-3': 'physnet1', 'host-4': 'physnet1',
              'host-5': 'physnet1', 'host-6': 'physnet1'},
-            ['host-%s' % x for x in range(1, 7)])
+            ['host-%s' % x for x in range(1, 7)],
+            {'host-%s' % x: ['az-a'] for x in range(1, 7)})
         expected = ['lrp-orp-id-a1']
-        self.assertItemsEqual(unhosted_gateways, expected)
+        self.assertCountEqual(unhosted_gateways, expected)
 
     def test_get_subnet_dhcp_options(self):
         self._load_nb_db()
@@ -692,16 +712,16 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             'ports': []}, subnet_options)
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'subnet-id-11-0-2-0')['subnet']
-        self.assertIsNone(subnet_options)
+        self.assertEqual({}, subnet_options)
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'port-id-30-0-1-0')['subnet']
-        self.assertIsNone(subnet_options)
+        self.assertEqual({}, subnet_options)
 
     def test_get_subnet_dhcp_options_with_ports(self):
         # Test empty
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'subnet-id-10-0-1-0', with_ports=True)
-        self.assertItemsEqual({'subnet': None, 'ports': []}, subnet_options)
+        self.assertCountEqual({'subnet': None, 'ports': []}, subnet_options)
         # Test loaded values
         self._load_nb_db()
         # Test getting both subnet and port dhcp options
@@ -714,7 +734,7 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                           'external_ids': dhcp_row.external_ids,
                           'options': dhcp_row.options,
                           'uuid': dhcp_row.uuid} for dhcp_row in dhcp_rows]
-        self.assertItemsEqual(expected_rows, [
+        self.assertCountEqual(expected_rows, [
             subnet_options['subnet']] + subnet_options['ports'])
         # Test getting only subnet dhcp options
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
@@ -725,12 +745,12 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
                           'external_ids': dhcp_row.external_ids,
                           'options': dhcp_row.options,
                           'uuid': dhcp_row.uuid} for dhcp_row in dhcp_rows]
-        self.assertItemsEqual(expected_rows, [
+        self.assertCountEqual(expected_rows, [
             subnet_options['subnet']] + subnet_options['ports'])
         # Test getting no dhcp options
         subnet_options = self.nb_ovn_idl.get_subnet_dhcp_options(
             'subnet-id-11-0-2-0', with_ports=True)
-        self.assertItemsEqual({'subnet': None, 'ports': []}, subnet_options)
+        self.assertCountEqual({'subnet': None, 'ports': []}, subnet_options)
 
     def test_get_subnets_dhcp_options(self):
         self._load_nb_db()
@@ -745,13 +765,13 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
             get_row_dict(
                 self._find_ovsdb_fake_row(self.dhcp_table, 'cidr', cidr))
             for cidr in ('10.0.1.0/24', '10.0.2.0/24')]
-        self.assertItemsEqual(expected_rows, subnets_options)
+        self.assertCountEqual(expected_rows, subnets_options)
 
         subnets_options = self.nb_ovn_idl.get_subnets_dhcp_options(
             ['subnet-id-11-0-2-0', 'subnet-id-20-0-1-0'])
         expected_row = get_row_dict(
             self._find_ovsdb_fake_row(self.dhcp_table, 'cidr', '20.0.1.0/24'))
-        self.assertItemsEqual([expected_row], subnets_options)
+        self.assertCountEqual([expected_row], subnets_options)
 
         subnets_options = self.nb_ovn_idl.get_subnets_dhcp_options(
             ['port-id-30-0-1-0', 'fake-not-exist'])
@@ -768,20 +788,6 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         address_sets = self.nb_ovn_idl.get_address_sets()
         self.assertEqual(len(address_sets), 4)
 
-    def test_get_port_group_not_supported(self):
-        self._load_nb_db()
-        # Make sure that PG tables doesn't exist in fake db.
-        self._tables.pop('Port_Group', None)
-        port_group = self.nb_ovn_idl.get_port_group(str(uuid.uuid4()))
-        self.assertIsNone(port_group)
-
-    def test_get_sg_port_groups_not_supported(self):
-        self._load_nb_db()
-        # Make sure that PG tables doesn't exist in fake db.
-        self._tables.pop('Port_Group', None)
-        port_groups = self.nb_ovn_idl.get_sg_port_groups()
-        self.assertEqual({}, port_groups)
-
     def test_get_router_floatingip_lbs(self):
         lrouter_name = 'rtr_name'
         # Empty
@@ -806,3 +812,61 @@ class TestNBImplIdlOvn(TestDBImplIdlOvn):
         lb_row = self._find_ovsdb_fake_row(self.lb_table, 'name', 'lb_2')
         lb = self.nb_ovn_idl.get_floatingip_in_nat_or_lb(fip_id)
         self.assertEqual(lb['_uuid'], lb_row.uuid)
+
+
+class TestSBImplIdlOvnBase(TestDBImplIdlOvn):
+
+    fake_set = {
+        'chassis': [
+            {
+                'hostname': 'fake-smartnic-dpu-chassis.fqdn',
+                'external_ids': {
+                    ovn_const.OVN_CMS_OPTIONS: (
+                        'firstoption,'
+                        'card-serial-number=fake-serial,'
+                        'thirdoption'),
+                },
+            },
+        ],
+    }
+    fake_associations = {}
+
+    def setUp(self):
+        super(TestSBImplIdlOvnBase, self).setUp()
+
+        self.chassis_table = fakes.FakeOvsdbTable.create_one_ovsdb_table()
+
+        self._tables = {}
+        self._tables['Chassis'] = self.chassis_table
+
+        with mock.patch.object(impl_idl_ovn.OvsdbSbOvnIdl, 'from_worker',
+                               return_value=mock.Mock()):
+            with mock.patch.object(ovs_idl.Backend, 'autocreate_indices',
+                                   create=True):
+                impl_idl_ovn.OvsdbSbOvnIdl.ovsdb_connection = None
+                self.sb_ovn_idl = impl_idl_ovn.OvsdbSbOvnIdl(mock.MagicMock())
+
+        self.sb_ovn_idl.idl.tables = self._tables
+
+    def _load_sb_db(self):
+        # Load Chassis
+        fake_chassis = TestSBImplIdlOvnBase.fake_set['chassis']
+        self._load_ovsdb_fake_rows(self.chassis_table, fake_chassis)
+
+
+class TestSBImplIdlOvnGetChassisByCardSerialFromCMSOptions(
+        TestSBImplIdlOvnBase):
+
+    def test_chassis_not_found(self):
+        self._load_sb_db()
+        self.assertRaises(
+            RuntimeError,
+            self.sb_ovn_idl.get_chassis_by_card_serial_from_cms_options,
+            'non-existent')
+
+    def test_chassis_found(self):
+        self._load_sb_db()
+        self.assertEqual(
+            'fake-smartnic-dpu-chassis.fqdn',
+            self.sb_ovn_idl.get_chassis_by_card_serial_from_cms_options(
+                'fake-serial').hostname)
