@@ -33,6 +33,7 @@ from neutron.common.ovn import hash_ring_manager
 from neutron.common.ovn import utils
 from neutron.conf.plugins.ml2.drivers.ovn import ovn_conf
 from neutron.db import ovn_hash_ring_db
+from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import impl_idl_ovn
 from neutron.plugins.ml2.drivers.ovn.mech_driver.ovsdb import ovsdb_monitor
 from neutron.services.ovn_l3 import plugin  # noqa
 from neutron.tests import base
@@ -99,33 +100,29 @@ class TestOvnDbNotifyHandler(base.BaseTestCase):
     def setUp(self):
         super(TestOvnDbNotifyHandler, self).setUp()
         self.handler = ovsdb_monitor.OvnDbNotifyHandler(mock.ANY)
-        # NOTE(ralonsoh): once the ovsdbapp library version is bumped beyond
-        # 1.5.0, the first assignation (using name mangling) can be deleted.
-        try:
-            self.watched_events = self.handler._RowEventHandler__watched_events
-        except AttributeError:
-            self.watched_events = self.handler._watched_events
 
     def test_watch_and_unwatch_events(self):
         expected_events = set()
-        networking_event = mock.Mock()
-        ovn_event = mock.Mock()
-        unknown_event = mock.Mock()
+        networking_event = mock.Mock(priority=1)
+        ovn_event = mock.Mock(priority=2)
+        unknown_event = mock.Mock(priority=3)
 
-        self.assertItemsEqual(set(), self.watched_events)
+        self.assertCountEqual(set(), set(self.handler._watched_events))
 
         expected_events.add(networking_event)
         self.handler.watch_event(networking_event)
-        self.assertItemsEqual(expected_events, self.watched_events)
+        self.assertCountEqual(expected_events,
+                              set(self.handler._watched_events))
 
         expected_events.add(ovn_event)
         self.handler.watch_events([ovn_event])
-        self.assertItemsEqual(expected_events, self.watched_events)
+        self.assertCountEqual(expected_events,
+                              set(self.handler._watched_events))
 
         self.handler.unwatch_events([networking_event, ovn_event])
         self.handler.unwatch_event(unknown_event)
         self.handler.unwatch_events([unknown_event])
-        self.assertItemsEqual(set(), self.watched_events)
+        self.assertCountEqual(set(), set(self.handler._watched_events))
 
     def test_shutdown(self):
         self.handler.shutdown()
@@ -133,7 +130,7 @@ class TestOvnDbNotifyHandler(base.BaseTestCase):
 
 # class TestOvnBaseConnection(base.TestCase):
 #
-# Each test is being deleted, but for reviewers sake I wanted to exaplain why:
+# Each test is being deleted, but for reviewers sake I wanted to explain why:
 #
 #     @mock.patch.object(idlutils, 'get_schema_helper')
 #     def testget_schema_helper_success(self, mock_gsh):
@@ -160,6 +157,7 @@ class TestOvnDbNotifyHandler(base.BaseTestCase):
 class TestOvnConnection(base.BaseTestCase):
 
     def setUp(self):
+        ovn_conf.register_opts()
         super(TestOvnConnection, self).setUp()
 
     @mock.patch.object(idlutils, 'get_schema_helper')
@@ -168,7 +166,9 @@ class TestOvnConnection(base.BaseTestCase):
                                idl_class, schema):
         mock_gsh.return_value = ovs_idl.SchemaHelper(
             location=schema_files[schema])
-        _idl = idl_class.from_server('punix:/tmp/fake', schema, mock.Mock())
+        impl_idl_ovn.Backend.schema = schema
+        helper = impl_idl_ovn.OvsdbNbOvnIdl.schema_helper
+        _idl = idl_class.from_server('punix:/fake', helper, mock.Mock())
         self.ovn_connection = connection.Connection(_idl, mock.Mock())
         with mock.patch.object(poller, 'Poller'), \
                 mock.patch('threading.Thread'):
@@ -199,6 +199,7 @@ class TestOvnConnection(base.BaseTestCase):
 class TestOvnIdlDistributedLock(base.BaseTestCase):
 
     def setUp(self):
+        ovn_conf.register_opts()
         super(TestOvnIdlDistributedLock, self).setUp()
         self.node_uuid = uuidutils.generate_uuid()
         self.fake_driver = mock.Mock()
@@ -216,14 +217,21 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
         self.mock_get_node = mock.patch.object(
             hash_ring_manager.HashRingManager,
             'get_node', return_value=self.node_uuid).start()
+        self.mock_update_tables = mock.patch.object(
+            self.idl, 'update_tables').start()
+
+    def _assert_has_notify_calls(self):
+        self.idl.notify_handler.notify.assert_has_calls([
+            mock.call(self.fake_event, self.fake_row, None, global_=True),
+            mock.call(self.fake_event, self.fake_row, None)])
+        self.assertEqual(2, len(self.idl.notify_handler.mock_calls))
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify(self, mock_touch_node):
         self.idl.notify(self.fake_event, self.fake_row)
 
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify_skip_touch_node(self, mock_touch_node):
@@ -233,8 +241,7 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
 
         # Assert that touch_node() wasn't called
         self.assertFalse(mock_touch_node.called)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
     def test_notify_last_touch_expired(self, mock_touch_node):
@@ -250,8 +257,7 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
 
         # Assert that touch_node() was invoked
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     @mock.patch.object(ovsdb_monitor.LOG, 'exception')
     @mock.patch.object(ovn_hash_ring_db, 'touch_node')
@@ -264,14 +270,77 @@ class TestOvnIdlDistributedLock(base.BaseTestCase):
         mock_touch_node.assert_called_once_with(mock.ANY, self.node_uuid)
         # Assert we are logging the exception
         self.assertTrue(mock_log.called)
-        self.idl.notify_handler.notify.assert_called_once_with(
-            self.fake_event, self.fake_row, None)
+        self._assert_has_notify_calls()
 
     def test_notify_different_node(self):
         self.mock_get_node.return_value = 'different-node-uuid'
         self.idl.notify('fake-event', self.fake_row)
         # Assert that notify() wasn't called for a different node uuid
-        self.assertFalse(self.idl.notify_handler.notify.called)
+        self.idl.notify_handler.notify.assert_called_once_with(
+            self.fake_event, self.fake_row, None, global_=True)
+
+    @staticmethod
+    def _create_fake_row(table_name):
+        # name is a parameter in Mock() so it can't be passed to constructor
+        table = mock.Mock()
+        table.name = table_name
+        return fakes.FakeOvsdbRow.create_one_ovsdb_row(
+            attrs={'_table': table, 'schema': ['foo']})
+
+    def test_handle_db_schema_changes_no_match_events(self):
+        other_table_row = self._create_fake_row('other')
+        database_table_row = self._create_fake_row('Database')
+
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_UPDATE, other_table_row)
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_CREATE, other_table_row)
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_UPDATE, database_table_row)
+
+        self.assertFalse(self.mock_update_tables.called)
+
+    def _test_handle_db_schema(self, agent_table, chassis_private_present):
+        database_table_row = self._create_fake_row('Database')
+        self.idl._tables_to_register[database_table_row.name] = 'foo'
+
+        self.fake_driver.agent_chassis_table = agent_table
+        if chassis_private_present:
+            self.idl.tables['Chassis_Private'] = 'foo'
+        else:
+            try:
+                del self.idl.tables['Chassis_Private']
+            except KeyError:
+                pass
+
+        self.idl.handle_db_schema_changes(
+            ovsdb_monitor.BaseEvent.ROW_CREATE, database_table_row)
+
+    def test_handle_db_schema_changes_old_schema_to_old_schema(self):
+        """Agents use Chassis and should keep using Chassis table"""
+        self._test_handle_db_schema('Chassis', chassis_private_present=False)
+        self.assertEqual('Chassis', self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_old_schema_to_new_schema(self):
+        """Agents use Chassis and should start using Chassis_Private table"""
+        self._test_handle_db_schema('Chassis', chassis_private_present=True)
+        self.assertEqual('Chassis_Private',
+                         self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_new_schema_to_old_schema(self):
+        """Agents use Chassis_Private and should start using Chassis table"""
+        self._test_handle_db_schema('Chassis_Private',
+                                    chassis_private_present=False)
+        self.assertEqual('Chassis', self.fake_driver.agent_chassis_table)
+
+    def test_handle_db_schema_changes_new_schema_to_new_schema(self):
+        """Agents use Chassis_Private and should keep using Chassis_Private
+           table.
+        """
+        self._test_handle_db_schema('Chassis_Private',
+                                    chassis_private_present=True)
+        self.assertEqual('Chassis_Private',
+                         self.fake_driver.agent_chassis_table)
 
 
 class TestPortBindingChassisUpdateEvent(base.BaseTestCase):
@@ -294,7 +363,7 @@ class TestPortBindingChassisUpdateEvent(base.BaseTestCase):
         pbtable = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'name': 'Port_Binding'})
         ovsdb_row = fakes.FakeOvsdbRow.create_one_ovsdb_row
-        self.driver._nb_ovn.lookup.return_value = ovsdb_row(attrs={'up': True})
+        self.driver.nb_ovn.lookup.return_value = ovsdb_row(attrs={'up': True})
         self._test_event(
             self.event.ROW_UPDATE,
             ovsdb_row(attrs={'_table': pbtable, 'chassis': 'one',
@@ -309,10 +378,14 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         super(TestOvnNbIdlNotifyHandler, self).setUp()
         helper = ovs_idl.SchemaHelper(schema_json=OVN_NB_SCHEMA)
         helper.register_all()
-        self.idl = ovsdb_monitor.OvnNbIdl(self.driver, "remote", helper)
+        self.idl = ovsdb_monitor.OvnNbIdl(self.mech_driver, "remote", helper)
         self.lp_table = self.idl.tables.get('Logical_Switch_Port')
-        self.driver.set_port_status_up = mock.Mock()
-        self.driver.set_port_status_down = mock.Mock()
+        self.mech_driver.set_port_status_up = mock.Mock()
+        self.mech_driver.set_port_status_down = mock.Mock()
+        mock.patch.object(self.idl, 'handle_db_schema_changes').start()
+        self._mock_hash_ring = mock.patch.object(
+            self.idl._hash_ring, 'get_node', return_value=self.idl._node_uuid)
+        self._mock_hash_ring.start()
 
     def _test_lsp_helper(self, event, new_row_json, old_row_json=None,
                          table=None):
@@ -335,32 +408,33 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
     def test_lsp_up_create_event(self):
         row_data = {"up": True, "name": "foo-name"}
         self._test_lsp_helper('create', row_data)
-        self.driver.set_port_status_up.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.mech_driver.set_port_status_up.assert_called_once_with("foo-name")
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_lsp_down_create_event(self):
         row_data = {"up": False, "name": "foo-name"}
         self._test_lsp_helper('create', row_data)
-        self.driver.set_port_status_down.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_up.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with(
+            "foo-name")
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
 
     def test_lsp_up_not_set_event(self):
         row_data = {"up": ['set', []], "name": "foo-name"}
         self._test_lsp_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_unwatch_logical_switch_port_create_events(self):
         self.idl.unwatch_logical_switch_port_create_events()
         row_data = {"up": True, "name": "foo-name"}
         self._test_lsp_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
         row_data["up"] = False
         self._test_lsp_helper('create', row_data)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_post_connect(self):
         self.idl.post_connect()
@@ -372,30 +446,31 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json = {"up": False}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=old_row_json)
-        self.driver.set_port_status_up.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.mech_driver.set_port_status_up.assert_called_once_with("foo-name")
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_lsp_down_update_event(self):
         new_row_json = {"up": False, "name": "foo-name"}
         old_row_json = {"up": True}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=old_row_json)
-        self.driver.set_port_status_down.assert_called_once_with("foo-name")
-        self.assertFalse(self.driver.set_port_status_up.called)
+        self.mech_driver.set_port_status_down.assert_called_once_with(
+            "foo-name")
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
 
     def test_lsp_up_update_event_no_old_data(self):
         new_row_json = {"up": True, "name": "foo-name"}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=None)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_lsp_down_update_event_no_old_data(self):
         new_row_json = {"up": False, "name": "foo-name"}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=None)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_lsp_other_column_update_event(self):
         new_row_json = {"up": False, "name": "foo-name",
@@ -403,25 +478,28 @@ class TestOvnNbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json = {"addresses": ["10.0.0.3"]}
         self._test_lsp_helper('update', new_row_json,
                               old_row_json=old_row_json)
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     def test_notify_other_table(self):
         new_row_json = {"name": "foo-name"}
         self._test_lsp_helper('create', new_row_json,
                               table=self.idl.tables.get("Logical_Switch"))
-        self.assertFalse(self.driver.set_port_status_up.called)
-        self.assertFalse(self.driver.set_port_status_down.called)
+        self.assertFalse(self.mech_driver.set_port_status_up.called)
+        self.assertFalse(self.mech_driver.set_port_status_down.called)
 
     @mock.patch.object(hash_ring_manager.HashRingManager, 'get_node')
     def test_notify_different_target_node(self, mock_get_node):
+        self._mock_hash_ring.stop()
         mock_get_node.return_value = 'this-is-a-different-node'
         row = fakes.FakeOvsdbRow.create_one_ovsdb_row()
+        row._table = mock.Mock(name='table-name')
         self.idl.notify_handler.notify = mock.Mock()
         self.idl.notify("create", row)
         # Assert that if the target_node returned by the ring is different
-        # than this driver's node_uuid, notify() won't be called
-        self.assertFalse(self.idl.notify_handler.notify.called)
+        # than this driver's node_uuid, only global notify() won't be called
+        self.idl.notify_handler.notify.assert_called_once_with(
+            "create", row, None, global_=True)
 
 
 class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
@@ -432,10 +510,12 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         super(TestOvnSbIdlNotifyHandler, self).setUp()
         sb_helper = ovs_idl.SchemaHelper(schema_json=OVN_SB_SCHEMA)
         sb_helper.register_table('Chassis')
-        self.sb_idl = ovsdb_monitor.OvnSbIdl(self.driver, "remote", sb_helper)
+        self.driver.agent_chassis_table = 'Chassis'
+        self.sb_idl = ovsdb_monitor.OvnSbIdl(self.mech_driver, "remote",
+                                             sb_helper)
         self.sb_idl.post_connect()
         self.chassis_table = self.sb_idl.tables.get('Chassis')
-        self.driver.update_segment_host_mapping = mock.Mock()
+        self.mech_driver.update_segment_host_mapping = mock.Mock()
         self.l3_plugin = directory.get_plugin(n_const.L3)
         self.l3_plugin.schedule_unhosted_gateways = mock.Mock()
 
@@ -445,6 +525,10 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
             "external_ids": ['map', [["ovn-bridge-mappings",
                                       "fake-phynet1:fake-br1"]]]
         }
+        self._mock_hash_ring = mock.patch.object(
+            self.sb_idl._hash_ring, 'get_node',
+            return_value=self.sb_idl._node_uuid)
+        self._mock_hash_ring.start()
 
     def _test_chassis_helper(self, event, new_row_json, old_row_json=None):
         row_uuid = uuidutils.generate_uuid()
@@ -463,14 +547,14 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
 
     def test_chassis_create_event(self):
         self._test_chassis_helper('create', self.row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
+        self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
 
     def test_chassis_delete_event(self):
         self._test_chassis_helper('delete', self.row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
+        self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', [])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis='fake-name')
@@ -480,7 +564,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json['external_ids'][1][0][1] = (
             "fake-phynet2:fake-br2")
         self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
+        self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
@@ -491,7 +575,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json['external_ids'][1][1][1] = (
             "foo_value")
         self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_not_called()
+        self.mech_driver.update_segment_host_mapping.assert_not_called()
         self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
 
     def test_chassis_update_event_reschedule_lost_physnet(self):
@@ -505,7 +589,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json = copy.deepcopy(self.row_json)
         self.row_json['external_ids'][1][0][1] += ',foo_physnet:foo_br'
         self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
+        self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['fake-phynet1', 'foo_physnet'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
@@ -514,7 +598,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
         old_row_json = copy.deepcopy(self.row_json)
         self.row_json['external_ids'][1][0][1] = 'foo_physnet:foo_br'
         self._test_chassis_helper('update', self.row_json, old_row_json)
-        self.driver.update_segment_host_mapping.assert_called_once_with(
+        self.mech_driver.update_segment_host_mapping.assert_called_once_with(
             'fake-hostname', ['foo_physnet'])
         self.l3_plugin.schedule_unhosted_gateways.assert_called_once_with(
             event_from_chassis=None)
@@ -527,7 +611,7 @@ class TestOvnSbIdlNotifyHandler(test_mech_driver.OVNMechanismDriverTestCase):
             'ovsdb_monitor.ChassisEvent.'
                 'handle_ha_chassis_group_changes') as mock_ha:
             self._test_chassis_helper('update', self.row_json, old_row_json)
-            self.driver.update_segment_host_mapping.assert_not_called()
+            self.mech_driver.update_segment_host_mapping.assert_not_called()
             self.l3_plugin.schedule_unhosted_gateways.assert_not_called()
             mock_ha.assert_not_called()
 
@@ -536,8 +620,8 @@ class TestChassisEvent(base.BaseTestCase):
 
     def setUp(self):
         super(TestChassisEvent, self).setUp()
-        self.driver = mock.Mock()
-        self.nb_ovn = self.driver._nb_ovn
+        self.driver = mock.MagicMock()
+        self.nb_ovn = self.driver.nb_ovn
         self.driver._ovn_client.is_external_ports_supported.return_value = True
         self.event = ovsdb_monitor.ChassisEvent(self.driver)
         self.is_gw_ch_mock = mock.patch.object(
@@ -553,44 +637,76 @@ class TestChassisEvent(base.BaseTestCase):
         self.assertFalse(self.nb_ovn.ha_chassis_group_del_chassis.called)
 
     def _test_handle_ha_chassis_group_changes_create(self, event):
+        # Chassis
+        ext_ids = {
+            'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
         row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'name': 'SpongeBob'})
+            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+        # HA Chassis
         ch0 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'priority': 10})
         ch1 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
             attrs={'priority': 9})
-        default_grp = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'ha_chassis': [ch0, ch1]})
-        self.nb_ovn.ha_chassis_group_get.return_value.execute.return_value = (
-            default_grp)
+        ch2 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'priority': 10})
+        # HA Chassis Groups
+        ha_ch_grp0 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'ha_chassis': [ch0, ch1], 'name': 'neutron-ha-ch-grp0',
+                   'external_ids': {
+                        ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: 'az-0,az-1'}})
+        ha_ch_grp1 = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'ha_chassis': [ch2], 'name': 'neutron-ha-ch-grp1',
+                   'external_ids': {
+                        ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: 'az-2'}})
+
+        self.nb_ovn.db_list_rows.return_value.execute.return_value = [
+            ha_ch_grp0, ha_ch_grp1]
         self.event.handle_ha_chassis_group_changes(event, row, mock.Mock())
-        # Assert the new chassis has been added to the default
-        # group with the lowest priority
+        # Assert the new chassis has been added to "neutron-ha-ch-grp0"
+        # HA Chassis Group with the lowest priority
         self.nb_ovn.ha_chassis_group_add_chassis.assert_called_once_with(
-            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, 'SpongeBob', priority=8)
+            'neutron-ha-ch-grp0', 'SpongeBob', priority=8)
 
     def test_handle_ha_chassis_group_changes_create(self):
         self._test_handle_ha_chassis_group_changes_create(
             self.event.ROW_CREATE)
 
     def _test_handle_ha_chassis_group_changes_delete(self, event):
+        # Chassis
+        ext_ids = {
+            'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
         row = fakes.FakeOvsdbTable.create_one_ovsdb_table(
-            attrs={'name': 'SpongeBob'})
+            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+        # HA Chassis
+        ha_ch = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'priority': 10})
+        # HA Chassis Group
+        ha_ch_grp = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'ha_chassis': [ha_ch], 'name': 'neutron-ha-ch-grp',
+                   'external_ids': {
+                        ovn_const.OVN_AZ_HINTS_EXT_ID_KEY: 'az-0'}})
+        self.nb_ovn.db_list_rows.return_value.execute.return_value = [
+            ha_ch_grp]
+
         self.event.handle_ha_chassis_group_changes(event, row, mock.Mock())
         # Assert chassis was removed from the default group
         self.nb_ovn.ha_chassis_group_del_chassis.assert_called_once_with(
-            ovn_const.HA_CHASSIS_GROUP_DEFAULT_NAME, 'SpongeBob',
-            if_exists=True)
+            'neutron-ha-ch-grp', 'SpongeBob', if_exists=True)
 
     def test_handle_ha_chassis_group_changes_delete(self):
         self._test_handle_ha_chassis_group_changes_delete(
             self.event.ROW_DELETE)
 
-    def test_handle_ha_chassis_group_changes_update_still_gw(self):
+    def test_handle_ha_chassis_group_changes_update_no_changes(self):
         # Assert nothing was done because the update didn't
-        # change the gateway chassis status
+        # change the gateway chassis status or the availability zones
+        ext_ids = {
+            'ovn-cms-options': 'enable-chassis-as-gw,availability-zones=az-0'}
+        new = fakes.FakeOvsdbTable.create_one_ovsdb_table(
+            attrs={'name': 'SpongeBob', 'external_ids': ext_ids})
+        old = new
         self.assertIsNone(self.event.handle_ha_chassis_group_changes(
-            self.event.ROW_UPDATE, mock.Mock(), mock.Mock()))
+            self.event.ROW_UPDATE, new, old))
         self.assertFalse(self.nb_ovn.ha_chassis_group_add_chassis.called)
         self.assertFalse(self.nb_ovn.ha_chassis_group_del_chassis.called)
 
@@ -610,6 +726,10 @@ class TestChassisEvent(base.BaseTestCase):
 
 
 class TestShortLivingOvsdbApi(base.BaseTestCase):
+    def setUp(self):
+        ovn_conf.register_opts()
+        super().setUp()
+
     def test_context(self):
         api_class = mock.Mock()
         idl = mock.Mock()
