@@ -59,7 +59,7 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
         # the new engine facade is resulting in changes being spread over
         # other sessions so we can end up getting stale reads in the parent
         # session if objects remain in the identity map.
-        if not self._ctx.session.is_active:
+        if not db_api.is_session_active(self._ctx.session):
             self._ctx.session.expire_all()
         return self._ctx
 
@@ -155,8 +155,20 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
                 raise db_exc.DBDeadlock()
             db_api.sqla_listen(se.Session, 'before_commit',
                                concurrent_increment)
+
+            # Despite the revision number is bumped twice during the session
+            # transaction, the revision number is tested only once the first
+            # time the revision number service is executed for this session and
+            # object.
             self._update('ports', port['port']['id'], new,
                          headers={'If-Match': 'revision_number=%s' % rev},
+                         expected_code=exc.HTTPOk.code)
+            self._update('ports', port['port']['id'], new,
+                         headers={'If-Match': 'revision_number=%s' %
+                                              str(int(rev) + 2)},
+                         expected_code=exc.HTTPOk.code)
+            self._update('ports', port['port']['id'], new,
+                         headers={'If-Match': 'revision_number=1'},
                          expected_code=exc.HTTPPreconditionFailed.code)
 
     def test_port_ip_update_revises(self):
@@ -178,6 +190,22 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
             new_rev = response['port']['revision_number']
             self.assertGreater(new_rev, rev)
 
+    def test_network_description_bumps_revision(self):
+        with self.network() as net:
+            rev = net['network']['revision_number']
+            data = {'network': {'description': 'Test Description'}}
+            response = self._update('networks', net['network']['id'], data)
+            new_rev = response['network']['revision_number']
+            self.assertEqual(rev + 1, new_rev)
+
+    def test_subnet_description_bumps_revision(self):
+        with self.subnet() as subnet:
+            rev = subnet['subnet']['revision_number']
+            data = {'subnet': {'description': 'Test Description'}}
+            response = self._update('subnets', subnet['subnet']['id'], data)
+            new_rev = response['subnet']['revision_number']
+            self.assertEqual(rev + 1, new_rev)
+
     def test_security_group_rule_ops_bump_security_group(self):
         s = {'security_group': {'tenant_id': 'some_tenant', 'name': '',
                                 'description': 's'}}
@@ -192,6 +220,7 @@ class TestRevisionPlugin(test_plugin.Ml2PluginV2TestCase):
                                      'remote_ip_prefix': '0.0.0.0/0',
                                      'ethertype': 'IPv4',
                                      'remote_group_id': None,
+                                     'remote_address_group_id': None,
                                      'direction': 'ingress',
                                      'security_group_id': sg['id']}}
         rule = self.cp.create_security_group_rule(self.ctx, r)
