@@ -45,7 +45,6 @@ class NeutronKeystoneContextTestCase(base.BaseTestCase):
         response = self.request.get_response(self.middleware)
         self.assertEqual('200 OK', response.status)
         self.assertEqual('testuserid', self.context.user_id)
-        self.assertEqual('testuserid', self.context.user)
 
     def test_with_tenant_id(self):
         self.request.headers['X_PROJECT_ID'] = 'testtenantid'
@@ -53,7 +52,7 @@ class NeutronKeystoneContextTestCase(base.BaseTestCase):
         response = self.request.get_response(self.middleware)
         self.assertEqual('200 OK', response.status)
         self.assertEqual('testtenantid', self.context.tenant_id)
-        self.assertEqual('testtenantid', self.context.tenant)
+        self.assertEqual('testtenantid', self.context.project_id)
 
     def test_roles_no_admin(self):
         self.request.headers['X_PROJECT_ID'] = 'testtenantid'
@@ -86,7 +85,7 @@ class NeutronKeystoneContextTestCase(base.BaseTestCase):
         self.assertEqual('testuserid', self.context.user_id)
         self.assertEqual('testusername', self.context.user_name)
         self.assertEqual('testtenantid', self.context.tenant_id)
-        self.assertEqual('testtenantname', self.context.tenant_name)
+        self.assertEqual('testtenantname', self.context.project_name)
 
     def test_request_id_extracted_from_env(self):
         req_id = 'dummy-request-id'
