@@ -15,23 +15,30 @@
 
 """Test of Policy Engine For Neutron"""
 
-import mock
+import copy
+import re
+from unittest import mock
+
+from neutron_lib.api import attributes
 from neutron_lib import constants
+from neutron_lib import context
 from neutron_lib import exceptions
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
+from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_policy import fixture as op_fixture
 from oslo_policy import policy as oslo_policy
 from oslo_serialization import jsonutils
 from oslo_utils import importutils
+from oslo_utils import uuidutils
 
 import neutron
-from neutron.api.v2 import attributes
-from neutron.common import constants as n_const
-from neutron import context
-from neutron import manager
 from neutron import policy
 from neutron.tests import base
 
+_uuid = uuidutils.generate_uuid
+
 
 class PolicyFileTestCase(base.BaseTestCase):
     def setUp(self):
@@ -71,13 +78,91 @@ class PolicyTestCase(base.BaseTestCase):
             "example:early_or_success": "@ or !",
             "example:lowercase_admin": "role:admin or role:sysadmin",
             "example:uppercase_admin": "role:ADMIN or role:sysadmin",
+            "example:only_system_admin_allowed": (
+                "role:admin and system_scope:all"),
+            "example:only_project_user_allowed": (
+                "role:reader and tenant_id:%(tenant_id)s")
         }
         policy.refresh()
+        self._register_default_rules()
         # NOTE(vish): then overload underlying rules
         policy.set_rules(oslo_policy.Rules.from_dict(rules))
         self.context = context.Context('fake', 'fake', roles=['member'])
         self.target = {}
 
+    def _register_default_rules(self):
+        rules_with_scope = [
+            oslo_policy.DocumentedRuleDefault(
+                name='get_example:only_project_user_allowed',
+                description="Test rule only",
+                check_str='role:reader and project_id:%(project_id)s',
+                operations=[
+                    {
+                        'method': 'GET',
+                        'path': '/example',
+                    },
+                ],
+                scope_types=['project'])]
+        policy._ENFORCER.register_defaults(rules_with_scope)
+
+    def _test_check_system_admin_allowed_action(self, enforce_new_defaults):
+        action = "example:only_system_admin_allowed"
+        cfg.CONF.set_override(
+            'enforce_new_defaults', enforce_new_defaults, group='oslo_policy')
+        project_admin_ctx = context.Context(
+            user="fake", project_id="fake",
+            roles=['admin', 'member', 'reader'])
+        system_admin_ctx = context.Context(
+            user="fake",
+            roles=['admin', 'member', 'reader'],
+            system_scope='all')
+        if not enforce_new_defaults:
+            system_admin_ctx.project_id = 'fake'
+        self.assertTrue(policy.check(system_admin_ctx, action, self.target))
+        if enforce_new_defaults:
+            self.assertFalse(
+                policy.check(project_admin_ctx, action, self.target))
+        else:
+            self.assertTrue(
+                policy.check(project_admin_ctx, action, self.target))
+
+    def test_check_only_system_admin_new_defaults(self):
+        self._test_check_system_admin_allowed_action(enforce_new_defaults=True)
+
+    def test_check_only_system_admin_old_defaults(self):
+        self._test_check_system_admin_allowed_action(
+            enforce_new_defaults=False)
+
+    def _test_enforce_system_admin_allowed_action(self, enforce_new_defaults):
+        action = "example:only_system_admin_allowed"
+        cfg.CONF.set_override(
+            'enforce_new_defaults', enforce_new_defaults, group='oslo_policy')
+        project_admin_ctx = context.Context(
+            user="fake", project_id="fake",
+            roles=['admin', 'member', 'reader'])
+        system_admin_ctx = context.Context(
+            user="fake",
+            roles=['admin', 'member', 'reader'],
+            system_scope='all')
+        if not enforce_new_defaults:
+            system_admin_ctx.project_id = 'fake'
+        self.assertTrue(policy.enforce(system_admin_ctx, action, self.target))
+        if enforce_new_defaults:
+            self.assertRaises(
+                oslo_policy.PolicyNotAuthorized,
+                policy.enforce, project_admin_ctx, action, self.target)
+        else:
+            self.assertTrue(
+                policy.enforce(project_admin_ctx, action, self.target))
+
+    def test_enforce_only_system_admin_new_defaults(self):
+        self._test_enforce_system_admin_allowed_action(
+            enforce_new_defaults=True)
+
+    def test_enforce_only_system_admin_old_defaults(self):
+        self._test_enforce_system_admin_allowed_action(
+            enforce_new_defaults=False)
+
     def test_enforce_nonexistent_action_throws(self):
         action = "example:noexist"
         self.assertRaises(oslo_policy.PolicyNotAuthorized, policy.enforce,
@@ -101,6 +186,19 @@ class PolicyTestCase(base.BaseTestCase):
                                 might_not_exist=True)
         self.assertTrue(result_2)
 
+    def test_check_invalid_scope(self):
+        cfg.CONF.set_override(
+            'enforce_new_defaults', True, group='oslo_policy')
+        cfg.CONF.set_override(
+            'enforce_scope', True, group='oslo_policy')
+        action = "get_example:only_project_user_allowed"
+        target = {'project_id': 'some-project'}
+        system_admin_ctx = context.Context(
+            user="fake",
+            roles=['admin', 'member', 'reader'],
+            system_scope='all')
+        self.assertFalse(policy.check(system_admin_ctx, action, target))
+
     def test_enforce_good_action(self):
         action = "example:allowed"
         result = policy.enforce(self.context, action, self.target)
@@ -121,6 +219,21 @@ class PolicyTestCase(base.BaseTestCase):
                           policy.enforce, self.context,
                           action, target)
 
+    def test_enforce_invalid_scope(self):
+        cfg.CONF.set_override(
+            'enforce_new_defaults', True, group='oslo_policy')
+        cfg.CONF.set_override(
+            'enforce_scope', True, group='oslo_policy')
+        action = "get_example:only_project_user_allowed"
+        target = {'project_id': 'some-project'}
+        system_admin_ctx = context.Context(
+            user_id="fake",
+            roles=['admin', 'member', 'reader'],
+            system_scope='all')
+        self.assertRaises(
+            oslo_policy.InvalidScope,
+            policy.enforce, system_admin_ctx, action, target)
+
     def test_templatized_enforcement(self):
         target_mine = {'tenant_id': 'fake'}
         target_not_mine = {'tenant_id': 'another'}
@@ -152,7 +265,7 @@ class DefaultPolicyTestCase(base.BaseTestCase):
 
     def setUp(self):
         super(DefaultPolicyTestCase, self).setUp()
-        tmpfilename = self.get_temp_file_path('policy.json')
+        tmpfilename = self.get_temp_file_path('policy.yaml')
         self.rules = {
             "default": '',
             "example:exist": '!',
@@ -182,7 +295,13 @@ FAKE_RESOURCES = {"%ss" % FAKE_RESOURCE_NAME:
                             'validate': {'type:dict':
                                          {'sub_attr_1': {'type:string': None},
                                           'sub_attr_2': {'type:string': None}}}
-                            }},
+                            },
+                   'list_attr': {'allow_post': True,
+                                 'allow_put': True,
+                                 'is_visible': True,
+                                 'default': None,
+                                 'enforce_policy': True
+                                 }},
                   # special plural name
                   "%s" % FAKE_SPECIAL_RESOURCE_NAME.replace('y', 'ies'):
                   {'attr': {'allow_post': True,
@@ -196,34 +315,63 @@ FAKE_RESOURCES = {"%ss" % FAKE_RESOURCE_NAME:
                             }}}
 
 
+class CustomRulesTestCase(base.BaseTestCase):
+
+    def test_field_check__boolean_value(self):
+        check = policy.FieldCheck('field', 'networks:router:external=True')
+        self.assertEqual('networks', check.resource)
+        self.assertEqual('router:external', check.field)
+        # TODO(stephenfin): I expected this to get converted to a boolean :-\
+        self.assertEqual('True', check.value)
+        self.assertIsNone(check.regex)
+
+    def test_field_check__regex_value(self):
+        check = policy.FieldCheck('field', 'port:device_owner=~^network:')
+        self.assertEqual('port', check.resource)
+        self.assertEqual('device_owner', check.field)
+        self.assertEqual('~^network:', check.value)
+        self.assertEqual(re.compile('^network:'), check.regex)
+
+    def test_field_check_deepcopy(self):
+        check_a = policy.FieldCheck('field', 'port:device_owner=~^network:')
+        check_b = copy.deepcopy(check_a)
+
+        self.assertIsNot(check_a, check_b)
+        self.assertEqual(check_a.resource, check_b.resource)
+        self.assertEqual(check_a.field, check_b.field)
+        self.assertEqual(check_a.value, check_b.value)
+        self.assertEqual(check_a.regex, check_b.regex)
+
+    def test_owner_check_deepcopy(self):
+        check_a = policy.OwnerCheck('tenant_id', '%(tenant_id)s')
+        check_b = copy.deepcopy(check_a)
+
+        self.assertIsNot(check_a, check_b)
+        self.assertEqual(check_a.target_field, check_b.target_field)
+
+
 class NeutronPolicyTestCase(base.BaseTestCase):
 
     def fakepolicyinit(self, **kwargs):
-        enf = policy._ENFORCER
-        enf.set_rules(oslo_policy.Rules(self.rules))
+        policy._ENFORCER = oslo_policy.Enforcer(cfg.CONF)
+        policy._ENFORCER.set_rules(oslo_policy.Rules(self.rules))
 
     def setUp(self):
         super(NeutronPolicyTestCase, self).setUp()
-        policy.refresh()
         # Add Fake resources to RESOURCE_ATTRIBUTE_MAP
-        attributes.RESOURCE_ATTRIBUTE_MAP.update(FAKE_RESOURCES)
+        attributes.RESOURCES.update(FAKE_RESOURCES)
         self._set_rules()
 
-        def remove_fake_resource():
-            del attributes.RESOURCE_ATTRIBUTE_MAP["%ss" % FAKE_RESOURCE_NAME]
-
         self.patcher = mock.patch.object(neutron.policy,
                                          'init',
                                          new=self.fakepolicyinit)
         self.patcher.start()
-        self.addCleanup(remove_fake_resource)
+        policy.refresh()
+        self.addCleanup(policy.refresh)
         self.context = context.Context('fake', 'fake', roles=['user'])
         plugin_klass = importutils.import_class(
             "neutron.db.db_base_plugin_v2.NeutronDbPluginV2")
-        self.manager_patcher = mock.patch('neutron.manager.NeutronManager')
-        fake_manager = self.manager_patcher.start()
-        fake_manager_instance = fake_manager.return_value
-        fake_manager_instance.plugin = plugin_klass()
+        directory.add_plugin(plugin_constants.CORE, plugin_klass())
 
     def _set_rules(self, **kwargs):
         rules_dict = {
@@ -250,6 +398,14 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             "create_port:mac": "rule:admin_or_network_owner or "
                                "rule:context_is_advsvc",
             "create_port:device_owner": "not rule:network_device",
+            "create_port:fixed_ips": (
+                "rule:context_is_advsvc or rule:admin_or_network_owner or "
+                "rule:shared"),
+            "create_port:fixed_ips:ip_address": (
+                "rule:context_is_advsvc or rule:admin_or_network_owner"),
+            "create_port:fixed_ips:subnet_id": (
+                "rule:context_is_advsvc or rule:admin_or_network_owner or "
+                "rule:shared"),
             "update_port": "rule:admin_or_owner or rule:context_is_advsvc",
             "get_port": "rule:admin_or_owner or rule:context_is_advsvc",
             "delete_port": "rule:admin_or_owner or rule:context_is_advsvc",
@@ -257,39 +413,16 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             "create_fake_resource:attr": "rule:admin_or_owner",
             "create_fake_resource:attr:sub_attr_1": "rule:admin_or_owner",
             "create_fake_resource:attr:sub_attr_2": "rule:admin_only",
+            "create_fake_resource:list_attr": "rule:admin_only_or_owner",
+            "create_fake_resource:list_attr:admin_element": "rule:admin_only",
+            "create_fake_resource:list_attr:user_element": (
+                "rule:admin_or_owner"),
 
             "create_fake_policy:": "rule:admin_or_owner",
-            "get_firewall_policy": "rule:admin_or_owner or "
-                            "rule:shared",
-            "get_firewall_rule": "rule:admin_or_owner or "
-                            "rule:shared",
-
-            "insert_rule": "rule:admin_or_owner",
-            "remove_rule": "rule:admin_or_owner",
         }
         rules_dict.update(**kwargs)
         self.rules = oslo_policy.Rules.from_dict(rules_dict)
 
-    def test_firewall_policy_insert_rule_with_admin_context(self):
-        action = "insert_rule"
-        target = {}
-        result = policy.check(context.get_admin_context(), action, target)
-        self.assertTrue(result)
-
-    def test_firewall_policy_insert_rule_with_owner(self):
-        action = "insert_rule"
-        target = {"tenant_id": "own_tenant"}
-        user_context = context.Context('', "own_tenant", roles=['user'])
-        result = policy.check(user_context, action, target)
-        self.assertTrue(result)
-
-    def test_firewall_policy_remove_rule_without_admin_or_owner(self):
-        action = "remove_rule"
-        target = {"firewall_rule_id": "rule_id", "tenant_id": "tenantA"}
-        user_context = context.Context('', "another_tenant", roles=['user'])
-        result = policy.check(user_context, action, target)
-        self.assertFalse(result)
-
     def _test_action_on_attr(self, context, action, obj, attr, value,
                              exception=None, **kwargs):
         action = "%s_%s" % (action, obj)
@@ -303,10 +436,10 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             result = policy.enforce(context, action, target)
             self.assertTrue(result)
 
-    def _test_nonadmin_action_on_attr(self, action, attr, value,
+    def _test_nonadmin_action_on_attr(self, action, obj, attr, value,
                                       exception=None, **kwargs):
         user_context = context.Context('', "user", roles=['user'])
-        self._test_action_on_attr(user_context, action, "network", attr,
+        self._test_action_on_attr(user_context, action, obj, attr,
                                   value, exception, **kwargs)
 
     def _test_advsvc_action_on_attr(self, action, obj, attr, value,
@@ -317,15 +450,16 @@ class NeutronPolicyTestCase(base.BaseTestCase):
                                   value, exception, **kwargs)
 
     def test_nonadmin_write_on_private_fails(self):
-        self._test_nonadmin_action_on_attr('create', 'shared', False,
-                                           oslo_policy.PolicyNotAuthorized)
+        self._test_nonadmin_action_on_attr(
+            'create', 'network', 'shared', False,
+            oslo_policy.PolicyNotAuthorized)
 
     def test_nonadmin_read_on_private_fails(self):
-        self._test_nonadmin_action_on_attr('get', 'shared', False,
+        self._test_nonadmin_action_on_attr('get', 'network', 'shared', False,
                                            oslo_policy.PolicyNotAuthorized)
 
     def test_nonadmin_write_on_shared_fails(self):
-        self._test_nonadmin_action_on_attr('create', 'shared', True,
+        self._test_nonadmin_action_on_attr('create', 'network', 'shared', True,
                                            oslo_policy.PolicyNotAuthorized)
 
     def test_create_port_device_owner_regex(self):
@@ -344,6 +478,45 @@ class NeutronPolicyTestCase(base.BaseTestCase):
                 'create', 'port', 'device_owner', val
             )
 
+    def test_create_port_fixed_ips_on_shared_network(self):
+
+        def fakegetnetwork(*args, **kwargs):
+            return {'tenant_id': 'fake',
+                    'shared': True}
+
+        kwargs = {'network_id': _uuid()}
+        with mock.patch.object(directory.get_plugin(),
+                               'get_network', new=fakegetnetwork):
+            self._test_nonadmin_action_on_attr(
+                'create', 'port',
+                'fixed_ips', [{'subnet_id': 'test-subnet-id'}],
+                **kwargs)
+            self._test_nonadmin_action_on_attr(
+                'create', 'port',
+                'fixed_ips', [{'ip_address': '1.2.3.4'}],
+                exception=oslo_policy.PolicyNotAuthorized,
+                **kwargs)
+
+    def test_create_port_fixed_ips_on_nonshared_network(self):
+
+        def fakegetnetwork(*args, **kwargs):
+            return {'tenant_id': 'fake',
+                    'shared': False}
+
+        kwargs = {'network_id': _uuid()}
+        with mock.patch.object(directory.get_plugin(),
+                               'get_network', new=fakegetnetwork):
+            self._test_nonadmin_action_on_attr(
+                'create', 'port',
+                'fixed_ips', [{'subnet_id': 'test-subnet-id'}],
+                exception=oslo_policy.PolicyNotAuthorized,
+                **kwargs)
+            self._test_nonadmin_action_on_attr(
+                'create', 'port',
+                'fixed_ips', [{'ip_address': '1.2.3.4'}],
+                exception=oslo_policy.PolicyNotAuthorized,
+                **kwargs)
+
     def test_advsvc_get_network_works(self):
         self._test_advsvc_action_on_attr('get', 'network', 'shared', False)
 
@@ -358,7 +531,7 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         self._test_advsvc_action_on_attr('get', 'port', 'shared', False)
 
     def test_advsvc_update_port_works(self):
-        kwargs = {n_const.ATTRIBUTES_TO_UPDATE: ['shared']}
+        kwargs = {constants.ATTRIBUTES_TO_UPDATE: ['shared']}
         self._test_advsvc_action_on_attr('update', 'port', 'shared', True,
                                          **kwargs)
 
@@ -370,38 +543,7 @@ class NeutronPolicyTestCase(base.BaseTestCase):
                                          oslo_policy.PolicyNotAuthorized)
 
     def test_nonadmin_read_on_shared_succeeds(self):
-        self._test_nonadmin_action_on_attr('get', 'shared', True)
-
-    def test_check_is_admin_with_admin_context_succeeds(self):
-        admin_context = context.get_admin_context()
-        # explicitly set roles as this test verifies user credentials
-        # with the policy engine
-        admin_context.roles = ['admin']
-        self.assertTrue(policy.check_is_admin(admin_context))
-
-    def test_check_is_admin_with_user_context_fails(self):
-        self.assertFalse(policy.check_is_admin(self.context))
-
-    def test_check_is_admin_with_no_admin_policy_fails(self):
-        del self.rules[policy.ADMIN_CTX_POLICY]
-        admin_context = context.get_admin_context()
-        self.assertFalse(policy.check_is_admin(admin_context))
-
-    def test_check_is_advsvc_with_admin_context_fails(self):
-        admin_context = context.get_admin_context()
-        self.assertFalse(policy.check_is_advsvc(admin_context))
-
-    def test_check_is_advsvc_with_svc_context_succeeds(self):
-        svc_context = context.Context('', 'svc', roles=['advsvc'])
-        self.assertTrue(policy.check_is_advsvc(svc_context))
-
-    def test_check_is_advsvc_with_no_advsvc_policy_fails(self):
-        del self.rules[policy.ADVSVC_CTX_POLICY]
-        svc_context = context.Context('', 'svc', roles=['advsvc'])
-        self.assertFalse(policy.check_is_advsvc(svc_context))
-
-    def test_check_is_advsvc_with_user_context_fails(self):
-        self.assertFalse(policy.check_is_advsvc(self.context))
+        self._test_nonadmin_action_on_attr('get', 'network', 'shared', True)
 
     def _test_enforce_adminonly_attribute(self, action, **kwargs):
         admin_context = context.get_admin_context()
@@ -415,14 +557,15 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         self._test_enforce_adminonly_attribute('create_network')
 
     def test_enforce_adminonly_attribute_update(self):
-        kwargs = {n_const.ATTRIBUTES_TO_UPDATE: ['shared']}
+        kwargs = {constants.ATTRIBUTES_TO_UPDATE: ['shared']}
         self._test_enforce_adminonly_attribute('update_network', **kwargs)
 
     def test_reset_adminonly_attr_to_default_fails(self):
-        kwargs = {n_const.ATTRIBUTES_TO_UPDATE: ['shared']}
-        self._test_nonadmin_action_on_attr('update', 'shared', False,
-                                           oslo_policy.PolicyNotAuthorized,
-                                           **kwargs)
+        kwargs = {constants.ATTRIBUTES_TO_UPDATE: ['shared']}
+        self._test_nonadmin_action_on_attr(
+            'update', 'network', 'shared', False,
+            oslo_policy.PolicyNotAuthorized,
+            **kwargs)
 
     def test_enforce_adminonly_attribute_nonadminctx_returns_403(self):
         action = "create_network"
@@ -463,6 +606,12 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         result = policy._build_match_rule(action, target, None)
         self.assertEqual("rule:" + action, str(result))
 
+    def test_build_match_rule_normal_pluralized_when_update(self):
+        action = "update_" + FAKE_RESOURCE_NAME
+        target = {}
+        result = policy._build_match_rule(action, target, None)
+        self.assertEqual("rule:" + action, str(result))
+
     def test_enforce_subattribute(self):
         action = "create_" + FAKE_RESOURCE_NAME
         target = {'tenant_id': 'fake', 'attr': {'sub_attr_1': 'x'}}
@@ -490,18 +639,6 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         result = policy.enforce(self.context, action, target)
         self.assertTrue(result)
 
-    def test_enforce_firewall_policy_shared(self):
-        action = "get_firewall_policy"
-        target = {'shared': True, 'tenant_id': 'somebody_else'}
-        result = policy.enforce(self.context, action, target)
-        self.assertTrue(result)
-
-    def test_enforce_firewall_rule_shared(self):
-        action = "get_firewall_rule"
-        target = {'shared': True, 'tenant_id': 'somebody_else'}
-        result = policy.enforce(self.context, action, target)
-        self.assertTrue(result)
-
     def test_enforce_tenant_id_check(self):
         # Trigger a policy with rule admin_or_owner
         action = "create_network"
@@ -515,7 +652,7 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             return {'tenant_id': 'fake'}
 
         action = "create_port:mac"
-        with mock.patch.object(manager.NeutronManager.get_instance().plugin,
+        with mock.patch.object(directory.get_plugin(),
                                'get_network', new=fakegetnetwork):
             target = {'network_id': 'whatever'}
             result = policy.enforce(self.context, action, target)
@@ -530,7 +667,7 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         # so long that we verify that, if *f* blows up, the behavior of the
         # policy engine to propagate the exception is preserved
         action = "create_port:mac"
-        with mock.patch.object(manager.NeutronManager.get_instance().plugin,
+        with mock.patch.object(directory.get_plugin(),
                                'get_network', new=fakegetnetwork):
             target = {'network_id': 'whatever'}
             self.assertRaises(NotImplementedError,
@@ -539,10 +676,27 @@ class NeutronPolicyTestCase(base.BaseTestCase):
                               action,
                               target)
 
+    def test_enforce_subattribute_as_list(self):
+        action = "create_" + FAKE_RESOURCE_NAME
+        target = {
+            'tenant_id': 'fake',
+            'list_attr': [{'user_element': 'x'}]}
+        result = policy.enforce(self.context,
+                                action, target, None)
+        self.assertTrue(result)
+
+    def test_enforce_subattribute_as_list_forbiden(self):
+        action = "create_" + FAKE_RESOURCE_NAME
+        target = {
+            'tenant_id': 'fake',
+            'list_attr': [{'admin_element': 'x'}]}
+        self.assertRaises(oslo_policy.PolicyNotAuthorized, policy.enforce,
+                          self.context, action, target, None)
+
     def test_retryrequest_on_notfound(self):
         failure = exceptions.NetworkNotFound(net_id='whatever')
         action = "create_port:mac"
-        with mock.patch.object(manager.NeutronManager.get_instance().plugin,
+        with mock.patch.object(directory.get_plugin(),
                                'get_network', side_effect=failure):
             target = {'network_id': 'whatever'}
             try:
@@ -560,7 +714,7 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             admin_or_network_owner="role:admin or "
                                    "tenant_id:%(network_tenant_id)s")
         action = "create_port:mac"
-        with mock.patch.object(manager.NeutronManager.get_instance().plugin,
+        with mock.patch.object(directory.get_plugin(),
                                'get_network', new=fakegetnetwork):
             target = {'network_id': 'whatever'}
             result = policy.enforce(self.context, action, target)
@@ -573,6 +727,17 @@ class NeutronPolicyTestCase(base.BaseTestCase):
             oslo_policy.Rules.from_dict,
             {'test_policy': 'tenant_id:(wrong_stuff)'})
 
+    def test_tenant_id_check_caches_extracted_fields(self):
+
+        plugin = directory.get_plugin()
+        with mock.patch.object(plugin, 'get_network',
+                               return_value={'tenant_id': 'fake'}) as getter:
+            action = "create_port:mac"
+            for i in range(2):
+                target = {'network_id': 'whatever'}
+                policy.enforce(self.context, action, target)
+        self.assertEqual(1, getter.call_count)
+
     def _test_enforce_tenant_id_raises(self, bad_rule):
         self._set_rules(admin_or_owner=bad_rule)
         # Trigger a policy with rule admin_or_owner
@@ -656,3 +821,26 @@ class NeutronPolicyTestCase(base.BaseTestCase):
         result = policy._is_attribute_explicitly_set(
             attr, resource, target, action)
         self.assertFalse(result)
+
+    @mock.patch("neutron_lib.services.constants.EXT_PARENT_RESOURCE_MAPPING",
+                {'parentresource': 'registered_plugin_name'})
+    @mock.patch("neutron_lib.plugins.directory.get_plugin")
+    def test_enforce_tenant_id_check_parent_resource_owner(
+            self, mock_get_plugin):
+
+        def fakegetparent(*args, **kwargs):
+            return {'tenant_id': 'fake'}
+        mock_plugin = mock.Mock()
+        mock_plugin.get_parentresource = fakegetparent
+        mock_get_plugin.return_value = mock_plugin
+
+        self._set_rules(
+            admin_or_ext_parent_owner="rule:context_is_admin or "
+                                      "tenant_id:%(ext_parent:tenant_id)s",
+            create_parentresource_subresource="rule:admin_or_ext_parent_owner")
+        self.fakepolicyinit()
+        action = 'create_parentresource_subresource'
+        target = {'ext_parent_parentresource_id': 'whatever', 'foo': 'bar'}
+        result = policy.enforce(self.context, action, target)
+        mock_get_plugin.assert_called_with('registered_plugin_name')
+        self.assertTrue(result)
